#|
	Might use this as the base of a power method some day.... ael
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 eigen-power
;;
(defun eigen-power(a
		 &key
		 (its 100)
		 lambda
		 (eps 1e-12)
		 (its 30)
		 verbose
		 power
		 )
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          eigen-power Doc:
;;
;; Author: Andy Long
;; Created: Wed Nov 26 11:21:05 1997
;;
;; Description/Arguments:

       (eigen-power matrix
		 &key
		 (its 100)
		 lambda
		 (eps 1e-12)
		 (its 30)
		 verbose
		 power
		 )
  
	This is a routine for generating the largest and smallest eigenvalues
and associated eigenvectors of a general nxn matrix.

;; Returns: a list of two lists: the eigenvalues, and the eigenvectors.

;; Notes:
	Uses a power method (described in Numerical Recipes) to compute
the eigenvectors.

;; See Also: eigen, eigenvalues, eigenvectors, generalized-eigenvectors

;; Examples:
 ;; Strang's Introduction to Applied Mathematics:
 (def strang (matrix '(3 3) '(1 -1 0 -1 2 -1 0 -1 1)))
 ;; eigenvalues: 0, 1, 3
 (setq eig (eigen-power strang :verbose t))

 (def strang (matrix '(2 2) '(0 -1 1 0)))
 ;; eigenvalues: i, -i
 (setq eig (eigen-power strang :verbose t))

 
"
  (let* (
	 (n (first (size a)))
	 (id (id n))
	 (b 
	  (apply
	   #'bind-columns
	   (mapcar (lambda(x) (normalize (- (* 2 (rand.u n)) 1))) (iseq n))))
	 eigenvectors
	 norm
	 )
    (dolist
     (i '(1))
     (let* (
	    (p a)
	    (y (normalize 
		(mapcar #'mean 
			(mapcar #'normalize
				(column-list (matmult p b))))))
	    (change 1e31)
	    )
       (do
	(
	 (i 0 (1+ i))
	 (yold y y)
	 )
	((or (> i its) (< change eps)))
	(setq
	 y (matmult p y)
	 norm (norm y)
	 y (/ y norm)
	 change (norm (- y yold))
	 )
	)
       (setq eigenvectors (append eigenvectors (list y)))
       )
     )
    (list norm eigenvectors)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
|#
