#|

                   Section 8.6 of Luke's book:

	I find this bitmap-editor useful for making toys for my neural nets to
munch on.
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          bitmap-edit-proto
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defproto bitmap-edit-proto '(bitmap h v) nil graph-window-proto)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :isnew
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth bitmap-edit-proto :isnew (width height)
  (call-next-method)
  (setf (slot-value 'bitmap)
        (make-array (list height width) :initial-element 0)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                     accessors for :bitmap, :v, :h 
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth bitmap-edit-proto :bitmap (&optional (bitmap nil set) )
  (if set (setf (slot-value 'bitmap) bitmap) (slot-value 'bitmap)))
(defmeth bitmap-edit-proto :v () (slot-value 'v))
(defmeth bitmap-edit-proto :h () (slot-value 'h))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :resize
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth bitmap-edit-proto :resize ()
  (let ((m (array-dimension (send self :bitmap) 0))
        (n (array-dimension (send self :bitmap) 1))
        (height (send self :canvas-height))        
        (width (send self :canvas-width)))
    (setf (slot-value 'v)
          (coerce (floor (* (Iseq 0 m) (/ height m)))
                  'vector))
    (setf (slot-value 'h)
          (coerce (floor (* (Iseq 0 n) (/ width n)))
                  'vector))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :draw-pixel
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth bitmap-edit-proto :draw-pixel (i j)
  (let* ((b (send self :bitmap))
         (v (send self :v))
         (h (send self :h))
         (left (aref h j))
         (right (aref h (+ j 1)))
         (top (aref v i))
         (bottom (aref v (+ i 1))))
    (send self (if (= 1 (aref b i j)) :paint-rect :erase-rect)
          left top (- right left) (- bottom top))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :redraw
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth bitmap-edit-proto :redraw ()
  (let* ((b (send self :bitmap))
         (m (array-dimension b 0))
         (n (array-dimension b 1))
         (width (send self :canvas-width))
         (height (send self :canvas-height)))
    (send self :start-buffering)
    (send self :erase-rect 0 0 width height)
    (dotimes (i m)
             (dotimes (j n)
                      (send self :draw-pixel i j)))
    (send self :buffer-to-screen)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :set-pixel
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth bitmap-edit-proto :set-pixel (x y)
  (let* ((b (send self :bitmap))
         (m (array-dimension b 0))
         (n (array-dimension b 1))
         (width (send self :canvas-width))
         (height (send self :canvas-height))
         (i (min (floor (* y (/ m height))) (- m 1)))
         (j (min (floor (* x (/ n width))) (- n 1))))
    (setf (aref b i j) (if (= (aref b i j) 1) 0 1))
    (send self :draw-pixel i j)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :do-click
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth bitmap-edit-proto :do-click (x y m1 m2)
  (send self :set-pixel x y))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :do-key
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth bitmap-edit-proto :do-key (c m1 m2)
"
  The following keys are currently defined: typing them while a scatterplot is
active will give the following results:

			q - remove the plot 
			h - help (this message)
"
  (case c
	((or #\h #\H)
	 (princ
"
  The following keys are currently defined: typing them while a scatterplot is
active will give the following results:

			q - remove the plot 
			h - help (this message)
")
	 )
	((or #\q #\Q) (send self :remove))
	(#\v (let((size (send self :size)))
	       (send self :size
		     (floor (* .9 (first size)))
		     (floor (* .9 (second size))))))
	(#\V (let((size (send self :size)))
	       (send self :size
		     (floor (* 1.1 (first size)))
		     (floor (* 1.1 (second size))))))

	)
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :name-bitmap
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth bitmap-edit-proto :name-bitmap ()
  (let ((str (get-string-dialog "Symbol for the bitmap:")))
    (if str
        (let ((name (with-input-from-string (s str) (read s))))
          (setf (symbol-value name) (send self :bitmap))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        :bitmap-as-cursor
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth bitmap-edit-proto :bitmap-as-cursor (yes)
  (if yes (make-cursor 'temp-cursor (send self :bitmap)))
  (send self :cursor (if yes 'temp-cursor 'arrow)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                bitmap
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun bitmap(&key (rows 16) (columns 16))
  "
;; Description: function bitmap(&key (rows 16) (columns 16))

Bitmap maker, from Luke's book 'LispStat'.

;; See also: show-bitmap

;; Examples:
 (bitmap)
 (bitmap :rows 5 :columns 5)
"
  (let ((w (send bitmap-edit-proto :new rows columns)))
    (setf bitmap-edit-menu (send menu-proto :new "Bitmap"))
    (setf name-item
	  (send menu-item-proto :new "Name Bitmap..."
		:action #'(lambda () (send w :name-bitmap))))
    (setf cursor-item
	  (send menu-item-proto :new "Use as Cursor"
		:action
		#'(lambda ()
		    (let ((mark (send cursor-item :mark)))
		      (send w :bitmap-as-cursor (not mark))
		      (send cursor-item :mark (not mark))))))
    (send bitmap-edit-menu :append-items name-item cursor-item)
    (send w :menu bitmap-edit-menu)
    (if *title* (send w :title *title*))
    w
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 show-bitmap
;;
(defun show-bitmap(bitmap)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          show-bitmap Doc:
;;
;; Author: Andy Long
;; Created: Sun Nov 30 21:44:18 1997
;;
;; Description/Arguments:

       (show-bitmap bitmap)
  
;; Returns:

;; Notes:

;; See also: bitmap

;; Examples:
 (show-bitmap (id 5))
"
  (let* (
	 (size (size bitmap))
	 (rows (first size))
	 (columns (second size))
	 (w (send bitmap-edit-proto :new rows columns))
	 )
    (setf bitmap-edit-menu (send menu-proto :new "Bitmap"))
    (setf name-item
	  (send menu-item-proto :new "Name Bitmap..."
		:action #'(lambda () (send w :name-bitmap))))
    (setf cursor-item
	  (send menu-item-proto :new "Use as Cursor"
		:action
		#'(lambda ()
		    (let ((mark (send cursor-item :mark)))
		      (send w :bitmap-as-cursor (not mark))
		      (send cursor-item :mark (not mark))))))
    (send bitmap-edit-menu :append-items name-item cursor-item)
    (send w :bitmap (signum bitmap))
    (send w :menu bitmap-edit-menu)
    (if *title* (send w :title *title*))
    w
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
