;;;;
;;;; help.lsp XLISP-STAT help functions
;;;; XLISP-STAT 2.1 Copyright (c) 1990, by Luke Tierney
;;;; Additions to Xlisp 2.1, Copyright (c) 1989 by David Michael Betz
;;;; You may give out copies of this software; for conditions see the file
;;;; COPYING included with this distribution.
;;;;

;;; This I (ael) stole from the above, and remade in my own image.

;;;;
;;;; Help Functions
;;;;
     
;; ael: I've changed help from a function to a macro, so that you don't have to
;; quote names as you hand them to it. It also will handle multiple names at
;; once. It would be even better if given names with asterix in them it would
;; do a help*, but heh.

(defmacro help (&rest s)
  "
       Help provides some information about how to use many of the commands
found in xlisp-stat. In many cases, examples are provided so that you can try a
particular command. Another useful command is about, e.g.

                           (about help)

which will give you a list of commands containing the word \"help\".

       If you're new to xlisp-stat, you may want to try a tutorial or two. Type

                           (tutorials)

for your options. These files are in the tutorials subdirectory of the calculus
subdirectory of the main xlisp-stat directory. Have a look at them to see how
any one of them works. Then create your own!
"
  (let (is-doc)
    (if s
	(dolist (i s is-doc)
		;; handling the case where someone wants to use it in the
		;; old-fashioned way
		(setq is-doc (if (listp i) (helpless (second i)) (helpless i)))
		)
      (helpless)
      )
    )
  )

(setq *line-length* 80) ;; I cannot tell a lie: I set this. ael.

#|
    (princ "
        For an introduction to the features of Andy Long's modified xlispstat
distribution, you might try a tutorial or two. Type

                           (tutorials)

for your options. These files are in the tutorials subdirectory of the calculus
subdirectory of the main xlisp-stat directory. Have a look at them to see how
any one of them works. Then create your own!

        If you have further questions, or want more information, contact me at

                     ael@maya.medctr.luc.edu

Good luck, have fun, and let me know what you do or don't like about this
software. I may modify it to suit you better!

                                                    Andy Long
                                                    November, 1997
")
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                helpless
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun helpless (&optional s (stream t))
  "
;; Description: function helpless

Args: (&optional symbol (stream t))
Prints the documentation associated with SYMBOL.  With no argument, 
this function calls hulp, the menu system for help."
  (cond 
   ((null s)
    (hulp)
    )
   (t (let ((docf (documentation s 'function))
	    (docv (documentation s 'variable))
	    (doct (documentation s 'type))
	    (docs (documentation s 'setf)))
	(if (or docf docv doct docs)
	    (flet ((put-doc (sym type str)
			    (format stream "~s" sym)
			    (dotimes (i (- *line-length*
					   (length (string sym))
					   (length (string type))))
				     (format stream " "))
			    (format stream "~a~%~a~%" type str)
			    ))
		  (if docf (put-doc s "[function-doc]" docf))
		  (if docv (put-doc s "[variable-doc]" docv))
		  (if doct (put-doc s "[type-doc]" doct))
		  (if docs (put-doc s "[setf-doc]" docs))
		  t
		  )
	  (let ()
	    (find-and-show-substring (string-downcase (symbol-name s)))
	    nil
	    )
	  )
	)
      )
   )
  )


(defun help* (sl)
"Args: (string)
Prints the documentation associated with those symbols whose print names
contain STRING as substring.  STRING may be a symbol, in which case the
print-name of that symbol is used."
  (dotimes (i *line-length*) (princ "-"))
  (terpri)
  (dolist (s (mapcar #'intern 
                     (sort-data (mapcar #'string (apropos-list sl)))))
          (eval `(help ,s))
          (dotimes (i *line-length*) (princ "-"))
          (terpri)))
