#|
	This is the first version, which I've updated below....


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 postscript
;;
(defun postscript(filename
		  &key plot (bulsize 1) width height no-axes no-box axes-box
		  (fmt "~,3,3g")
		  )
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; postscript Doc:    Creation Date:1/27/97        Author: Andy Long
;;
;; Description/Arguments:

       (postscript filename
		  &key plot (bulsize 1) width height no-axes no-box axes-box
		       (fmt \"~,3,3g\")
		  )
  
	Requires:
		filename (a string).

	Options:
		plot is a plot-name, if you don't want to use the most recent;
		bulsize is the size of points;
		width is the width of the postscript window;
		height is the height of the postscript window;
		no-axes is true if you want to can the axes;
		no-box is true if you want to can the box;
		axes-box is true if you want the axes part of the box;
		fmt is a format for the axis labels (defaults to \"~,3,3g\").

;; Notes:

;; See Also: postscript-3d

;; Examples:
 (new-plot)
 (plot sin)
 (postscript \"postscript.example0.ps\")
 (plot (sample (iseq 100) 10) (sample (iseq 100) 10))
 (postscript \"postscript.example1.ps\")
 (plot (rand.u 10) (rand.u 10))
 (postscript \"postscript.example2.ps\")
"
  (let* (
	 (plot (if plot plot (first (agw))))
	 (f (open filename :direction :output))
	 (stuff (send plot :plot-item))
	 (variable-labels (send plot :variable-label '(0 1)))
	 (width (if width width 400))
	 (height (if height height 400))
	 (domain (send plot :range 0))
	 (dmin (first domain))
	 (dmax (second domain))
	 (ddiff (- dmax dmin))
	 (range (send plot :range 1))
	 (rmin (first range))
	 (rmax (second range))
	 (rdiff (- rmax rmin))
	 started bullet-size
	 lineson linewidth linetype point-labels plot-symbol
	 text
	 (tinyx (/ ddiff 100))
	 (tinyy (/ rdiff 100))
	 (labels-showing (send plot :showing-labels))
	 turn-me-off
	 )
    (format f "%!PS 
%%BoundingBox: 0 0 ~d ~d
/m {moveto} def
/l {lineto} def
/r {rlineto} def
/s {stroke} def
/n {newpath} def
/c {closepath} def
2 setlinejoin
/rtext{dup stringwidth pop -1 div 0 rmoveto show} def
/ltext{show} def
/ctext{dup stringwidth pop -2 div 0 rmoveto show} def
/1b{1 0 360 arc c fill} def
/2b{2 0 360 arc c fill} def
/3b{3 0 360 arc c fill} def
/4b{4 0 360 arc c fill} def
/5b{5 0 360 arc c fill} def
/6b{6 0 360 arc c fill} def
1.000000 1.000000 scale
/Courier-Bold findfont 14 scalefont setfont
"
	    (+ 100 width) (+ 100 height)
	    )
    (if no-box
	(if axes-box
	    (format f "
s n
100 100 m
100 ~d l
100 100 m
~d 100 l
s n
" 
		    (+ 100 width) (+ 100 height)
		    )
	  )
      (format f "
s n
100 100 m
100 ~d l
100 ~d m
~d ~d l
~d ~d m
~d 100 l
~d 100 m
100 100 l
s n
" 
	      (+ 100 width) (+ 100 width)
	      (+ 100 width) (+ 100 height) (+ 100 width) (+ 100 height)
	      (+ 100 height) (+ 100 height)
	      )
      )
    (if no-axes ()
      (let ()
	;; Plot the y-axis if visible:
	(if (and (< 0 dmax) (> 0 dmin))
	    (setq stuff (append stuff
				(list
				 (list 'lines '(0 0) (list rmin rmax))
				 ))))
	
	;; Plot the x-axis if visible:
	(if (and (< 0 rmax) (> 0 rmin))
	    (setq stuff (append stuff
				(list
				 (list 'lines (list dmin dmax) '(0 0))
				 ))))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                axis tic marks:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	(setq stuff (append stuff
			    (mapcar
			     #'(lambda(x)
				 (list 'lines (list dmin (- dmin tinyx))
				       (list x x))
				 )
			     (rseq rmin rmax *y-tics*)
			     )))
	(setq stuff
	      (append stuff
		      (mapcar
		       #'(lambda(x)
			   (list 'lines (list x x) (list rmin (- rmin tinyy)))
			   )
		       (rseq dmin dmax *x-tics*)
		       )))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                axis tic labels:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	(setq stuff (append stuff
			    (mapcar
			     #'(lambda(x)
				 (list
				  (format nil fmt x)
				  (- dmin (* 5 tinyx))
				  (- x (* tinyy (size (format nil fmt x))))
				  90
				  ))
			     (rseq rmin rmax *y-tics*)
			     )))
	(setq stuff (append stuff
			    (mapcar
			     #'(lambda(x)
				 (list
				  (format nil fmt x)
				  (- x (* tinyx (size (format nil fmt x))))
				  (- rmin (* 5 tinyy))
				  ))
			     (rseq dmin dmax *x-tics*)
			     )))
	)
      )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                          variable labels:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (if variable-labels
	(setq stuff
	      (append
	       stuff
	       (list
		(list (first variable-labels)
		      (+ dmin (half ddiff)
			 (- (* tinyx (size (first variable-labels)))))
		      (- rmin (* 10 tinyy))
		      )
		(list (second variable-labels)
		      (- dmin (* 10 tinyx))
		      (+ rmin (half rdiff)
			 (- (* tinyy (size (second variable-labels))))
			 )
		      90
		      )
		)
	       )
	      )
      )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                Usual plot-item stuff, now:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    (dolist
     (i stuff)
     (setq bullet-size bulsize)
     (setq lineson *lines*)
     (setq point-labels nil)
     (if lineson
	 (let ()
	   (setq linewidth *line-width*)
	   (setq linetype *line-type*)
	   ))
     (setq text nil)
     (cond
      ((or (functionp i) (listp i))
       (cond
	((functionp i)
	 (let ()
	   (setq x (rseq dmin dmax *points*))
	   (setq y (mapcar i x))
	   )
	 )
	((listp (first i))
	 (let ()
	   (setq x (first i))
	   (setq y (second i))
	   )
	 (if (and *point-labels* (not lineson) labels-showing
		  (= (length *point-labels*) (length x))
		  )
	     (let ()
	       (setq point-labels *point-labels*)
	       (setq angle nil)
	       )
	   )
	 )
	((equal (first i) 'lines)
	 (let ()
	   (setq x (second i))
	   (setq y (third i))
	   (setq lineson t)
	   (setq linewidth (if (fifth i) (fifth i) linewidth)) 
	   (setq linetype (if (sixth i) (sixth i) linetype)) 
	   )
	 )
	((equal (first i) 'points)
	 (let ()
	   (setq x (second i))
	   (setq y (third i))
	   (setq lineson nil)
	   (setq plot-symbol (if (fifth i) (fifth i)))
	   (setq bullet-size (if (sixth i) (sixth i) bulsize))
	   )
	 )
	((equal (first i) 'function)
	 (let ()
	   (setq x (rseq dmin dmax *points*))
	   (setq y (mapcar (second i) x))
	   ;; third is color, can't use it yet
	   (if (fourth i) (setq lineson (fourth i)))
	   (setq linewidth (if (fifth i) (fifth i) linewidth)) 
	   (setq linetype (if (sixth i) (sixth i) linetype)) 
	   )
	 )
	((stringp (first i))
	 (let ()
	   (setq x (list (second i)))
	   (setq y (list (third i)))
	   (setq text (first i))
	   (setq lineson t)
	   (setq angle (if (fourth i) (fourth i) nil))
	   )
	 )
	(t (print "Search me!"))
	)
       (setq started nil)
       (dolist
	(j (weave 
	    (mapcar
	     #'(lambda(x) (+ 100 (floor (* width (/ (- x dmin) ddiff)))))
	     x
	     )
	    (mapcar
	     #'(lambda(x) (+ 100 (floor (* height (/ (- x rmin) rdiff)))))
	     y
	     )
	    ))

	#|
	;; What follows is my attempt at trimming the plot. It also trimmed the
	;; axis labels, so I decided to get back to it when I get some time....
	(setq j
	      (list
	       (if (< (first j) 0) 0
		 (if (> (first j) (+ 100 width)) (+ 100 width)
		   (first j)))
	       (if (< (second j) 0) 0
		 (if (> (second j) (+ 100 height)) (+ 100 height)
		   (second j)))
	       )
	      )
|#
	
	(setq j
	      (list
	       (if (> (first j) (+ 100 width)) (+ 100 width) (first j))
	       (if (> (second j) (+ 100 height)) (+ 100 height) (second j))
	       )
	      )

	(if (and lineson (not started) linewidth)
	    (if (or (equal linetype 'solid) (equal linetype 'normal))
		(format f "~%~d setlinewidth~%0 setgray" linewidth)
	      (let ()
		(setq turn-me-off t)
		(format f "~%~d setlinewidth~%.5 setgray" linewidth)
		)
	      )
	  )
	(if started 
	    (if lineson
		(format f
			"~%~d ~d l~%s n ~d ~d m"
			(first j) (second j) (first j) (second j))
	      (let ()
		(format f "~%s n ~d ~d ~db" (first j) (second j) bullet-size)
		(if point-labels
		    (let(
			 (text (first point-labels))
			 )
		      (format f "~%s n ~d ~d m" (first j) (second j))
		      (setq point-labels (circulate point-labels))
		      (if angle
			  (format f "~%~g rotate~%(~a) ltext~%-~g rotate~%"
				  angle text angle)
			(format f "~%(~a) ltext" text)
			)
		      )
		  )
		)
	      )
	  (let ()
	    (if lineson
		(format f "~%s n ~d ~d m" (first j) (second j))
	      (format f "~%s n ~d ~d ~db" (first j) (second j) bullet-size)
	      )
	    (let()
	      (if (or text point-labels)
		  (let()
		    (if point-labels
			(let()
			  (setq text (first point-labels))
			  (setq point-labels (circulate point-labels))
			  ))
		    (format f "~%s n ~d ~d m" (first j) (second j))
		    (if angle
			(format f "~%~g rotate~%(~a) ltext~%-~g rotate~%"
				angle text angle)
		      (format f "~%(~a) ltext" text)
		      )
		    )
		)
	      )
	    )
	  )
	(if turn-me-off
	    (let ()(print "hi")(setq turn-me-off nil)(format f "~%0 setgray~%")))
	(setq started t)
	)
       )
      (t (print "can't handle it yet!"))
      )
     )
    (format f "~%showpage~%")
    (close f)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
|#
