;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                dir
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun dir ()
  "
Dir runs ls from a unix prompt. Reads and lists a directory.
"
  (system "ls")
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                lpq
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun lpq ()
  "
lpq runs lpq from a unix prompt. Checks the print queue.
"
  (system "lpq"))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                lpr
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun lpr (file &optional (like-this "lpr -h "))
  "
;; Description: function  lpr (file &optional (like-this \"lpr -h \"))

lpr runs command 

	lpr -h file

from a unix prompt, where file is given by a string.
"
  (system (strcat like-this file))
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                print-pic
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun print-pic(&key rv portrait (name "print-pic.ps") force)
  "
        print-pic(&key rv portrait (name \"print-pic.ps\") force)
 
	Print-pic sends a picture to the printer for me. It uses
my own stuff, so probably won't work for you!
	If a filename is given, then the file will not be printed.

Set options to t:
	rv - reverse video
	portrait - portrait, rather than landscape
	force - If true, the filename will be overwritten if it exists.

						Andy Long

	print-pic takes an optional filename: if any name other than
print-pic.ps is given, then the file will be saved: otherwise, the
file will be destroyed after it is sent to the printer.
"
  (let (
	(rv (if rv " -rv " ""))
	(portrait (if portrait " -p " ""))
	)
    (if (equal name "print-pic.ps")
	(let ()
	  (system "/bin/rm -f print-pic.ps")
	  (system (strcat "xwdtops " rv portrait name))
	  (system (strcat "lpr -h " name))
	  (system "lpq")
	  (system "/bin/rm -f print-pic.ps")
	  )
      (if (probe-file name)
	  (if force
	      (let ()
		(system (strcat "/bin/rm -f " name))
		(system (strcat "xwdtops " rv portrait name))
		)
	    (let ((temp name))
	      (format
	       t
	       "File \"~a\" exists: type it again or another to continue~%"
	       name)
	      (setq name (read))
	      (if (symbolp name) (setq name (string-downcase (symbol-name
							      name))))
	      (if (equal name temp) (system (strcat "/bin/rm -f " name)))
	      (system (strcat "xwdtops " rv portrait name))
	      )
	    )
	(system (strcat "xwdtops " rv portrait name))
	)
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                edit-pic
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun edit-pic(&key rv (name "edit-pic.ps"))
  "
        edit-pic(&key rv (name \"edit-pic.ps\"))
 
	edit-pic sends a picture to the printer for me. It uses
my own stuff, so probably won't work for you! Set options to t:

	rv - reverse video

xv is used to read in the xwd image: save it as postscript.

						Andy Long

	edit-pic takes an optional filename: if any name other than
edit-pic.ps is given, then the file will be saved: otherwise, the
file will be destroyed after it is sent to the printer.
"
  (let (
	(rv (if rv " -rv " ""))
	)
    (system (strcat "if test -f " name "; then rm -i " name "; fi"))
    (system (strcat "xwd > " name))
    (system (strcat "xv " rv name))
    (system (strcat "lpr -h " name))
    (system "lpq")
    (if (equal name "edit-pic.ps") (system "/bin/rm -f edit-pict.ps"))
    )
  ) 
 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            date
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun date()
  "
;; Documentation: (date)

	Only works on unix systems at present.

;; Examples:
 (date)
"
  (system "date")
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            xv
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun xv(&key (filename "xv-tmp.xwd")
	       (directory "/pme3/ael/public_html/dad/"))
  "
;; Documentation:
 (xv &key (filename \"xv-tmp.xwd\")
	  (directory \"/pme3/ael/public_html/dad/\"))

;; Examples:
 (plot sin)
 (xv :filename \"sin-plot.xwd\" :directory \".\")
"
  (let ((both (strcat directory "/" filename)))
    (system (strcat "/bin/rm -f " both))
    (system (strcat "xwd > " both))
    (system (strcat "cd " directory "; xv " filename " &"))
    (system (strcat "chmod 644 " both))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 gv
;;
(defmacro gv(filename)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; gv Doc:    Creation Date:1/28/97        Author: Andy Long
;;
;; Description/Arguments: macro

       (gv filename)
  
;; Notes: only included under UNIX (#+unix).

;; See Also:

;; Examples:
 ;; create a plot:
 (plot sin)
 (postscript \"test.ps\")
 ;; Now view it using ghostview:
 (gv \"test.ps\")
 (gv test.ps)
"
  (let (
	(xx (if (eval (stringp `,filename)) `,filename
	      (nstring-downcase (symbol-name `,filename))))
	)
    (system 
     (eval `(strcat
	     "for i in " ,xx
	     ";do ghostview -display $DISPLAY -magstep 0 $i;done"
	     ))
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 compile-and-load-code
;;
;; only defined in the unix environment:
(defun compile-and-load-code(filestem &key (compile "cc -I. -c") (linux t))
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          compile-and-load-code Doc:
;;
;; Author: Andy Long
;; Created: Wed Oct 22 15:31:49 1997
;;
;; Description/Arguments:

       (compile-and-load-code filestem &key (compile \"cc -I. -c\") (linux t))
  
	filestem - the filename (with complete relative path information)
	compile  - the compile command that you wish to use
	linux    - the linux key was added because my linux system needs a
		   special ld command to make the object shared.

;; Notes: 

;; See Also: dyn-load

;; Examples:
 (with-open-file
  (f \"foo.c\" :direction :output)
  (format f \"~%
 /*----------------------------------------------------------------------*/~%
 foo(m, n, x, sum)~%
      int *n, *m\\\;~%
      double *x, *sum\\\;~%
 {~%
   int i, j, k\\\;~%
   k=*n\\\;~%
   for (i=0,*sum=0\\\; i < *m\\\; i++)~%
     {~%
       for (j=0\\\; j < *n\\\; j++)~%
 	{~%
 	  *sum += x[i*k+j]\\\;~%
 	}~%
     }~%
 }~%
 ~%
 /* This one allows us to define a local matrix, which would be useful if we~%
 needed to pass a matrix to a C-code: */~%
 fooc(m, n, x, sum)~%
      int *n, *m\\\;~%
      double *x, *sum\\\;~%
 {~%
   int i, j, k\\\;~%
   double xx[*m,*n]\\\;~%
   k=*n\\\;~%
   for (i=0,*sum=0\\\; i < *m\\\; i++)~%
     {~%
       for (j=0\\\; j < *n\\\; j++)~%
 	{~%
 	  xx[i,j] = x[i*k+j]\\\;~%
           *sum += xx[i,j]\\\;~%
 	}~%
     }~%
   printf(\\\"~%
 %f\\\",*sum)\\\;~%
   return *xx\\\;~%
 }~%
 /*----------------------------------------------------------------------*/~%
 \")
  )
 (compile-and-load-code \"./foo\")

 (def mat (matrix '(3 2) (rand.u 6)))
 (def size (size mat))
 (print (call-cfun \"foo\" (first size) (second size) (matrix-to-list mat) 0.0))
 (print (sum mat))
 (print (call-cfun \"fooc\" (first size) (second size) (matrix-to-list mat) 0.0))
"
  (system (strcat compile " " filestem ".c -o " filestem ".o"))
  ;; the following command was added because my linux system appeared to need it:
  (if linux 
      (system (strcat "ld -lm -o " filestem ".so -shared " filestem ".o")))
  (dyn-load (strcat filestem ".so"))
  )
;;
