(defun xgobips(name reds greens blues x y
		    &key
		    (type 'lines)
		    title
		    (title-color '(0 0 0)) ;; black
		    (xlabel "x")
		    (ylabel "y")
		    (axes t)
		    (axis-color '(0 0 0)) ;; black
		    (nxtics 4)
		    (nytics 4)
		    (xtics 
		     (let ((tics (get-nice-range (min x) (max x) nxtics)))
		       (rseq (first tics) (second tics) (third tics))))
		    (ytics
		     (let ((tics (get-nice-range (min y) (max y) nytics)))
		       (rseq (first tics) (second tics) (third tics))))
		    (xtickfmt "~,3g")
		    (ytickfmt "~,3g")
		    (pointsize 11)
		    morelines
		    morelabels
		    points
		    (line-type "ln")
		    xmax
		    ymax
		    xmin
		    ymin
		    yscale
		    )
  "
function xgobips
	Arguments
		name reds greens blues x y
        &key
		title
       	        (title-color '(0 0 0)) ;; black
		(xlabel \"x\")
		(ylabel \"y\")
		(axes t) ;; if t, then axes and labels will be added
		(axis-color '(0 0 0)) ;; black
		(nxtics 4) ;; this number of tics may be overruled by xtics
		(nytics 4) ;; this number of tics may be overruled by ytics
		(xtics 
		     (let ((tics (get-nice-range (min x) (max x) nxtics)))
		       (rseq (first tics) (second tics) (third tics))))
		(ytics
		     (let ((tics (get-nice-range (min y) (max y) nytics)))
		       (rseq (first tics) (second tics) (third tics))))
		(xtickfmt \"~,3g\")
		(ytickfmt \"~,3g\")
		(pointsize 11)
		morelines
		morelabels
		points
		(line-type \"ln\")
		xmax
		ymax
		xmin
		ymin
		yscale

Descriptions:

	name    - file name of postscript output
	reds	- real between 0 and 1: red value
	greens	- real between 0 and 1: green value
	blues	- real between 0 and 1: blue value
	x	- list of x coordinates
	y	- list of y coordinates

	Keys:

	title   - A title for your plot
	title-color	- list of three red green blue values
			  '(0 0 0) (black) by default
	xlabel 	- string label for x axis
	ylabel 	- string label for y axis
	axis-color	- list of three red green blue values
			  '(0 0 0) is black
	nxtics	- number of x tics
	nytics  - number of y tics
	xtics	- a list of tics for the x axis
	ytics	- a list of tics for the y axis
	xtickfmt - format for x tick labels
	ytickfmt - format for y tick labels
	pointsize - size of font used for labels

	line-type - either ln or lndashed

	morelines - a list of lines, containing list of components
			(red green blue xs ys line-type)
	morelabels - a list of labels, containing lists of components
		     of the form
			(string (list red green blue) (list x y))
	points	- a list containing components as follows:
		(reds greens blues glyph-types glyph-sizes x y)
		Glyph Types available:
		      1 plus
		      2 xglyph
		      3 opensquare
		      4 filledsquare
		      5 opencircle
		      6 filledcircle
		      7 point
	xmax	- set the boundaries of the rectangle to show
	ymax
	xmin
	ymin
	yscale - scales the ymin/ymax by a factor (expand, contract about the
		 mean) 

;; Examples:
 (xgobips \"xgobips.ps\"
   1  ;; red   level 
   0  ;; green
   0  ;; blue
   (rand.u 10) (rand.u 10)
   :xtics '(.2 .3 .5)
   )
 (xgobips \"xgobips.ps\"
   1  ;; red   level 
   0  ;; green
   0  ;; blue
   (rand.u 10) (rand.u 10)
   :points (list (list
		(repeat 0 10)
		(repeat 0 10)
		(repeat 1 10)
		(sample (iseq 1 7) 10 t)
		(repeat 2 10)
		(rand.u 10) (rand.u 10)))
   :xtics '(.2 .3 .5)
   )
 (xgobips \"xgobips.ps\"
   1  ;; red   level 
   0  ;; green
   0  ;; blue
   (rand.u 10) (rand.u 10)
   :morelines (list (list 0 0 1 (rand.u 10) (rand.u 10) \"lndashed\"))
   :xtics '(.2 .3 .5)
   :xmin 0
   :xmax 1
   :ymin 0
   :ymax 1
   )
"
  (printall x)

  (if title
      (setq morelabels
	    (if morelabels
		(append morelabels
			(list title title-color (list 300 500)))
	      (list (list title title-color (list 300 500)))
	      )
	    )
    )

  (let* (
	 (n-1 (1- (length x)))
	 (xmin (if xmin xmin (min x)))
	 (xmax (if xmax xmax (max x)))
	 (ymin (if ymin ymin (min y)))
	 (ymax (if ymax ymax (max y)))
	 (yscale
	  (if yscale
	      (let* (
		     (ymean (mean (list ymax ymin)))
		     (yshift (* (/ yscale) (- ymax ymean)))
		     )
		(list (- ymean yshift) (+ ymean yshift))
		)
	    (list ymin ymax)
	    )
	  )
	 (ymin (first yscale))
	 (ymax (second yscale))
	 (plot-shift (if axes 100 0))
	 (xs (- 600 (* 2 plot-shift)))
	 (ys (- 500 (* 2 plot-shift)))
	 (x (+ plot-shift (* xs (/ (- x xmin) (- xmax xmin)))))
	 (y (+ plot-shift (* ys (/ (- y ymin) (- ymax ymin)))))
	 etcfmt
	 )
  (with-open-file 
   (f name :direction :output)
   (format f "%!PS-Adobe-3.0 EPSF-3.0
%%BoundingBox: 0 0 600 500
%%Creator: Xlispstat
%%Title: XGobi-style plot window
%%CreationDate: 

%%EndComments
%%!
/pointsize ~d def
% Start of postscript setup
% $Header: /afs/sph.umich.edu/user/a/aelon/xlispstat/calculus/examples/RCS/xgobips2.lsp,v 1.2 1999/11/15 17:24:50 aelon Exp aelon $

/Helvetica findfont pointsize scalefont setfont
/ticksize pointsize 2 div def
/gobiinit {  % fg.r fg.g fg.b bg.r bg.g bg.b xmin ymin xmax ymax -> init state
	/ymax exch def /xmax exch def
	/ymin exch def /xmin exch def
	/blue exch def /green exch def /red exch def
	/fblue exch def /fgreen exch def /fred exch def
	newpath
	xmin ymin moveto
	xmin ymax lineto
	xmax ymax lineto
	xmax ymin lineto
	closepath
	clip
	% fill background
	red green blue setrgbcolor clippath fill
	fred fgreen fblue setrgbcolor
	newpath clippath stroke
} def

% use a dictionary so we can index into an array of functions
% addglyph
%/gobobdict 7 dict def
/gobobdict 8 dict def

/plus {	% size -> plus on page in that size at current point
	save
	exch
	/size exch def
	/incr size 2 div def
	save
	0 incr rmoveto
	0 0 size sub  rlineto
	stroke
	restore
	0 incr sub 0 rmoveto
	size 0 rlineto
	stroke
	restore
} def
gobobdict 1 /plus cvx put

/xglyph { 	% size -> x drawn at current point
	save
	exch
	/size exch def
	/incr size 2 div def
	save
	incr incr rmoveto
	0 size sub dup rlineto
	stroke
	restore
	0 incr sub incr rmoveto
	size 0 size sub rlineto
	stroke
	restore
} def
gobobdict 2 /xglyph cvx put

% addglyph
/opensquare	% size -> opensquare at current point
{
	save
	exch
	/size exch def
 	/incr size 2 div def
 	incr incr rmoveto
 	0 size sub 0 rlineto
 	0 0 size sub rlineto
 	size 0 rlineto
	closepath
	stroke
	restore
} def
gobobdict 3 /opensquare cvx put

% addglyph
/filledsquare	% size -> filledsquare at current point
{
	save
	exch
	/size exch def
 	/incr size 2 div def
 	incr incr rmoveto
 	0 size sub 0 rlineto
 	0 0 size sub rlineto
 	size 0 rlineto
    fill  % will this do it?  dfs
	closepath
	stroke
	restore
} def
gobobdict 4 /filledsquare cvx put

/opencircle { % size -> circle on page in that size at current point
	save
	exch
	/dotsize exch def
	currentpoint /myy exch def /myx exch def
	% put out a point at currentpoint
	newpath myx myy dotsize 2 div 0 360  arc closepath stroke
	restore
} def
% addglyph
gobobdict 5 /opencircle cvx put

/filledcircle { % size -> circle on page in that size at current point
	save
	exch
	/dotsize exch def
	currentpoint /myy exch def /myx exch def
	% put out a point at currentpoint
	newpath myx myy dotsize 2 div 0 360 arc fill closepath stroke
	restore
} def
% addglyph
gobobdict 6 /filledcircle cvx put

/point { % size -> dot on page in that size at current point
% Realize a point as a 1x1 filled rectangle.
%	save
%	exch
%	/dotsize exch def
%	currentpoint /myy exch def /myx exch def
%	put out a point at currentpoint
%	newpath myx myy dotsize 360 0  arc closepath fill
%	restore
	save
	exch
	/size exch def
 	/incr size 2 div def
 	incr incr rmoveto
 	0 size sub 0 rlineto
 	0 0 size sub rlineto
 	size 0 rlineto
	closepath
    fill
	stroke
	restore
} def
% addglyph
gobobdict 7 /point cvx put

% r g b type size x y pg -> print it
/pg { 
	save
	8 1 roll	% put save obj at bottom of our args
	/y exch def /x exch def /size exch def /type exch def
    /myblue exch def /mygreen exch def /myred exch def
	% got args, now do something
	
	myred mygreen myblue setrgbcolor
	x y moveto 
	size
	gobobdict type get
	exec
	restore

} def

% r g b x1 y1 x2 y2 ln
/ln {
	save
	8 1 roll	% put save obj at bottom of our args
	/y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
	myred mygreen myblue setrgbcolor
	newpath
	x1 y1 moveto
	x2 y2 lineto
	stroke
	restore
} def

/lndashed {
    [2] 1 setdash
	save
	8 1 roll	% put save obj at bottom of our args
	/y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
	myred mygreen myblue setrgbcolor
	newpath
	x1 y1 moveto
	x2 y2 lineto
	stroke
	restore
    [] 0 setdash
} def

/arbax { % arbitrary axis; label r g b x1 y1 x2 y2
	save
	9 1 roll	% put save obj at bottom of our args
	/y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
	/label exch def
	myred mygreen myblue setrgbcolor
	newpath
	x1 y1 moveto
	x2 y2 lineto
	stroke
	x2 y2 moveto
	pointsize 2 div dup rmoveto
	label show
	restore
} def

/yax { % yax: (label) red green blue x1 y1 x2 y2
	save
	9 1 roll	% put save obj at bottom of our args
    /y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
	myred mygreen myblue setrgbcolor
	newpath
	x1 y1 moveto
	x2 y2 lineto
	stroke
% axis label
	newpath
	x1 y1 moveto
	/diff y2 y1 sub def
	0 diff 2 div rmoveto
	label stringwidth	% wx wy on stack
	pop
	2 div neg
	0 exch rmoveto
	save
	90 rotate
	0 pointsize 3.0 mul rmoveto
	label show
	restore
	restore
} def

/xax { % xax: (label) red green blue x1 y1 x2 y2
	save
	9 1 roll	% put save obj at bottom of our args
    /y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
	myred mygreen myblue setrgbcolor
	newpath
	x1 y1 moveto
	x2 y2 lineto
	stroke
% axis label
	newpath
	x1 y1 moveto
	/diff x2 x1 sub def
	diff 2 div 0 rmoveto
	label stringwidth	% wx wy on stack
	pop
	2 div 
	neg 0 rmoveto
	0 pointsize -3.3 mul rmoveto
	label show
	restore
} def

/xaxlabel { % xax: (label) red green blue x1 y1
	save
	7 1 roll	% put save obj at bottom of our args
        /y1 exch def /x1 exch def
        /myblue exch def /mygreen exch def /myred exch def
        /label exch def
	myred mygreen myblue setrgbcolor
	newpath
	x1 y1 moveto
	stroke
% axis label
	newpath
	x1 y1 moveto
	label stringwidth	% wx wy on stack
	pop
	2 div 
	neg 0 rmoveto
	0 pointsize -3.3 mul rmoveto
	label show
	restore
} def

/ytx { % ytx: (label) red green blue x y
	save
	7 1 roll	% put save  obj at bottom of our args
    /y exch def /x exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
	myred mygreen myblue setrgbcolor
	newpath
	x y moveto
	ticksize neg 0 rlineto stroke
	x y moveto
	save
	label stringwidth	% wx wy on stack
	pop
	2 div
	90 rotate
	%neg pointsize 1.5 mul rmoveto
	neg pointsize 1.2 mul rmoveto
	label show
	restore
	restore
} def

/xtx { % xtx: (label) red green blue x y
	save
	7 1 roll
    /y exch def /x exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
	myred mygreen myblue setrgbcolor
	newpath
	x y moveto
	0 ticksize neg rlineto stroke
	x y moveto
	label stringwidth	% wx wy on stack
	pop
	2 div
	%neg pointsize neg 2 mul rmoveto
	neg pointsize neg 1.6 mul rmoveto
	label show
	restore
} def

/sticky { % sticky: (string) red green blue x y
	save
	7 1 roll	% put save obj at bottom of our args
    /y exch def /x exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
	myred mygreen myblue setrgbcolor
	x y moveto
	pointsize 4 div dup rlineto stroke
	x y moveto
	pointsize 4 div dup rmoveto
	label show
	restore
} def

% Routine for drawing the bitmaps
% r g b size x y bitmap -> print it
/bitmap {
    save
    7 1 roll    % put save obj at bottom of our args
    /y exch def /x exch def /size exch def
    /myblue exch def /mygreen exch def /myred exch def

    myred mygreen myblue setrgbcolor
    x y moveto
 	0 0 size sub rlineto
 	0 size sub 0 rlineto
 	0 size rlineto
	closepath
	stroke
	restore
} def

% Routine for drawing the arcs on the projection pursuit window
% r g b radius ang1 ang2 x y pparc -> print it
/pparc {
    save
    9 1 roll    % put save obj at bottom of our args
    /y exch def /x exch def
    /ang2 exch def /ang1 exch def
    /radius exch def
    /myblue exch def /mygreen exch def /myred exch def

    myred mygreen myblue setrgbcolor
    x y moveto
    x y radius ang1 ang2 arc
	closepath
	fill
    stroke
	restore
} def

% Routine for drawing y axis and label for projection pursuit plot
% ppyax: (label) red green blue x1 y1 x2 y2
/ppyax {
    save
    9 1 roll    % put save obj at bottom of our args
    /y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
    myred mygreen myblue setrgbcolor
    newpath
    x1 y1 moveto
    x2 y2 lineto
    stroke
% axis label
    newpath
    x1 y1 moveto
    /diff y2 y1 sub def
    0 diff 2 div rmoveto
    label stringwidth   % wx wy on stack
    pop
    2 div neg
    0 exch rmoveto
    save
    90 rotate
    0 pointsize 3.0 mul rmoveto
    label show
    restore
    restore
} def

% Routine for drawing y-axis tick marks and labels on pp plot
% ppytx: (label) red green blue x y
/ppytx {
    save
    7 1 roll    % put save  obj at bottom of our args
    /y exch def /x exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
    myred mygreen myblue setrgbcolor
    newpath
    x y moveto
    ticksize neg 0 rlineto stroke
    x y moveto
    save
    label stringwidth   % wx wy on stack
    pop
    2 div
    90 rotate
    %neg pointsize 1.5 mul rmoveto
    neg pointsize 1.2 mul rmoveto
    label show
    restore
    restore
} def

% ppln: red green blue width x1 y1 x2 y2
/ppln {
    save
    9 1 roll    % put save obj at bottom of our args
    /y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /width exch def
    /myblue exch def /mygreen exch def /myred exch def
    myred mygreen myblue setrgbcolor
    newpath
    x1 y1 moveto
    x2 y2 lineto
    width setlinewidth
    stroke
    restore
} def
% ael: not sure about the point of this, so I scrapped it:
% xgobiinit: red green blue max_x_size max_y_size
% 0.000000 0.000000 0.501961 1.000000 1.000000 1.000000 72.000000 94.725014 540.000000 697.275024 gobiinit
% ln: red green blue x1 y1 x2 y2 line_type
~{~,3f ~,3f ~,3f ~,3f ~,3f ~,3f ~,3f ~a~%~}
" 
	   pointsize
	   (combine
	    (weave (repeat reds n-1) (repeat greens n-1) (repeat blues n-1)
		   (butlast x) 
		   (butlast y) (rest x) (rest y) (repeat line-type n-1)))
	   )
   (if morelines
       (dolist
	(line morelines)
	(let* (
	       (reds (first line))
	       (greens (second line))
	       (blues (third line))
	       (x (fourth line))
	       (y (fifth line))
	       (x (+ plot-shift (* xs (/ (- x xmin) (- xmax xmin)))))
	       (y (+ plot-shift (* ys (/ (- y ymin) (- ymax ymin)))))
	       (line-type (sixth line))
	       (n-1 (1- (length x)))
	       )
	  (format f "% ln: red green blue x1 y1 x2 y2 line_type~%")
	  (mapcar
	   (lambda (red green blue x1 y1 x2 y2 line_type)
	     (format f "~,3f ~,3f ~,3f ~,3f ~,3f ~,3f ~,3f ~a~%" 
		     red green blue x1 y1 x2 y2 line_type)
	     )
	   (repeat reds n-1) (repeat greens n-1) (repeat blues n-1) (butlast x)
	   (butlast y) (rest x) (rest y) (repeat line-type n-1))
	  )
	)
     )
   (if points
       (dolist
	(line points)
	(let* (
	       (reds (first line))
	       (greens (second line))
	       (blues (third line))
	       (glyph-types (fourth line))
	       (glyph-sizes (fifth line))
	       (x (sixth line))
	       (y (seventh line))
	       (x (+ plot-shift (* xs (/ (- x xmin) (- xmax xmin)))))
	       (y (+ plot-shift (* ys (/ (- y ymin) (- ymax ymin)))))
	       )
	  (format f "
% pg: red green blue glyph_type glyph_size x y
~{~,3f ~,3f ~,3f ~d ~d ~,3f ~,3f pg~%~}
" 
		  (combine
		   (weave reds greens blues glyph-types glyph-sizes x y)
		   )
		  )
	  )
	)
     )
   (if axes
       (let ()
	 (setq
	  xtickfmt (strcat "~{(" xtickfmt ") ~,6f ~,6f ~,6f ~,6f ~,6f xtx~%~}")
	  ytickfmt (strcat "~{(" ytickfmt ") ~,6f ~,6f ~,6f ~,6f ~,6f ytx~%~}")
	  etcfmt (strcat
		  "
% sticky: (string) red green blue x y
%  place string a small distance to the right and
%  above x,y
% yax: (label) red green blue x1 y1 x2 y2
%  draw y axis and label
(~a) ~{~,6f ~} ~{~,6f ~} yax
% ytx: (label) red green blue x y
%  draw y axis tick and label
"
ytickfmt    
"
% xax: (label) red green blue x1 y1 x2 y2
%  draw x axis and label
(~a) ~{~,6f ~} ~{~,6f ~} xax
% xtx: (label) red green blue x y
%  draw x axis tick and label
"
xtickfmt    
)
     )

   (format f etcfmt
  ylabel axis-color (list 50 50 50 450)
  (if ytics
    (combine 
      (mapcar (lambda(tic)(combine tic
               axis-color 50 (+ 100 (* 300 (/ (- tic ymin) (- ymax ymin))))))
       ytics))
  (combine 
    (mapcar (lambda(tic)(combine (+ ymin (* (- ymax ymin) (/ (- tic 100) 300)))
               axis-color 50 tic))
      (rseq 100 400 nytics)))
  )
  xlabel axis-color (list 50 50 550 50)
  (if xtics
    (combine 
      (mapcar (lambda(tic)(combine tic
               axis-color (+ 100 (* 400 (/ (- tic xmin) (- xmax xmin)))) 50))
       xtics))
  (combine 
    (mapcar (lambda(tic)(combine (+ xmin (* (- xmax xmin) (/ (- tic 100) 400)))
               axis-color tic 50))
      (rseq 100 500 nxtics)))
)
)
)
)
   (if morelabels
       (dolist
	(label morelabels)
	 (setq
	  etcfmt (strcat "
% another label: (label) red green blue x1 y1
(~a) ~{~,6f ~} ~{~,6f ~} xaxlabel
")
          )
        (format f etcfmt (first label)
          (second label)
          (third label))
	)
      )
   (format f "
showpage
%%EOF
")
)
)
)

(defun xgobips-points
  (name reds greens blues glyph-type glyph-size x y
	&key
        title
       	(title-color '(0 0 0)) ;; black
	(xlabel "x")
	(ylabel "y")
	(axes t)
	(axis-color '(0 0 0)) ;; black
	(nxtics 4)
	(nytics 4)
	(xtics 
	 (let ((tics (get-nice-range (min x) (max x) nxtics)))
	   (rseq (first tics) (second tics) (third tics))))
	(ytics
	 (let ((tics (get-nice-range (min y) (max y) nytics)))
	   (rseq (first tics) (second tics) (third tics))))
	(xtickfmt "~,3g")
	(ytickfmt "~,3g")
	(pointsize 11)
	morelines
	morelabels
	points
	(line-type "ln")
	xmax
	ymax
	xmin
	ymin
	yscale
	)
  "
function xgobips
	Arguments
		name reds greens blues glyph-type glyph-size x y
        &key
        	title
        	(title-color '(0 0 0)) ;; black
		(xlabel \"x\")
		(ylabel \"y\")
		(axes t) ;; if t, then axes and labels will be added
		(axis-color '(0 0 0)) ;; black
		(nxtics 4) ;; this number of tics may be overruled by xtics
		(nytics 4) ;; this number of tics may be overruled by ytics
		(xtics 
		     (let ((tics (get-nice-range (min x) (max x) nxtics)))
		       (rseq (first tics) (second tics) (third tics))))
		(ytics
		     (let ((tics (get-nice-range (min y) (max y) nytics)))
		       (rseq (first tics) (second tics) (third tics))))
		(xtickfmt \"~,3g\")
		(ytickfmt \"~,3g\")
		(pointsize 11)
		morelines
		morelabels
		points
		(line-type \"ln\")
		xmax
		ymax
		xmin
		ymin
		yscale

Descriptions:

	name    - file name of postscript output
	reds	- real between 0 and 1: red value
	greens	- real between 0 and 1: green value
	blues	- real between 0 and 1: blue value
	glyph-type - Glyph Types available:
		      1 plus
		      2 xglyph
		      3 opensquare
		      4 filledsquare
		      5 opencircle
		      6 filledcircle
		      7 point
	glyph-size - integer > 0
	x	- list of x coordinates
	y	- list of y coordinates

	Keys:

	title   - A title for your plot
	title-color	- list of three red green blue values
			  '(0 0 0) (black) by default
	xlabel 	- string label for x axis
	ylabel 	- string label for y axis
	axis-color	- list of three red green blue values
			  '(0 0 0) is black
	nxtics	- number of x tics
	nytics  - number of y tics
	xtics	- a list of tics for the x axis
	ytics	- a list of tics for the y axis
	xtickfmt - format for x tick labels
	ytickfmt - format for y tick labels
	pointsize - size of font used for labels

	line-type - either ln or lndashed

	morelines - a list of lines, containing list of components
			(red green blue xs ys line-type)
	morelabels - a list of labels, containing lists of components
		     of the form
			(string (list red green blue) (list x y))
	points	- a list containing components as follows:
		(reds greens blues glyph-types glyph-sizes x y)
	xmax	- set the boundaries of the rectangle to show
	ymax
	xmin
	ymin
	yscale - scales the ymin/ymax by a factor (expand, contract about the
		 mean) 

;; Examples:
 (xgobips \"xgobips.ps\"
   1  ;; red   level 
   0  ;; green
   0  ;; blue
   (rand.u 10) (rand.u 10)
   :xtics '(.2 .3 .5)
   )
 (xgobips \"xgobips.ps\"
   1  ;; red   level 
   0  ;; green
   0  ;; blue
   (rand.u 10) (rand.u 10)
   :points (list (list
		(repeat 0 10)
		(repeat 0 10)
		(repeat 1 10)
		(sample (iseq 1 7) 10 t)
		(repeat 2 10)
		(rand.u 10) (rand.u 10)))
   :xtics '(.2 .3 .5)
   )
 (xgobips \"xgobips.ps\"
   1  ;; red   level 
   0  ;; green
   0  ;; blue
   (rand.u 10) (rand.u 10)
   :morelines (list (list 0 0 1 (rand.u 10) (rand.u 10) \"lndashed\"))
   :xtics '(.2 .3 .5)
   :xmin 0
   :xmax 1
   :ymin 0
   :ymax 1
   )
"
  (if title
      (setq morelabels
	    (if morelabels
		(append morelabels
			(list title title-color (list 300 500)))
	      (list (list title title-color (list 300 500)))
	      )
	    )
    )

  (let* (
	 (n-1 (1- (length x)))
	 (xmin (if xmin xmin (min x)))
	 (xmax (if xmax xmax (max x)))
	 (ymin (if ymin ymin (min y)))
	 (ymax (if ymax ymax (max y)))
	 (yscale
	  (if yscale
	      (let* (
		     (ymean (mean (list ymax ymin)))
		     (yshift (* (/ yscale) (- ymax ymean)))
		     )
		(list (- ymean yshift) (+ ymean yshift))
		)
	    (list ymin ymax)
	    )
	  )
	 (ymin (first yscale))
	 (ymax (second yscale))
	 (plot-shift (if axes 100 0))
	 (xs (- 600 (* 2 plot-shift)))
	 (ys (- 500 (* 2 plot-shift)))
	 (x (+ plot-shift (* xs (/ (- x xmin) (- xmax xmin)))))
	 (y (+ plot-shift (* ys (/ (- y ymin) (- ymax ymin)))))
	 etcfmt
	 )
  (with-open-file 
   (f name :direction :output)
   (format f "%!PS-Adobe-3.0 EPSF-3.0
%%BoundingBox: 0 0 600 500
%%Creator: Xlispstat
%%Title: XGobi-style plot window
%%CreationDate: 

%%EndComments
%%!
/pointsize ~d def
% Start of postscript setup
% $Header: /afs/sph.umich.edu/user/a/aelon/xlispstat/calculus/examples/RCS/xgobips2.lsp,v 1.2 1999/11/15 17:24:50 aelon Exp aelon $

/Helvetica findfont pointsize scalefont setfont
/ticksize pointsize 2 div def
/gobiinit {  % fg.r fg.g fg.b bg.r bg.g bg.b xmin ymin xmax ymax -> init state
	/ymax exch def /xmax exch def
	/ymin exch def /xmin exch def
	/blue exch def /green exch def /red exch def
	/fblue exch def /fgreen exch def /fred exch def
	newpath
	xmin ymin moveto
	xmin ymax lineto
	xmax ymax lineto
	xmax ymin lineto
	closepath
	clip
	% fill background
	red green blue setrgbcolor clippath fill
	fred fgreen fblue setrgbcolor
	newpath clippath stroke
} def

% use a dictionary so we can index into an array of functions
% addglyph
%/gobobdict 7 dict def
/gobobdict 8 dict def

/plus {	% size -> plus on page in that size at current point
	save
	exch
	/size exch def
	/incr size 2 div def
	save
	0 incr rmoveto
	0 0 size sub  rlineto
	stroke
	restore
	0 incr sub 0 rmoveto
	size 0 rlineto
	stroke
	restore
} def
gobobdict 1 /plus cvx put

/xglyph { 	% size -> x drawn at current point
	save
	exch
	/size exch def
	/incr size 2 div def
	save
	incr incr rmoveto
	0 size sub dup rlineto
	stroke
	restore
	0 incr sub incr rmoveto
	size 0 size sub rlineto
	stroke
	restore
} def
gobobdict 2 /xglyph cvx put

% addglyph
/opensquare	% size -> opensquare at current point
{
	save
	exch
	/size exch def
 	/incr size 2 div def
 	incr incr rmoveto
 	0 size sub 0 rlineto
 	0 0 size sub rlineto
 	size 0 rlineto
	closepath
	stroke
	restore
} def
gobobdict 3 /opensquare cvx put

% addglyph
/filledsquare	% size -> filledsquare at current point
{
	save
	exch
	/size exch def
 	/incr size 2 div def
 	incr incr rmoveto
 	0 size sub 0 rlineto
 	0 0 size sub rlineto
 	size 0 rlineto
    fill  % will this do it?  dfs
	closepath
	stroke
	restore
} def
gobobdict 4 /filledsquare cvx put

/opencircle { % size -> circle on page in that size at current point
	save
	exch
	/dotsize exch def
	currentpoint /myy exch def /myx exch def
	% put out a point at currentpoint
	newpath myx myy dotsize 2 div 0 360  arc closepath stroke
	restore
} def
% addglyph
gobobdict 5 /opencircle cvx put

/filledcircle { % size -> circle on page in that size at current point
	save
	exch
	/dotsize exch def
	currentpoint /myy exch def /myx exch def
	% put out a point at currentpoint
	newpath myx myy dotsize 2 div 0 360 arc fill closepath stroke
	restore
} def
% addglyph
gobobdict 6 /filledcircle cvx put

/point { % size -> dot on page in that size at current point
% Realize a point as a 1x1 filled rectangle.
%	save
%	exch
%	/dotsize exch def
%	currentpoint /myy exch def /myx exch def
%	put out a point at currentpoint
%	newpath myx myy dotsize 360 0  arc closepath fill
%	restore
	save
	exch
	/size exch def
 	/incr size 2 div def
 	incr incr rmoveto
 	0 size sub 0 rlineto
 	0 0 size sub rlineto
 	size 0 rlineto
	closepath
    fill
	stroke
	restore
} def
% addglyph
gobobdict 7 /point cvx put

% r g b type size x y pg -> print it
/pg { 
	save
	8 1 roll	% put save obj at bottom of our args
	/y exch def /x exch def /size exch def /type exch def
    /myblue exch def /mygreen exch def /myred exch def
	% got args, now do something
	
	myred mygreen myblue setrgbcolor
	x y moveto 
	size
	gobobdict type get
	exec
	restore

} def

% r g b x1 y1 x2 y2 ln
/ln {
	save
	8 1 roll	% put save obj at bottom of our args
	/y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
	myred mygreen myblue setrgbcolor
	newpath
	x1 y1 moveto
	x2 y2 lineto
	stroke
	restore
} def

/lndashed {
    [2] 1 setdash
	save
	8 1 roll	% put save obj at bottom of our args
	/y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
	myred mygreen myblue setrgbcolor
	newpath
	x1 y1 moveto
	x2 y2 lineto
	stroke
	restore
    [] 0 setdash
} def

/arbax { % arbitrary axis; label r g b x1 y1 x2 y2
	save
	9 1 roll	% put save obj at bottom of our args
	/y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
	/label exch def
	myred mygreen myblue setrgbcolor
	newpath
	x1 y1 moveto
	x2 y2 lineto
	stroke
	x2 y2 moveto
	pointsize 2 div dup rmoveto
	label show
	restore
} def

/yax { % yax: (label) red green blue x1 y1 x2 y2
	save
	9 1 roll	% put save obj at bottom of our args
    /y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
	myred mygreen myblue setrgbcolor
	newpath
	x1 y1 moveto
	x2 y2 lineto
	stroke
% axis label
	newpath
	x1 y1 moveto
	/diff y2 y1 sub def
	0 diff 2 div rmoveto
	label stringwidth	% wx wy on stack
	pop
	2 div neg
	0 exch rmoveto
	save
	90 rotate
	0 pointsize 3.0 mul rmoveto
	label show
	restore
	restore
} def

/xax { % xax: (label) red green blue x1 y1 x2 y2
	save
	9 1 roll	% put save obj at bottom of our args
    /y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
	myred mygreen myblue setrgbcolor
	newpath
	x1 y1 moveto
	x2 y2 lineto
	stroke
% axis label
	newpath
	x1 y1 moveto
	/diff x2 x1 sub def
	diff 2 div 0 rmoveto
	label stringwidth	% wx wy on stack
	pop
	2 div 
	neg 0 rmoveto
	0 pointsize -3.3 mul rmoveto
	label show
	restore
} def

/xaxlabel { % xax: (label) red green blue x1 y1
	save
	7 1 roll	% put save obj at bottom of our args
        /y1 exch def /x1 exch def
        /myblue exch def /mygreen exch def /myred exch def
        /label exch def
	myred mygreen myblue setrgbcolor
	newpath
	x1 y1 moveto
	stroke
% axis label
	newpath
	x1 y1 moveto
	label stringwidth	% wx wy on stack
	pop
	2 div 
	neg 0 rmoveto
	0 pointsize -3.3 mul rmoveto
	label show
	restore
} def

/ytx { % ytx: (label) red green blue x y
	save
	7 1 roll	% put save  obj at bottom of our args
    /y exch def /x exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
	myred mygreen myblue setrgbcolor
	newpath
	x y moveto
	ticksize neg 0 rlineto stroke
	x y moveto
	save
	label stringwidth	% wx wy on stack
	pop
	2 div
	90 rotate
	%neg pointsize 1.5 mul rmoveto
	neg pointsize 1.2 mul rmoveto
	label show
	restore
	restore
} def

/xtx { % xtx: (label) red green blue x y
	save
	7 1 roll
    /y exch def /x exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
	myred mygreen myblue setrgbcolor
	newpath
	x y moveto
	0 ticksize neg rlineto stroke
	x y moveto
	label stringwidth	% wx wy on stack
	pop
	2 div
	%neg pointsize neg 2 mul rmoveto
	neg pointsize neg 1.6 mul rmoveto
	label show
	restore
} def

/sticky { % sticky: (string) red green blue x y
	save
	7 1 roll	% put save obj at bottom of our args
    /y exch def /x exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
	myred mygreen myblue setrgbcolor
	x y moveto
	pointsize 4 div dup rlineto stroke
	x y moveto
	pointsize 4 div dup rmoveto
	label show
	restore
} def

% Routine for drawing the bitmaps
% r g b size x y bitmap -> print it
/bitmap {
    save
    7 1 roll    % put save obj at bottom of our args
    /y exch def /x exch def /size exch def
    /myblue exch def /mygreen exch def /myred exch def

    myred mygreen myblue setrgbcolor
    x y moveto
 	0 0 size sub rlineto
 	0 size sub 0 rlineto
 	0 size rlineto
	closepath
	stroke
	restore
} def

% Routine for drawing the arcs on the projection pursuit window
% r g b radius ang1 ang2 x y pparc -> print it
/pparc {
    save
    9 1 roll    % put save obj at bottom of our args
    /y exch def /x exch def
    /ang2 exch def /ang1 exch def
    /radius exch def
    /myblue exch def /mygreen exch def /myred exch def

    myred mygreen myblue setrgbcolor
    x y moveto
    x y radius ang1 ang2 arc
	closepath
	fill
    stroke
	restore
} def

% Routine for drawing y axis and label for projection pursuit plot
% ppyax: (label) red green blue x1 y1 x2 y2
/ppyax {
    save
    9 1 roll    % put save obj at bottom of our args
    /y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
    myred mygreen myblue setrgbcolor
    newpath
    x1 y1 moveto
    x2 y2 lineto
    stroke
% axis label
    newpath
    x1 y1 moveto
    /diff y2 y1 sub def
    0 diff 2 div rmoveto
    label stringwidth   % wx wy on stack
    pop
    2 div neg
    0 exch rmoveto
    save
    90 rotate
    0 pointsize 3.0 mul rmoveto
    label show
    restore
    restore
} def

% Routine for drawing y-axis tick marks and labels on pp plot
% ppytx: (label) red green blue x y
/ppytx {
    save
    7 1 roll    % put save  obj at bottom of our args
    /y exch def /x exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
    myred mygreen myblue setrgbcolor
    newpath
    x y moveto
    ticksize neg 0 rlineto stroke
    x y moveto
    save
    label stringwidth   % wx wy on stack
    pop
    2 div
    90 rotate
    %neg pointsize 1.5 mul rmoveto
    neg pointsize 1.2 mul rmoveto
    label show
    restore
    restore
} def

% ppln: red green blue width x1 y1 x2 y2
/ppln {
    save
    9 1 roll    % put save obj at bottom of our args
    /y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /width exch def
    /myblue exch def /mygreen exch def /myred exch def
    myred mygreen myblue setrgbcolor
    newpath
    x1 y1 moveto
    x2 y2 lineto
    width setlinewidth
    stroke
    restore
} def
% ael: not sure about the point of this, so I scrapped it:
% xgobiinit: red green blue max_x_size max_y_size
% 0.000000 0.000000 0.501961 1.000000 1.000000 1.000000 72.000000 94.725014 540.000000 697.275024 gobiinit
% pg: red green blue glyph_type glyph_size x y
~{~,7f ~,7f ~,7f ~d ~d ~,7f ~,7f pg~%~}
" 
#|
% ln: red green blue x1 y1 x2 y2 line_type
~{~,3f ~,3f ~,3f ~,3f ~,3f ~,3f ~,3f ~a~%~}
|#
	   pointsize
	   (combine
	    (weave
	     (repeat reds n) (repeat greens n) (repeat blues n)
	     (repeat glyph-type n)
	     (repeat glyph-size n)
	     x
	     y)
	    )
	   )
   (if morelines
       (dolist
	(line morelines)
	(let* (
	       (reds (first line))
	       (greens (second line))
	       (blues (third line))
	       (x (fourth line))
	       (y (fifth line))
	       (x (+ plot-shift (* xs (/ (- x xmin) (- xmax xmin)))))
	       (y (+ plot-shift (* ys (/ (- y ymin) (- ymax ymin)))))
	       (line-type (sixth line))
	       (n-1 (1- (length x)))
	       )
	  (format f "% ln: red green blue x1 y1 x2 y2 line_type~%")
	  (mapcar
	   (lambda (red green blue x1 y1 x2 y2 line_type)
	     (format f "~,3f ~,3f ~,3f ~,3f ~,3f ~,3f ~,3f ~a~%" 
		     red green blue x1 y1 x2 y2 line_type)
	     )
	   (repeat reds n-1) (repeat greens n-1) (repeat blues n-1) (butlast x)
	   (butlast y) (rest x) (rest y) (repeat line-type n-1))
	  )
	)
     )
   (if points
       (dolist
	(line points)
	(let* (
	       (reds (first line))
	       (greens (second line))
	       (blues (third line))
	       (glyph-types (fourth line))
	       (glyph-sizes (fifth line))
	       (x (sixth line))
	       (y (seventh line))
	       (x (+ plot-shift (* xs (/ (- x xmin) (- xmax xmin)))))
	       (y (+ plot-shift (* ys (/ (- y ymin) (- ymax ymin)))))
	       )
	  (format f "
% pg: red green blue glyph_type glyph_size x y
~{~,3f ~,3f ~,3f ~d ~d ~,3f ~,3f pg~%~}
" 
		  (combine
		   (weave reds greens blues glyph-types glyph-sizes x y)
		   )
		  )
	  )
	)
     )
   (if axes
       (let ()
	 (setq
	  xtickfmt (strcat "~{(" xtickfmt ") ~,6f ~,6f ~,6f ~,6f ~,6f xtx~%~}")
	  ytickfmt (strcat "~{(" ytickfmt ") ~,6f ~,6f ~,6f ~,6f ~,6f ytx~%~}")
	  etcfmt (strcat
		  "
% sticky: (string) red green blue x y
%  place string a small distance to the right and
%  above x,y
% yax: (label) red green blue x1 y1 x2 y2
%  draw y axis and label
(~a) ~{~,6f ~} ~{~,6f ~} yax
% ytx: (label) red green blue x y
%  draw y axis tick and label
"
ytickfmt    
"
% xax: (label) red green blue x1 y1 x2 y2
%  draw x axis and label
(~a) ~{~,6f ~} ~{~,6f ~} xax
% xtx: (label) red green blue x y
%  draw x axis tick and label
"
xtickfmt    
)
     )

   (format f etcfmt
  ylabel axis-color (list 50 50 50 450)
  (if ytics
    (combine 
      (mapcar (lambda(tic)(combine tic
               axis-color 50 (+ 100 (* 300 (/ (- tic ymin) (- ymax ymin))))))
       ytics))
  (combine 
    (mapcar (lambda(tic)(combine (+ ymin (* (- ymax ymin) (/ (- tic 100) 300)))
               axis-color 50 tic))
      (rseq 100 400 nytics)))
  )
  xlabel axis-color (list 50 50 550 50)
  (if xtics
    (combine 
      (mapcar (lambda(tic)(combine tic
               axis-color (+ 100 (* 400 (/ (- tic xmin) (- xmax xmin)))) 50))
       xtics))
  (combine 
    (mapcar (lambda(tic)(combine (+ xmin (* (- xmax xmin) (/ (- tic 100) 400)))
               axis-color tic 50))
      (rseq 100 500 nxtics)))
)
)
)
)
   (if morelabels
       (dolist
	(label morelabels)
	 (setq
	  etcfmt (strcat "
% another label: (label) red green blue x1 y1
(~a) ~{~,6f ~} ~{~,6f ~} xaxlabel
")
          )
        (format f etcfmt (first label)
          (second label)
          (third label))
	)
      )
   (format f "
showpage
%%EOF
")
)
)
)

(defun xgobips(name reds greens blues glyph-types glyph-sizes x y &key
		    (xlabel "x")
		    (ylabel "y")
		    (axis-color '(0 0 0)) ;; black
		    (nxtics 4)
		    (nytics 4)
		    (xtics 
		     (let ((tics (get-nice-range (min x) (max x) nxtics)))
		       (rseq (first tics) (second tics) (third tics))))
		    (ytics
		     (let ((tics (get-nice-range (min y) (max y) nytics)))
		       (rseq (first tics) (second tics) (third tics))))
		    )
  "
function xgobips
	Arguments
		name reds greens blues glyph-types glyph-sizes x y
        &key
		(xlabel \"x\")
		(ylabel \"y\")
		(axis-color '(0 0 0)) ;; black
		(nxtics 4) ;; this number of tics may be overruled by xtics
		(nytics 4) ;; this number of tics may be overruled by ytics
		(xtics 
		     (let ((tics (get-nice-range (min x) (max x) nxtics)))
		       (rseq (first tics) (second tics) (third tics))))
		(ytics
		     (let ((tics (get-nice-range (min y) (max y) nytics)))
		       (rseq (first tics) (second tics) (third tics))))

reds	- list of red values (between 1 and 0)
greens	- list of green values
blues	- list of blue values
glyph-types - list of glyph-type values
glyph-sizes - list of glyph-size values
x	- list of x coordinates
y	- list of y coordinates

	Keys:

	xlabel 	- string label for x axis
	ylabel 	- string label for y axis
	axis-color	- list of three red green blue values
			  '(0 0 0) is black
	nxtics	- number of x tics
	nytics  - number of y tics
	xtics	- a list of tics for the x axis
	ytics	- a list of tics for the y axis

Glyph Types available:
      1 plus
      2 xglyph
      3 opensquare
      4 filledsquare
      5 opencircle
      6 filledcircle
      7 point

;; Examples:
 ;; first five are red; next five blue
 ;; glyph-type is randomly chosen; size increases from 1 to 10.
 (xgobips \"xgobips.ps\"
   (combine (repeat 1 5) (repeat 0 5))  ;; red   level 
   (combine (repeat 0 10))              ;; green
   (combine (repeat 0 5) (repeat 1 5))  ;; blue
   (sample (iseq 1 7) 10 t)             ;; glyph-type
   (combine (iseq 1 10))                ;; glyph-size
   (rand.u 10) (rand.u 10)
   :xtics '(.2 .3 .5)
   )
"
  (let* (
	(xmax (max x))
	(xmin (min x))
	(ymax (max y))
	(ymin (min y))
	(xs 500)
	(ys 600)
	(x (+ 100 (* 400 (/ (- x xmin) (- xmax xmin)))))
	(y (+ 100 (* 300 (/ (- y ymin) (- ymax ymin)))))
	)
  (with-open-file 
   (f name :direction :output)
   ;; 72 94 541 698
   (format f "%!PS-Adobe-3.0 EPSF-3.0
%%BoundingBox: 0 0 600 500
%%Creator: Xlispstat
%%Title: XGobi-style plot window
%%CreationDate: 

%%EndComments
%%!
/pointsize 11 def
% Start of postscript setup
% $Header: /fs/31/m/21251/dfs/xgobi/sun4/ps/gobi.head.ps,v 1.14 91/11/26 13:29:26 dfs Exp Locker: dfs $

/Helvetica findfont pointsize scalefont setfont
/ticksize pointsize 2 div def
/gobiinit {  % fg.r fg.g fg.b bg.r bg.g bg.b xmin ymin xmax ymax -> init state
	/ymax exch def /xmax exch def
	/ymin exch def /xmin exch def
	/blue exch def /green exch def /red exch def
	/fblue exch def /fgreen exch def /fred exch def
	newpath
	xmin ymin moveto
	xmin ymax lineto
	xmax ymax lineto
	xmax ymin lineto
	closepath
	clip
	% fill background
	red green blue setrgbcolor clippath fill
	fred fgreen fblue setrgbcolor
	newpath clippath stroke
} def

% use a dictionary so we can index into an array of functions
% addglyph
%/gobobdict 7 dict def
/gobobdict 8 dict def

/plus {	% size -> plus on page in that size at current point
	save
	exch
	/size exch def
	/incr size 2 div def
	save
	0 incr rmoveto
	0 0 size sub  rlineto
	stroke
	restore
	0 incr sub 0 rmoveto
	size 0 rlineto
	stroke
	restore
} def
gobobdict 1 /plus cvx put

/xglyph { 	% size -> x drawn at current point
	save
	exch
	/size exch def
	/incr size 2 div def
	save
	incr incr rmoveto
	0 size sub dup rlineto
	stroke
	restore
	0 incr sub incr rmoveto
	size 0 size sub rlineto
	stroke
	restore
} def
gobobdict 2 /xglyph cvx put

% addglyph
/opensquare	% size -> opensquare at current point
{
	save
	exch
	/size exch def
 	/incr size 2 div def
 	incr incr rmoveto
 	0 size sub 0 rlineto
 	0 0 size sub rlineto
 	size 0 rlineto
	closepath
	stroke
	restore
} def
gobobdict 3 /opensquare cvx put

% addglyph
/filledsquare	% size -> filledsquare at current point
{
	save
	exch
	/size exch def
 	/incr size 2 div def
 	incr incr rmoveto
 	0 size sub 0 rlineto
 	0 0 size sub rlineto
 	size 0 rlineto
    fill  % will this do it?  dfs
	closepath
	stroke
	restore
} def
gobobdict 4 /filledsquare cvx put

/opencircle { % size -> circle on page in that size at current point
	save
	exch
	/dotsize exch def
	currentpoint /myy exch def /myx exch def
	% put out a point at currentpoint
	newpath myx myy dotsize 2 div 0 360  arc closepath stroke
	restore
} def
% addglyph
gobobdict 5 /opencircle cvx put

/filledcircle { % size -> circle on page in that size at current point
	save
	exch
	/dotsize exch def
	currentpoint /myy exch def /myx exch def
	% put out a point at currentpoint
	newpath myx myy dotsize 2 div 0 360 arc fill closepath stroke
	restore
} def
% addglyph
gobobdict 6 /filledcircle cvx put

/point { % size -> dot on page in that size at current point
% Realize a point as a 1x1 filled rectangle.
%	save
%	exch
%	/dotsize exch def
%	currentpoint /myy exch def /myx exch def
%	put out a point at currentpoint
%	newpath myx myy dotsize 360 0  arc closepath fill
%	restore
	save
	exch
	/size exch def
 	/incr size 2 div def
 	incr incr rmoveto
 	0 size sub 0 rlineto
 	0 0 size sub rlineto
 	size 0 rlineto
	closepath
    fill
	stroke
	restore
} def
% addglyph
gobobdict 7 /point cvx put

% r g b type size x y pg -> print it
/pg { 
	save
	8 1 roll	% put save obj at bottom of our args
	/y exch def /x exch def /size exch def /type exch def
    /myblue exch def /mygreen exch def /myred exch def
	% got args, now do something
	
	myred mygreen myblue setrgbcolor
	x y moveto 
	size
	gobobdict type get
	exec
	restore

} def

/ln {
	save
	8 1 roll	% put save obj at bottom of our args
	/y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
	myred mygreen myblue setrgbcolor
	newpath
	x1 y1 moveto
	x2 y2 lineto
	stroke
	restore
} def

/lndashed {
    [2] 1 setdash
	save
	8 1 roll	% put save obj at bottom of our args
	/y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
	myred mygreen myblue setrgbcolor
	newpath
	x1 y1 moveto
	x2 y2 lineto
	stroke
	restore
    [] 0 setdash
} def

/arbax { % arbitrary axis; label r g b x1 y1 x2 y2
	save
	9 1 roll	% put save obj at bottom of our args
	/y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
	/label exch def
	myred mygreen myblue setrgbcolor
	newpath
	x1 y1 moveto
	x2 y2 lineto
	stroke
	x2 y2 moveto
	pointsize 2 div dup rmoveto
	label show
	restore
} def

/yax { % yax: (label) red green blue x1 y1 x2 y2
	save
	9 1 roll	% put save obj at bottom of our args
    /y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
	myred mygreen myblue setrgbcolor
	newpath
	x1 y1 moveto
	x2 y2 lineto
	stroke
% axis label
	newpath
	x1 y1 moveto
	/diff y2 y1 sub def
	0 diff 2 div rmoveto
	label stringwidth	% wx wy on stack
	pop
	2 div neg
	0 exch rmoveto
	save
	90 rotate
	0 pointsize 3.0 mul rmoveto
	label show
	restore
	restore
} def

/xax { % xax: (label) red green blue x1 y1 x2 y2
	save
	9 1 roll	% put save obj at bottom of our args
    /y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
	myred mygreen myblue setrgbcolor
	newpath
	x1 y1 moveto
	x2 y2 lineto
	stroke
% axis label
	newpath
	x1 y1 moveto
	/diff x2 x1 sub def
	diff 2 div 0 rmoveto
	label stringwidth	% wx wy on stack
	pop
	2 div 
	neg 0 rmoveto
	0 pointsize -3.3 mul rmoveto
	label show
	restore
} def

/ytx { % ytx: (label) red green blue x y
	save
	7 1 roll	% put save  obj at bottom of our args
    /y exch def /x exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
	myred mygreen myblue setrgbcolor
	newpath
	x y moveto
	ticksize neg 0 rlineto stroke
	x y moveto
	save
	label stringwidth	% wx wy on stack
	pop
	2 div
	90 rotate
	%neg pointsize 1.5 mul rmoveto
	neg pointsize 1.2 mul rmoveto
	label show
	restore
	restore
} def

/xtx { % xtx: (label) red green blue x y
	save
	7 1 roll
    /y exch def /x exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
	myred mygreen myblue setrgbcolor
	newpath
	x y moveto
	0 ticksize neg rlineto stroke
	x y moveto
	label stringwidth	% wx wy on stack
	pop
	2 div
	%neg pointsize neg 2 mul rmoveto
	neg pointsize neg 1.6 mul rmoveto
	label show
	restore
} def

/sticky { % sticky: (string) red green blue x y
	save
	7 1 roll	% put save obj at bottom of our args
    /y exch def /x exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
	myred mygreen myblue setrgbcolor
	x y moveto
	pointsize 4 div dup rlineto stroke
	x y moveto
	pointsize 4 div dup rmoveto
	label show
	restore
} def

% Routine for drawing the bitmaps
% r g b size x y bitmap -> print it
/bitmap {
    save
    7 1 roll    % put save obj at bottom of our args
    /y exch def /x exch def /size exch def
    /myblue exch def /mygreen exch def /myred exch def

    myred mygreen myblue setrgbcolor
    x y moveto
 	0 0 size sub rlineto
 	0 size sub 0 rlineto
 	0 size rlineto
	closepath
	stroke
	restore
} def

% Routine for drawing the arcs on the projection pursuit window
% r g b radius ang1 ang2 x y pparc -> print it
/pparc {
    save
    9 1 roll    % put save obj at bottom of our args
    /y exch def /x exch def
    /ang2 exch def /ang1 exch def
    /radius exch def
    /myblue exch def /mygreen exch def /myred exch def

    myred mygreen myblue setrgbcolor
    x y moveto
    x y radius ang1 ang2 arc
	closepath
	fill
    stroke
	restore
} def

% Routine for drawing y axis and label for projection pursuit plot
% ppyax: (label) red green blue x1 y1 x2 y2
/ppyax {
    save
    9 1 roll    % put save obj at bottom of our args
    /y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
    myred mygreen myblue setrgbcolor
    newpath
    x1 y1 moveto
    x2 y2 lineto
    stroke
% axis label
    newpath
    x1 y1 moveto
    /diff y2 y1 sub def
    0 diff 2 div rmoveto
    label stringwidth   % wx wy on stack
    pop
    2 div neg
    0 exch rmoveto
    save
    90 rotate
    0 pointsize 3.0 mul rmoveto
    label show
    restore
    restore
} def

% Routine for drawing y-axis tick marks and labels on pp plot
% ppytx: (label) red green blue x y
/ppytx {
    save
    7 1 roll    % put save  obj at bottom of our args
    /y exch def /x exch def
    /myblue exch def /mygreen exch def /myred exch def
    /label exch def
    myred mygreen myblue setrgbcolor
    newpath
    x y moveto
    ticksize neg 0 rlineto stroke
    x y moveto
    save
    label stringwidth   % wx wy on stack
    pop
    2 div
    90 rotate
    %neg pointsize 1.5 mul rmoveto
    neg pointsize 1.2 mul rmoveto
    label show
    restore
    restore
} def

% ppln: red green blue width x1 y1 x2 y2
/ppln {
    save
    9 1 roll    % put save obj at bottom of our args
    /y2 exch def /x2 exch def /y1 exch def /x1 exch def
    /width exch def
    /myblue exch def /mygreen exch def /myred exch def
    myred mygreen myblue setrgbcolor
    newpath
    x1 y1 moveto
    x2 y2 lineto
    width setlinewidth
    stroke
    restore
} def
% ael: not sure about the point of this, so I scrapped it:
% xgobiinit: red green blue max_x_size max_y_size
% 0.000000 0.000000 0.501961 1.000000 1.000000 1.000000 72.000000 94.725014 540.000000 697.275024 gobiinit
% pg: red green blue glyph_type glyph_size x y
~{~,7f ~,7f ~,7f ~d ~d ~,7f ~,7f pg~%~}
% sticky: (string) red green blue x y
%  place string a small distance to the right and
%  above x,y
% yax: (label) red green blue x1 y1 x2 y2
%  draw y axis and label
(~a) ~{~,6f ~} ~{~,6f ~} yax
% ytx: (label) red green blue x y
%  draw y axis tick and label
~{(~,3g) ~,6f ~,6f ~,6f ~,6f ~,6f ytx~%~}
% xax: (label) red green blue x1 y1 x2 y2
%  draw x axis and label
(~a) ~{~,6f ~} ~{~,6f ~} xax
% xtx: (label) red green blue x y
%  draw x axis tick and label
~{(~,3g) ~,6f ~,6f ~,6f ~,6f ~,6f xtx~%~}
showpage
%%EOF
" (combine (weave reds greens blues glyph-types glyph-sizes x y))
  ylabel axis-color (list 50 50 50 450)
  (if ytics
    (combine 
      (mapcar (lambda(tic)(combine tic
               axis-color 50 (+ 100 (* 300 (/ (- tic ymin) (- ymax ymin))))))
       ytics))
  (combine 
    (mapcar (lambda(tic)(combine (+ ymin (* (- ymax ymin) (/ (- tic 100) 300)))
               axis-color 50 tic))
      (rseq 100 400 nytics)))
  )
  xlabel axis-color (list 50 50 550 50)
  (if xtics
    (combine 
      (mapcar (lambda(tic)(combine tic
               axis-color (+ 100 (* 400 (/ (- tic xmin) (- xmax xmin)))) 50))
       xtics))
  (combine 
    (mapcar (lambda(tic)(combine (+ xmin (* (- xmax xmin) (/ (- tic 100) 400)))
               axis-color tic 50))
      (rseq 100 500 nxtics)))
)
)
)
)
)