;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;       Original File name:      tools.lsp, extracted from XlispOOP.doc
;       Original Author:         Tim Mikkelsen
;
;       Xlisp-stat translator:   Andy Long, Fall 1995
;
;       Description:    Object-oriented example program
;       From Language:       XLISP 2.0
;       To Language:         XLISP-STAT 3.44
;
;       Date Created:      10-Jan-1988
;       Date Updated:       2-Apr-1989
;       Date Translated:    1-Nov-1995
;       
;       (c) Copyright 1988, by Tim Mikkelsen, all rights reserved.
;           Permission is granted for unrestricted non-commercial use.
;
; I took that permission and ran with it! ael.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;       Define the superclasses and classes
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; make TOOLS superclass
;       with a different :ISNEW method
;       added methods are :BORROW and :RETURN
;       class variables are     NUMBER          contains # of tool instances
;                               ACTIVE-LIST     contains list of current objects
;       instance variables are  POWER           list - (AC BATTERY HAND)
;                               MOVEABLE        CAN-CARRY or CAN-ROLL or FIXED
;                               OPERATIONS      list
;                               MATERIAL        list - (WOOD METAL PLASTIC ...)
;                               PIECES          list
;                               LOCATION        HOME or person's name
;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; The tools proto:
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defproto tools
  '(power 
    moveable 
    operations 
    material 
    pieces 
    location) 
  '(number active-list)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; The tools methods:
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth tools :number (&optional new)
  (when new (setf (slot-value 'number) new))
  (slot-value 'number)
  )
(send tools :number 0)
#| ael:

	The preceding statement initialize the tool class. We have to do that
before using :isnew, or else! I don't know how to initialize the proto on the
fly.
|#

(defmeth tools :active-list (&optional new)
  (when new (setf (slot-value 'active-list) new))
  (slot-value 'active-list)
  )
(send tools :active-list '()) ; initializes active-list


(defmeth tools :isnew (&optional objct)
  (setf (slot-value 'number) (1+ (slot-value 'number)))
  (setf (slot-value 'active-list) (cons objct (slot-value 'active-list)))
  (setf (slot-value 'location) 'home)
  objct
  )

(defmeth tools :borrow (by-whom)
  (if (eq (slot-value 'location) 'home) ; then
      (setq borrow-out (setf (slot-value 'location) by-whom)) ; otherwise,
     (setq borrow-out (princ-to-string (list "It's being used by " (slot-value 'location)". Try later.")))
     )
  borrow-out
  )

(defmeth tools :return ()
  (if (eq (slot-value 'location) 'home) ; then
      (list (princ "We've got it on the shelf.") (terpri)) ; otherwise,
    (setf (slot-value 'location) 'home))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; make HAND-TOOLS proto
;       with a different :ISNEW method
;       new instance variable   WEIGHT          <number> of pounds
;       the rest is inherited from TOOLS 
; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defproto hand-tools '(weight) '(number tool-list) tools)

(defmeth hand-tools :number (&optional new)
  (when new (setf (slot-value 'number) new))
  (slot-value 'number)
  )
(send hand-tools :number 0)

(defmeth hand-tools :tool-list (&optional new)
  (when new (setf (slot-value 'tool-list) new))
  (slot-value 'tool-list)
  )
(send hand-tools :tool-list '())

(defmeth hand-tools :isnew (pow op mat parts w-in name)
  (setf (slot-value 'power) pow)
  (setf (slot-value 'moveable) 'can-carry)
  (setf (slot-value 'operations) op)
  (setf (slot-value 'material) mat)
  (setf (slot-value 'pieces) parts)
  (setf (slot-value 'weight) w-in)
  (send hand-tools :number (1+ (send hand-tools :number)))
  (send hand-tools :tool-list (append (send hand-tools :tool-list) name))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; make SHOP-TOOLS proto
;       with a different :ISNEW method
;       no new instance variables
;       the rest is inherited from TOOLS 
; 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defproto shop-tools '() '(number tool-list) tools)

(defmeth shop-tools :number (&optional new)
  (when new (setf (slot-value 'number) new))
  (slot-value 'number)
  )
(send shop-tools :number 0)

(defmeth shop-tools :tool-list (&optional new)
  (when new (setf (slot-value 'tool-list) new))
  (slot-value 'tool-list)
  )
(send shop-tools :tool-list '())

(defmeth shop-tools :isnew (pow mov op mat parts name)
  (setf (slot-value 'power) pow)
  (setf (slot-value 'moveable) mov)
  (setf (slot-value 'operations) op)
  (setf (slot-value 'material) mat)
  (setf (slot-value 'pieces) parts)
  (send shop-tools :number (1+ (send shop-tools :number)))
  (send shop-tools :tool-list (append (send shop-tools :tool-list) name))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;       Create instances of the various tool classes 
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(setq hand-drill (apply #'send hand-tools :new
					; make an instance - HAND-DRILL
			'(ac) 
			'(drill polish grind screw)
			'(wood metal plastic)
			'(drill drill-bits screw-bits buffer)
			'2.5
			'(hand-drill)
			nil)
      )
(send tools :isnew hand-drill)

(setq hand-saw (send hand-tools :new
			'(elbow grease) 
			'(cut)
			'(wood metal plastic)
			'(handle blade)
			'1.2
			'(hand-saw)
			)
      )
(send tools :isnew hand-saw)

(setq table-saw (apply #'send shop-tools :new
					; make an instance - TABLE-SAW
		       '(ac)
		       'fixed
		       '(rip cross-cut)
		       '(wood plastic)
		       '(saw blades fence)
		       '(table-saw)
		       'nil)
      )
(send tools :isnew table-saw)
(setq radial-arm (apply #'send shop-tools :new
					; make an instance = RADIAL-ARM
			'(ac)
			'can-roll
			'(rip cross-cut)
			'(wood plastic)
			'(saw blades dust-bag)
			'(radial-arm)
			nil)
      )
(send tools :isnew radial-arm)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; The following session shows how to use the tool definitions.
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(send hand-drill :borrow 'fred)
(send table-saw :return)
(send hand-drill :borrow 'joe)
(send hand-drill :return)

#|
So, Fred was able to borrow the HAND-DRILL.  When an attempt was made to
return the  TABLE-SAW,  it was  already  at home.  A second  attempt  to
borrow the HAND-DRILL  indicated that "you can't" because it was already
lent out.  Lastly, the HAND-DRILL was returned successfully.  (Note that
the "got it  already"  and "you  can't"  strings  show up  twice  in the
display because the methods both print and return the string.)
|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; The following session shows the structure of the TOOLS object:
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(send tools :show)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; The two TOOLS sub-classes HAND-TOOLS and SHOP-TOOLS structure looks like:
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(send hand-tools :show)
(send shop-tools :show)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Instances of the sub-classes look like
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(send hand-drill :show)
(send table-saw :show)
(send radial-arm :show)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; Some other interesting options:
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(send hand-drill :own-slots)
(send hand-drill :precedence-list)
(send hand-drill :OWN-METHODS)
(send hand-drill :add-slot 'expensive 'yes)
(send hand-drill :own-slots)

(defmeth hand-drill :expensive (&optional new)
  (when new (setf (slot-value 'expensive) new))
  (slot-value 'expensive)
  )

(send shop-tools :add-slot 'expensive 'nil)

(defmeth shop-tools :expensive (&optional new)
  (when new (setf (slot-value 'expensive) new))
  (slot-value 'expensive)
  )

(send shop-tools :own-slots)
(send table-saw :own-slots)

;; (send table-saw :has-slot 'expensive)
;; will cause error
;; Error: object does not own slot - EXPENSIVE
;; Happened in: #<Subr-SLOT-VALUE: #119a6cc>


(send shop-tools :expensive 'yes)
(send table-saw :expensive)

(def whats (send tools :active-list))
whats
(dotimes (i (send tools :number)) (send (select whats i) :show))

(defmeth tools :slots ()
  (let (
	(slot-vals nil)
	)
    (dolist (i (send self :slot-names))
	     (setq slot-vals (adjoin (list (slot-value i)) slot-vals))
	     )
    slot-vals
    )
  )


(defun showslots (tooltype)
  (let (
	(crum (send tooltype :slots))
	(crumnames (reverse (send tooltype :slot-names)))
	)
    (dotimes (i  (length crum))
	     (princ (element crumnames i))
	     (princ ":")
	     (print (first (element crum i)))
	     (terpri)
	     )
    )
  )

(showslots hand-drill)

(def crum (send hand-drill :slots))
(def crumnames (reverse (send hand-drill :slot-names)))
(first (element crum (search (list 'pieces) crumnames)))

(send hand-drill :slot-value 'pieces)

#|

Let me see if I can summarize:

1) You define the class using a defproto:
------------------------------------------------------------------------------
DEFPROTO                                                        [function-doc]
Syntax (defproto name &optional ivars cvars (parent *object*) doc)
Makes a new object prototype with instance variables IVARS, 'class'
variables CVARS and parents PARENT. PARENT can be a single object or
a list of objects. IVARS and CVARS must be lists.
------------------------------------------------------------------------------
There are slots for 

instances (e.g. hand-tools)
class variables (everyone inherits these)

2) you define sub-classes similarly, etc.

3) everyone below inherits what's above.

I think that I'm ready for traffic lights!

Above, in the method :slots, I define a way of actually accessing any particular slot. It seems to me that we ought to avoid giving any particular sub-object the same-named slot: I haven't been so careful above.

________________________________________________________________________________

Some of the changes from the xlisp2.0 code:

1) We need to replace class by defproto, and change setq to defmeth.
2) remove :answer
3) Methods, as defined by Tim, contain the extra ().

|#
