#|
				Adji

	This is file adji.lsp, created to play one version of mancala and beat
the old men of Cotonou. I created this file for Rock, when we got to talking
about how the old men like to sit around and play. I told him that it would be
fun to program a computer to beat them (I wondered what they'd say to that!).

							Andy Long
							November, 1997
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                player proto: same as for bele:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(require "bele")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 methods:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :move (player pit &key pits check started print)
  "
pits are labelled 0 to 11: 0-5 are for player 0, 6-11 for player 1.
"
  ;; First we check to make sure that the entry is valid:
  (if (not started) (let () (setq started t) (check-start player pit)))

  (let* (
	 (pits (if pits pits (copy-list (send self 'pits))))
	 (result (adji-move pits pit))
	 (pits (first result))
	 (final-pit (second result))
	 (score (send self 'winnings))
	 )

    (print-move print player pits pit)

    (while
	(or	      
	 ;; opposite of bele: you have to fall on your opponent's side:
	 (and (= player 1)
	      (and (< final-pit 6) (> final-pit -1))
	      (if (> final-pit -1) (search (list (elt pits final-pit)) '(2 3)))
	      )
	 (and (= player 0) (>= final-pit 6))
	 )
      ;; then there is the potential for winnings:
      (let ((winnings (elt pits final-pit)))
	(if (search (list winnings) '(2 3)) ;; then there are winnings!
	    (let ()
	      (setf (elt pits final-pit) 0)
	      (setq score (+ score (* winnings (list (- 1 player) player))))
	      )
	  )
	)
      (setq final-pit (decf final-pit))
      )

    (if (not check)
	(let () (send self 'winnings score) (send self 'pits pits)))

    (list pits (elt score player))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             adji
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun adji(&key print)
  "
;; Examples:
 (adji :print t)
"
  (let ((new-game (send game :new (iseq 2) (repeat 4 12) '(0 0))))
    (do
     ()
     ((not (send new-game :one-round :print print)))
     (format t "The score is: ~s" (send new-game 'winnings))
     (break)
     )
    (format t "The final score is: ~s" (send new-game 'winnings))
    (send game :dispose new-game)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             play-adji
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun play-adji(&key print (player 1))
  "
;; Examples:
 (play-adji :print t)
 (play-adji :print t :player 0) ;; to start the game
"
  (let ((new-game (send game :new (iseq 2) (repeat 4 12) '(0 0))))
    (do
     ()
     ((not (send new-game :one-round :print print :interactive player)))
     (format t "The score is: ~s" (send new-game 'winnings))
     )
    (format t "The final score is: ~s" (send new-game 'winnings))
    (send game :dispose new-game)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                         replay-adji
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun replay-adji (
		       &key print
			    order
			    names
			    (player 1) 
			    (pits (repeat 4 12))
			    (score '(0 0)))
  "
;; Examples:
  ;; (setq newd (send game :new (iseq 2) 
 (replay-adji :print t :order '(1 0) :player 0 :names '(\"Rock\" \"Ord\")
    :pits '(2 1 9 2 0 4 3 0 2 1 5 1) :score '(7 11))

 (replay-adji :print t :player 0 :order '(0 1) :names '(\"Anna\" \"Ord\")
    :pits '(1 0 0 2 2 3 0 0 0 0 15 0)
    :score '(15 10))
"
  (let ((new-game (send game :new (iseq 2) pits score)))
    (print-pits pits)
    (do
     ()
     ((not (send new-game :one-round :print print :interactive player
		 :order order :names names)))
     (if (/= 48 (sum (combine (send new-game 'winnings) 
			      (send new-game 'pits))))
	 (let ()
	   (printall 48 (send new-game 'winnings) (send new-game 'pits))
	   (break)
	   ))
     (format t "The score is: ~s" (send new-game 'winnings))
     )
    (send game :dispose new-game)
    )
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                adji-move
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun adji-move (pits pit)
  "
pits are labelled 0 to 11: 0-5 are for player 0, 6-11 for player 1.

;; Examples:
 (adji-move '(1 0 0 0 0 0 1 2 3 4 5 6) 0)
 (adji-move '(1 0 0 0 0 0 1 2 3 4 5 6) 6)
 (adji-move '(1 0 0 0 0 0 1 2 3 4 5 24) 6)
 (adji-move (combine 1  12  9  9  4  2 (reverse '(9  0  0  0  1  1))) 5)

 (setq ad (new game '(0 1) 
   (combine 1  12  9  9  4  2 (reverse '(9  0  0  0  1  1)))
   '(0 0)))
 (send ad :move 0 5 :print t)
"
  (let* (
	 (pebbles (elt pits pit))
	 (pits-to-add
	  (mapcar (lambda(x) (mod x 12)) (iseq (+ pit 1) (+ pit pebbles)))
	  )
	 (temp (setf (elt pits pit) 0))
	 (temp
	  (mapcar
	   (lambda(pit) (setf (elt pits pit) (incf (elt pits pit))))
	   pits-to-add))
	 (temp (elt pits pit)) ;; we don't put any back in this spot:
	 (tmp (setf (elt pits pit) 0))
	 (pit (final pits-to-add))
	 )
    ;;    (print pits)
    ;;    (print temp)
    (do
     ()
     ((= 0 temp))
     (setf temp (decf temp))
     (setf pit (mod (incf pit) 12))
     (setf (elt pits pit) (incf (elt pits pit)))
     )	    
    (list pits pit)
    )
  )
