#|
	I'm going to start with the whole sas file that someone prepared for
the Agresti data, and after it append anything that I do in lisp. In other
words, I'm not going to translate this file, only use as necessary.

Andy Long, Dec. 1996

File:


This document contains tables from the Appendix of "An Introduction to
Categorical Data Analysis," by Alan Agresti, published by John Wiley
and Sons, Inc., January 1996.  The tables show SAS code for the analyses
conducted in that text, and contain the major data sets from that text.
See the text for further information about the datasets.

The text provides an applied introduction to basic methods of
categorical data analysis, particularly logistic regression and other
generalized linear models.  The technical level is appropriate for
applied statisticians and nonstatisticians having a background in
standard statistical methods such as regression and ANOVA.  The Tables
show a variety of SAS procedures, with particular emphasis on GENMOD,
the SAS procedure for generalized linear modeling.  

These tables can be freely used and distributed for non-commercial
purposes.  Please direct any comments about errors in the tables or in
the text itself to Alan Agresti, Dept. of Statistics, Univ. of
Florida, Gainesville, Florida 32611-8545, e-mail AA@STAT.UFL.EDU.

A.1  SAS file for chi-squared tests with aspirin data in Table 2.3

data aspirin;
input group mi count @@;
cards;
1 1 189  1 2 10845  
2 1 104  2 2 10933  
;
proc freq order=data; weight count;
   tables group*mi / chisq expected measures;
proc genmod;
   model count = group mi / dist=poi link=log obstats;
run;


A.2  SAS file for analyses of infant malformation data in Table 2.7

data infants;
input malform alcohol count @@;
cards;
1 0   17066  1 0.5 14464  1 1.5   788  1 4.0   126  1 7.0    37
2 0      48  2 0.5    38  2 1.5     5  2 4.0     1  2 7.0     1 
;
proc freq; weight count;
   tables malform*alcohol / chisq cmh1;
proc freq; weight count;
   tables malform*alcohol / cmh1 scores=ridit;
run;


A.3  SAS file for Fisher's exact test with tea-tasting data in Table 2.8

data tea;
input poured guess count @@; 
cards; 
1  1  3    1  2  1  
2  1  1    2  2  3 
; 
proc freq;  weight count; 
   tables  poured*guess /  exact; 
run;


A.4  SAS file for CMH analysis of Chinese smoking data in Table 3.3

data cmh;
input center smoke cancer count @@;
cards;
1 1 1 126  1 1 2 100   1 2 1  35  1 2 2  61
2 1 1 908  2 1 2 688   2 2 1 497  2 2 2 807
3 1 1 913  3 1 2 747   3 2 1 336  3 2 2 598
4 1 1 235  4 1 2 172   4 2 1  58  4 2 2 121
5 1 1 402  5 1 2 308   5 2 1 121  5 2 2 215
6 1 1 182  6 1 2 156   6 2 1  72  6 2 2  98
7 1 1  60  7 1 2  99   7 2 1  11  7 2 2  43
8 1 1 104  8 1 2  89   8 2 1  21  8 2 2  36
;
proc freq; mweight count;
   tables center*smoke*cancer / cmh;
run;


A.5  SAS file for binary regression models with snoring data in Table 4.1

data glm;
input snoring disease total;
cards;
0  24  1379 
2  35   638 
4  21   213 
5  30   254
; 
proc genmod; model disease/total = snoring  /  dist=bin  link=identity ; 
proc genmod; model disease/total = snoring  /  dist=bin  link=logit ; 
proc genmod; model disease/total = snoring  /  dist=bin  link=probit;
run;


A.6  SAS file for analyses of horseshoe crab data in Table 4.2

data crab;
input  color spine  width  satell  weight;
if satell>0 then y=1; if satell=0 then y=0; n=1; 
weight = weight/1000;  color = color - 1;
if color=4 then dark=0; if color < 4 then dark=1;
cards;
3  3  28.3  8  3050

4  3  22.5  0  1550

2  1  26.0  9  2300

4  3  24.8  0  2100

4  3  26.0  4  2600

3  3  23.8  0  2100

2  1  26.5  0  2350

4  2  24.7  0  1900

3  1  23.7  0  1950

4  3  25.6  0  2150

4  3  24.3  0  2150

3  3  25.8  0  2650

3  3  28.2  11 3050

5  2  21.0  0  1850

3  1  26.0  14  2300

2  1  27.1  8  2950

3  3  25.2  1  2000

3  3  29.0  1  3000

5  3  24.7  0  2200

3  3  27.4  5  2700

3  2  23.2  4  1950

2  2  25.0  3  2300

3  1  22.5  1  1600

4  3  26.7  2  2600

5  3  25.8  3  2000

5  3  26.2  0  1300

3  3  28.7  3  3150

3  1  26.8  5  2700

5  3  27.5  0  2600

3  3  24.9  0  2100

2  1  29.3  4  3200

2  3  25.8  0  2600

3  2  25.7  0  2000

3  1  25.7  8  2000

3  1  26.7  5  2700

5  3  23.7  0  1850

3  3  26.8  0  2650

3  3  27.5  6  3150

5  3  23.4  0  1900

3  3  27.9  6  2800

4  3  27.5  3  3100

2  1  26.1  5  2800

2  1  27.7  6  2500

3  1  30.0  5  3300

4  1  28.5  9  3250

4  3  28.9  4  2800

3  3  28.2  6  2600

3  3  25.0  4  2100

3  3  28.5  3  3000

3  1  30.3  3  3600

5  3  24.7  5  2100

3  3  27.7  5  2900

2  1  27.4  6  2700

3  3  22.9  4  1600

3  1  25.7  5  2000

3  3  28.3  15  3000

3  3  27.2  3  2700

4  3  26.2  3  2300

3  1  27.8  0  2750

5  3  25.5  0  2250

4  3  27.1  0  2550

4  3  24.5  5  2050

4  1  27.0  3  2450

3  3  26.0  5  2150

3  3  28.0  1  2800

3  3  30.0  8  3050

3  3  29.0  10 3200

3  3  26.2  0  2400

3  1  26.5  0  1300

3  3  26.2  3  2400

4  3  25.6  7  2800

4  3  23.0  1  1650

4  3  23.0  0  1800

3  3  25.4  6  2250

4  3  24.2  0  1900

3  2  22.9  0  1600

4  2  26.0  3  2200

3  3  25.4  4  2250

4  3  25.7  0  1200

3  3  25.1  5  2100

4  2  24.5  0  2250

5  3  27.5  0  2900

4  3  23.1  0  1650

4  1  25.9  4  2550

3  3  25.8  0  2300

5  3  27.0  3  2250

3  3  28.5  0  3050

5  1  25.5  0  2750

5  3  23.5  0  1900

3  2  24.0  0  1700

3  1  29.7  5  3850

3  1  26.8  0  2550

5  3  26.7  0  2450

3  1  28.7  0  3200

4  3  23.1  0  1550

3  1  29.0  1  2800

4  3  25.5  0  2250

4  3  26.5  1  1967

4  3  24.5  1  2200

4  3  28.5  1  3000

3  3  28.2  1  2867

3  3  24.5  1  1600

3  3  27.5  1  2550

3  2  24.7  4  2550

3  1  25.2  1  2000

4  3  27.3  1  2900

3  3  26.3  1  2400

3  3  29.0  1  3100

3  3  25.3  2  1900

3  3  26.5  4  2300

3  3  27.8  3  3250

3  3  27.0  6  2500

4  3  25.7  0  2100

3  3  25.0  2  2100

3  3  31.9  2  3325

5  3  23.7  0  1800

5  3  29.3  12  3225

4  3  22.0  0  1400

3  3  25.0  5  2400

4  3  27.0  6  2500

4  3  23.8  6  1800

2  1  30.2  2  3275

4  3  26.2  0  2225

3  3  24.2  2  1650

3  3  27.4  3  2900

3  2  25.4  0  2300

4  3  28.4  3  3200

5  3  22.5  4  1475

3  3  26.2  2  2025

3  1  24.9  6  2300

2  2  24.5  6  1950

3  3  25.1  0  1800

3  1  28.0  4  2900

5  3  25.8  10 2250

3  3  27.9  7  3050

3  3  24.9  0  2200

3  1  28.4  5  3100

4  3  27.2  5  2400

3  2  25.0  6  2250

3  3  27.5  6  2625

3  1  33.5  7  5200

3  3  30.5  3  3325

4  3  29.0  3  2925

3  1  24.3  0  2000

3  3  25.8  0  2400

5  3  25.0  8  2100

3  1  31.7  4  3725

3  3  29.5  4  3025

4  3  24.0  10 1900

3  3  30.0  9  3000

3  3  27.6  4  2850

3  3  26.2  0  2300

3  1  23.1  0  2000

3  1  22.9  0  1600

5  3  24.5  0  1900

3  3  24.7  4  1950

3  3  28.3  0  3200

3  3  23.9  2  1850

4  3  23.8  0  1800

4  2  29.8  4  3500

3  3  26.5  4  2350

3  3  26.0  3  2275

3  3  28.2  8  3050

5  3  25.7  0  2150

3  3  26.5  7  2750

3  3  25.8  0  2200

4  3  24.1  0  1800

4  3  26.2  2  2175

4  3  26.1  3  2750

4  3  29.0  4  3275

2  1  28.0  0  2625

5  3  27.0  0  2625

3  2  24.5  0  2000
; 
proc genmod; model satell = width / dist=poi link=log;
proc genmod; model satell = width / dist=poi link=identity;
proc genmod;
   model satell = width / dist=nor link=identity;
 proc genmod;
   model y/n = width / dist=bin  link=logit obstats ;
proc genmod; class color;
   model y/n = color width / dist=bin link=logit;
proc genmod; 
   model y/n = color width / dist=bin link=logit; 
proc genmod; class color spine;
   model y/n = color spine width weight / dist=bin link=logit type3;
proc genmod;
   model y/n =  / dist=bin  link=logit ;
proc logistic;
   model y = width;
proc logistic;
   model y = color weight width  / selection=backward;
proc genmod; 
   model y/n = weight dark / dist=bin link=logit; 
run;


A.7  SAS file for Poisson regression with grouped crab data in Table 4.3

data crabs;
input width cases satell;
log_case = log(cases);
cards;
22.69  14  14
23.84  14  20
24.77  28  67
25.84  39 105
26.79  22  63
27.74  24  93
28.67  18  71
30.41  14  72
;
proc genmod; 
   model satell = width / dist=poi link=log offset=log_case obstats ; 
run;


A.8  SAS file for logistic regression with Table 5.1

data crabs;
input width cases satell; 
cards;
22.69 14  5
23.84 14  4
24.77 28 17
25.84 39 21
26.79 22 15
27.74 24 20 
28.67 18 15 
30.41 14 14 
;
proc genmod; 
   model satell/cases = width / dist=bin link=logit waldci lrci alpha=.01; 
proc logistic; 
   model satell/cases = width / influence ; 
output out=predict  p=pi_hat  lower=LCL  upper=UCL;
proc print  data=predict;
proc logistic;
   model satell/cases = / influence ;
run;


A.9  SAS file for analyses of AIDS-AZT data in Table 5.5

data aids;
input race $ azt $ yes no @@; 
cases = yes + no; 
cards; 
white  y  14  93  white  n  32  81 
black  y  11  52  black  n  12  43
; 
proc genmod   order=data; class race  azt; 
   model yes/cases = race azt /  dist=bin  link=logit  obstats type3; 
run;


A.10  SAS file for loglinear model (AM, CM) with Table 6.3

data drugs;
input a c m count @@; 
cards;
 1 1 1 911   1 1 2 538   1 2 1 44   1 2 2 456
2 1 1   3   2 1 2 43   2 2 1 2   2 2 2 279
;
proc genmod; class a c m;
   model  count = a c m a*m  c*m / dist=poi link=log obstats;
proc catmod; weight count;
   model a*c*m = _response_; loglin a|m c|m;
run;


A.11  SAS file for linear-by-linear association with Table 7.3

data sex;
input premar  birth  count @@;
assoc = premar*birth; 
cards;
1 1  38   1 2  60   1 3 68   1 4 81
2 1  14   2 2  29   2 3 26   2 4 24
3 1  42   3 2  74   3 3 41   3 4 18
4 1 157   4 2 161   4 3 57   4 4 36
;
proc genmod; class premar birth;
   model  count = premar birth assoc / dist=poi link=log;
run;


A.12  SAS file for CMH analyses of Table 7.3

data cmh;
input gender $ income satisf count @@;
cards;
F  3 1 1  F 3  3 3  F  3 4 11  F  3 5 2 
F 10 1 2  F 10 3 3  F 10 4 17  F 10 5 3 
F 20 1 0  F 20 3 1  F 20 4  8  F 20 5 5
F 35 1 0  F 35 3 2  F 35 4 4   F 35 5 2
M  3 1 1  M  3 3 1  M  3 4 2   M  3 5 1
M 10 1 0  M 10 3 3  M 10 4 5   M 10 5 1
M 20 1 0  M 20 3 0  M 20 4 7   M 20 5 3
M 35 1 0  M 35 3 1  M 35 4 9   M 35 5 6
proc freq; weight count;
   tables gender*income*satisf /  cmh;
run;


A.13  SAS file for baseline-category logit model with Table 8.1

data gator;
input length choice $ @@;
cards; 
1.24 I  1.30 I  1.30 I  1.32 F  1.32 F  1.40 F  1.42 I  1.42 F
1.45 I  1.45 O  1.47 I  1.47 F  1.50 I  1.52 I  1.55 I  1.60 I  
1.63 I  1.65 O  1.65 I  1.65 F  1.65 F  1.68 F  1.70 I  1.73 O  
1.78 I  1.78 I  1.78 O  1.80 I  1.80 F  1.85 F  1.88 I  1.93 I
1.98 I  2.03 F  2.03 F  2.16 F  2.26 F  2.31 F  2.31 F  2.36 F
2.36 F  2.39 F  2.41 F  2.44 F  2.46 F  2.56 O  2.67 F  2.72 I
2.79 F  2.84 F  3.25 O  3.28 O  3.33 F  3.56 F  3.58 F  3.66 F
3.68 O  3.71 F  3.89 F
;
proc catmod; response logits; direct length;
   model choice = length / pred=prob  pred=freq;
run;       


A.14  SAS file for ordinal logit models with Table 8.6

data ideology;
input party ideology count @@;
cards;
1 1 80   1 2 81   1 3 171   1 4 41   1 5 55
0 1 30   0 2 46   0 3 148   0 4 84   0 5 99
;
proc logistic; weight count;
   model ideology = party;
proc catmod; weight count; response clogits; 
   model ideology = _response_ party;
proc catmod; weight count; response alogits; 
   model ideology = _response_ party;
run;


A.15  SAS file for square-table models with Table 9.6

data sex;
input premar  extramar  symm  qi  count @@;
cards;
 1 1 1 1 144   1 2 2 5 2  1 3 3 5 0   1 4 4 5 0
2 1 2 5 33    2 2 5 2 4   2 3 6 5 2   2 4 7 5 0
3 1 3 5 84   3 2 6 5 14   3 3 8 3 6   3 4 9 5 1
4 1 4 5 126   4 2 7 5 29   4 3 9 5 25  4 4 10 4 5
;
proc genmod; class symm;
   model  count = symm / dist=poi link=log;
proc genmod; class extramar premar symm;
   model  count = symm extramar premar / dist=poi link=log;
proc genmod; class symm;
   model  count = symm extramar premar / dist=poi link=log;
proc genmod; class extramar premar qi;
   model count = extramar premar qi / dist=poi link=log;
proc freq; weight count;
   tables premar*extramar / agree;
data sex2;
input score below above @@; trials = below + above;
cards;
1 33 2   1 14 2   1 25 1  2 84 0  2 29 0  3 126 0
;
proc genmod; 
   model above/trials = score / dist=bin link=logit noint;
proc genmod;
   model above/trials = / dist=bin link=logit noint;
run;


A.16  SAS file for Bradley-Terry model with Table 9.9

data tennis;
input wins matches seles graf sabat navrat sanchez; 
cards;
2  5  1 -1  0  0  0
1  1  1  0 -1  0  0
3  6  1  0  0 -1  0
2  2  1  0  0  0 -1
6  9  0  1 -1  0  0
3  3  0  1  0 -1  0
7  8  0  1  0  0 -1
1  3  0  0  1 -1  0
3  5  0  0  1  0 -1
3  4  0  0  0  1 -1
;
proc genmod; 
   model wins/matches = seles graf sabat navrat sanchez / 
   dist=bin link=logit noint covb;
proc genmod; 
   model wins/matches =  / dist=bin link=logit noint; 
run;

|#

;; data crab;
(data crab
3  3  28.3  8  3050
4  3  22.5  0  1550
2  1  26.0  9  2300
4  3  24.8  0  2100
4  3  26.0  4  2600
3  3  23.8  0  2100
2  1  26.5  0  2350
4  2  24.7  0  1900
3  1  23.7  0  1950
4  3  25.6  0  2150
4  3  24.3  0  2150
3  3  25.8  0  2650
3  3  28.2  11 3050
5  2  21.0  0  1850
3  1  26.0  14  2300
2  1  27.1  8  2950
3  3  25.2  1  2000
3  3  29.0  1  3000
5  3  24.7  0  2200
3  3  27.4  5  2700
3  2  23.2  4  1950
2  2  25.0  3  2300
3  1  22.5  1  1600
4  3  26.7  2  2600
5  3  25.8  3  2000
5  3  26.2  0  1300
3  3  28.7  3  3150
3  1  26.8  5  2700
5  3  27.5  0  2600
3  3  24.9  0  2100
2  1  29.3  4  3200
2  3  25.8  0  2600
3  2  25.7  0  2000
3  1  25.7  8  2000
3  1  26.7  5  2700
5  3  23.7  0  1850
3  3  26.8  0  2650
3  3  27.5  6  3150
5  3  23.4  0  1900
3  3  27.9  6  2800
4  3  27.5  3  3100
2  1  26.1  5  2800
2  1  27.7  6  2500
3  1  30.0  5  3300
4  1  28.5  9  3250
4  3  28.9  4  2800
3  3  28.2  6  2600
3  3  25.0  4  2100
3  3  28.5  3  3000
3  1  30.3  3  3600
5  3  24.7  5  2100
3  3  27.7  5  2900
2  1  27.4  6  2700
3  3  22.9  4  1600
3  1  25.7  5  2000
3  3  28.3  15  3000
3  3  27.2  3  2700
4  3  26.2  3  2300
3  1  27.8  0  2750
5  3  25.5  0  2250
4  3  27.1  0  2550
4  3  24.5  5  2050
4  1  27.0  3  2450
3  3  26.0  5  2150
3  3  28.0  1  2800
3  3  30.0  8  3050
3  3  29.0  10 3200
3  3  26.2  0  2400
3  1  26.5  0  1300
3  3  26.2  3  2400
4  3  25.6  7  2800
4  3  23.0  1  1650
4  3  23.0  0  1800
3  3  25.4  6  2250
4  3  24.2  0  1900
3  2  22.9  0  1600
4  2  26.0  3  2200
3  3  25.4  4  2250
4  3  25.7  0  1200
3  3  25.1  5  2100
4  2  24.5  0  2250
5  3  27.5  0  2900
4  3  23.1  0  1650
4  1  25.9  4  2550
3  3  25.8  0  2300
5  3  27.0  3  2250
3  3  28.5  0  3050
5  1  25.5  0  2750
5  3  23.5  0  1900
3  2  24.0  0  1700
3  1  29.7  5  3850
3  1  26.8  0  2550
5  3  26.7  0  2450
3  1  28.7  0  3200
4  3  23.1  0  1550
3  1  29.0  1  2800
4  3  25.5  0  2250
4  3  26.5  1  1967
4  3  24.5  1  2200
4  3  28.5  1  3000
3  3  28.2  1  2867
3  3  24.5  1  1600
3  3  27.5  1  2550
3  2  24.7  4  2550
3  1  25.2  1  2000
4  3  27.3  1  2900
3  3  26.3  1  2400
3  3  29.0  1  3100
3  3  25.3  2  1900
3  3  26.5  4  2300
3  3  27.8  3  3250
3  3  27.0  6  2500
4  3  25.7  0  2100
3  3  25.0  2  2100
3  3  31.9  2  3325
5  3  23.7  0  1800
5  3  29.3  12  3225
4  3  22.0  0  1400
3  3  25.0  5  2400
4  3  27.0  6  2500
4  3  23.8  6  1800
2  1  30.2  2  3275
4  3  26.2  0  2225
3  3  24.2  2  1650
3  3  27.4  3  2900
3  2  25.4  0  2300
4  3  28.4  3  3200
5  3  22.5  4  1475
3  3  26.2  2  2025
3  1  24.9  6  2300
2  2  24.5  6  1950
3  3  25.1  0  1800
3  1  28.0  4  2900
5  3  25.8  10 2250
3  3  27.9  7  3050
3  3  24.9  0  2200
3  1  28.4  5  3100
4  3  27.2  5  2400
3  2  25.0  6  2250
3  3  27.5  6  2625
3  1  33.5  7  5200
3  3  30.5  3  3325
4  3  29.0  3  2925
3  1  24.3  0  2000
3  3  25.8  0  2400
5  3  25.0  8  2100
3  1  31.7  4  3725
3  3  29.5  4  3025
4  3  24.0  10 1900
3  3  30.0  9  3000
3  3  27.6  4  2850
3  3  26.2  0  2300
3  1  23.1  0  2000
3  1  22.9  0  1600
5  3  24.5  0  1900
3  3  24.7  4  1950
3  3  28.3  0  3200
3  3  23.9  2  1850
4  3  23.8  0  1800
4  2  29.8  4  3500
3  3  26.5  4  2350
3  3  26.0  3  2275
3  3  28.2  8  3050
5  3  25.7  0  2150
3  3  26.5  7  2750
3  3  25.8  0  2200
4  3  24.1  0  1800
4  3  26.2  2  2175
4  3  26.1  3  2750
4  3  29.0  4  3275
2  1  28.0  0  2625
5  3  27.0  0  2625
3  2  24.5  0  2000
)
(def crab (matrix (list (/ (length crab) 5) 5) crab))

;; input  color spine  width  satell  weight;
(def color (column 1 crab))
(def spine (column 2 crab))
(def width (column 3 crab))
(def satell (column 4 crab))
(def weight (column 5 crab))
(def y (mapcar #'(lambda(x) (if (> x 0) 1 0)) satell))

(def weight (calc weight/1000))
(def color (calc color - 1))
(def dark (mapcar #'(lambda(x) (if (= x 4) 0 (if (< 4 x) 1))) color))

(logistic weight y)
(logistic (list color weight width) y)
