(princ "
	This is Dan Kalman's latest code. He's also got a test routine at the
end which he called testf, but which I've renamed kalmans-test. Try it via:

			(try-out kalmans-test)
")
#|
From kalman@email.cas.american.edu Thu Jan 16 21:00 CST 1997
To: ael@maya.medctr.luc.edu
Subject: newer version of lisp code
Content-Type: text
Content-Length: 5982
Status: R

Dear Andy,

I found a newer version of the lisp that goes with the paper I handed
out in San Diego.  This version really implements everything just as
in the talk.  I have not worked with it very much, and suspect it may
perform worse than the other version.  But the code should be more
understandable.  Hope you have fun with it.

-dan
|#

(defun DSpair (part1 part2)
  "join two parts to make a DS"
  (cond
   ((and part1 part2) (list part1 part2))
   (t nil)))
	
;(defun DSmake-const (n m &optional (value 0))
(defun DSmake-const (n m value)
  "  Creates a constant element of DS(N,M), zero by default
  or equal to the optional VALUE "
  (cond 
   ((or (< n 0) (< m 0)) nil)
   ((or (= m 0) (= n 0)) value)
   (t (DSpair (DSmake-const (- n 1) m value) (DSmake-const n (- m 1) 0)))))

(defun DSmake-var (n m var-num value)
  "Make an element of DS(n,m) for the VAR-NUM th variable"
  (cond
   ((or (< m 1) (< n 1)) value)
   ((or (< var-num 1) (> var-num n)) nil)
   ((=  n var-num) (DSpair (DSmake-const (- n 1) m value) 
			 (DSmake-const n (- m 1) 1)))
   (t              (DSpair (DSmake-var (- n 1) m var-num value) 
			 (DSmake-const n (- m 1) 0)))))
(defun DS-V (d-struc)
  "V projection - reduce variable index by 1"
  (cond
   ((not (listp d-struc)) d-struc)
   (t (car d-struc))))

(defun DS-D (d-struc)
  "D projection - throw away the first component"
  (cond
   ((not (listp d-struc)) d-struc)
   (t (car (cdr d-struc)))))

(defun DS-H (d-struc)
  "slice off highest order entries"
  (cond
   ((not (listp d-struc)) d-struc)
   ((listp (car (cdr d-struc)))   ;; m > 1 case
    (DSpair (DS-H (DS-V d-struc)) (DS-H (DS-D d-struc))))
   (t (DS-H (DS-V d-struc)))))

(defun DS-val (d-struc)
  "extract the function value"
  (if (listp d-struc) (DS-val (DS-H d-struc)) 
    d-struc))

(defun DS-extract (d-struc p-list)
  "Extract a partial derivative from d-struc.  p-list specifies the 
   number of derivatives for each variable, listed in reverse order.  The
   first entry in the list is interpreted as the number of derivatives 
   with respect to the last variable in the d-struc"
  (cond
   ((not p-list) (DS-val d-struc))
   ((= 0 (car p-list)) (DS-extract (DS-V d-struc) (cdr p-list)))
   (t (let* (
	     (n (car p-list))
	     (n1 (- n 1))
	     (new-p-list (cons n1 (cdr p-list))))
	(DS-extract (DS-D d-struc) new-p-list)))))


(defun DS+const (ds const)
  "add a const to a ds"
  (cond
   ((or (not ds) (not const)) nil)
   ((listp const) nil)
   ((listp ds) (DSpair (DS+const (DS-V ds) const) (DS-D ds)))
   (t (+ ds const))))

(defun DS*const (ds const)
  "multiply a ds and a const"
  (cond
   ((or (not ds) (not const)) nil)
   ((listp const) nil)
   ((listp ds) (DSpair (DS*const (DS-V ds) const)
			(DS*const (DS-D ds) const)))
   (t (* ds const))))

(defun DS+ (ds1 ds2)
  "DS addition"
  (cond
   ((or (not ds1) (not ds2)) nil)
   ((and (listp ds1) (listp ds2)) (DSpair (DS+ (DS-V ds1) (DS-V ds2))
				      (DS+ (DS-D ds1) (DS-D ds2))))
   ((or (listp ds1) (listp ds2)) nil)
   (t (+ ds1 ds2))))

(defun DS- (ds1 ds2)
  "DS subtraction"
   (DS+ ds1 (DS*const ds2 -1)))

(defun DS* (ds1 ds2)
  "DS multiplication"
  (cond
   ((or (not ds1) (not ds2)) nil)
   ((and (listp ds1) (listp ds2)) 
    (DSpair (DS* (DS-V ds1) (DS-V ds2))
	     (DS+ 
	      (DS* (DS-D ds1) (DS-H ds2))
	      (DS* (DS-H ds1) (DS-D ds2)))))
   ((or (listp ds1) (listp ds2)) nil)
   (t (* ds1 ds2))))
  
;;(defun DSrecur (a real-f f df &rest args)
(defun DSrecur (a real-f f df args)
  "General purpose recursion shell for auto diff for the function f.
First arg is A, the DS which is being operated on.  Second arg REAL-F 
is the procedure to perform if A is real.   Args F and DF are 
the function and its derivative. The remaining args list ARGS is provided 
to permit parameters in function definitions.  For example, an integer 
power function can be defined with the power as a parameter."
  (cond 
   ((not a) nil)
   ((not (listp a)) (apply real-f a args))
   (t (let* ((Va   (DS-V a))
             (Ha   (DS-H a))
             (Da   (DS-D a))
             (fVa  (apply f Va args))
             (dfHa (apply df Ha args)))           
        (DSpair fVa (DS* dfHa Da))))))

(defun DSpwr (ds n)
  "ds^n"
  (DSrecur ds 'int-pwr 'DSpwr 'DSpwr-deriv (list n)))

(defun int-pwr (a n)
  (let* (
	 (n1 (- n 1)))
    (cond
     ((< n 0) nil)
     ((= n 0) 1 )
     (t (* a (int-pwr a n1))))))

(defun DSpwr-deriv (ds n)
  "derivative function for DSpwr"
  (let* (
	 (n1 (- n 1)))
    (if (= n 0) (DS*const ds 0) (DS*const (DSpwr ds n1) n))))
	 

(defun real-recip (a) (if (/= a 0) (/ 1 a) nil))
(defun DSdrecip (a) (DS*const (DSrecip (DS* a a)) -1))
(defun DSrecip (a) (DSrecur a 'real-recip 'DSrecip 'DSdrecip nil))

(defun real-sqrt (a) (if (>= a 0) (sqrt a) nil))
(defun DSdsqrt (a) (DSrecip (DS*const (DSsqrt a) 2)))
(defun DSsqrt (a) (DSrecur a 'real-sqrt 'DSsqrt 'DSdsqrt nil))

(defun DSexp (a) (DSrecur a 'exp 'DSexp 'DSexp nil))
(defun DSsin (a) (DSrecur a 'sin 'DSsin 'DScos nil))
(defun DSdcos (a) (DS*const (DSsin a) -1))
(defun DScos (a) (DSrecur a 'cos 'DScos 'DSdcos nil))

(defun real-log (a) (if (< 0 a) (log a) nil))
(defun DSlog (a) (DSrecur a 'real-log 'DSlog 'DSrecip nil))



(setq dsmap '(((0 (x (xx xxx))) ((y (xy xxy)) ((yy xyy) yyy))) (((z (xz xxz)) ((yz xyz) yyz)) (((zz xzz) yzz) zzz))))


(defun DS-print (ds)
  (cond
   ((not ds) nil)
   ((not (listp  ds)) (prin1 ds) (princ "\n\n"))
   ((not (listp (car ds))) (prin1 ds) (princ "\n"))
   (t  (DS-print (DS-V ds))
       (DS-print (DS-D ds)))))

(progn (princ "\n\n\n") (DS-print dsmap))

(defun kalmans-test (x-in y-in z-in w-in nderiv p-list)
  "
;; Examples:
 (kalmans-test 1 2 3 2 3 nil)
"
  (let* (
	 (x (DSmake-var 4 nderiv 1 x-in))
	 (y (DSmake-var 4 nderiv 2 y-in))
	 (z (DSmake-var 4 nderiv 3 z-in))
	 (w (DSmake-var 4 nderiv 4 w-in))
	 (x3 (DSpwr x 3))
	 (y2 (DSpwr y 2))
	 (xy2 (DS* x y2))
	 (x3z (DS* x3 z))
	 (w4 (DSpwr w 4))
	 (f (DS* w4 (DS+ xy2 x3z))))
    (princ "\n")
    (DS-print f)
    (DS-extract f p-list)))

