#|
From majordom@stat.umn.edu Tue Apr  1 21:07 CST 1997
Received: from umnstat.stat.umn.edu (umnstat.stat.umn.edu [128.101.51.1]) by maya.medctr.luc.edu (8.7.3/8.6.9) with SMTP id VAA13023 for <ael@maya.medctr.luc.edu>; Tue, 1 Apr 1997 21:07:38 -0600 (CST)
Received: by umnstat.stat.umn.edu
	(1.38.193.4/16.2) id AA29805; Tue, 1 Apr 1997 20:15:33 -0600
Received: from salmon.maths.tcd.ie by umnstat.stat.umn.edu with SMTP
	(1.38.193.4/16.2) id AA29538; Tue, 1 Apr 1997 19:47:36 -0600
Received: from orange.stats.tcd.ie by salmon.maths.tcd.ie  with SMTP id aa09807;
          2 Apr 97 2:47 +0100
Received: from  by stats.tcd.ie (4.1/SMI-4.1)
	id AB07790; Wed, 2 Apr 97 02:34:43 BST
X-Sender: dbearly@orange.stats.tcd.ie
Message-Id: <l03010d01af6767e5c2fe@[194.125.134.126]>
Mime-Version: 1.0
Date: Wed, 2 Apr 1997 01:47:16 +0000
To: stat-lisp-news@umnstat.stat.umn.edu
From: "Dom. Early" <Dom.Early@tcd.ie>
Subject: barchart class (part 2 of 3)
Sender: owner-stat-lisp-news@stat.umn.edu
Precedence: bulk
Content-Type: text/plain; charset="us-ascii"
Content-Length: 18313
Status: R
|#
(require "domutils")
(require "graphs")
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; BARCHART.lsp : Ver 1.0 beta (2 April 1997)
;;
;; A prototype class for a bar chart
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;;
;; (c) Dom. Early 1997
;;
;; Dom. Early
;; Dept. of Statistics
;; Trinity College Dublin
;; Ireland
;;
;;
;;DEVELOPMENT ENVIRONMENT:
;;
;;            Machine:	PowerMac 7200/90
;;   Operating system:	MacOS 7.5.5
;;         XLISP-STAT:  3.50 Beta (PowerMac Binary)
;;
;;
;;SOURCE CODE
;;   Source code is optimised for a 4 space tab width and indented according
;;   to my own personal preferences (sorry, I know hardened lisp'ers will hate
;;   it, so pretty print it! 8)
;;
;;
;;COMMENTS:
;;   Users are free to use this code, UNMODIFIED, in their packages, provided
;;   they clearly acknowledge it's use and credit the author.
;;
;;   If you have any bug reports, suggestions or comments about this code,
;;   or if you have suggestions on how to improve it, please let me know
;;   via email.  Also, if you find this code useful, I'd appreciate your
;;   letting me know that as well.
;;
;;   This code has only been tested on non Power Mac platforms so I am unsure
;;   as to how well or badly it performs on Windows/Unix boxes.
;;   Comments from "beta testers" for these platforms are especially welcome.
;;
;;
;;							Enjoy,
;;
;;	Dom.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;;OVERVIEW:
;;   The primary function in the code below is to produce a well-integrated
;;   barchart type plot to XLisp-Stat.  Other implementations of barcharts
;;   are designed to provide basic barchart facilities only, and lack many
;;   of the features of the built-in plots.
;;
;;   My barchart-proto is implemented as a subclass of histogram-proto
;;   and I have tried to implement the package as closely to the standard
;;   histogram interface as possible.
;;
;;   Essentially a barchart of say, 5 categories is built using a histogram
;;   of 9 bins, with every second bin containing zero elements.  My code then
;;   adds labels to each bin, the label being the value of the category.
;;
;;
;;USAGE:
;;   Use in a similar manner to histogram except barchart accepts non-numeric
;;   data.  e.g.
;;
;;   >(barchart '(foo bar foobar foo bar foo 1 1 2 3 "foo" "FOOBAR"))
;;
;;   This produces a barchart with the following categories:
;;					'(1 2 3 BAR FOO FOOBAR foo)
;;
;;   (note that it converts all elements to strings so foobar is equivalent to
;;   "FOOBAR").
;;
;;   In physically small barcharts or barcharts with a lot of categories labels
;;   can obscure each other so I have provided an option to switch to
;;   vertically
;;   oriented labels.  Select Options... from the Barchart menu when the
;;   barchart is the active window and click on the Vertical Labels checkbox.
;;
;;
;;ADVANTAGES:
;;   Has all features of standard plots including selecting and brushing,
;;   linking, hiding/focusing selections, selection colors, and plot options.
;;
;;
;;DISADVANTAGES:
;;   Limited to 25 categories
;;
;;
;;KNOWN PROBLEMS
;;   A problem with to do with scaling/rounding errors that occurs when the
;;	 the barchart is resized to a very small size, or when the barchart
;;   contains a lot of categories could leave two contiguous non-empty bars.
;;   A fix simply tests for this problem and then resizes the barchart until
;;   the problem 'goes away'.  Certain other functions which call the plot
;;   method :adjust-to-data may also cause the graph size to be adjusted
;;   slightly under certain circumstances.
;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; PROTOTYPE DECLARATION
;;
;; sBarLabelList   	- Used to store the labels for each bar
;; sVerticalLabels 	- Boolean flags whether to display labels vertically
;; sScaleFix		- Boolean flags whether in scale fix cycle
;; sDoScaleFix		- Boolean flags whether scale fix is enabled
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defproto Barchart-Proto '(sBarLabelList sVerticalLabels sScaleFix
sDoScaleFix)
							() histogram-proto)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defmeth Barchart-Proto :BarLabel(pBarIndex)
"Args: (pBarIndex:NATURAL)
Return nth bar label from our list of labels"
	(GetElement (send self :BarLabelList) pBarIndex)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defmeth Barchart-Proto :VerticalLabels(&optional (pVerticalLabels nil pSet))
"Args: (pVerticalLabels:BOOLEAN)
Gettor/Settor for sVerticalLabels flag, which flags whether the labels
should be printed horizontally or vertically"
	(when pSet
		(send self :slot-value 'sVerticalLabels pVerticalLabels)
		(send self :AdjustMargins)
	)
	(send self :slot-value 'sVerticalLabels)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defmeth Barchart-Proto :ScaleFix(&optional (pScaleFix nil pSet))
"Args: (pScaleFix:BOOLEAN)
Gettor/Settor for sScaleFix flag, which flags whether are in the
scale fix cycle"
	(if pSet
		(send self :slot-value 'sScaleFix pScaleFix)
	)
	(send self :slot-value 'sScaleFix)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defmeth Barchart-Proto :DoScaleFix(&optional (pDoScaleFix nil pSet))
"Args: (pDoScaleFix:BOOLEAN)
Gettor/Settor for sDoScaleFix flag, which flags whether scale fixing
is enabled"
	(if pSet
		(send self :slot-value 'sDoScaleFix pDoScaleFix)
	)
	(send self :slot-value 'sDoScaleFix)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defmeth Barchart-proto :BarLabelList(&optional (pList nil pSet))
"Args: (pList:LIST OF STRINGS)
Gettor/Settor for sBarLabelList slot, which contains the list of
labels for the barchart bars"
	(if pSet
		(send self :slot-value 'sBarLabelList pList)
	)
	(send self :slot-value 'sBarLabelList)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defmeth Barchart-proto :AdjustMargins()
"Args: ()
Sets the margins of the plot to accomadate the labels of the bars,
according to the label orientation"
	(when (send self :VerticalLabels)
		;; Adjust the margins to allow for vertical label strings
		;; Algorithm :
		;; Find the biggest label string length
		;; Set self margin to allow for that string length
		;; Resize ourself to include the new margin
		(let(	(maxtw 0)
			)
			(dolist(label (send self :BarLabelList) nil)
				(if(< maxtw (send self :text-width label))
					(setf maxtw (send self :text-width
label))
				)
			)
			(send self :margin 0 0 0 (- maxtw 20))
			(send self :size
				(+ (car (send self :size)) maxtw)
				(+ (cadr (send self :size)) maxtw)
			)
		)
	)

	(when (not (send self :VerticalLabels))
		(send self :margin 0 0 0 0)
	)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defmeth Barchart-proto :CountElements(pList)
"Args: (pList:LIST OF VALUES)
Inputs a list of values
Returns a list of unique values from the input list, together with
a list of the count of each unique element in the input list"
	(let(	(elementlist (sort (UniqueList pList) #'string<))
			(elementcount '())
			(curindex 0)
		)
		(dotimes (i (length elementlist))
			(setf elementcount (AddElement elementcount 0))
		)
		(dolist(element pList nil)
			(setf curindex (GetElementIndex element elementlist))
			(setf elementcount (SetElement elementcount curindex
									(+
1 (GetElement elementcount curindex))
							   )
			)
		)
		(list elementlist elementcount)
	)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defmeth Barchart-proto :CheckBinCounts()
"Args: ()
Ensure that every second bar is empty so histogram appears as a barchart.
Returns true if any of the odd bars is non-empty.  Used to fix the scaling
problem"
	(let(	(count 0)
			(warning false)
			(element nil)
			(bincounts (send self :bin-counts))
		)
		(loop
			(if (or warning (Null bincounts))
				(return)
			)
			(setf element (car bincounts))
			(setf bincounts (cdr bincounts))
			(if (oddp count)
				(if (Neq 0 element)
					(setf warning true)
				)
			)
			(setf count (+ count 1))
		)
		warning
	)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defmeth  Barchart-proto :AdjustFrameSize()
"Args: ()
Increments the horizontal aspect of the fame-size by one.  Used to fix
the scaling problem"
	(send self :frame-size (1+ (car (send self :frame-size)))
								(cadr (send
self :frame-size))
	)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defmeth Barchart-proto :Redraw()
"Args: ()
Overloads the histogram-proto method to redraw the barchart.
Draws the histogram and then adds the labels"
	(when (not (send self :DoScaleFix))
		(send self :start-buffering)
		(send self :erase-window)
		(call-next-method)
		(send self :LabelBars)
		(send self :buffer-to-screen)
	)

	(when (send self :DoScaleFix)
		(when (not (send self :ScaleFix))
			(send self :start-buffering)
		)
;;		(send self :erase-window)
		(call-next-method)
		(when (not (send self :ScaleFix))
			(send self :ScaleFix true)
			(if (send self :CheckBinCounts)
				(loop
					(send self :AdjustFrameSize)
					(send self :adjust-to-data :draw false)
					(send self :resize)
					(send self :erase-window)
					(send self :redraw)
					(if (not (send self :CheckBinCounts))
						(return)
					)
				)
			)
			(send self :ScaleFix false)
		)
		(when (not (send self :ScaleFix))
			(send self :LabelBars)
			(send self :buffer-to-screen)
		)
	)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defmeth Barchart-proto :LabelBars()
"Args: ()
Draws the labels for each bar either vertically or horizontally"
	(let*(	(crect (send self :content-rect))
			(left (car crect))
			(top (car (cdr crect)))
			(width (car (cdr (cdr crect))))
			(height (car (cdr (cdr (cdr crect)))))
			(bottom (+ top height))
	        (nbars (send self :num-bins))
	        (bwidth (/  (third crect) nbars))
	        (ta (send self :text-ascent))
	        (tw 0)
	        (maxtw 0)
	        (curlabel "")
	 	)
		(dotimes(i nbars)
			(when (evenp i)
				(setf curlabel (send self :BarLabel (/ i 2)))
				(if(< maxtw (send self :text-width curlabel))
					(setf maxtw (send self :text-width
curlabel))
				)
			)
		)
		(dotimes(i nbars)
			(when (evenp i)
				(setf curlabel (send self :BarLabel (/ i 2)))
				(setf tw (send self :text-width curlabel))

				(if (send self :VerticalLabels)
					(send self :draw-string-up
						curlabel
						(round (+ (* i bwidth) left
ta))
						(+ bottom maxtw 5)
					)
					(send self :draw-string
						curlabel
						(round (+ (* i bwidth) left))
						(+ bottom (+ ta 5))
					)
				)
			)
		)
	)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defmeth Barchart-proto :adjust-to-data(&key (draw true))
  "Args: (draw:BOOLEAN)
Overloads the histogram-proto method, redrawing if draw is true.
Resets the number of bins and sets the scaled range"
  (call-next-method :draw draw)
  
  (send self :num-bins (- (* (length (send self :BarLabelList)) 2) 1))
  (send self :x-axis true nil 0)
  (send self :scaled-range 0 0 (- (length (send self :BarLabelList)) 1))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun barchart(pList &key (title "Barchart"))
  "
Args: (pList:LIST title:STRING)

Creates a barchart from the data in pList, which presumably contains several
repetitions of data values.

;; Examples:
 ;; This produces a barchart with the categories 1 2 3 BAR FOO FOOBAR foo:
 (barchart '(foo bar foobar foo bar foo 1 1 2 3 \"foo\" \"FOOBAR\"))
"
  (let*(
	(bchart (send Barchart-proto :new 1 :show false))
	(strlist (mapcar #'ToString pList))
	(barcounts (send bchart :CountElements strlist))
	(labels (car barcounts))
	(counts (cadr barcounts))
	histlist
	)
    (when (< (length labels) 25)
	  (dolist(element strlist nil)
		 (setf histlist
		       (AddElement 
			histlist (* 1 (GetElementIndex element labels)))))
	  (mapcar
	   (lambda(x) (apply #'send bchart x))
	   (list
	    (list :title title)
	    (list :add-points histlist)
	    (list :scaled-range 0)
	    (list :BarLabelList labels)
	    (list :adjust-to-data :draw false)
	    (list :VerticalLabels false)
	    (list :DoScaleFix true)
	    (list :show-window)
	    )))
    (when (not (< (length labels) 25))
	  (Message-Dialog
	   "cannot plot barcharts of more than 24 categories")
	  (setf bchart nil)
	  )
    bchart
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
(defmeth Barchart-proto :make-options-dialog-items()
"Args: ()
Overload the histogram-proto method to add an option to draw labels
vertically"
	(remove
   		nil
   		(AddElement (call-next-method)
			(send graph-toggle-item-proto :new
				"Vertical Labels" self :VerticalLabels)
		)
	)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(send Barchart-proto :title "Barchart")
(send Barchart-proto :menu-title "Barchart")
(send Barchart-proto :fixed-aspect nil)
(send Barchart-proto :size 250 125)
(send Barchart-proto :menu-template '(	link
;;
	showing-labels

	mouse

	resize-brush
;;
	---------------

	dash
                                        erase-selection
                                        focus-on-selection
                                        show-all
;;                                      symbol

	color
                                        selection
;;
	---------------

	dash
;;
	slicer
                                        rescale
                                        options
#+unix                                  save-image
									)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(provide "BARCHART")
