#|
A little help file to aid in understanding Bithell's statistic. In his paper of
  author = 	"J. F. Bithell",
  title = 	"The choice of test for detecting raised disease risk
		 near a point source",
  journal = 	"Statistics in Medicine",
  year = 	"1995",
  volume = 	"14",
  OPTnumber = 	"",
  pages = 	"2309--2322",
he presents some sample Relative Risk Functions (RRFs).
|#

(setq
 alpha .1
 beta .1
 gamma .1
 delta .1
 eta 1
 xsi 1
 )
(spin-points 10)
;; This is a stupid risk function, tending to huge values as d -> 0:
(func d(x y) (sqrt (x^2+y^2)))
(func f0(x y) (eta * exp(alpha/(d x y))))
(plot f0)
(func f1(x y) (eta * (1+xsi*exp(-(d x y)/beta))))
(plot f1)
(func f2(x y) (eta * (1+xsi*exp(-(x^2+y^2)/gamma^2))))
(plot f2)
(func f3(x y) (eta * (1+xsi/(1 + (d x y)/delta))))
(plot f3)

#|
Now to create a data set which I can use to test Bithell:
|#
(setq
 x (rand.u 100)
 y (rand.u 100)
 focus.x .3
 focus.y .2
 x.shift (- x focus.x)
 y.shift (- y focus.y)
 weave (weave x y)
 names (iseq (length x))
 sizes (repeat 1000 (length x))
 mat (bind-columns
      x y
      (floor (* 50 (mapcar #'f0 x.shift y.shift)))
      (floor (* 50 (mapcar #'f1 x.shift y.shift)))
      (floor (* 50 (mapcar #'f2 x.shift y.shift)))
      (floor (* 50 (mapcar #'f3 x.shift y.shift)))
      )
 done (lines off)
 )
(plot x.shift y.shift (column 3 mat))
(plot x.shift y.shift (column 4 mat))
(plot x.shift y.shift (column 5 mat))
(plot x.shift y.shift (column 6 mat))
(write-matrix-to-file 
 (bind-columns names (select-columns mat '(0 1 2)) sizes)
 "bithell0.txt")
(write-matrix-to-file 
 (bind-columns names (select-columns mat '(0 1 3)) sizes)
 "bithell1.txt")
(write-matrix-to-file 
 (bind-columns names (select-columns mat '(0 1 4)) sizes)
 "bithell2.txt")
(write-matrix-to-file 
 (bind-columns names (select-columns mat '(0 1 5)) sizes)
 "bithell3.txt")
