;;;udina oct 99


(defun convex-hull (lpoints &key draw)
  "
;; Function: convex-hull (lpoints &key draw)

	Returns a list of points to draw the convex hull of the given lpoints.

;; Arguments:

	LPOINTS - should be a list of points. Result is a list of points, given
	          counterclockwise around the mass center and closed.

;; Examples:
 (convex-hull (transpose (list (normal-rand 500) 
                              (normal-rand 500))) 
                  :draw t)
"
  (let ((plopo (when draw
		 (plot-points (transpose lpoints))))
	nlpoints bcnt pord hull v1 v2)
    (labels ((ascent (x y) "angle from axis x>0 to vector x y"
		     (cond ((zerop x)
			    (cond ((> y 0) (/ pi 2))
				  ((< y 0) (* 1.5 pi))
				  (t (error " in ascent, in convex-hull"))))
			   ((> x 0)
			    (if (>= y 0) (atan (/ y x))
			      (+ (* 1.5 pi) (atan (abs (/ x y))))))
			   (t (if (> y 0) (+ (/ pi 2) (atan (abs (/ x y))))
				(+ pi (atan (/ y x)))))))
	     (ascent-lt (p1 p2) "p1 has ascent greater than p2"
			(< (apply #'ascent p1)
			   (apply #'ascent p2)))
	     (gir-convex (p1 p2 p3) "going through p1 p2 p3 is a left turn"
			 ;;sign of 3rd comp of vectorial product decides
			 (>= (apply #'- (* (- p2 p1) (reverse (- p3 p2)))) 0)
			 )
	     (zero-point (p)
			 (and (zerop (car p))
			      (zerop (second p))))
	     (eqpt (p1 p2)
		   (zero-point (- p1 p2)))
	     (treu-zeros (lp) "remove (0 0) from the list lp"
			 (delete-if #'zero-point lp))
	     (treu-repes (lp) "remove repeats"
			 (if lp
			     (union (list (car lp))
				    (treu-repes (cdr lp))
				    :test #'equal)
			   nil)))
	    ;;body
	    (setq nlpoints (treu-repes (copy-list lpoints)))
	    ;;center all points respect mass center
	    (setq bcnt (/ (apply #'+ nlpoints) (length nlpoints)))
	    (setq nlpoints (mapcar #'(lambda (p) (- p bcnt))
				   nlpoints))
	    (setq nlpoints (treu-zeros nlpoints))
	    ;;sort points counterclockwise
	    ;;most of computing time is spent here,
	    ;;probably this can be faster...
	    ;;(setq nlpoints (sort nlpoints #'ascent-lt))
	    (setq nlpoints (select nlpoints 
				   (order (mapcar #'(lambda (p) 
						      (apply #'ascent p))
						  nlpoints))))
	    (when draw
	      (send plopo :add-lines (transpose (setq sortli (mapcar 
							      #'(lambda (p) (+ p bcnt))
							      nlpoints)))))
	    (setq hull (select nlpoints '(1 0)))
	    (setq pord (cddr nlpoints))
	    (loop; do all points counterclockwise
	     (unless pord (return (reverse hull)))
	     (while (and (>= (length hull) 2)
			 (not (gir-convex (cadr hull)
					  (car hull)
					  (car pord))))
	       (setq hull (cdr hull)))
	     (setq hull (cons (car pord) hull)); hull is built in reverse order
	     (setq pord (cdr pord)))
	    (setq canvi t)
	    (loop ;look at joining end to begining
	     (unless canvi (return))
	     (setq canvi nil)
	     (unless (gir-convex (car hull);look at the first point (last in hull)
			     (car (last hull))
			     (car (last (butlast hull))))
	       (setq hull (butlast hull))
	       (setq canvi t))
	     (unless (gir-convex (second hull); look at the end point (car in hull)
				 (car hull)
				 (car (last hull)))
	       (setq hull (cdr hull))
	       (setq canvi t)))
	    ;;close the path in hull
	    (setq canvi (car hull))
	    (setq hull (cons canvi (reverse hull)))
	    ;;uncenter
	    (setq hull (mapcar #'(lambda (p) (+ p bcnt))
			       hull))
	    (when draw (send plopo :add-lines (transpose hull)))
	    hull
	    )))
