;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
Hi Dad,
 
        Here's a kludgy way to get an m x n case to work out (in the sense of
getting curve interpolation with as much differentiability as existed in the
original curves):
 
        Assume m < n: step over the n dimension m curves at a time, and do an
mxm case (n-m+1 times). You now have n-m+1 patch functions, which you can
multiply by bump functions (infinitely differentiable, finitely supported) and
add together to maintain interpolation and differentiability.
 
        It's kludgy, but would work. I'm trying to recall what the bump
functions look like (I've got my analysis notes at home), but it seems to me
that the basic form is:
 
                {exp(-1/(1-x^2))        |x| < 1
        f(x) =  {
                {0                      |x| >= 1
 
 
        Now if you used a pair of these for each patch, essentially only using
each patch at its ends, and put them together in just the right way....

	Well here, let me show you in xlispstat:
Start with the canonical bump-function we'll need:
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(func
 canonical-bump-function(x)
 (if ((abs x) < 1)
     (exp(1-1/(1-x^2)))
   (0))
 )
(def half-area-under-canonical-bump-function
     (simpson canonical-bump-function -1 0 1e-6))
(def area-under-canonical-bump-function
     (* 2 half-area-under-canonical-bump-function))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
	I need a bump function which possesses more symmetry than the canonical
bump function: in particular, I need that

 				f(x)=1-f(1-x).

So I integrate it (from -infinity to x) to get an infinitely differentiable
function from 0 to alpha (which I renormalize to 1).

	I paste two of these together back-to-back, and I have a more symmetric
bump function, called the cbf. Then these are combined to form bi-humped
animals, built up of pairs of cbf functions.
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun integral-canonical-bump-function(X)
  (cond
   ((<= x -1) 0)
   ((<= x 0)
    (/
     (eval `(simpson canonical-bump-function -1 ,x .001))
     area-under-canonical-bump-function)
    )
   ((<= x 1)
    (- 1 (integral-canonical-bump-function (- x)))
    )
   (t 1)
   )
  )

(defun new-bf(x a b c d)
  (cond
   ((<= x a)
    0
    )
   ((<= x b)
    (integral-canonical-bump-function (- (/ (* 2 (- x a)) (- b a)) 1))
    )
   ((<= x c)
    1
    )
   ((<= x d)
    (integral-canonical-bump-function (+ (/ (* -2 (- x c)) (- d c)) 1))
    )
   (t 0)
   )
  )
(defun front-humped-bump-function(x c d)
  (cond
   ((<= x c)
    1
    )
   ((<= x d)
    (integral-canonical-bump-function (+ (/ (* -2 (- x c)) (- d c)) 1))
    )
   (t 0)
   )
  )
(defun back-humped-bump-function(x a b)
  (cond
   ((<= x a)
    0
    )
   ((<= x b)
    (integral-canonical-bump-function (- (/ (* 2 (- x a)) (- b a)) 1))
    )
   (t 1)
   )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
	Plot the bump functions, if desired:
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun plot-bumps()
  (plot canonical-bump-function -2 2)
  (plot integral-canonical-bump-function -2 2)
  (plot (lambda(x) (new-bf x -1.5 -1 0 1)) -2 2)
  (plot (lambda(x) (front-humped-bump-function x -1.5 -1)) -2 2)
  (plot (lambda(x) (back-humped-bump-function x 0 1)) -2 2)
  )
;; (plot-bumps)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
	Start with some complicated function (which generates a full matrix a):
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(func f(x y) sin(6*ln(1+x)*y) + y*x + (exp y*(1-x)))
(def m 3)
(def n 5)
(title (num-to-string n))
(def xs (rseq 0 1 m))
(def ys (rseq 0 1 n))
(def A (outer-product xs ys #'f))
(spin-domain 0 1 0 1)
(spin-points n)

(def m-1 (1- m))
(def n-m+1 (1+ (- n m)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
	Now I'm going to create the monster function, which is the functions
from the previous example times the appropriate grid functions.
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def matrices
     (mapcar
      (lambda(i)
	(let (
	      (suba (submatrix a 0 m-1 i (+ i m-1)))
	      (some (select ys (iseq i (+ i m-1))))
	      )
	  (inverse suba)
	  )
	)
      (iseq n-m+1)
      )
     )

(def G-functions
     (mapcar
      (lambda(i)
	(let ((some (select ys (iseq i (+ i m-1)))))
	  (lambda(x) (mapcar (lambda(y) (f x y)) some))
	  )
	)
      (iseq n-m+1)
      )
     )

;; types: 'f (front), 'n (normal), 'b (back):
(def types (combine 'f (if (= n 4) 'b (combine (repeat 'n (- n 4)) 'b))))
(def H-functions
     (mapcar
      (lambda(i)
	(let (
	      (some (select ys (iseq i (+ i m-1))))
	      (type (elt types i))
	      )
	  (lambda(y) (* 
		      (cond
		       ((equalp type 'f)
			(front-humped-bump-function
			 y (final (butlast some)) (final some))
			)
		       ((equalp type 'b)
			(back-humped-bump-function
			 y (first some) (second some))
			)
		       (t
			(new-bf y
				(first some) (second some)
				(second (reverse some)) (first (reverse some))
				)
			)
		       )
		      (mapcar (lambda(x) (f x y)) xs)
		      )
	    )
	  )
	)
      (iseq n-m+1)
      )
     )
#|
(mapcar (lambda(x) (eval `(plot (lambda(x) (first (funcall ,x x))) 0 1)))
	h-functions)
(mapcar (lambda(x) (eval `(plot (lambda(x) (second (funcall ,x x))) 0 1)))
	h-functions)
(mapcar (lambda(x) (eval `(plot (lambda(x) (third (funcall ,x x))) 0 1)))
	h-functions)
|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
	Now we create the skin function of these elements:
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun skin(x y)
	(sum
	 (mapcar
	  (lambda(G A-1 H)
	    (matprod (funcall G x) A-1 (funcall H y))
	    )
	  G-functions
	  matrices
	  H-functions
	  )
	 )
	)

(plot f skin)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
	Increase the number of points, and plot it again:
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(spin-points (+ n (1- n)))
(plot f skin)
(plot (lambda(x y) (- (f x y) (skin x y))))
(print 
 (max (outer-product
       (rseq 0 1 (spin-points (+ n (1- n))))
       (rseq 0 1 (spin-points (+ n (1- n))))
       (lambda(x y) (- (f x y) (skin x y)))))
 )

(defun type-check(n)
  (let (
	(ys (rseq 0 1 n))
	(types (combine 'f (if (= n 4) 'b (combine (repeat 'n (- n 4)) 'b))))
	)
    (flet ((lam(x)
	       (sum
		(mapcar
		 (lambda(i type xstart xend)
		   (let (
			 (some (select ys (iseq i (+ i m-1))))
			 )
		     (cond
		      ((equalp type 'f)
		       (front-humped-bump-function
			x (final (butlast some)) (final some))
		       )
		      ((equalp type 'b)
		       (back-humped-bump-function
			x (first some) (second some))
		       )
		      (t
		       (new-bf x
			       (first some) (second some)
			       (second (reverse some)) (first (reverse some))
			       )
		       )
		      )
		     )
		   )
		 (iseq (length types))
		 types
		 (select ys (iseq (length types)))
		 (select-from-end ys (iseq (length types)))
		 ))))
	  (max (abs (- 1 (mapcar #'lam (rseq 0 1 100)))))
	  )
    )
  )
;; (mapcar #'type-check (iseq 4 8))

(plot (lambda(x) (skin x 0)) (lambda(x) (f x 0)) 0 1)
(plot (lambda(x) (skin x .25)) (lambda(x) (f x .25)) 0 1)
(plot (lambda(x) (skin x .5)) (lambda(x) (f x .5)) 0 1)
(plot (lambda(x) (skin x .75)) (lambda(x) (f x .75)) 0 1)
(plot (lambda(x) (skin x 1)) (lambda(x) (f x 1)) 0 1)

(plot (lambda(x) (skin 0 x)) (lambda(x) (f 0 x)) 0 1)
(plot (lambda(x) (skin .5 x)) (lambda(x) (f .5 x)) 0 1)
(plot (lambda(x) (skin 1 x)) (lambda(x) (f 1 x)) 0 1)
(plot (lambda(x) (skin .75 x)) (lambda(x) (f .75 x)) 0 1)

(plot (lambda(x) (- (skin x 0)  (f x 0))) 0 1)
(plot (lambda(x) (- (skin x .25) (f x .25))) 0 1)
(plot (lambda(x) (- (skin x .5)  (f x .5))) 0 1)
(plot (lambda(x) (- (skin x .75)  (f x .75))) 0 1)
(plot (lambda(x) (- (skin x 1) (f x 1))) 0 1)

(plot (lambda(x) (- (skin 0 x) (f 0 x))) 0 1)
(plot (lambda(x) (- (skin .5 x) (f .5 x))) 0 1)
(plot (lambda(x) (- (skin 1 x) (f 1 x))) 0 1)
