#|
Please find enclosed yet another help system. I like it because is simple.

To use it, just put the args in the doc string, as in

(Defun foo (arg1 arg2 &key (karg 33))
"ARGS: arg1 arg2 &key (karg 33)
A silly function"
(list arg1 arg2 karg))

and call
(dhelp 'foo)
to get a dialog window to fill the arguments.


 Frederic Udina
____________________________________________________________________
Dept. D'Economia i Empresa          | voice: 34- 935 42 17 56/17 63
Universitat Pompeu Fabra            |        fax:  34- 935 42 17 46
       Ramon Trias Fargas, 25-27    |          e-mail: udina@upf.es
       08005 Barcelona SPAIN        |  WWWeb: http://libiya.upf.es/
____________________________________~_______________________________
 Everybody knows that the dice are loaded ...              L. Cohen
|#

;;; ::::::::::::::::::::::::::::::
;;; a simple help system
;;; f. udina, jul 99
;;; ::::::::::::::::::::::::::::::

#| 
Examples:
(defun make-kregression (data headers col1 col2 &rest args)
"ARGS: data headers col1 col2 &rest args"
(...))

(defun draw-cross-summary (data headers row col what &key (output t) (col-width 8) (sorted t))
"ARGS: data headers row col what &key (output t) (col-width 8) (sorted t)
Draws a cross-summary of WHAT in DATA with headers using
ROW and COL (ROW values will be used as row headers,
COL values as column headers in the table drawn).
 Keys: output is an output stream, col-width is cell width,
sorted t means sort the values, nil means do not."
(...))

|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||#

(defun dhelp (funname &optional (what 'dialog))
  "ARGS: funname &optional (what 'dialog)

 This file implements a simple help system for alling lisp functions
 through a dialog window that shows the requiread (and optional)
 arguments.
 
 It requires the documentation string having a particular format. 
 The first line must begin with \"ARGS: \" followed by the lambda list of 
 the arguments in the same format as appear in the defun (see below for
 a couple of examples).
 
 If the function FOO is defined with a conforming doc string, then calling
 (dhelp foo)
 will produce a dialog window showing the required arguments for the call
 and also key arguments, optional arguments or rest arguments.
 
 After loading dhelp.lsp, try (dhelp 'dhelp)
 
 The current version does'nt support more than one 
 keyword of the key, optional, rest.
 
 It would be easy to adapt dhelp to take the lambda list directly from the
 function-lambda-expression (not available for compiled functions) or from 
 the file where the functions are defined. My option was for the simpler 
 but useful way.
 
 ;; Notes:
  Args need to be quoted.
  Given a funname, if what is template, it simply prints a
 template of the call. If what is dialog (default), a dialog window 
 appear for the call arguments being filled. An optional or key arg
 left empty, will not appear in the call.

;; Examples:

 (Defun foo (arg1 arg2 &key (karg 33))
   \"ARGS: arg1 arg2 &key (karg 33)\"
    (list arg1 arg2 karg))
 (dhelp 'foo)
"
  ;;(unless (fboundp funname)
  ;;  (error "~a is'nt bound to any function" funname))
  (let* ((doc (documentation funname 'function))
	 (1stline (second (get-lambda-expression (eval `(function ,funname)))))
	 (1stline (format nil "~{~a ~}" 1stline))
	 (1stline (subseq 1stline 0 (1- (length 1stline))))
	 (args (if (and (stringp 1stline)
			(< 0 (length 1stline))
			)
		   1stline
		 (error
		  "~a doesn't have info enough about the template"
		  funname)))
	 pos pargs kargs oargs rargs
	 )
    (setq pos (position #\& args))
    (when pos
	  (setq pargs (if (= 0 pos) nil
			(subseq args 0 (1- pos))))
	  (cond ((string= "&KEY " (string-upcase (subseq args pos (+ pos 5))))
		 (setq kargs (subseq args (+ pos 5))))
		((string= "&REST " (string-upcase (subseq args pos (+ pos 6))))
		 (setq rargs (subseq args (+ pos 6))))
		((string= "&OPTIONAL " (string-upcase (subseq args pos (+ pos 10))))
		 (setq oargs (subseq args (+ pos 10)))))
	  (when (or (and kargs (position #\& kargs))
		    (and oargs (position #\& oargs))
		    (and rargs (position #\& rargs)))
		(error "more than one keyword not supported: ~a, ~a" funname
		       args)))
    (unless pos (setq pargs args))
    (ecase what
	   (template (format nil "(~a ~a)~%" funname args))
	   (dialog (dhelp-dialog-call funname (dhelp-split-args pargs) 
				      (dhelp-split-args kargs t) 
				      (dhelp-split-args oargs)
				      (dhelp-split-args rargs))))))

(defun dhelp-split-args (args &optional are-keys)
  "given a string, returns a list of words,
 a pair (key value) would be a single word"
  (let* ((rest args)
	 wordlist sppos parpos clparpos)
    (loop
     ;;(format t "rest: ~a~%" rest)
     (loop ;skip spaces
      (when (or (= 0 (length rest))
		(char/= #\space (select rest 0)))
	    (return))
      (setq rest (subseq rest 1)))
     (when (= 0 (length rest)) (return (reverse wordlist)))
     
     (setq sppos (position #\space rest :test #'char=))
     (setq parpos (position #\( rest :test #'char=))
     (cond ((and parpos
		 (< parpos sppos))
	    (setq clparpos (position #\) rest :test #'char=))
	    (unless clparpos (error "bad parenthesis: ~a" args))
	    (push (format nil (if are-keys ":~a ~a" "~a ~a")
			  (subseq rest 1 sppos)
			  (subseq rest (1+ sppos) clparpos))
		  wordlist)
	    (setq rest (subseq rest (1+ clparpos))))
	   (sppos
	    (push (subseq rest 0 sppos) wordlist)
	    (setq rest (subseq rest (1+ sppos))))
	   (t (push rest wordlist)
	      (setq rest ""))))
    ))

(defun dhelp-dialog-call (funname args keyargs optargs restargs)
  (let* ((size '(220 20))
	 (prompt (format nil "Fill the arguments for\n ~a and click OK" funname))
	 (args-items
	  (mapcar #'(lambda (arg)
		      (list (send text-item-proto :new arg :size size)
			    (send text-item-proto :new (cond ((string= arg "data")
							      "*data*")
							     ((string= arg "headers")
							      "*headers*")
							     (t ""))
				  :size size :editable t)))
		  args))
	 (kor-prompt (format nil "~a arguments:" (cond (keyargs "Key")
						       (optargs "Optional")
						       (restargs "Rest"))))
	 splout tmp sppos sp2pos
	 (split-key #'(lambda (str)
			;;:key -> (key "") and :key val -> (key val)
			(setq sppos (position #\space str :test #'char=))
			(if sppos
			    (setq sp2pos (position #\space str :test #'char=
						   :start (1+ sppos)))
			  (setq sp2pos nil))
			(cond (sp2pos
			       (list (subseq str 0 sppos)
				     (subseq str (1+ sppos) sp2pos)))
			      (sppos
			       (list (subseq str 0 sppos)
				     (subseq str (1+ sppos))))
			      (t (list str "")))))
	 (other-items (cond
		       ((or keyargs optargs)
			(setq splout (mapcar split-key
					     (if keyargs keyargs optargs)))
			(mapcar #'(lambda (pair)
				    (list (send text-item-proto :new
						(first pair) :size size)
					  (send text-item-proto :new
						(second pair) :size size :editable t)))
				splout))
		       (restargs
			(list (list (send text-item-proto :new
					  (car restargs) :size size)
				    (send text-item-proto :new
					  "" :size size :editable t))))
		       (t nil)))
	 (ok-btn (Send button-item-proto :new "OK"))
	 (cancel-btn (Send button-item-proto :new "Cancel"))
	 (print-btn (Send button-item-proto :new "Print it"))
	 (help-btn (Send button-item-proto :new "Help"))
	 items dlg repl fcall scall)
    (setq items (list prompt
		      (list args-items)
		      ))
    (when other-items
	(setq items (append items
			    (list kor-prompt)
			    (list (list other-items)))))
    (setq items (append items
			(list (list ok-btn print-btn cancel-btn help-btn))))
    (setq dlg (send dialog-proto :new items))
    (loop (setq repl (Send dlg :modal-dialog))
	  (when (member repl (list ok-btn cancel-btn print-btn))
		(return))
	  (when (eql repl help-btn)
		(message-dialog (documentation funname 'function))))
    (cond ((eq repl cancel-btn)
	   (send dlg :dispose))
	  ((member repl (list ok-btn print-btn))
	   (setq fcall (list (symbol-name funname)))
	   (dolist (pair args-items)
		   (push (send (Second pair) :Text) fcall))
	   (when other-items
		 (dolist (pair other-items)
			 (cond (keyargs
				(unless (string= "" (send (second pair) :text))
					(push (send (first pair) :text) fcall)
					(push (send (second pair) :text) fcall)))
				 ((or optargs restargs)
				  (push (send (second pair) :text) fcall)))))
	   (setq fcall (Reverse fcall))
	   (setq scall "(")
	   (dolist (w fcall)
		   (setq scall (concatenate 'string scall w " ")))
	   (setq scall (concatenate 'string scall ")"))
	   (send dlg :dispose)
	   (if (eq print-btn repl)
	       (format t "~a~%" scall)
	     (progn (format t "DHELP is calling ~s~%" scall)
		    (eval (car (eval-string scall)))))))
))

(defun eval-string (aString)
  "Returns a list of the expressions obtained 
on evaluation of the characters in aString"  
  (let ((st (make-string-input-stream aString))   
        (result nil)   
        (expr nil))  
    (loop (setq expr (read st nil 'eof))  
          (when (eql expr 'eof)    
                (return (reverse result)))  
          (setf result (cons expr result)))))

