;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; DOMUTILS.lsp : Ver 1.0 beta (2 April 1997)
;;
;; A set of utility functions and constants
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;;
;; (c) Dom. Early 1997
;;
;; Dom. Early
;; Dept. of Statistics
;; Trinity College Dublin
;; Ireland
;;
;;
;;DEVELOPMENT ENVIRONMENT:
;;
;;            Machine:	PowerMac 7200/90
;;   Operating system:	MacOS 7.5.5
;;         XLISP-STAT:  3.50 Beta (PowerMac Binary)
;;
;;
;;SOURCE CODE
;;   Source code is optimised for a 4 space tab width and indented according
;;   to my own personal preferences (sorry, I know hardened lisp'ers will hate
;;   it, so pretty print it! 8)
;;
;;
;;COMMENTS:
;;   Users are free to use this code, UNMODIFIED, in their packages, provided
;;   they clearly acknowledge it's use and credit the author.
;;
;;   If you have any bug reports, suggestions or comments about this code,
;;   or if you have suggestions on how to improve it, please let me know
;;   via email.  Also, if you find this code useful, I'd appreciate your
;;   letting me know that as well.
;;
;;   This code has only been tested on non Power Mac platforms so I am unsure
;;   as to how well or badly it performs on Windows/Unix boxes.
;;   Comments from "beta testers" for these platforms are especially welcome.
;;
;;
;;							Enjoy,
;;
;;	Dom.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;;
;;OVERVIEW:
;;   The primary aim of the code below is to provide clearer constants,
;;   and utility functions to make coding simpler and code more readable.
;;   These functions are general in nature and can be used by a wide range
;;   of packages.  I hope the identifiers used don't clash with other
;;   people's code.
;;
;;   Note: In speed-critical areas some functions may impede performance
;;         where used
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Define a global true for readability
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(setf true t)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; define a global false for readability
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(setf false nil)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Define a nil predicate for readability
;; PARAMETERS
;;	Value	- the value to be tested
;;
;; RETURNS
;;	boolean
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun Nil-qm(pValue)
  "Args: (pValue)
Returns true if pValue is nil, otherwise false"
  (eq nil pValue)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Define a list type predicate for readability
;; PARAMETERS
;;	Value	- the value to be tested
;;
;; RETURNS
;;	boolean
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun List-qm(pValue)
  "Args: (pValue)
Returns true if pValue is a list, otherwise false"
  (listp pValue)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Define a number type predicate for readability
;; PARAMETERS
;;	Value	- the value to be tested
;;
;; RETURNS
;;	boolean
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun Number-qm(pValue)
  "Args: (pValue)
Returns true if pValue is a number, otherwise false"
  (numberp pValue)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Define a real number type predicate for readability
;; PARAMETERS
;;	Value	- the value to be tested
;;
;; RETURNS
;;	boolean
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun Real-qm(pValue)
  "Args: (pValue)
Returns true if pValue is a real, otherwise false"
  (realp pValue)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Define an integer number type predicate for readability
;; PARAMETERS
;;	Value	- the value to be tested
;;
;; RETURNS
;;	boolean
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun Integer-qm(pValue)
  "Args: (pValue)
Returns true if pValue is an integer, otherwise false"
  (integerp pValue)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Define a natural number type predicate for readability
;; PARAMETERS
;;	Value	- the value to be tested
;;
;; RETURNS
;;	boolean
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun Natural-qm(pValue)
  "Args: (pValue)
Returns true if pValue is a natural number, otherwise false"
  (and (integerp pValue) (>= pValue 0))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Define a string type predicate for readability
;; PARAMETERS
;;	Value	- the value to be tested
;;
;; RETURNS
;;	boolean
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun String-qm(pValue)
  "Args: (pValue)
Returns true if pValue is a string, otherwise false"
  (stringp pValue)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Define a character type predicate for readability
;; PARAMETERS
;;	Value	- the value to be tested
;;
;; RETURNS
;;	boolean
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun Character-qm(pValue)
  "Args: (pValue)
Returns true if pValue is a character, otherwise false"
  (if (symbolp pValue)
      (= 1 (length (symbol-name pValue)))
    (characterp pValue)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Define an empty list predicate for readability
;; PARAMETERS
;;	Value	- the value to be tested
;;
;; RETURNS
;;	boolean
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun Empty-qm(pValue)
  "Args: (pValue)
Returns true if pValue is an empty list (nil), otherwise false"
  (nil-qm pValue)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Define a Not Equal operator for readability
;; PARAMETERS
;;	Value	- the value to be tested
;;
;; RETURNS
;;	boolean
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun Neq(pValue1 pValue2)
  "Args: (pValue1 pValue2)
Returns true if pValue1 is not equal to pValue2, otherwise false"
  (not (eq pValue1 pValue2))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Define an Error Handling Routine
;; PARAMETERS
;;	String	-	File containing routine
;;	String	-	Routine
;;	String	-	Error message
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun DomsErr(pFileName pRoutine pErrorMsg)
  "Args: (pFilename pRoutine pErrorMsg)
Diagnostic display of the error catching routine"
  (princ (format nil "   File : ~a" pFileName))
  (terpri)
  (princ (format nil "Routine : ~a" pRoutine))
  (terpri)
  (princ (format nil "  Error : ~a" pErrorMsg))
  (terpri)
  nil
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Function to generate a list of random numbers
;; PARAMETERS
;;	N		- the number of elements in output list, must be
;; greater than 0
;;	Range 	- random numbers will be between 0 and Range, if integer
;; entered
;;			  then a list of integers is generated, if real
;; then a list of
;;			  reals
;;
;; RETURNS
;;	list
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun RandomSequence(pN pRange)
  "Args: (pN pRange)
Returns a list of pN random numbers in the range 0 - pRange"
  (if (and (natural-qm pN) (number-qm pRange) (< 0 pN))
      (let	(	(output '())
	    		)
	(dotimes(i pN output)
		(setf output (cons (random pRange) output))
	     	)
	)
    (DomsErr "domutils.lsp" "RandomSequence" "Bad parameters")
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Function to test if a value is in a list
;; PARAMETERS
;;	Value	- the value to be searched for in the list
;;	List	- the list to be searched, must be a list
;;
;; RETURNS
;;	boolean
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Member test required to catch strings

(defun InList-qm(pValue pList)
  "Args: (pValue pList)
Returns true if an item of value pValue is in pList, otherwise false"
  (if (list-qm pList)
      (not (empty-qm (member pValue pList :test #'equal)))
    false
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Function to return the index of an element if it is in a list,
;; indexes begin at 0
;; PARAMETERS
;;	Value	- the value to be searched for
;;	List	- the list to be searched, must be a list
;;
;; RETURNS
;;	number
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun GetElementIndex(pValue pList &optional(pIndex 0))
  "Args: (pValue pList)
Returns the index of the first item of value pValue in pList, or nil if
the item is not in the list.
Note: Indexing begins at index 0"
  (if (list-qm pList)
      (let(	(tail (member pValue pList :test #'equal))
		)
	(if tail
	    (- (length pList) (length tail))
	  nil
	  )
	)
    (DomsErr "domutils.lsp" "GetElementIndex"

	     "Parameter must be a list")
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Function to get a value from a list given its index, indexes begin at 0
;; PARAMETERS
;;	List	- the list to be searched, must be a list
;;	Index	- the index of the element
;;
;; RETURNS
;;	element
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun GetElement(pList pIndex)
  "Args: (pList pIndex)
Returns the item at index pIndex in pList, or nil if
the item is not in the list.
Note: Indexing begins at index 0"
  (if (and (list-qm pList) (natural-qm pIndex))
      (nth pIndex pList)
    (DomsErr "domutils.lsp" "GetElement" "Bad parameters")
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Destructive Function to set a value in a list given its index,
;; indexes begin at 0
;; PARAMETERS
;;	List	- the list to be searched, must be a list
;;	Index	- the index of the element
;;	Value	- the new value of the element
;;
;; RETURNS
;;	List
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun SetElement(pInList pIndex pValue)
  "Args: (pList pIndex pValue)
Returns a list similar to pList but with item at index pIndex set to pValue.
Note: Indexing begins at index 0"
  (cond
   ((and (list-qm pInList) (natural-qm pIndex) (> (length pInList)
						  pIndex))
    (setf (nth pIndex pInList) pValue)
    pInList
    )
   (true
    (DomsErr "domutils.lsp" "SetElement" "Bad parameters")
    )
   )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Function to add a value to a list
;; PARAMETERS
;;	List	- the list into which the value is stored, must be a list
;;	any		- the value to be added
;;
;; RETURNS
;;	List
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun AddElement(pList pValue)
  "Args: (pList pValue)
Returns a list with pValue appended to pList"
  (if (list-qm pList)
      (append pList (list pValue))
    (DomsErr "domutils.lsp" "AddElement" "Bad parameters")
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Function to get last character from a string
;; PARAMETERS
;;	String	- the list to be searched, must be a string
;;
;; RETURNS
;;	char
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun GetStringTerminator(pString)
  "Args: (pString)
Returns the last character of pString"
  (if (and (string-qm pString) (not (eq (length pString) 0)))
      (char pString (- (length pString) 1))
    (DomsErr "domutils.lsp" "GetStringTerminator"
	     "Parameter
must be a string")
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Function to partially copy a string
;; PARAMETERS
;;	String	- the string to be copied, must be a string
;;	N		- the number of characters to copy, if greater than
;;			  the length of the string, the string only is returned
;;
;; RETURNS
;;	string
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun StrnCpy(pSrc pN &optional (pDest "") (pCount 0))
  "Args: (pString pN)
Returns a string containing up to n characters of pString"
  (if (and (string-qm pSrc) (natural-qm pN))
      (if (or (eq pCount pN) (eq pCount (length pSrc)))
	  pDest
	(StrnCpy pSrc pN
		 (concatenate 'String pDest (string (char
						     pSrc pCount)))
		 (+ pCount 1)
		 )
	)
    (DomsErr "domutils.lsp" "StrnCpy" "Bad parameters")
    )
  )

#| - ael - Fri Apr 24 18:32:18 1998
pithed this as it caused problems - not sure why, as it seems to jive with my
own:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Function to concanenate n strings
;; PARAMETERS
;;	String...String
;;		strings to be concatenated together
;;
;; RETURNS
;;	string
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun StrCat(&rest pArgs)
  "Args: (&rest pStrings)
Returns a string containing the contcatenation of all argument strings"
  (let(	(result "")
	)
    (if (some #'(lambda(x) (not (string-qm x))) pArgs)
	(DomsErr "domutils.lsp" "StrnCpy" "Bad parameters")
      (dolist(astr pArgs result)
	     (setf result (concatenate 'String result astr))
	     )
      )
    )
  )
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Function to subtract 2 strings
;; PARAMETERS
;;	String1 String2
;;		string2 to be subtracted from string1
;;
;; RETURNS
;;	string
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun StrDiff(pStr1 pStr2)
  "Args: (pStr1 pStr2)
Returns a pStr1 less pStr2 if pStr1 is of form <*><pStr2>"
  (let(	(result pStr1)
	(lstr2 (length pStr2))
	)
    (if (or (not (string-qm pStr1)) (not (string-qm pStr2)))
	(DomsErr "domutils.lsp" "StrnCpy" "Bad parameters")
      (dotimes(i lstr2 result)
	      (setf result
		    (string-right-trim
		     (tostring (char pStr2 (-
					    lstr2 (+ i 1))))
		     result
		     )
		    )
	      )
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Function to remove the last character from a string
;; PARAMETERS
;;	String	- the string to be modified, must be a string. If
;;			  the string is empty it returns an empty string
;;
;; RETURNS
;;	string
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun RemoveStringTerminator(pString)
  "Args: (pString)
Returns a string less the last character of the string"
  (if (string-qm pString)
      (if (eq pString "")
	  ""
	(StrnCpy pString (- (length pString) 1))
	)
    (DomsErr "domutils.lsp" "RemoveStringTerminator"
	     "Parameter must be a string")
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Function to create a menu 'divider'
;; PARAMETERS
;;	none
;;
;; RETURNS
;;	menu-item-proto
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun NewDashedMenuItem()
  "Args: ()
returns a new menu dividing menu-item-proto object"
  (send menu-item-proto :new "-")
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Function to remove dupliacates from a list of elements
;; eg (1 2 3 3 4 4 5) -> (1 2 3 4 5)
;; PARAMETERS
;;	List - A list of elements, possibly containing duplicates
;;
;; RETURNS
;;	List - List of unique elements in same order as input
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun UniqueList(pList)
  "Args: (pList)
Returns a list containing each unique element in pList"
  (let(	(outlist '())
	)
    (dolist(element pList nil)
	   (setf outlist
		 (if (not (inlist-qm element outlist))
		     (AddElement outlist element)
		   outlist
		   )
		 )
	   )
    outlist
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
					;
(defun GetType(pX)
  "Args: (pItem)
Returns the type of pItem"
  (if (String-qm pX)
      'STRING
    (if (Integer-qm pX)
	'INTEGER
      (if (Real-qm pX)
	  'REAL
	nil
	)
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
					;
(defun ToString(pValue &optional pForce false)
  "Args: (pValue &optional pForce)
Converts pValue to a string, if pForce is TRUE then it converts even strings
to nested strings"
  (if (or pForce (not (String-qm pValue)))
      (with-output-to-string (s) (prin1 pValue s))
    pValue
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun String2Symbol(pString)
  "Args (pString)
Converts a string to a symbol"
  (if (string-qm pString)
      (read (make-string-input-stream pString))
    (DomsErr "domutils.lsp" "String2Symbol" "Parameter must be a string")
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
(defun Available-qm(pString)
  "Args: (pString)
Returns true if the Module specified in pString is loaded"
  (if (string-qm pString)
      (dolist(element *modules* nil)
	     (if (string= element pString)
		 (return true)
	       )
	     )
    false
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun FlattenTree(pExp &optional(pResult nil))
  (if (nil-qm pExp)
      pResult
    (if (List-qm (car pExp))
	(append (append pResult (FlattenTree (car pExp)))

		(FlattenTree (cdr pExp)))
      (append (append pResult (list (car pExp)))

	      (Flattentree (cdr pExp)))
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(defun IsInstance-qm(pProto pObject)
  (if (objectp pobject)
      (eq pProto (send pObject :slot-value 'PROTO-NAME))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
;; Note - this does not handle multiple inheritance yet
(defun IsDescendant-qm(pProto pObject)
  (if (objectp pObject)
      (if (eq pProto (send pObject :slot-value 'PROTO-NAME))
	  true
	(IsDescendant-qm pProto (car (send pObject :parents)))
	)
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
(provide "domutils")
