;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 eig-show
;;
(defun eig-show(A &optional domain)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          eig-show Doc:
;;
;; Author: Andy Long
;; Created: Sat Jan 31 10:15:58 1998
;;
;; Description/Arguments:

       (eig-show A &optional domain)
  
where A is a square matrix, and domain an (optional) list of x0 and x1.

;; Returns: a plot of the determinant of (A - lambda I). Where the determinant
  is zero, we have an eigenvalue.

;; Examples:
 (eig-show #2a((1 2) (2 4)))
 (eig-show #2a((1 2) (2 4)) '(-1 6))
 (eig-show #2a((1 2) (-2 4)) '(-1 6))
"
  (let ((id (id (first (size a)))))
    (if domain 
	(plot (lambda(x) (determinant (- (* x id) a)))
	      (first domain)
	      (second domain))
      (plot (lambda(x) (determinant (- (* x id) a))))
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun muller-eigs(A)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          muller-eigs Doc:
;;
;; Author: Andy Long
;; Created: Sat Jan 31 10:15:58 1998
;;
;; Description/Arguments:

       (muller-eigs A)
  
;; Returns: the eigenvalues, obtained by solving for the roots of the function

				f(x) = det(A - x I)

;; See Also: muller, muller2

;; Examples:
 (muller-eigs #2a((1 2) (2 4)))
 (eigenvalues #2a((1 2) (2 4)))

 (muller-eigs #2a((1 2.1) (2 4)))
 (eigenvalues #2a((1 2.1) (2 4)))

 (muller-eigs #2a((1 2.1) (-2 4)))
 (eigenvalues #2a((1 2.1) (-2 4)))
"
  (let* (
	 (n (first (size a)))
	 (id (id n))
	 (guesses (rand.n 3))
	 )
    (muller2 (lambda(x) (determinant (- (* x id) a)))
	     (elt guesses 0) (elt guesses 1) (elt guesses 2)
	     :num-roots n)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
