;; 				Flesh.lsp:
#|
	I want to test a notion about the interpolation of the zero matrix. My
belief is that if we choose as many function pairs as we have elements of the
zero matrix (plus one), then we can generally make a linear combination which
will interpolate the zero mesh without being zero between matrix elements.
|#
#|
	I'm going to start with a 2x2 example:
|#
;; Define two 4-vector functions, chosen pretty much at random:
(defun g(x) (list (sin x) (sinc x) (ln x) (square x)))
(defun h(x) (list (cos x) (exp x) (cube x) (* 2 x)))

;; f is a vector function, representing the four functions multiplied together
;; pairwise:
(defun f(x y) (* (g x) (h y)))

;; this is the bonus pair which will "anchor" the team. It receives a
;; coefficient of 1:
(defun e(x y) (* (cosh x) (sqrt y)))

;; I evaluate the vector function f at the four corners of our domain, and 
;; then bind them together as the rows of a matrix:
(def mat (apply #'bind-rows (mapcar #'f '(1 1 2 2) '(1 2 1 2))))

;; The coefficients lambda are obtained by solving the system Mat lambda = -e:
(def lambda (solve mat (- (mapcar #'e '(1 1 2 2) '(1 2 1 2)))))

;; Now ff is the scalar function of the inner product 
(defun ff(x y) (+ (dot (f x y) lambda) (e x y)))

(spin-domain 1 2 1 2)
(spin-points 20)
(plot ff)

(format t "verify that the following four points are zero:
~{~g ~}~%" (mapcar #'ff '(1 1 2 2) '(1 2 1 2)))

