#|
			Gabor Filters

	Joel Hounsou has me thinking about Gabor filters. He want to duplicate
some results he's seen in a few papers, with the unltimate goal of signature
discrimination.

	

						Andy Long, IMSP, 1997
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 gabor-in-glory
;;
(defun gabor-in-glory(x y w theta phi xcenter ycenter sigma)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          gabor-in-glory Doc:
;;
;; Author: Andy Long
;; Created: Fri Oct 24 11:02:11 1997
;;
;; Description/Arguments:

       (gabor-in-glory x y w theta phi xcenter ycenter sigma)
  
	This definition of the Gabor function comes from:
Fogel, I. and D. Sagi, Gabor Filters as Texture Discriminators, Biological
Cybernetics, 1989, 61, 103--113.

	I call it gabor-in-glory because this is the Gabor function with all
the parameters to specify. Yuck!

;; Notes: I created this function and gabor because of some image processing
;; work that I'm starting with Joel Hounsou at the IMSP.

;; See Also: gabor

;; Examples:
 (setq w 1 theta 0 phi 0 xcenter 0 ycenter 0 sigma 1)
 (gabor-in-glory .5 .3 w theta phi xcenter ycenter sigma)
 (gabor-in-glory '(2 .5) '(2.5 .3) w theta phi xcenter ycenter sigma)
"
  (cond
   ((numberp x)
    (let (
	  (xdiff (- x xcenter))
	  (ydiff (- y ycenter))
	  )
      (*
       (exp
	(/ (- (+ (* xdiff xdiff) (* ydiff ydiff)))
	   (* 2 (* sigma sigma))))
       (sin
	(+ (* w (- (* x (cos theta)) (* y (sin theta)))) phi))
       )
      ))
   ((listp x)
    (mapcar
     #'(lambda(x y)
	 (gabor-in-glory x y w theta phi xcenter ycenter sigma))
     x y))
   ((vectorp x)
    (mapcar
     #'(lambda(x y)
	 (gabor-in-glory x y w theta phi xcenter ycenter sigma))
     (coerce x 'list) (coerce y 'list)))
   )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 gabor
;;
(defun gabor(x y)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          gabor Doc:
;;
;; Author: Andy Long
;; Created: Fri Oct 24 11:02:11 1997
;;
;; Description/Arguments:

       (gabor x y)
  
	This is the part of the gabor function relavent to image processing
work. An image is given in (x,y) coordinates, and we convolve gabor with the
images to create a decomposition of the image.

;; Notes:

;; See Also: gabor-in-glory

;; Examples:
 (plot gabor)
 (setq theta pi/4)
 (contour gabor)
"
  (let (
	(w (if (boundp 'w) w 1))
	(theta (if (boundp 'theta) theta 0))
	(phi (if (boundp 'phi) phi 0))
	(xcenter (if (boundp 'xcenter) xcenter 0))
	(ycenter (if (boundp 'ycenter) ycenter 0))
	(sigma (if (boundp 'sigma) sigma 1))
	)
    (gabor-in-glory x y w theta phi xcenter ycenter sigma)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
