#|
		Did this one for Jay's BMI slope/mean SBP paper:

Kaufman, J. S., M. C. Asuzu, J. Mufunda, T. Forrester, R. Wilks, A. Luke,
\qquad A. E. Long, and R. S. Cooper. {\em Relationship Between Blood Pressure
\qquad and Body Mass Index in Lean Populations}. Hypertension, Vol. 30, \#6,
\qquad December, 1997:1511-1516.
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def jays
     '(
nigeria-urban   m       1135    20.2    1.22    122.8   3.495306
nigeria-urban   f       1745    21.8    0.41    122.5   4.659502
nigeria-rural   m       1079    20.9    0.4     116.8   2.800701
nigeria-rural   f       1076    21.9    0.15    113.2   3.650162
zimbabwe        m       393     21.7    1.7     122.5   3.240743
zimbabwe        f       400     26.2    1.04    127.1   5.377118
nigeria-urban   m       502     21.7    1.47    122.9   3.561942
nigeria-urban   f       649     22.6    0.52    118.7   4.689355
camer-rural   m       742     23.5    1.07    118.2   3.075269
camer-rural   f       717     23.5    1.01    113.6   4.340001
camer-urban   m       614     25.1    0.89    123.7   3.57317
camer-urban   f       752     27      0.66    116.9   4.646045
jamaica         m       596     23.8    1       119.7   4.340909
jamaica         f       831     27.9    0.54    118     6.528397
))
(def names '(site sex n bmi adj-b adj-sbp sd-bmi))
(def jays (matrix (list 14 (length names)) jays))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        Combined Men and Women:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def jay jays)

(def mean-bmi (column 4 jay))
(def beta (column 5 jay))
(def mean-sbp (column 6 jay))
(def sd (column 7 jay))
(def alpha (- mean-sbp (* beta mean-bmi)))
(mprint (bind-columns alpha beta mean-bmi mean-sbp sd))
(def prop 1)

(lines off)
(plot mean-bmi mean-sbp)
(mapcar
 (lambda (mean sd alpha beta)
   (send my-plot :add-plot-item
	  (list
	   'lines
	   (list (- mean (+ prop sd)) (+ mean (+ prop sd)))
	   (mapcar
	    (lambda(x) (+ alpha (* beta x)))
	    (list (- mean (+ prop sd)) (+ mean (+ prop sd)))
	    )
	   'blue
	   )
	  )
   )
 mean-bmi sd alpha beta
 )
(send my-plot :update-plot)
;; (postscript "jaylines.ps")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Just men:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def jay (select-rows jays (where (column 2 jays) 'm)))
(def mean-bmi (column 4 jay))
(def beta (column 5 jay))
(def mean-sbp (column 6 jay))
(def sd (column 7 jay))
(def alpha (- mean-sbp (* beta mean-bmi)))
(mprint (bind-columns alpha beta mean-bmi mean-sbp sd))
(def prop 1)

(lines off)
(plot mean-bmi mean-sbp)
(mapcar
 (lambda (mean sd alpha beta)
   (send my-plot :add-plot-item
	  (list
	   'lines
	   (list (- mean (+ prop sd)) (+ mean (+ prop sd)))
	   (mapcar
	    (lambda(x) (+ alpha (* beta x)))
	    (list (- mean (+ prop sd)) (+ mean (+ prop sd)))
	    )
	   'black
	   )
	  )
   )
 mean-bmi sd alpha beta
 )
(send my-plot :update-plot)
(make-title "Men")
;; (postscript "jaymen.ps")
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Just women:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def jay (select-rows jays (where (column 2 jays) 'f)))
(def mean-bmi (column 4 jay))
(def beta (column 5 jay))
(def mean-sbp (column 6 jay))
(def sd (column 7 jay))
(def alpha (- mean-sbp (* beta mean-bmi)))
(mprint (bind-columns alpha beta mean-bmi mean-sbp sd))
(def prop 1)

(lines off)
(plot mean-bmi mean-sbp)
(mapcar
 (lambda (mean sd alpha beta)
   (send my-plot :add-plot-item
	  (list
	   'lines
	   (list (- mean (+ prop sd)) (+ mean (+ prop sd)))
	   (mapcar
	    (lambda(x) (+ alpha (* beta x)))
	    (list (- mean (+ prop sd)) (+ mean (+ prop sd)))
	    )
	   'black
	   )
	  )
   )
 mean-bmi sd alpha beta
 )
(send my-plot :update-plot)
(make-title "Women")
;; (postscript "jaywomen.ps")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        Correlated variables:
#|
	Jay asked me if I'd check something for him: he generated two variables
(normal 0 1) with a certain correlation, and he ....
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(digits 5)
(def n 10000)
(def x1 (rand.n n))
(def x1 (select x1 (order x1)))
(def r .5)
(def x2 (+ (* (rand.n n) (sqrt (- 1 (^ r 2)))) (* r x1)))
(correlation x1 x2)
(def mat (bind-columns x1 x2))

(def g1 (select-rows mat (iseq 0 2499)))
(def g2 (select-rows mat (iseq 2500 4999)))
(def g3 (select-rows mat (iseq 5000 7499)))
(def g4 (select-rows mat (iseq 7500 9999)))

(mapcar
 (lambda(x)
   (print (send (regression-model (column 1 x) (column 2 x)) :beta)))
 (list g1 g2 g3 g4)
 )

#|
	Combined:
|#
(def n 1000)
(def r .8)
(def x1 (rand.n n))
(def x1 (select x1 (order x1)))
(def x2 (+ (* (rand.n n) (sqrt (- 1 (^ r 2)))) (* r x1)))
(def n/4 (/ n 4))
(def groups (combine 
	     (repeat 'g1 n/4)
	     (repeat 'g2 n/4)
	     (repeat 'g3 n/4)
	     (repeat 'g4 n/4)
	     ))
(def i1 (indicator groups (where groups 'g1)))
(def i2 (indicator groups (where groups 'g2)))
(def i3 (indicator groups (where groups 'g3)))

(def crap 
     (regression-model (list x1 i1 (* i1 x1) i2 (* i2 x1) i3 (* i3 x1)) x2)
     )
(send crap :hypothesis (bind-columns (repeat 0 6) (id 6)))
