;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        Automatic Differentiation
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
	I got the beginnings of this stuff from Dan Kalman, at American
University. He gave a talk at the combined meetings, and did such a very nice
job that I wanted to implement this stuff right away!
	The original codes are in the accompanying fill kalman.orig. I've made
a number of changes in the code that follows, some cosmetic, others not.
	He said at the meeting that he had some code for the basic situation,
so I decided to avail myself of the opportunity to get it from him. That way I
can save some typing, and will get the chance to study something that already
works.

To: Andy Long <ael@maui.medctr.luc.edu>
cc: dkalman@maa.org
Subject: lisp code
Date: Wed, 15 Jan 1997 13:21:05 -0500
From: Dan Kalman <kalman@email.cas.american.edu>

Andy,

appended to this message is a suite of lisp functions.  It is pretty
well documented.  Let me know if there is anything that doesn't make
sense.  

-dan

   ========================================================================
   |                                            |                         |
   |  Dan Kalman                                |  dkalman@maa.org        |
   |  Associate Executive Director              |                         |
   |  The Mathematical Association of America   |  (202) 387-5200         |
   |  1529 Eighteenth Street, NW                |                         |
   |  Washington, DC 20036                      |                         |
   |                                            |                         |
   ========================================================================
   |                                      				  |
   |  This message originated at American University, but I am at the     |
   |  MAA currently.  Mail reaches me at both of these addresses:         |
   |      kalman@email.cas.american.edu       dkalman@maa.org         	  |
   |                                                            	  |
   ========================================================================
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                         My improvements, etc.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;  This is an older version of the suite of functions for Auto Diff.
;;  It represents a derivative structure as a list (a b c ...) rather
;;  than as a pair (a (b c ...)).  The relation ship between the pair
;;  (a b) discussed in the talk and paper, and the list that is used
;;  in this code is this:  if the list in this code is s, then the
;;  pair that is referred to in the paper is ((car s) (cdr s)).

(defun DS-make-const (n m &optional (value 0))
  "  Creates a constant element of DS(N,M), zero by default
  or equal to the optional VALUE "
  (cond 
   ((or (< n 0) (< m 0)) nil)
   ((and (= m 0) (= n 0)) value)
   ((= m 0) (list (DS-make-const (- n 1) m value)))
   ((= 1 n) (append (DS-make-const n (- m 1) value) (list 0)))
   (t (cons (DS-make-const (- n 1) m value) (DS-make-const n (- m 1))))))

(defun DS-make-var (n m var-num &optional (value 0))
  "Make an element of DS(n,m) for the VAR-NUM th variable"
  (cond
   ((< m 1) nil)
   ((< n var-num) (DS-make-const n m))
   ((> n var-num) (cons (DS-make-var (- n 1) m var-num value) 
			(DS-make-const n (- m 1))))
   ((= n 1) (cons value (cons 1 (DS-make-const 1 (- m 2)))))
   ( t (let* ((a0 (DS-make-const (- n 1) m value))
	      (a1 (DS-make-const (- n 1) (- m 1) 1)))
	 (cons a0 (cons a1 (DS-make-const n (- m 2))))))))

(defun DSpi-m (a)
  "Projection that deletes car of the list.  This is the D operator"
  (cdr a))

(defun DSpi-0 (a)
  "Projection pi 0 removes outermost elements of A.  This is the H
      operator"
  (cond
   ((not a) nil)
   ((not (listp a)) nil)
   (t (let* ((a0 (car a))
	     (len (length a))
	     (len-1 (- len 1))
	     (pi0a0 (DSpi-0 a0)))
	(cond 
	 ((not (listp a0)) (sublist a len-1))
	 (pi0a0 (cons (DSpi-0 a0) (DSpi-0 (cdr a))))
	 (t nil))))))

;;  This next function predates the development of the extraction
;;  theorem.  It gives a more direct mapping from the list of
;;  differential operators to a value than the extraction theorem
;;  would.  I have not yet implemented a version of the extraction theorem

(defun DS-extract (a dlist)
  "Assuming first argument A is a derivative structure, return the 
entry specified by second argument DLIST.  The car of DLIST specifies 
one element of the list A, the next element of DLIST specifies which element 
of that element of A, etc.  Or put another way, if DLIST is (d1 d2 d3 ...) 
then the value returned is the partial derivative resulting from d1 
differentiations relative to the last variable, d2 differentiations relative
to the next to the last variable, etc."
  (cond
   ((not dlist) a)
   ((not (and a (listp a))) nil)
   (t (DS-extract (nth (car dlist) a) (cdr dlist)))))

(defun DS+ (a b &rest args)
  (if args (DS+ a (apply 'DS+ b args))
    (let* ((a-is-number (not (listp a)))
	   (b-is-number (not (listp b))))
      (cond
       ((not (and a b)) nil)
       ((and a-is-number b-is-number) (+ a b))
       (a-is-number (DS-add-value b a))
       (b-is-number (DS-add-value a b))
       (t (cons (DS+ (car a) (car b)) (DS+ (cdr a) (cdr b))))))))

(defun DS- (a b)
  (let* ((a-is-number (not (listp a)))
	 (b-is-number (not (listp b))))
    (cond
     ((not (and a b)) nil)
     ((and a-is-number b-is-number) (- a b))
     (a-is-number (DS-add-value (DS* -1 b) a))
     (b-is-number (DS-sub-value a b))
     (t (cons (DS- (car a) (car b)) (DS- (cdr a) (cdr b)))))))

;;  This definition does not use the recursive form of multiplication.
;;  Instead, this is based directly on Leibniz rule.  It permits
;;  multiplying 2 or more operands, in the style of the native lisp
;;  multiplication.  The loop at the bottom marches through the
;;  elements of the list making up a DS element; the call to bdot then
;;  computes an appropriate derivative of a product using leibniz
;;  rule.  The actual computation of the binomial coefficients is done
;;  by brute force by the nCm function.  That could be done more efficiently.

(defun DS* (a b &rest args)
  (if args (DS* a (apply 'DS* b args))
    (let* ((a-is-number (not (listp a)))
	   (b-is-number (not (listp b))))
      (cond
       ((not (and a b)) nil)
       ((and a-is-number b-is-number) (* a b))
       ( a-is-number (cons (DS* a (car b)) (DS* a (cdr b))))
       ( b-is-number (cons (DS* b (car a)) (DS* b (cdr a))))
       (t (let* ((a-order (- (length a) 1))
		 (b-order (- (length b) 1))
		 (result-order (min a-order b-order))
		 (result nil)
		 (counter result-order))
	    (loop while (>= counter 0)
		  do (progn (setq result (cons (bdot a b counter) result))
			    (setq counter (- counter 1))))
	    result))))))

(defun bdot (a b n)
  (if (= n 0) (DS* (car a) (car b))
    (let* ((a-sub (sublist a (+ n 1)))
	   (b-rev (reverse (sublist b (+ n 1))))
	   (k 0))
      (accum-bdot a-sub b-rev k n))))

(defun accum-bdot (a b k n)
  "accumulates a sum of list elements of A and B multiplied by
   binomial coefficients"
  (if (= k n) (DS* (car a) (car b))
    (DS+ 
     (DS* (nCm n k) (DS* (car a) (car b)))
     (accum-bdot (cdr a) (cdr b) (+ k 1) n))))

(defun DS/ (a b)
  (DS* a (DSrecip b)))

(defun DSrecur (a real-f f df &rest args)
  "General purpose recursion shell for auto diff for the function f.
First arg is A, the DS which is being operated on.  Second arg REAL-F 
is the procedure to perform if A is real.   Args F and DF are 
the function and its derivative. The remaining args list ARGS is provided 
to permit parameters in function definitions.  For example, an integer 
power function can be defined with the power as a parameter."
  (cond 
   ((not a) nil)
   ((not (listp a)) (apply real-f a args))
   (t (let* ((a0   (car a))
	     (pi0a (DSpi-0 a))
	     (pima (DSpi-m a))
	     (fa0  (apply f a0 args))
	     (dfa  (apply df pi0a args)))	    
	(cons fa0 (DS* dfa pima))))))

(defun real-recip (a) (if (= a 0) nil (/ 1 a)))
(defun DSdrecip (a) (DS* -1 (DSrecip (DS* a a))))
(defun DSrecip (a) 
  "
;; Examples:
 (DSrecip '(3 1))
 ;; this is the beginnings for 1/x:
 (DSrecip '(1 -1 2 -6))
"
  (DSrecur a 'real-recip 'DSrecip 'DSdrecip))

(defun real-sqrt (a) (if (>= a 0) (sqrt a) nil))
(defun DSdsqrt (a) (DSrecip (DS* 2 (DSsqrt a))))
(defun DSsqrt (a) (DSrecur a 'real-sqrt 'DSsqrt 'DSdsqrt))

(defun DSexp (a) (DSrecur a 'exp 'DSexp 'DSexp))
(defun DSsin (a)
  "
 (DSsin (DSrecip '(1 -1 2 -6)))
"
  (DSrecur a 'sin 'DSsin 'DScos))

(defun DSdcos (a) (DS* -1 (DSsin a)))
(defun DScos (a) (DSrecur a 'cos 'DScos 'DSdcos))

(defun real-log (a) (if (< 0 a) (log a) nil))
(defun DSlog (a) (DSrecur a 'real-log 'DSlog 'DSrecip))


(defun sublist (list n)
  " Create new list from LIST using first N elements."
  (cond
   ((<= (length list) n) list)
   ((= 0 n) nil)
   (t (cons (car list) (sublist (cdr list) (- n 1))))))

(defun nCm (n m)
  "Binomial coefficient"
  (cond
   ((= m 0) 1)
   ((= n m) 1)
   (t (+ (nCm (- n 1) (- m 1)) (nCm (- n 1) m)))))

(defun DS-update-value (op ds val)
  "Use operation OP to update the value entry of the DS by VAL"
  (let ((ds0 (car ds)))
    (if (listp ds0) (DS-update-value op ds0 val)
      (rplaca ds (apply op ds0 val nil))))
  ds)

(defun DS-add-value (ds val) 
  "
;; Examples:
 (DS-ADD-VALUE '(3 1) 3)
"
  (DS-update-value '+ ds val))

(defun DS-sub-value (ds val) (DS-update-value '- ds val))

