#|
	Kishore Karamshandani of Pulmonary came to me with plots of lung
function for seven lung-transplant patients whose lung function was
declining. At 20% decline, they are said to show BOS, a lung syndrome.

	He wondered if we could show that a treatment they have implemented has
either arrested the decline, or improved the situation. The method we used was
to perform a linear regression of FEV (lung-function) on days, and
days*treatment, with a common intercept at the date of treatment. We then
checked to see if the days*treatment coefficient was significant.

	Conclusion: in four of the seven cases, the change in slope was
significant, always in the positive direction. In the other three cases, there
was no significant change in slope.

							Andy Long
							1997
							Loyola U. Chicago
|#

(def variables (list "day" "fev" "treatment"))
(def p1 (list 
1 1.44 0
70 1.27 0
154 1.31 0
289 1.31 0
390 1.28 0
439 1.28 0
566 1.16 0
611 1.05 0
619 1.09 0
756 0.96 0
757 0.96 1
791 1.01 1
849 0.97 1
887 1.01 1
952 0.99 1
977 0.93 1
1008 1.05 1
))
(def p2 (list
1 1.67 0
69 1.8 0
168 1.44 0
209 1.56 0
217 1.45 0
229 1.67 0
250 1.46 0
292 1.4 0
394 1.26 0
395 1.26 1
429 1.41 1
465 1.4 1
575 1.01 1
602 0.77 1
622 0.72 1
651 0.66 1
691 0.73 1
726 0.77 1
760 0.81 1
780 0.7 1
))
(def p3 (list
1 1.03 0
14 0.99 0
27 0.92 0
94 1.05 0
140 0.96 0
146 0.92 0
318 0.95 0
416 1 0
460 0.72 0
560 0.72 0
610 0.72 0
649 0.76 0
711 0.82 0
800 0.87 0
904 0.68 0
919 0.88 0
965 0.69 0
966 0.69 1
986 0.76 1
992 0.81 1
1004 0.77 1
1080 0.74 1
1119 0.8 1
))
(def p4 (list
1 2.41 0
24 2.15 0
74 2.12 0
80 2.09 0
107 1.82 0
108 1.82 1
146 2.12 1
218 2.18 1
231 2.07 1
267 1.84 1
308 1.9 1
398 1.97 1
439 1.89 1
468 1.86 1
498 1.98 1
))
(def p5 (list
1 3.22 0
28 2.96 0
35 2.95 0
101 2.89 0
126 2.84 0
176 2.94 0
382 1.65 0
409 1.66 0
445 1.82 0
506 1.76 0
529 1.84 0
546 1.98 0
552 1.76 0
565 1.84 0
588 1.98 0
677 1.8 0
698 1.91 0
720 1.91 0
781 1.84 1
809 1.76 1
816 1.81 1
849 1.78 1
902 1.9 1
917 1.9 1
))
(def p6 (list
1 2.25 0
31 2.37 0
113 1.56 0
148 1.62 0
175 1.41 0
206 1.47 0
226 1.31 0
257 0.99 0
258 0.99 1
278 1.06 1
287 1.01 1
327 0.9 1
348 0.89 1
375 0.85 1
403 0.64 1
))
(def p7 (list
1 1.42 0
8 1.37 0
16 1.34 0
24 1.34 0
57 1.05 0
59 0.98 0
70 1.21 0
99 1.17 0
113 0.98 0
116 0.92 0
127 1.05 0
204 0.99 0
211 0.92 0
212 0.72 0
267 0.72 1
302 0.88 1
323 0.92 1
342 0.82 1
377 0.88 1
))
(def patients
     (mapcar (lambda(x) (split-list x 3)) (list p1 p2 p3 p4 p5 p6 p7)))
(def patients-names (mapcar #'symbol-name '(p1 p2 p3 p4 p5 p6 p7)))

(lines off)
(mapcar
 (lambda(x y)
   (let* (
;;	  (days (/ (mapcar #'first x) 365))
	  (days (/ (mapcar #'first x) 1))
	  (fev (mapcar #'second x))
	  (ind (mapcar #'third x))
	  (treat (search '(1) ind))
	  (date (elt days treat))
	  (level (elt fev treat))
	  (days (- days date))
	  (fev (- fev level))
	  (reg
	   (regress
	    (list
	     days
	     (* (heaviside days) days)
	     )
	    fev
	    :print nil
	    )
	   )
	  (beta (send reg :beta))
	  (before (where days 0 #'<))
	  (after (where days 0 #'>=))
	  (int (first beta))
	  (sec-beta (second beta))
	  (sum-rest-beta (sum (rest beta)))
	  tmp
	  )
     (format
      t "Intercept ~,3f ~20t  ~,2f ~30t  ~,2f~%"
      (+ level int) sec-beta sum-rest-beta)
     (title y)
     (variable-labels "Days" "FEV 1")
     (plotter days (+ level fev))
     (send my-plot :add-plot-item

	   (list 'points (select days before) (+ level (select fev before))
		 'blue 'cross 2)
	   (list 'points (select days after) (+ level (select fev after))
		 'black 'cross 2)

	   (list 'function
		 (lambda(x)
		   (if (> x 0) nil
		     (+ level int (* sec-beta x))))
		 'blue 'lines 2 'dotted)
	   (list 'function
		 (lambda(x)
		   (if (< x 0) nil
		     (+ level int (* sum-rest-beta x))))
		 'black 'lines)
;;	   (list (format nil "R-squared: ~,3f" (send reg :r-squared)) 10 1.4)
	   )
#|
     (make-title y :sub-title 
		 (num-to-string (send reg :hypothesis #2a((0 1)))))
|#
     (make-title y)
     (postscript (strcat y ".ps") :no-squish t)
     (list
      (send reg :hypothesis #2a((0 1)))
      (send reg :r-squared)
      )
     )
   )
 patients patients-names
 )
(print (round-to '(0.05584836006081573 0.13807908235422484 0.18774376224878675 0.017767486190551462 0.008669766962917103 0.019862741092945058 0.005179684228339587) .01))
