#|
				Mancala

	This is file mancala.lsp, created to play one version of mancala and
beat the old men of Cotonou. I created this file for Rock, when we got to
talking about how the old men like to sit around and play. I told him that it
would be fun to program a computer to beat them (I wondered what they'd say to
that!).

	In this file we consider two different ways of playing: there's
bele, and there's adji: 

BELE: in bele, a player takes a pile from a pit on her side, and drops one
stone in the next pit counter-clock-wise from it. She continues to play until
she lands on an empty space. If that empty space is again on her side, then if
there are any in the space across, that pile and the single one on which one
lands are picked up.

ADJI: in bele, a player takes a pile from her side, and plays just once. If
she lands on the opponent's side, and there are two or three in the resultant
pile, then they are picked up, as are any other piles preceding the final pit
on the opponent's side which contain two or three stones (until one reaches a
pit without two or three, at which point one stops amassing stones, and the
other plays).

	If you pick up a pile with twelve or more in it, then you will not drop
any stone in the pit from which you picked up, skipping over it and continuing
on to the next pit.

	In both games, we assume that play stops when one player has no stones
on her side, at which point any stones on the other sides belong to the
opponent. In any case, one must play so that the other player can play (has no
empty slots in front of him/her) if possible. I'm not at all sure that that's
the correct way, although I've had weak support for that idea.

	Try this one sometime: play bele with pits (5 3 2 1 0 4 3 0 1 3 5 14),
starting from the first pit (5)! You'll never finish, because you get into a
13518 cycle!

(trace bele-move)
(send (send game :new (iseq 2) '(5 3 2 1 0 4 3 0 1 3 5 14)) :move 1 6)
(send (send game :new (iseq 2) '(1 0 4 1 2 1 26 1 2 1 0 2)) :move 1 6)

eventually get you to:
(0 5 14 5 3 2 1 0 2 3 2 4) 
(26 1 2 1 0 2 1 0 4 1 2 1) 

Here's another cute (11) cycle that my program found:
(0 1 0 1 0 2 1 0 1 0 1 2)
(1 2 0 1 0 2 1 0 1 0 1 0) 
(1 0 1 2 0 2 1 0 1 0 1 0) 
(1 0 1 0 1 3 1 0 1 0 1 0) 
(1 0 1 0 1 0 2 1 2 0 1 0) 
(1 0 1 0 1 0 2 1 0 1 2 0) 
(2 0 1 0 1 0 2 1 0 1 0 1) 
(0 1 2 0 1 0 2 1 0 1 0 1) 
(0 1 0 1 2 0 2 1 0 1 0 1) 
(0 1 0 1 0 1 3 1 0 1 0 1) 
(0 1 0 1 0 1 0 2 1 2 0 1) 
(0 1 0 1 0 1 0 2 1 0 1 2) 

							Andy Long
							November, 1997
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                player proto:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(uber-proto game (players pits winnings type moves) nil nil "

			Game of Mancala.

;; Examples:
 (setq new-game (send game :new :players (iseq 2) :pits (repeat 4 12)
   :winnings '(0 0)))
 (ask new-game pits)
 (send new-game :move 0 0 :check t)
 (send new-game :maximize 0)
 (send new-game :move 0 5 :check t)
 (mapcar #'(lambda(n) (send new-game :move 0 n :check t)) (iseq 6))
 (mapcar #'(lambda(n) (send new-game :move 1 n :check t)) (iseq 6 11))
")
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
#|
 	(send game :help 'proto) 

 retrieves this help, and 

 	(try-out proto :proto game)

 tries it out!
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                Initialize game:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(send game :players (iseq 2))
(send game :pits (repeat 4 12))
(send game :winnings '(0 0))
(send game :type 'bele)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 methods:
;;
;;                                 :move
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :move (player pit &key pits check started print)
  (let ((type (send self :type)))
    (send self 
	  (case type
		('bele :bele-move)
		('adji :adji-move)
		(t (error (format nil "Unrecognized type of game: ~s" type)))
		)
	  player pit :pits pits :check check :started started :print print))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :bele-move
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :bele-move (player pit &key pits check started print)
  "
pits are labelled 0 to 11: 0-5 are for player 0, 6-11 for player 1.
"
  ;; First we check to make sure that the entry is valid:
  (if (not started) (let () (setq started t) (check-start player pit)))
  (let* (
	 (pits (if pits pits (copy-list (send self :pits))))
	 (result (bele-move pits pit))
	 (pits (first result))
	 (final-pit (second result))
	 )

    (print-move print player pits pit)

    ;; we're done, and should tally up:
    (let* ((other-pit (- 11 final-pit))
	   (winnings (elt pits other-pit))
	   )
      (if (or	      
	   (and (= player 1) (< other-pit 6))
	   (and (= player 0) (>= other-pit 6))
	   )
	  ;; then there are winnings!
	  (let ((winnings (elt pits other-pit)))
	    (setf (elt pits other-pit) 0)
	    (if (/= winnings 0) 
		(let ()
		  (setq winnings (1+ winnings))
		  (setf (elt pits final-pit) 0)
		  ))
	    (if (not check)
		(let ()
		  (send self :pits pits)
		  (send self :winnings
			(+ (send self :winnings)
			   (* winnings (list (- 1 player) player))))
		  ))
	    (list pits winnings)
	    )
	(let ()
	  (if (not check) (send self :pits pits))
	  (list pits 0))
	)
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 :adji-move
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :adji-move (player pit &key pits check started print)
  "
pits are labelled 0 to 11: 0-5 are for player 0, 6-11 for player 1.
"
  ;; First we check to make sure that the entry is valid:
  (if (not started) (let () (setq started t) (check-start player pit)))

  (let* (
	 (pits (if pits pits (copy-list (send self :pits))))
	 (result (adji-move pits pit))
	 (pits (first result))
	 (final-pit (second result))
	 (score (send self :winnings))
	 )

    (print-move print player pits pit)

    (while
	(or	      
	 ;; opposite of bele: you have to fall on your opponent's side:
	 (and (= player 1)
	      (and (< final-pit 6) (> final-pit -1))
	      (if (> final-pit -1) (search (list (elt pits final-pit)) '(2 3)))
	      )
	 (and (= player 0) (>= final-pit 6))
	 )
      ;; then there is the potential for winnings:
      (let ((winnings (elt pits final-pit)))
	(if (search (list winnings) '(2 3)) ;; then there are winnings!
	    (let ()
	      (setf (elt pits final-pit) 0)
	      (setq score (+ score (* winnings (list (- 1 player) player))))
	      )
	  )
	)
      (setq final-pit (decf final-pit))
      )

    (if (not check)
	(let () (send self :winnings score) (send self :pits pits)))

    (list pits (elt score player))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :one-round
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :one-round (&key print
			       interactive
			       (order '(0 1))
			       (names '("r1" "r2"))
			       (depth 3)
			       )
  (let* (
	 (zero (first order))
	 (one (second order))
;;	 (m1 (send self :maximize zero))
	 (m1 (if interactive
		 (if (= interactive zero) (interact zero self)
		   (send self :choose-site zero depth))
	       (send self :choose-site zero depth)))
	 (r1 (if m1 (send self :move zero m1 :print (if print (elt names zero)))))
;;	 (m2 (if m1 (send self :maximize one)))
	 (m2 (if m1
		 (if interactive
		     (if (= interactive one) (interact one self)
		       (send self :choose-site one depth))
		   (send self :choose-site one depth))))
	 (r2 (if m2 (send self :move one m2 :print (if print (elt names one)))))
	 )
    (format t "~%Next round!~%")
    (if (not (and r1 r2)) ;; game over!
	(let (
	      (pits (send self :pits))
	      (winnings (send self :winnings))
	      )
	  (send self :winnings
		(+ winnings (list (sum (select pits (iseq 6)))
				  (sum (select pits (iseq 6 11))))))
	  (values nil (list m1 m2))
	  )
      (values t (list m1 m2))
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :try-one-play
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :try-one-play (player pit &key (check t))
  "
;; Examples:
 (def new-game (send game :new))
 (send new-game :try-one-play 0 5)
 (ask new-game pits)
 (send new-game :try-one-play 1 8 :check nil)
 (ask new-game pits)
 (send game :dispose new-game)
"
  (let* (
	 (true-pits (send self :pits))
	 (true-score (send self :winnings))
	 (legal (valid-move player pit true-pits :play-on t))
	 pits score
	 )
    (if legal
	(let ()
	  (send self :move player pit)
	  (setq score (send self :winnings))
	  (setq pits (send self :pits))

	  (if check
	      (let ()
		(send self :pits true-pits)
		(send self :winnings true-score)
		))))
    (values score pits)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :try-one-round
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :try-one-round (pit1 pit2 &key (players '(0 1)) (check t))
  "
;; Examples:
 (def new-game (send game :new))
 (send new-game :try-one-round 5 8)
 (ask new-game pits)
 (send new-game :try-one-round 5 8 :check nil)
 (ask new-game pits)
 (send new-game :try-one-round 4 8 :check nil) ;; illegal move! nothing in 4:
 (ask new-game pits)
 (send game :dispose new-game)
"
  (let* (
	 (true-pits (send self :pits))
	 (p1 (first players))
	 (p2 (second players))
	 (true-score (send self :winnings))
	 pits score
	 (legal (valid-move p1 pit1 true-pits :play-on t))
	 )
    (if legal
	(let ()
	  (send self :move p1 pit1)
	  (setq legal (valid-move p2 pit2 (send self :pits) :play-on t))
	  (if legal
	      (let ()
		(send self :move p2 pit2)
		(setq score (send self :winnings))
		(setq pits (send self :pits))
		))))
    (if (and check legal)
	(let ()
	  (send self :pits true-pits)
	  (send self :winnings true-score)
	  ))
    (values score pits)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                interact
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun interact(interactive game)
  (let (
	(pits (send game :pits))
	(score (send game :winnings))
	ans
	)
    (print-pits pits)
    (format t 
	    (if (= 1 interactive)
		"~%The score is computer ~d, bad guys ~d."
	      "~%The score is bad guys ~d, computer ~d.")
	    (first score) (second score))
    (format t "~%Make your move: you have pits~{ ~d~}: "
	    (if (= interactive 0) (iseq 1 6) (iseq 12 7)))
    (tagbody
     read
     (setq ans (read))
     (if (not (integerp ans))
	 (let ()
	   (format t "Need an integer! Make your move:~%")
	   (go read)))
     (setq ans (mod (1- ans) 12))
     (if (>= (mod (- ans (* interactive 6)) 12) 6)
	 (let ()
	   (format t "~% You chose an illegal pit (~d)! Try again: " (1+ ans))
	   (go read)))
     (if (= 0 (elt pits ans))
	 (let ()
	   (format t "~% Pit ~d is empty. Choose a non-empty pit: "
		   (1+ ans))
	   (go read)))
     )
    ans
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :report-winnings
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :report-winnings (interactive &key (names '("computer" "bad guys")))
  (let (
	(pits (send self :pits))
	(score (send self :winnings))
	)
    (if interactive
	(let ()
	  (format t "~2%~{	~d~}~2%" (iseq 12 7))
	  (format t "~{	~d~}~%" (select pits (iseq 11 6)))
	  (format t "~{	~d~}~2%" (select pits (iseq 6)))
	  (format t "~{	~d~}~%" (iseq 1 6))
	  (format t "~%The final score is ~a ~d, ~a ~d."
		  (elt names 0) (first score)
		  (elt names 1) (second score))
	  )
      )
    score
    ))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :possibilities
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :possibilities (player)
  "Makes all possible next moves, and gives the winnings.

;; Examples:
 (send new-game :possibilities 0)
"
  (let* (
	 ;; All the pits:
	 (pits (send self :pits))
	 ;; Let's select only the player's pits to consider:
	 (player-pits (+ (* player 6) (iseq 6)))
	 ;; the bead values of those pits:
	 (player-pit-vals (select pits player-pits))
	 ;; if the player has any beads in front of him, then he can
	 ;; play - otherwise, he'll pass:
	 (vals
	  (if (/= 0 (sum player-pit-vals))
	      (mapcar
	       #'(lambda(n) (append (send self :move player n :check t) (list n)))
	       (select player-pits (where player-pit-vals 0 #'/=)))))
	 )
    (if vals vals) ;; otherwise nil - no possibilities, i.e. player passes!
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :possible-plays
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :possible-plays (player)
  (let* (
	 ;; Select only the player's pits to consider:
	 (player-pits (+ (* player 6) (iseq 6)))
	 ;; the bead values of those pits:
	 (player-pit-vals (select (send self :pits) player-pits))
	 )
    ;; if the player has any beads in front of him, then he can
    ;; play - otherwise, he'll pass:
    (select player-pits (where player-pit-vals 0 #'/=))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :maximize
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :maximize (player &key minimize)
  "Maximize maximizes the player's gain, given that he can look ahead at his
opponent's next move and minimize the maximum.

;; Examples:
 (def new-game (send game :new))
 (send new-game :maximize 0)
 (send new-game :maximize 0 :minimize t)
"
  (let* (
	 ;; Usually we want to maximize with maximize! But if we want to
	 (maxmin (list #'max #'min))
	 ;; minimize, we'll swap these:
	 (max (if minimize (second maxmin) (first maxmin)))
	 (min (if minimize (first maxmin) (second maxmin)))

	 ;; the other player:
	 (other-player (- 1 player))
	 ;; keep the true pits to put back later:
	 (true-pits (send self :pits))
	 ;; Let's see what player can do:
	 (vals (send self :possibilities player))
	 ;; Now for each one of these possibilities, we want to see what the
	 ;; other player can do:
	 (result
	  (mapcar
	   (lambda(val)
	     (send self :pits (first val))
	     (mapcar
	      (lambda(y)
		(list (- (second val) (second y)) (third val) (third y)))
	      (send self :possibilities other-player))
	     )
	   vals)
	  )
	 (win-loss (mapcar (lambda(x) (mapcar #'first x)) result))
	 ;; it may be that the game is over (i.e. no possibilities):
	 (win-loss (if (some (lambda(x) x) win-loss)
		       ;; The minimum of this is the worst they can do to us,
		       (mapcar min win-loss)))

	 (max-win-loss (remove-if #'null win-loss))
	 ;; Again, if there are some of these:
	 ;; then we want to take the maximum of the minima:
	 (max-win-loss (if max-win-loss (funcall max max-win-loss)))
	 (minimax (if max-win-loss 
		      (select result
			      (where win-loss max-win-loss))))
	 )
    ;; make sure that we put the pits back right:
    ;; (break)
    (send self :pits true-pits)
    (if minimax
	(if (length=1 minimax)
	    (values (second (first (first minimax))) max-win-loss)
	  ;; There may be ties: in that case, we take the one with the highest
	  ;; average as well:
	  (let* (
		 (avg
		  (mapcar
		   #'mean
		   (mapcar (lambda(x) (mapcar #'first x)) minimax)))
		 (maxiavg (select minimax (where avg (funcall max avg))))
		 )	    
	    (values (second (first (first maxiavg))) max-win-loss)
	    )
	  )
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                print-pits
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun print-pits(pits)
  "
;; Examples:
 (PRINT-PITS '(2 1 9 2 0 4 3 0 2 1 5 1))
"
  (format t "~%~{~s~}~%" (repeat '- 80))
  (eval
   `(format t  "~18t \#~d~28t \#~d~38t \#~d~48t \#~d~58t \#~d~68t \#~d~%"
	    ,@(iseq 12 7)))
  (eval `(format t    "~20t~a~30t~a~40t~a~50t~a~60t~a~70t~a~%"
		 ,@(repeat "_____ " 6)))
  (eval `(format t   "~19t~a~29t~a~39t~a~49t~a~59t~a~69t~a~%"
		 ,@(repeat "/    /" 6)))
  (eval
   `(format t
	    "~18t/ ~d ~22t/~28t/ ~d ~32t/~38t/ ~d ~42t/~48t/ ~d ~52t/~58t/ ~d ~
~62t/~68t/ ~d ~72t/~%"
	    ,@(select pits (iseq 11 6))))
  (eval `(format t "~17t~a~27t~a~37t~a~47t~a~57t~a~67t~a~2%" 
		 ,@(repeat "/____/" 6)))
  (eval `(format t    "~20t~a~30t~a~40t~a~50t~a~60t~a~70t~a~%"
		 ,@(repeat "_____ " 6)))
  (eval `(format t   "~19t~a~29t~a~39t~a~49t~a~59t~a~69t~a~%"
		 ,@(repeat "/    /" 6)))
  (eval `(format t  "~18t/ ~d ~22t/~28t/ ~d ~32t/~38t/ ~d ~42t/~48t/ ~d ~52t/~58t/ ~d ~
~62t/~68t/ ~d ~72t/~%"
		 ,@(select pits (iseq 6))))
  (eval `(format t "~17t~a~27t~a~37t~a~47t~a~57t~a~67t~a~2%"
		 ,@(repeat "/____/" 6)))
  (eval
   `(format t  "~18t \#~d~28t \#~d~38t \#~d~48t \#~d~58t \#~d~68t \#~d~%"
	    ,@(iseq 1 6)))
  (format t "~{~s~}~%" (repeat '- 80))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                bele-move
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun bele-move (pits pit &key print)
  "
pits are labelled 0 to 11: 0-5 are for player 0, 6-11 for player 1.

;; Examples:
 (bele-move '(1 0 0 0 0 0 1 2 3 4 5 6) 0)
 (bele-move '(1 0 0 0 0 0 1 2 3 4 5 6) 6)
"
  (do*
   (
    (pebbles (elt pits pit) (elt pits pit))
    (pits-to-add
     (mapcar (lambda(x) (mod x 12)) (iseq (+ pit 1) (+ pit pebbles)))
     (mapcar (lambda(x) (mod x 12)) (iseq (+ pit 1) (+ pit pebbles)))
     )
    (temp (setf (elt pits pit) 0) (setf (elt pits pit) 0))
    (temp
     (mapcar
      (lambda(pit) (setf (elt pits pit) (incf (elt pits pit))))
      pits-to-add)
     (mapcar
      (lambda(pit) (setf (elt pits pit) (incf (elt pits pit))))
      pits-to-add)
     )
    (pit (final pits-to-add) (final pits-to-add))
    )
   ((= 1 (elt pits pit)) (list pits pit))
   (if print (print pits))
   )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                adji-move
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun adji-move (pits pit)
  "
pits are labelled 0 to 11: 0-5 are for player 0, 6-11 for player 1.

;; Examples:
 (adji-move '(1 0 0 0 0 0 1 2 3 4 5 6) 0)
 (adji-move '(1 0 0 0 0 0 1 2 3 4 5 6) 6)
 (adji-move '(1 0 0 0 0 0 1 2 3 4 5 24) 6)
 (adji-move (combine 1  12  9  9  4  2 (reverse '(9  0  0  0  1  1))) 5)

 (setq ad (new game '(0 1) 
   (combine 1  12  9  9  4  2 (reverse '(9  0  0  0  1  1)))
   '(0 0)))
 (send ad :move 0 5 :print t)
"
  (let* (
	 (pebbles (elt pits pit))
	 (pits-to-add
	  (mapcar (lambda(x) (mod x 12)) (iseq (+ pit 1) (+ pit pebbles)))
	  )
	 (temp (setf (elt pits pit) 0))
	 (temp
	  (mapcar
	   (lambda(pit) (setf (elt pits pit) (incf (elt pits pit))))
	   pits-to-add))
	 (temp (elt pits pit)) ;; we don't put any back in this spot:
	 (tmp (setf (elt pits pit) 0))
	 (pit (final pits-to-add))
	 )
    ;;    (print pits)
    ;;    (print temp)
    (do
     ()
     ((= 0 temp))
     (setf temp (decf temp))
     (setf pit (mod (incf pit) 12))
     (setf (elt pits pit) (incf (elt pits pit)))
     )	    
    (list pits pit)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                check-start
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun check-start (player pit &key play-on)
  (if
      (and
       (integerp pit)
       (< pit 12)
       (< (+ (* player (- 11 pit)) (* (- 1 player) pit)) 6)
       )
      t
    (if play-on
	nil
      (error (format nil "Bad input:~%Player: ~d~%Pit: ~d" player (1+ pit)))))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                valid-move
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun valid-move (player pit pits &key play-on)
  (if
      (and
       (integerp pit)
       (< pit 12)
       (< (+ (* player (- 11 pit)) (* (- 1 player) pit)) 6)
       (/= 0 (elt pits pit))
       )
      t
    (if play-on
	nil
      (error (format nil "Bad input:~%Player: ~d~%Pit: ~d" player (1+ pit)))))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                print-move
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun print-move (print player pits pit)
  (if print 
      (let ()
	(if (stringp print) (format t "~%~a pit: ~d" print (1+ pit))
	  (format t "~%pit: ~d" (1+ pit)))
	(print-pits pits)))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                         mancala
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun mancala (
		&key (print t)
		     (names '("Bad Guys" "Ordinateur") set)
		     (order '(0 1))
		     (type 'bele)
		     (player 0) 
		     (pits (repeat 4 12))
		     (score '(0 0))
		     (depth 2)
		     augment-depth
		     )
  "
;; Examples:
 ;; plays bele to a (34 14) route.
 (mancala :player nil :names '("Kodjo" "Koffi"))
 ;; plays to a (24 24) draw.
 (mancala :player nil :names '("Kodjo" "Koffi") :type 'adji)

 (mancala :print t)
 (mancala :print t :player 1) ;; to play second

 ;; to play a game from a point at which it was stopped:
 (mancala :order '(1 0) :names '(\"Rock\" \"Ord\")
    :pits '(2 1 9 2 0 4 3 0 2 1 5 1) :score '(7 11))
"
  (if (not player) (setq player 3))
  (setq names (if set names (if (= 1 player) (reverse names) names)))
  (let ((new-game
	 (send game :new :players (iseq 2) :pits pits :winnings score
	       :type type))) 
    (unwind-protect
	(let ()
	  (print-pits pits)
	  (do
	   ((i 
	     (cmv (send new-game :one-round :print print :interactive player
		       :order order :names names :depth depth))
	     (cmv (send new-game :one-round :print print :interactive player
		       :order order :names names :depth depth))
	     ))
	   ((not (first i)))
	   (send new-game :moves (append (send new-game :moves) (second i)))
	   (format t "The score is: ~s" (send new-game :winnings))
	   
	   ;; adapt the depth to take bigger risks:
	   (if augment-depth
	       (setq depth
		     (cond
		      ((> (sum (send new-game :winnings)) 24)
		       (+ 1 depth))
		      ((> (sum (send new-game :winnings)) 36)
		       (+ 1 depth))
		      (t depth)))
	     )
	   )
	  (send new-game :report-winnings player :names names)
	  )
      ;; we want to make sure to dispose of the game, after showing how to
      ;; reproduce it:
      (format t "~%Moves: ~{ ~d~}~%" (1+ (send new-game :moves)))
      (send game :dispose new-game)
      new-game
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :maximize-depth
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :maximize-depth (player depth)
  "
	meth game :maximize-depth (player depth)

Maximize-depth maximizes the player's gain, given that he can look ahead at
the next depth moves (a move is made up of one move by each player).

;; Examples:
 (send game :maximize-depth 0 3)
"
  (let* (
	 ;; the other player:
	 (other-player (- 1 player))
	 ;; keep the true pits to put back later:
	 (true-pits (send self :pits))
	 ;; Let's see what player can do:
	 (vals (send self :possibilities player))
	 ;; Now for each one of these possibilities, we want to see what the
	 ;; other player can do:
	 (result
	  (mapcar
	   (lambda(val)
	     (send self :pits (first val))
	     (mapcar
	      (lambda(y)
		(list (- (second val) (second y)) (third val) (third y)))
	      (send self :possibilities other-player))
	     )
	   vals)
	  )
	 ;; The minimum of this is the worst they can do to us,
	 (win-loss (mapcar (lambda(x) (mapcar #'first x)) result))
	 (win-loss (if (some (lambda(x) x) win-loss)
		       (mapcar #'min win-loss)))
	 ;; and we want to take the maximum of the minima:
	 (max-win-loss (remove-if #'null win-loss))
	 (max-win-loss (if max-win-loss (max max-win-loss)))
	 (minimax (if max-win-loss 
		      (select result
			      (where win-loss max-win-loss))))
	 )
    ;; make sure that we put the pits back right:
    ;; (break)
    (send self :pits true-pits)
    (if minimax
	(if (length=1 minimax)
	    (values (second (first (first minimax))) max-win-loss)
	  ;; There may be ties: in that case, we take the one with the highest
	  ;; average as well:
	  (let* (
		 (avg
		  (mapcar
		   #'mean
		   (mapcar (lambda(x) (mapcar #'first x)) minimax)))
		 (maxiavg (select minimax (where avg (max avg))))
		 )	    
	    ;; take the first (generally a single list anyway, but could have
	    ;; several:
	    (values (second (first (first maxiavg))) max-win-loss)
	    )
	  )
      )
    )
  )

#|
;; the list of winnings for the possible move:



(def new-game (send game :new))
(dolist
 (possible (send new-game :possibilities 0))
 (let (
       (pits (first possible))
       (winnings (second possible))
       (pit-choice (third possible))
       )
   (print possible)
 )

(def new-game (send game :new))
(send new-game :move 0 2)
(setq winnings
      (mapcar
       #'second
       (mapcar
	(lambda(n) (send new-game :move 1 n :check t))
	(iseq 6 11)))
      )
(send new-game :move 1 9)
(send new-game :winnings)
;; down 2.

(def new-game (send game :new))
(send new-game :move 0 5)
(setq winnings
      (mapcar
       #'second
       (mapcar
	(lambda(n) (send new-game :move 1 n :check t))
	(iseq 6 11)))
      )
(send new-game :move 1 10)
(send new-game :winnings)
;; down 9!

;; Here's the algorithm:

(def new-game (send game :new))
(send new-game :try-one-round 5 10)

|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                test-game
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun test-game (pits moves &key (score '(0 0)) (type 'bele) (players '(0 1))
		       play-on)
  "
;; Examples:
 (test-game (repeat 4 12) '(2 9))
 (test-game (repeat 4 12) '(5 8))
 (test-game (repeat 4 12) (1- '(2 12 6 8 1 7 6 10 1 12 5 10)))
"
  (let* (
	 (self (send game :new :players (iseq 2) :pits pits :winnings score
		     :type type))
	 (player (first players))
	 )
    (unwind-protect
	(let ()
	  (dolist
	   (move moves)
	   (if (not (send self :try-one-play player move :check nil))
	       (error (format nil "Illegal move: Player ~d, pit ~d" player move)))
	   (setq player (- 1 player))
	   )
	  (setq score (send self :winnings))
	  (setq pits (send self :pits))
	  (list score pits)
	  )
      (send game :dispose self)
      )
    )
  )

(defmeth game :score nil (send self :winnings))


(defmeth game :iterate-game (player n &key plays)
  "
;; Examples
 (def ng (send game :new))
 (def 1-deep (send ng :iterate-game 0 1))
 (def winnings (mapcar (lambda(x) (apply #'- x)) 1-deep))
 (select 1-deep (where winnings (max winnings)))
 
 (def 2-deep (send ng :iterate-game 0 2))
 (let win nil)
 (dolist
   (1-deep 2-deep)
   (let ((winnings (mapcar (lambda(x) (apply #'- x)) 1-deep)))
     (setq win (combine win (select 1-deep (where winnings (min winnings)))
     ))))
 (def win (rest win))
 (select 2-deep (where winnings (max winnings)))

 (def 3-deep (send ng :iterate-game 0 3))
 (def 4-deep (send ng :iterate-game 0 4))
 (def split (split-list 4-deep 2))
 (def scores (mapcar #'(lambda(x) (apply #'- x)) (xs split)))
 (def scores (mapcar #'- (xs split)))
 (def paths (mapcar #'first (ys 4-deep)))

 (send game :dispose ng)

 (def ng (send game :new))
 (send ng :pits '(0 1 1 0 0 0 0 1 2 2 0 0))
 (send ng :iterate-game 0 1)
 (send ng :iterate-game 0 2)
 (send ng :iterate-game 0 8)
 (send game :dispose ng)

 (def ng (send game :new))
 (send ng :pits '(0 0 0 0 0 0 0 1 2 2 0 0))
 (send ng :iterate-game 0 1)
 (send game :dispose ng)
"
  (if (= n 0)
      (list (send self :score) plays)
    (let (
	  (true (send self :pits))
	  (score (send self :winnings))
	  (possible (send self :possible-plays player))
	  (other (- 1 player))
	  play
	  result
	  )
      (if possible
	  (dolist
	   (choice possible)
	   (setq play (cmv (send self :try-one-play player choice :check nil)))
	   (setq
	    result
	    (append
	     result
	     (if plays
		 (send self :iterate-game other (1- n)
		       :plays (combine plays choice))
	       (list
		(send self :iterate-game other (1- n)
		      :plays (list choice))))
	     )
	    )
	   (send self :pits true)
	   (send self :winnings score)
	   )
	)
      result
      )
    )
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                :choose-site
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth game :choose-site(player depth)
  (let*(
	#| - ael - Fri May 1 08:37:44 1998
	(games (send self :iterate-game player depth))
 - lea - |#
	(games (remove-if #'null (send self :iterate-game player depth)))
	(vals
	 (mapcar
	  (lambda(choice)
	    (let* (
		   (split (split-list choice 2))
		   (scores (mapcar #'(lambda(x) (apply #'- x)) (xs split)))
		   )
	      (if (= 0 player) scores (- scores))
	      )
	    )
	  games
	  ))
	(avg (if vals (mapcar #'mean vals)))
	(vals (if vals (mapcar #'max (mapcar #'min vals))))
	(locations (if vals (where vals (max vals))))
	(avg (select avg locations))
	)
    (if vals
	(first
	 (second
	  (elt
	   games
	   (elt locations (if avg (first (where avg (max avg))) 0))
	   )
	  )
	 )
      )
    )
  )

;; (mancala :type 'adji)
;; 4 2 5 2 1 5 1 1 1 2
;; (mancala :type 'adji :depth 3)
;; 
;; Anna walloped it!
;; 
;; (mancala :type 'adji :depth 4)
;; The final score is Bad Guys 44, Ordinateur 4.
;; Moves:  5 10 1 12 1 8 6 8 1 7 4 12 6 9 5 11 5 7 1 12 6 9 4 7 5 10 6 11 3 8 5
;; 10 1 9 4 10 5 11 1 12 
;; 