#|
Sat Sep 18 12:44:29 EDT 1999

This is a model of malarial resistance that Steve Meshnick is working on. I'm
back in the D.E. game!

|#
(defun quinine-thailand()
  ;; Parameters:
  (setq
   G .16
   H .8
   gamg 12
   gamh gamg
   gamr 26
   gam0 3
   squig .00091
   Cg .77
   a 2
   mu 26
   m 100
   b 1
   c 1
   )
  ;; Initial conditions:
  (setq ynot (list .1 0 0 0 .1 0))

  ;; Run length and duration
  (setq
   start 0
   end 40
   num 4000
   )
  )

(defun quinine-malawi()
  ;; Parameters:
  (setq
   G .001
   H .5
   gamg 12
   gamh gamg
   gamr 26
   gam0 3
   squig .00091
   Cg .77
   a 20
   mu 26
   m 100
   b 1
   c 1
   )
  ;; Initial conditions:
  (setq ynot (list .8 0 0 0 .1 0))
  ;; Run length and duration
  (setq
   start 0
   end 40
   num 4000
   )
  )

;; derivative vector:
(defun f (vals)
  (let* (
	 (y  (elt vals 0))
	 (yr (elt vals 1))
	 (z  (elt vals 2))
	 (zr (elt vals 3))
	 (yv (elt vals 4))
	 (zv (elt vals 5))
         )
    (list
     ;; dY/dt
     (- 
      (* m a b yv (- 1 y yr z zr))
      (* y (+ 
	    (* G Cg gamg)
	    (* H gamh)
	    (* gam0 (- 1 H (* G Cg)))
	    )
	 )
      (* y gamr G (- 1 Cg)))
     
     ;; dYr/dt
     (-
      (* y gamr  G (- 1 Cg) (- 1 (* squig (- 1 Cg) (/ gamg))))
      (* yr (+ gam0 (* (+ G H) (- gamh gam0))))
      )
	    
     ;; dZ/dt
     (- 
      (* m a b zv (- 1 y yr z zr))
      (* z (-
	    (* H gamh)
	    (* gam0 (- 1 H))
	    )
	 )
      (* z gamr G)
      )
     
     ;; dZr/dt
     (-
      (* gamr G (+ 
		 (* y (- 1 Cg) (- 1 Cg) (/ squig gamg))
		 z
		 )
	 )
      (* zr (+ gam0 (* (+ G H) (- gamh gam0))))
      )
	    
     ;; dYv/dt
     (- 
      (* a c (+ y yr) (- 1 yv zv))
      (* mu yv)
      )

     ;; dZv/dt
     (- 
      (* a c (+ z zr) (- 1 yv zv))
      (* mu zv)
      )
     )
    )
  )

;; (quinine-malawi)
(quinine-thailand)

(def points (rk-f #'f ynot start end num :verbose t))

(mvsq '(times y yr z zr yv zv) (transpose points))
(let ()
  (lines on)
  (plot times y)
  (send my-plot :add-plot-item

	(list 'lines '(20 25) '(.4 .4) 'blue)
	(list "Y" 30 .4)

	(list 'lines '(20 25) '(.3666 .3666) 'red)
	(list "Z" 30 .3666)

	(list 'lines '(20 25) '(.3333 .3333) 'green)
	(list "Yr" 30 .3333)

	(list 'lines '(20 25) '(.3 .3) 'black)
	(list "Zr" 30 .3)

	(list 'lines times z 'red)
	(list 'lines times yr 'green)
	(list 'lines times zr 'black)
	)
  )
