#|
Sat Sep 18 12:44:29 EDT 1999

This is a the problem model of malarial resistance that Steve Meshnick is
working on, from Malawi. I'm trying to figure out why things look so unstable
when z is 1 and all else is zero.
|#

(defun quinine-malawi()
  ;; Parameters:
  (setq
   G .001
   H .5
   gamg 12
   gamh gamg
   gamr 26
   gam0 3
   squig .00091
   Cg .77
   a 20
   mu 26
   m 100
   b 1
   c 1
   )
  ;; Initial conditions:
  (setq ynot (list 1 0 .43))
  ;; Run length and duration
  (setq
   start 0
   end 10
   num 1000
   )
  )

;; the setup:
(quinine-malawi)

;; Set up some big, ugly constants:
(setq
 mab (* m a b)
 ac (* a c)

 GCggamg (* G Cg gamg)
 Hgamh (* H gamh)
 1-H-GCg*gam0 (* (- 1 H (* G Cg)) gam0)
 1-Cg (- 1 Cg)
 gamrG (* gamr G)
 gamrG*1-Cg (* gamrG 1-Cg)
 G+H (+ G H)
 squig/gamg (/ squig gamg)

 ugly0 (+ GCggamg Hgamh 1-H-GCg*gam0 gamrG*1-Cg)
 ugly1 (* gamrG*1-Cg (- 1 (* squig/gamg 1-Cg)))
 ugly2 (+ gam0 (* G+H (- gamh gam0)))
 ugly3 (- Hgamh (* gam0 (- 1 H)))
 ugly4 (* 1-Cg 1-Cg squig/gamg)
 ugly5 (+ ugly3 gamrG)
 )

(printall
 mab
 ac
 ugly0
 ugly1
 ugly2
 ugly3
 ugly4
 ugly5
 )

(defun stablize (x) (min (max 0 x) 1))
(compile 'stablize)

(defun f (vals)
  (let* (
	 (z  (elt vals 0))
	 (zr (elt vals 1))
	 (zv  (elt vals 2))
	 (1-zv (- 1 zv))
	 (1-z-zr (- 1 z zr))
         )
    (list
     ;; dz/dt
     (- (* mab zv 1-z-zr) (* z ugly5))
     ;; dzr/dt
     (- (* gamrG z) (* zr ugly2))
     ;; dzv/dt
     (- (* ac (+ z zr) 1-zv) (* mu zv))
     )
    )
  )
(compile 'f)

;; Now let's run the model:
;; (def points (rk-f #'f ynot start end num :verbose t))
(def points (rk-f #'f ynot start end num))

(mvsq '(times z zr zv) (transpose points))
(let ()
  (lines on)
  (plot times z)
  (send my-plot :add-plot-item

	(list 'lines '(20 25) '(.4 .4) 'blue)
	(list "Z" 30 .4)

	(list 'lines '(20 25) '(.3666 .3666) 'red)
	(list "Zr" 30 .3666)

	(list 'lines '(20 25) '(.3333 .3333) 'green)
	(list "Zv" 30 .3333)

	(list 'lines times zr 'red)
	(list 'lines times zv 'green)
	)
  )
