#| - Thu Feb 26 22:52:20 1998

	I've got a function (below) which implements newton's method for
non-linear systems, and I want to use it to find the singular vectors of a
matrix. I consider below, in detail, an example of a 3x3 matrix.

	I worked up the analysis for the general matrix case, and coded it up,
and I've also considered it by AA^T. Both work, sort of, but instead of finding
the first singular vector/value, they find the last!

	I spent a good deal of time and effort trying to turn that around, but
realized that, given the way I was proceeding (seeking a zero of a non-linear
system, of which each singular value/vector pair is a zero), I had many zeros
and none which had precedence that I could see.

	So I gave up on trying to find the singular values in descending order,
and decided to change my objective function to include orthogonality to
preceding vectors. This now seems to work, and better than I'd expected.

	In the case of the 3x3 matrix, the grungy example shows the pattern
that the non-linear function and jacobian matrix follows - it generalizes
fairly easily, so that I was able to create a function which generates the svd
(called nlsvd, defined below). This is not the first time that I wrote a code
to solve for the SVD: I once coded the tranposed qr algorithm for the svd
(?robust-svd). What is essential here is that I may have worked up a method for
the calculation which I can generalize to the tensor case.

	The examples are giving excellent results.
	
							Andy Long
							Fearless Programming
							IMSP
							February 27, 1998

|#
;; this here's our function:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				non-linear-system-root
;;
(defun non-linear-system-root(f jack x0 &key (eps 1e-12) (maxits 100) verbose)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			non-linear-system-root Doc:
;;
;; Author: Andy Long
;; Created: Wed Feb 25 20:51:44 1998
;;
;; Description/Arguments: non-linear-system-root
  
;; Arguments: f jack x0 &key (eps 1e-12) (maxits 100)

;; Returns: a vector x which we hope gives the vectoral equation f(x)=0.

;; Notes: For DESS class at the IMSP

;; Examples:
 ;; How about trying to solve linear systems?
 (def A #2a((2 4)(5 2)))
 (def b #(2 1))
 ;; now we set up the two functions, Ax-b:
 (defun f(z)
  (let ((x (first z))
	(y (second z)))
    (list
     (- (+ (* 2 x) (* 4 y)) 2)
     (- (+ (* 5 x) (* 2 y)) 1))))
 (defun jack(z) A)
 ;; and compare solutions:
 (non-linear-system-root #'f #'jack (list 0 0) :eps 1e-12)
 (solve A b)
"
  (do*
   (
    (i 1 (1+ i))
    (x0 x0 x1)
    (x1 
     (+ x0 (solve (funcall jack x0) (- (funcall f x0))))
     (+ x0 (solve (funcall jack x0) (- (funcall f x0))))
     )
    )
   ((or (> i maxits) (< (norm (- x1 x0)) eps)) (print i) x1)
   (if verbose (print x1))
   )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				IT WORKS! Grungy example:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(def A #2a((2 4 -1)(5 6 2)(1 2 1)))
(def size (size A))
(defun funcs(z)
  (let* (
	 (size (size A))
	 (length (length z))
	 (u (select z (iseq (first size))))
	 (v (select z (iseq (first size) (1- (sum size)))))
	 (lambda (select z (list (- length 2) (- length 1))))
	 (l0 (first lambda))
	 (l1 (second lambda))
	 )
    (combine
     (- (matmult A v) (* 2 l0 u))
     (- (matmult u A) (* 2 l1 v))
     (* (- 1 (dot u u)))
     (* (- 1 (dot v v)))
     )))

(defun jack(z)
  (let* (
	 (size (size A))
	 (length (length z))
	 (u (select z (iseq (first size))))
	 (v (select z (iseq (first size) (1- (sum size)))))
	 (ulen (length u))
	 (vlen (length v))
	 (lambda (select z (list (- length 2) (- length 1))))
	 (l0 (first lambda))
	 (l1 (second lambda))
	 (ustuff (bind-columns (* -2 u) (zeros ulen 1)))
	 (vstuff (bind-columns (zeros vlen 1) (* -2 v)))
	 )
    (combine-matrices
     (list (* -2 l0 (id ulen))         A            ustuff)
     (list (transpose A)      (* -2 l1 (id vlen))   vstuff)
     (list (transpose ustuff) (transpose vstuff)  (zeros 2 2))
     )))
    
(setq first-one (non-linear-system-root #'funcs #'jack (rand.n 8)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;		Now let's find the second one:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def u1 (select first-one (iseq (first size))))
(def v1 (select first-one (iseq (first size) (1- (sum size)))))
(def lambda1 (sum (select-from-end first-one (iseq 2))))
			
(defun funcs(z)
  (let* (
	 (size (size A))
	 (length (length z))
	 (u (select z (iseq (first size))))
	 (v (select z (iseq (first size) (1- (sum size)))))
	 (lambda (select z (list (- length 2) (- length 1))))
	 (l0 (first lambda))
	 (l1 (second lambda))
	 )
    (combine
     (- (matmult A v) (* 2 l0 u))
     (- (matmult u A) (* 2 l1 v))
     (- (dot u u1))
     (- (dot v v1))
     (* (- 1 (dot u u)))
     (* (- 1 (dot v v)))
     )))

(defun jack(z)
  (let* (
	 (size (size A))
	 (length (length z))
	 (u (select z (iseq (first size))))
	 (v (select z (iseq (first size) (1- (sum size)))))
	 (ulen (length u))
	 (vlen (length v))
	 (lambda (select z (list (- length 2) (- length 1))))
	 (l0 (first lambda))
	 (l1 (second lambda))
	 (ustuff (bind-columns (- u1) (zeros ulen 1) (* -2 u) (zeros ulen 1)))
	 (vstuff (bind-columns (zeros vlen 1) (- v1) (zeros vlen 1) (* -2 v)))
	 (zeros (zeros 4 4))
	 )
    (combine-matrices
     (list (* -2 l0 (id ulen))         A            ustuff)
     (list (transpose A)      (* -2 l1 (id vlen))   vstuff)
     (list (transpose ustuff) (transpose vstuff)    zeros)
     )))
    
(def second-one (non-linear-system-root #'funcs #'jack (rand.n 10)))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;		Now let's find the third one:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def u2 (select second-one (iseq (first size))))
(def v2 (select second-one (iseq (first size) (1- (sum size)))))
(def lambda2 (sum (select-from-end second-one (iseq 2))))

(defun funcs(z)
  (let* (
	 (size (size A))
	 (length (length z))
	 (u (select z (iseq (first size))))
	 (v (select z (iseq (first size) (1- (sum size)))))
	 (lambda (select z (list (- length 2) (- length 1))))
	 (l0 (first lambda))
	 (l1 (second lambda))
	 )
    (combine
     (- (matmult A v) (* 2 l0 u))
     (- (matmult u A) (* 2 l1 v))
     (- (dot u u1))
     (- (dot v v1))
     (- (dot u u2))
     (- (dot v v2))
     (* (- 1 (dot u u)))
     (* (- 1 (dot v v)))
     )))

(defun jack(z)
  (let* (
	 (size (size A))
	 (length (length z))
	 (u (select z (iseq (first size))))
	 (v (select z (iseq (first size) (1- (sum size)))))
	 (ulen (length u))
	 (vlen (length v))
	 (lambda (select z (list (- length 2) (- length 1))))
	 (l0 (first lambda))
	 (l1 (second lambda))
	 (ustuff (bind-columns (- u1) (zeros ulen 1) 
			       (- u2) (zeros ulen 1) 
			       (* -2 u) (zeros ulen 1)))
	 (vstuff (bind-columns (zeros vlen 1) (- v1)
			       (zeros vlen 1) (- v2)
			       (zeros vlen 1) (* -2 v)))
	 (zeros (zeros 6 6))
	 )
    (combine-matrices
     (list (* -2 l0 (id ulen))         A            ustuff)
     (list (transpose A)      (* -2 l1 (id vlen))   vstuff)
     (list (transpose ustuff) (transpose vstuff)    zeros)
     )))
    
(def third-one (non-linear-system-root #'funcs #'jack (rand.n 12)))
(def u3 (select third-one (iseq (first size))))
(def v3 (select third-one (iseq (first size) (1- (sum size)))))
(def lambda3 (sum (select-from-end third-one (iseq 2))))
			
(mprint (bind-columns u3 u2 u1))
(mprint (list lambda3 lambda2 lambda1))
(mprint (bind-columns v3 v2 v1))
(mapcar #'mprint (butlast (svd a)))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	Now let's create a function that finds the SVD in general:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; add to manual: nlsvd
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				nlsvd
;;
(defun nlsvd(A &key verbose (eps 1e-12) (maxits 100) no-orthogonalization)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			nlsvd Doc:
;;
;; Author: Andy Long
;; Created: Fri Feb 27 10:35:52 1998
;;
;; Description/Arguments: nlsvd
  
;; Arguments: A &key verbose (eps 1e-12) (maxits 100) no-orthogonalization

;; Returns: the svd of a matrix, obtained using newton's method to solve for a
   root of a system of non-linear equations.

;; Notes: Stemming from my DESS class at the IMSP, and looking toward a method
   that generalizes for the TSVD (Tensor SVD).

	For the moment I've added the 'no-orthogonalization' key, to see if
  that speeds up (or slows down!) things. As it is, I'm taking the orthogonal
  complement of a random vector with the vectors already obtained.

;; See Also: non-linear-system-root

;; Examples:
 (def A #2a((2 4 -1)(5 6 2)(1 2 1)))
 (nlsvd A)

 (def m 3)
 (def n 4)
 (def A (matrix (list m n) (rand.n (* m n))))
 (nlsvd A)

 (def m 4)
 (def n 3)
 (def A (matrix (list m n) (rand.n (* m n))))
 (nlsvd A)
"
  (let* (
	 (size (size A))
	 (dim (min size))
	 (uvdims (sum size))
	 (ulen (first size))
	 (vlen (second size))
	 (ubunch (iseq ulen))
	 (vbunch (iseq ulen (1- uvdims)))
	 (uzeros (zeros ulen 1))
	 (vzeros (zeros vlen 1))
	 (lagrange-multipliers 0)
	 schmidt-pair us vs lambdas
	 order
	 )
    (dotimes
     (i dim)
     (setq lagrange-multipliers (+ 2 lagrange-multipliers))
     (flet (
	    (funcs(z)
		  (let* (
			 (u (select z ubunch))
			 (v (select z vbunch))
			 (other-lambda-indices
			  (set-difference
			   (iseq (length z))
			   (combine
			    ubunch vbunch
			    (select (reverse (iseq (length z))) (iseq 2)))))
			 (other-lambda (select z (order other-lambda-indices)))
			 (lambda (select-from-end z (iseq 2)))
			 (l0 (first lambda))
			 (l1 (second lambda))
			 )
		    (combine
		     #| - Wed Mar 4 17:34:42 1998

 This is very strange. I really should have this term corresponding to the
 other-lambda in the mix. But I actually converge faster in my test cases
 without it. Some sort of relaxation? It would be nice to find a test case where
 this works, but the other doesn't... or would it? It's clearly a lot faster if
 I don't have to multiply each time.
	I've checked, and it's not like the vector is zero or anything.... So
 for the moment I'm not using the other-lambda variable.

		     (if other-lambda
			 (- (matmult A v) (* 2 l0 u)
			    (apply
			     #'+
			     (mapcar (lambda(lam ui) (* lam ui))
				     (firsts other-lambda) us)))
		       (- (matmult A v) (* 2 l0 u)))
		     (if other-lambda
			 (- (matmult u A) (* 2 l1 v)
			    (apply
			     #'+
			     (mapcar (lambda(lam vi) (* lam vi))
				     (seconds other-lambda) vs)))
		       (- (matmult u A) (* 2 l1 v)))
|#

		     (- (matmult A v) (* 2 l0 u))
		     (- (matmult u A) (* 2 l1 v))
		     (if (= i 0)
			 (list (* (- 1 (dot u u))) (* (- 1 (dot v v))))
		       (combine
			(mapcar
			 (lambda(us vs) (list (- (dot u us)) (- (dot v vs))))
			 us vs)
			(* (- 1 (dot u u)))
			(* (- 1 (dot v v))))
		       ))))
	    (jack(z)
		 (let* (
			(u (select z ubunch))
			(v (select z vbunch))
			(lambda (select-from-end z (iseq 2)))
			(l0 (first lambda))
			(l1 (second lambda))
			(ustuff
			 (if (= i 0)
			     (bind-columns (* -2 u) uzeros)
			   (apply #'bind-columns
				  (smoosh
				   (list
				    (mapcar
				     (lambda(ui) (bind-columns (- ui) uzeros))
				     us)
				    (list (* -2 u)) uzeros)))))
			(vstuff
			 (if (= i 0)
			     (bind-columns vzeros (* -2 v))
			   (apply #'bind-columns
				  (smoosh
				   (list
				    (mapcar
				     (lambda(vi) (bind-columns vzeros (- vi)))
				     vs)
				    vzeros (list (* -2 v)))))))
			(zeros (zeros lagrange-multipliers lagrange-multipliers))
			(astuff
			 (combine-matrices
			  (list (* -2 l0 (id ulen))         A)
			  (list (transpose A)      (* -2 l1 (id vlen)))))
			(vecstuff (bind-rows ustuff vstuff))
			)
		   (combine-matrices
		    (list        astuff          vecstuff)
		    (list (transpose vecstuff)    zeros  )
		    )))
	    )
	   (setq
	    schmidt-pair
	    (non-linear-system-root
	     #'funcs #'jack
	     (combine 
	      (normalize
	       (if (or no-orthogonalization (= i 0)) (rand.n ulen)
		 (orthogonal-complement (rand.n ulen) (apply #'bind-columns us))))
	      (normalize
	       (if (or no-orthogonalization (= i 0)) (rand.n vlen)
		 (orthogonal-complement (rand.n vlen) (apply #'bind-columns vs))))
	      (rand.n lagrange-multipliers))
	     :verbose verbose :maxits maxits :eps eps
	     )
	    lambda (sum (select-from-end schmidt-pair (iseq 2)))
	    us (append us (list (select schmidt-pair ubunch)))
	    ;; change the sign of v if lambda is negative:
	    vs (append vs (list (* (signum lambda) (select schmidt-pair vbunch))))
	    ;; and make sure that lambda itself is positive:
	    lambdas (append lambdas (list (abs lambda)))
	    )
	   )
     )
    ;; order the singular values from largest to smallest:
    (setq
     order (reverse (order lambdas))
     lambdas (select lambdas order)
     us (apply #'bind-columns (select us order))
     vs (apply #'bind-columns (select vs order))
     )
    (if verbose
	(mapcar
	 #'mprint
	 (list us lambdas vs
	       (- A (matmult us (diagonal-to-full lambdas) (transpose vs)))
	       )
	 )
      )
    (list us lambdas vs)
    )
  )
(try-out nlsvd)

;; Examples:
;;  (def A #3a(((2 4)(1 -1))((5 6)(2 1))))
;;  (nltsvd A)

