#| - ael - Wed Apr 29 11:44:27 1998

	I'm going to create a hybrid, a sexual product of two (or more!)
prototypes. The question is how to do it. I could

	- give *object* a slot hybrid, which contains the names of other
	prototypes with which it shares all properties.

This would permit me to reparent if necessary to access a method.

	I've discovered that reparenting works very nicely. Although methods
are lost, slots are not. And the change is transparent - if I reparent a
distribution proto as a univariate function proto, the change is registered in
the active list of function proto. Cool! Unfortunately there is no transfer of
ownership in the active-list. I might try redefining :reparent, using
call-next-method, and seeing if I can take care of that too.

	At any rate, there's some communication going on behind my back, with
things as they should be.

 - lea - |#
#| - ael - Sat May 16 10:46:53 1998

	Added children with multiple parents, as described in Lisp-Stat. Works
fine! Slicker than reparenting everytime. Appears that "births" and "deaths"
are working properly this way too.

	The problem that will occur is this: if several parents all have a
method, only one will be called (unless there is a call-next-method involved,
which is sensible).

 - lea - |#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			A distribution child of function-proto:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(require "obj-func")

(uber-proto distribution-proto
	    (random quantile parameters) nil
	    function-proto
	    "A distribution instance.

;; Examples:

 (def standard-normal
      (new
       continuous-distribution-proto
       ;; the function itself is the density
       :function (lambda(x) (normal-dens x))
       :num-vars 1
       :domain 'analytic
       :plot-domain '(-4 4)
       :plot-points 100
       ;; This is the cdf, the cumulative distribution function
       :derivative (lambda(x) (nilly x))
       :anti-derivative (lambda(x) (normal-cdf x))
       ;; new stuff:
       :quantile (lambda(x) (if (and (> x 0) (< x 1)) (normal-quant x)))
       :random (lambda(n) (normal-rand n))
       ))
 (send standard-normal :what)
 (show standard-normal)
 (send standard-normal :plot)
 (send standard-normal :reparent univariate-function-proto)
 (send standard-normal :plot) 
 (send standard-normal :plot-all)
 (show standard-normal)
 ;; would cause an error: 
 ;; (send standard-normal :quantile 4)
 (send standard-normal :reparent continuous-distribution-proto)
 (send standard-normal :quantile)

 ;; Having created continuous-uni-distribution-proto as a proto with two
 ;; parents, the continuous-distribution-proto and the
 ;; univariate-function-proto, we can use it rather than reparent:
 (def new-standard-normal
     (new
      continuous-uni-distribution-proto
      ;; the function itself is the density
      :function (lambda(x) (normal-dens x))
      :num-vars 1
      :domain 'analytic
      :plot-domain '(-4 4)
      :plot-points 100
      ;; This is the cdf, the cumulative distribution function
      :derivative (lambda(x) (nilly x))
      :anti-derivative (lambda(x) (normal-cdf x))
      ;; new stuff:
      :quantile (lambda(x) (if (and (> x 0) (< x 1)) (normal-quant x)))
      :random (lambda(n) (normal-rand n))
      ))
 (send new-standard-normal :plot)
 (send new-standard-normal :plot-all)
 ;; no error here, as quantile is a method for one of the parents:
 (send new-standard-normal :quantile)

 (def poisson
      (new
       discrete-distribution-proto
       :function (lambda(n) (poisson-pmf n (send self :parameters)))
       :num-vars 1
       :parameters 1
       :domain 'non-negative-integers
       :plot-domain '(0 10)
       :plot-points 11
       ;; This is the cdf, the cumulative distribution function
       :derivative (lambda(x) (nilly x))
       :anti-derivative (lambda(x) (poisson-cdf x (send self :parameters)))
       ;; new stuff:
       :quantile (lambda(x) (if (and (> x 0) (< x 1))
 			       (poisson-quant x (send self :parameters))))
       :random (lambda(n) (poisson-rand n (send self :parameters)))
       ))
")
(send distribution-proto :help)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Some children:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(uber-proto continuous-distribution-proto nil nil distribution-proto)
(uber-proto discrete-distribution-proto nil nil distribution-proto)
(uber-proto mixed-distribution-proto nil nil distribution-proto)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;		children featuring Multiple Parent prototypes:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; It doesn't cost much to create a bunch of protos! Here's an attempt to use
;; the multiple parenting feature:
(uber-proto continuous-uni-distribution-proto
	    nil nil
	    (list continuous-distribution-proto univariate-function-proto)
	    )
(uber-proto continuous-bi-distribution-proto
	    nil nil
	    (list continuous-distribution-proto bivariate-function-proto)
	    )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			Take 'em for a spin:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(send distribution-proto :try-out)

(ask function-proto active-list)
(ask function-proto number)
(ask univariate-function-proto number)
(ask bivariate-function-proto number)
(ask distribution-proto number)

;; We can reparent so as to give a distribution properties of a univariate
;; function (say):
(send standard-normal :reparent univariate-function-proto)
(mapcar (lambda(x)(send x :plot)) (ask function-proto active-list))
(send standard-normal :reparent continuous-distribution-proto)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;		continuous-distribution-proto method :plot-all
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth continuous-distribution-proto :plot-all (&rest args)
  "
;; Description: Plots all available functions: the function itself;
   derivative, anti-derivative, and quantile function.

;; Examples:
 (def standard-normal
      (new
       continuous-distribution-proto
       ;; the function itself is the density
       :function (lambda(x) (normal-dens x))
       :num-vars 1
       :domain 'analytic
       :plot-domain '(-4 4)
       :plot-points 100
       ;; This is the cdf, the cumulative distribution function
       :derivative (lambda(x) (nilly x))
       :anti-derivative (lambda(x) (normal-cdf x))
       ;; new stuff:
       :quantile (lambda(x) (if (and (> x 0) (< x 1)) (normal-quant x)))
       :random (lambda(n) (normal-rand n))
       ))
 (inform-dialog
     \"    Given the density, we can deduce the cumulative
     ~%as the integral of the density. Then we can deduce
     ~%the quantile, as the inverse of the cumulative.\" :location '(320 400))
 (title \"Standard Normal Distribution\")
 (send standard-normal
   :plot-all
   :domain '(-3 3) :points 300 :location '(20 20) :size '(300 300))
 (pause 24)
 (inform-dialog
     \"    We can create a random variable based on
     ~%the quantile, by sampling the uniform distribution
     ~%on the interval [0,1] and reading the y-values
     ~%of the quantile function (blue lines).\" :location '(320 400))
 (plot (send standard-normal :quantile) 0 1
       :location (320 20) :size (300 300))
 (def uniform-sample (uniform-rand 40))
 (def normal-sample (mapcar (send standard-normal :quantile) uniform-sample))
 (apply #'send my-plot :add-plot-item
       (mapcar
	(lambda(x y) (list 'lines (list x x) (list 0 y) 'blue))
	uniform-sample
	normal-sample))
 (apply #'send my-plot :add-plot-item
       (mapcar
	(lambda(x y) (list 'lines (list 0 x) (list y y) 'red))
	uniform-sample
	normal-sample))
 (pause 24)
 (def histo (histogram normal-sample :show nil))
 ;; shifts the histo over so that it doesn't cover the plot:
 (send histo :location 20 380)
 (send histo :show-window)
 (pause 24)
 (inform-dialog
     \"    The blue lines are from the sample of the 
     ~%uniform distribution. These 'reflect' off of the
     ~%quantile function, giving the red lines,
     ~%whose y-values form a random sample from the
     ~%normal distribution (histogram at left).\" :location '(320 400))
"
  (let* (
	 (fa (catch-keys '(:points :domain) args))
	 (points (first fa))
	 (domain (second fa))
	 (args (remove-keys '(:points :domain) args))
	 )
    (points (if points points (send self :plot-points)))
    (print args)
    (plotter (combine
	      (send self :function)
	      (send self :derivative)
	      (send self :anti-derivative)
	      (send self :quantile)
	      (if domain domain (send self :plot-domain)))
             :special-plot-args
	     (if args (mapcar #'combine (split-list args 2)))
	     )
    )
  )
(try-out :plot-all :proto continuous-distribution-proto)

(defun report()
  (let ()
    (mapcar (lambda(x) (terpri) (mapcar (lambda(y) (format t "~%~s" y)) x))
	    (list
	     (ask univariate-function-proto active-list)
	     (ask distribution-proto active-list)
	     (ask function-proto active-list)))
    (list
     (ask univariate-function-proto number)
     (ask distribution-proto number)
     (ask function-proto number))))

(send standard-normal :new-parent univariate-function-proto)
(report)
(send standard-normal :new-parent continuous-distribution-proto)
(report)

(send poisson :new-parent univariate-function-proto)
(report)
(send poisson :new-parent discrete-distribution-proto)
(report)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;		More multiple parent tests
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def new-standard-normal
     (new
      continuous-uni-distribution-proto
      ;; the function itself is the density
      :function (lambda(x) (normal-dens x))
      :num-vars 1
      :domain 'analytic
      :plot-domain '(-4 4)
      :plot-points 100
      ;; This is the cdf, the cumulative distribution function
      :derivative (lambda(x) (nilly x))
      :anti-derivative (lambda(x) (normal-cdf x))
      ;; new stuff:
      :quantile (lambda(x) (if (and (> x 0) (< x 1)) (normal-quant x)))
      :random (lambda(n) (normal-rand n))
      ))
(defmeth continuous-distribution-proto :crapola ()
  (print 'cont)
  (call-next-method)
  )
(defmeth univariate-function-proto :crapola ()
  ;; Here we explicitly do not call-next-method, to stop the chain:
  (print 'uni)
  )
(mapcar {print x} (send new-standard-normal :precedence-list))
(send new-standard-normal :crapola)

