#|
	I'm trying to copy Ripley's function from the C code (pps.c in 
		/data/software/R/R-0.65.0/packages/spatial/
), to make sure that I know what's going on.
	His example (provided with the documentation) is the towns data.

	Okay, I've reproduced his R results.

	This routine features edge correction for a rectangular box.

	The simulations assume a Poisson distribution of points in the same
rectangular box.

|#
;; Ripley's data:
(setq 
 x
 '(0.84  0.84  4.24  2.56  9.36 12.76 13.60 17.04 17.04 18.72 17.88 20.44
 21.28 18.72 25.52 28.08 28.92 29.80 31.48 35.76 39.16 39.16  4.24  8.52
 10.20 15.32 23.84 18.72 22.12 26.40 27.24 20.44 30.64 34.88 35.76 38.28
  1.72  2.56  9.36 15.32 16.16 23.84 26.40 27.24 34.04 33.20 36.60  1.72
  5.96  5.96  9.36  9.36 15.32 17.04 17.88 22.96 24.68 28.08 29.80 35.76
 36.60  4.24  5.96  8.52 10.20 18.72 22.96 29.80 31.48)
 y
 '(39.16 33.20 34.04 32.36 37.44 37.44 39.16 36.60 34.88 38.28 34.88 35.76
 33.20 31.48 35.76 33.20 32.36 36.60 37.44 35.76 33.20 39.16 29.80 24.68
 26.40 30.64 30.64 23.84 25.52 23.84 28.92 29.80 25.52 24.68 28.08 28.08
 15.32 20.44 21.28 15.32 18.72 19.56 17.88 22.12 14.48 20.44 18.72  7.64
 13.60 11.08 13.60  7.64 11.92  5.96  8.52  7.64 13.60 11.92  7.64 12.76
 10.20  4.24  5.12  4.24  4.24  0.00  3.40  2.56  0.84)
 xl 0
 xu 40 
 yl 0
 yu 40
 )

(defun VR_sp_pp2(x y k fs
		   &key
		   (xu0 (max x))
		   (xl0 (min x))
		   (yu0 (max y))
		   (yl0 (min y))
		   )
  (let* (
	 (n (size x))
	 (kk k)
	 (fss fs)

	 dmin
	 lm
	 a

	 (alm 0)
	 (ax (- xu0 xl0))
	 (ay (- yu0 yl0))
	 (sarea (sqrt (* ax ay)))
	 (dm fss)
	 (g (/ 2.0 (* n n)))
	 (fs1 (min fss ( * 0.5 (sqrt (+ (* ax ax) (* ay ay))))))
	 (s1 (/ kk fss))
	 (k1 (floor (+ (* s1 fs1) 1e-3)))
	 (k k1)
	 (rr (* fs1 fs1))
	 (h (repeat 0 kk))
	 )

    (dolist
     (i (iseq 1 (1- n)))
     (let (
	   (xi (elt x i))
	   (yi (elt y i))
	   )
       (dotimes
	(j i)
	(let* (
	       (x1 (- (elt x j) xi))
	       (y1 (- (elt y j) yi))
	       (a (+ (* x1 x1) (* y1 y1)))
	       )
	  (if (< a rr)
	      (let ()
		(setq
		 a (sqrt a)
		 dm (min a dm)
		 ib (floor (* s1 a))
		 )
		(if (< ib k1)
		    (let ()
		    (setf (elt h ib)
			  (+ (elt h ib)
			     (* g 
				(+
				 (edge xi yi a xl0 xu0 yl0 yu0)
				 (edge (elt x j) (elt y j) a xl0 xu0 yl0 yu0)))))
		    
		    (print (list xi yi a))
		    (print (edge xi yi a xl0 xu0 yl0 yu0))
		    )
		  )
		)
	    )
	  )
	)
       )
     )
    (setq a 0.0)
    (dotimes
     (i k1)
     (setf
      a (+ a (elt h i))
      (elt h i) (* (sqrt (/ a pi)) sarea)
      alm (max alm (abs (- (elt h i) (/ (+ i 1) s1))))
      )
     )
    (setq 
     dmin dm
     lm alm
     )
    (list (/ (* (iseq 1 k1) fs) k) h n k dmin (max dmin lm) fs)
    )
  )

(defun edge (x y a xl0 xu0 yl0 yu0)
  "
;; Examples:
 (edge .1 .2 .03 0 40 0 40)
 (edge .1 .2 .3 0 40 0 40)
 (edge .1 .2 300 0 40 0 40)
"
  (let* (
	 (r (repeat 0 6))
	 (w (- x xl0))
	 (b 0.0)
	 )
    (if (> w (- y yl0)) (setq w (- y yl0)))
    (if (> w (- xu0 x)) (setq w (- xu0 x)))
    (if (> w (- yu0 y)) (setq w (- yu0 y)))
    (if (<= a w) 
	.5
      (let ()
	(setf
	 (elt r 4) (- x xl0)
	 (elt r 0) (- x xl0)
	 (elt r 5) (- yu0 y)
	 (elt r 1) (- yu0 y)
	 (elt r 2) (- xu0 x)
	 (elt r 3) (- y yl0)
	 )
	(dolist
	 (i (iseq 1 4))
	 (if (< (elt r i) a) 
	     (let ()
	       (if (= (elt r i) 0.0)
		   (setq b (+ b pi))
		 (let ()
		   (setq 
		    c (acos (/ (elt r i) a))
		    c1 (atan (/ (elt r (- i 1)) (elt r i)))
		    c2 (atan (/ (elt r (+ i 1)) (elt r i)))
		    b (+ b (min c c1) (min c c2))
		    )
		   )
		 )
	       )
	   )
	 )
	(if (< b 6.28)
	    (/ 1.0 (- 2.0 (/ b pi)))
	  0.0
	  )
	)
      )
    )
  )

(setq
 crap (vr_sp_pp2 x y 100 10 :xl0 xl :xu0 xu :yl0 yl :yu0 yu)
 hx (first crap)
 hy (second crap)
 rip-n (elt crap 2)
 rip-k (elt crap 3)
 rip-dmin (elt crap 4)
 rip-lm (elt crap 5)
 rip-fs (elt crap 6)
 )
;;     (list (/ (* (iseq 1 k1) fs) k) h n k dmin (max dmin lm) fs)

;; This is the output of the R function:
(def yout
     '(0.0000000 0.0000000 0.0000000 0.0000000 0.0000000 0.0000000 0.0000000
0.0000000 0.4625418 0.4625418 0.4625418 0.6541329 0.6541329 0.6541329 0.6541329
0.6541329 0.8011459 0.9250835 1.0342749 1.5024656 1.5024656 1.5024656 1.5024656
1.6490924 1.9085721 2.0268926 2.1794789 2.2755259 2.2755259 2.2755259 2.4710269
2.4710269 2.4710269 2.5139448 2.6098318 3.1199529 3.1877886 3.3599646 3.4903616
3.4903616 3.4903616 3.4903616 3.8245269 4.0341476 4.0341476 4.2990872 4.3749063
4.3749063 4.5957016 4.8009766 4.8558403 5.0720845 5.0720845 5.1418631 5.3050532
5.3050532 5.3988179 5.4679588 5.4679588 5.7074859 5.8608711 6.1883474 6.3188256
6.3188256 6.3758618 6.3758618 6.4458644 6.4624386 6.6504394 6.6504394 6.7969133
6.7969133 6.9633078 7.0758398 7.0758398 7.0758398 7.3762555 7.4472879 7.5931906
7.5931906 7.8978491 7.8978491 7.8978491 7.9630918 8.0861043 8.3091722 8.4274932
8.5585451 8.6664230 8.6664230 8.7525510 8.9325020 8.9827272 9.0883749 9.1160634
9.4019636 9.5457719 9.7197411 9.7197411 9.9018321))

(plot hx hy)
(plot hx yout)
(plot hy yout)