#| - ael - Sun Dec 6 16:58:02 1998

This is a complement to the book "Spatial Autocorrelation: A Primer" by Daniel
Griffith.

 - lea - |#


;; add to manual: shapiro-wilks
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				shapiro-wilks
;;
(defun shapiro-wilks(x a)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			shapiro-wilks Doc:
;;
;; Author: Andy Long
;; Created: Mon Dec 7 00:15:11 1998
;;
;; Description/Arguments: shapiro-wilks

	
;; Arguments: x coeffs

;; Returns: 

;; Notes: 
	As W -> 1, the distribution of x conforms to a normal distribution.
As W->0, it fails to conform. Page 21.

;; See Also: 

;; Key Functions:

;; Examples:
 ;; p. 50:
 (shapiro-wilks '(0 1 1 2) '(.6872 .1677))
"
  (let* (
	 (rank (rank x))
	 (n (length x))
	 (x (select x rank))
	 (xr (reverse x))
	 (n  (/ (if (evenp n) n (1+ n)) 2))
	 (b (sum (* a (abs (- (select x (iseq n)) (select xr (iseq n)))))))
	 )
    (printall a (abs (- (select x (iseq n)) (select xr (iseq n)))))
    (/ (* b b) (sum (square (- x (mean x)))))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				pattern-game
;;
(defun pattern-game(matrix &optional dimensions)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			pattern-game Doc:
;;
;; Author: Andy Long
;; Created: Sun Dec 6 17:02:40 1998
;;
;; Description/Arguments: pattern-game

	This is a function to support a cute lab experiment proposed by
Griffith. Arrange the matrix to either maximize or minimize the spatial
autocorrelation. 

;; Arguments: a matrix, or a list followed with the dimensions of the matrix.

;; Returns: spatial autocorrelation (developed in the book)

;; Notes: See page 17 of the book `Spatial Autocorrelation: A Primer` by Daniel
;; Griffith. The spatial autocorrelation coefficient is developed in the
;; software program in the appendix.

;; See Also: 

;; Key Functions:

;; Examples:
 (pattern-game
 (matrix
  '(4 4)
  '(16 78 83 13
    38 50 58 23
    55 41 38 75
    19 84 26 27)
  ))

 (pattern-game
 (matrix
  '(4 4)
  '(84 83 58 38
    78 75 50 27
    55 41 26 19
    38 23 16 13)
  ))

 (pattern-game '(19 83 84 13 38 55 58 26 50 41 38 75 16 78 23 27) '(4 4))
 (pattern-game '(50 23 58 38 19 84 16 55 78 13 83 27 38 75 26 41) '(4 4))
"
  (let* (
	 (a (if (matrixp matrix) matrix
	      (if dimensions (matrix dimensions (combine matrix))
		(error "can't deal with that thing you've passed."))))

	 (ss (sum (square (- a (mean a)))))

	 (dims (size a))
	 (rows (first dims))
	 (cols (second dims))

	 ;; since we're using the rook's definition, I just need to shift the
	 ;; matrix by 1 step (left/right and top/bottom), subtract, square, and
	 ;; add it all up:
	 (left (select a (iseq rows) (iseq (1- cols))))
	 (right (select a (iseq rows) (iseq 1 (1- cols))))
	 (top (select a (iseq (1- rows)) (iseq cols)))
	 (bot (select a (iseq 1 (1- rows)) (iseq cols)))
	 (dif (+ (sum (square (- left right)))
		 (sum (square (- top bot)))))
	 )
    ;; The formula in the book has a 32 in the denominator: this cancels with a
    ;; 2 which I have in the dif coefficient (he adds every pair twice, so I
    ;; should have a two in the numerator, which cancels with the 32):
    (if (= ss 0) 1 (- 1 (^ (/ (* 5 dif) (* 16 ss)) .9145063)))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; add to manual: geary-ratio
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				geary-ratio
;;
(defun geary-ratio(a C-matrix)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			geary-ratio Doc:
;;
;; Author: Andy Long
;; Created: Sun Dec 6 17:52:28 1998
;;
;; Description/Arguments: geary-ratio

	Described on page 36 of

;; Arguments: a list (or matrix), with the connectivity matrix.

;; Returns: The Geary Ratio, oddly enough

;; Notes: 
	As G-> 0, spatial autocorrelation increases;
	As G-> 2, spatial autocorrelation decreases.
The mean assuming randomness is 1.

;; See Also: 

;; Key Functions:

;; Examples:
 (geary-ratio '(1 2 3 2 3 4 3 4 5) (matrix-neighbors 3))
 (geary-ratio '(3 4 3 4 3 2 1 2 5) (matrix-neighbors 3))
 (geary-ratio (matrix '(3 3) '(4 1 4 2 5 2 3 3 3))
	      (matrix-neighbors 3))

 ;; from the book: Buffalo crime data, p. 39:
 (def buff-c
  (matrix
  '(14 14)
  '(0 1 0 1 1 0 0 1 0 0 0 0 0 0
    1 0 0 1 1 1 0 0 0 1 0 0 0 0
    0 0 0 1 0 0 0 1 0 0 1 0 0 1
    1 1 1 0 0 0 0 1 0 1 0 0 1 1
    1 1 0 0 0 1 1 0 0 0 0 1 0 0
    0 1 0 0 1 0 1 0 1 1 0 0 0 0
    0 0 0 0 1 1 0 0 1 0 0 1 0 0
    1 0 1 1 0 0 0 0 0 0 0 0 0 0
    0 0 0 0 0 1 1 0 0 1 0 0 1 0
    0 1 0 1 0 1 0 0 1 0 0 0 1 0
    0 0 1 0 0 0 0 0 0 0 0 0 0 1
    0 0 0 0 1 0 1 0 0 0 0 0 0 0
    0 0 0 1 0 0 0 0 1 1 0 0 0 1
    0 0 1 1 0 0 0 0 0 0 1 0 1 0)))
 (def prostitution '(220 74 1 15 0 1 0 7 0 10 0 0 3 0))
 (def arson '(13 23 48 33 72 46 44 53 23 80 30 33 65 24))
 (def population '(7695 17354 37558 39950 9846 26539
    28822 28077 29897 36005 29502 33998 60671 42863))

 (geary-ratio prostitution buff-c)
 (geary-ratio arson buff-c)

 (simulation population buff-c :type #'geary-ratio)
"
  (let* (
	 (x (combine a))
	 (ss (sum (square (- x (mean x)))))
	 (n (first (size c-matrix)))
	 (n-1 (1- n))
	 (gr 0)
	 (sc 0)
	 )
    (dolist
     (i (iseq n-1))
     (let ((xi (elt x i)))
       (dolist
	(j (iseq (1+ i) n-1))
	(setq
	 gr (+ gr (* (aref c-matrix i j) (square (- xi (elt x j)))))
	 sc (+ sc (aref c-matrix i j))
	 )
	)
       )
     )
    ;; if the ss is 1, the field is perfectly autocorrelated:
    (if (= ss 0) 1 (* n-1 gr (/ (* 2 sc ss))))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; add to manual: moran-I
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				moran-I
;;
(defun moran-I(a C-matrix)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			moran-I Doc:
;;
;; Author: Andy Long
;; Created: Sun Dec 6 17:52:28 1998
;;
;; Description/Arguments: moran-I

	Described on page 36 of

;; Arguments: a list (or matrix), with the connectivity matrix.

;; Returns: Moran's I, oddly enough

;; Notes: 
	Expected value: -1/(n-1)
	MC -> 1 => SA; MC -> 0 => dissimilar values in juxtaposition.

;; See Also: 

;; Key Functions:

;; Examples:
 (moran-I '(0 1 0 1 0 1 0 1 0) (matrix-neighbors 3))
 (moran-I '(1 2 3 2 3 4 3 4 5) (matrix-neighbors 3))
 (moran-I '(3 4 3 4 3 2 1 2 5) (matrix-neighbors 3))
 (moran-I (matrix '(3 3) '(4 1 4 2 5 2 3 3 3))
	      (matrix-neighbors 3))

 (histogram 
   (mapcar (lambda(i) (moran-I (sample (iseq 10) 9) (matrix-neighbors 3)))
           (iseq 100)))

 ;; from the book: Buffalo crime data, p. 40:
 (def buff-c
  (matrix
  '(14 14)
  '(0 1 0 1 1 0 0 1 0 0 0 0 0 0
    1 0 0 1 1 1 0 0 0 1 0 0 0 0
    0 0 0 1 0 0 0 1 0 0 1 0 0 1
    1 1 1 0 0 0 0 1 0 1 0 0 1 1
    1 1 0 0 0 1 1 0 0 0 0 1 0 0
    0 1 0 0 1 0 1 0 1 1 0 0 0 0
    0 0 0 0 1 1 0 0 1 0 0 1 0 0
    1 0 1 1 0 0 0 0 0 0 0 0 0 0
    0 0 0 0 0 1 1 0 0 1 0 0 1 0
    0 1 0 1 0 1 0 0 1 0 0 0 1 0
    0 0 1 0 0 0 0 0 0 0 0 0 0 1
    0 0 0 0 1 0 1 0 0 0 0 0 0 0
    0 0 0 1 0 0 0 0 1 1 0 0 0 1
    0 0 1 1 0 0 0 0 0 0 1 0 1 0)))
 (def prostitution '(220 74 1 15 0 1 0 7 0 10 0 0 3 0))
 (def arson '(13 23 48 33 72 46 44 53 23 80 30 33 65 24))
 (def population '(7695 17354 37558 39950 9846 26539
    28822 28077 29897 36005 29502 33998 60671 42863))

 (moran-I prostitution buff-c)
 (moran-I arson buff-c)
 (moran-I population buff-c)

 (simulation population buff-c)
"
  (let* (
	 (x (combine a))
	 (mean (mean x))
	 (z (- x mean))
	 (ss (sum (square z)))
	 (n (first (size c-matrix)))
	 (ones (repeat 1 n))
	 (gr (matmult z c-matrix z))
	 (sc (matmult ones c-matrix ones))
	 )
    ;; if the ss is 1, the field is perfectly autocorrelated:
    (if (= ss 0) 1 (* n gr (/ (* sc ss))))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; add to manual: matrix-neighbors
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				matrix-neighbors
;;
(defun matrix-neighbors(size)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			matrix-neighbors Doc:
;;
;; Author: Andy Long
;; Created: Sun Dec 6 19:14:31 1998
;;
;; Description/Arguments: matrix-neighbors

	

;; Arguments: size

;; Returns: 

;; Notes: 

;; See Also: 

;; Key Functions:

;; Examples:
 (matrix-neighbors 3)
"
  (let* (
	 (n (* size size))
	 (size-1 (1- size))
	 (id (id size))
	 (zero (zeros size size)) 
	 (off zero)
	 (off (dolist
	       (i (iseq (1- size)))
	       (let ((i+1 (1+ i)))
		 (setf
		  (aref off i i+1) 1
		  (aref off i+1 i) 1
		  ))))
	 (off zero)
	 (result (zeros n n))
	 )
    (setf
     (select
      result
      (iseq (* size size-1) (1- n))
      (iseq (* size size-1) (1- n)))
     off
     )
    (dolist
     (i (iseq 0 (- size 2)))
     (let ((rown (* i size)))
       (setf
	(select
	 result
	 (iseq rown (+ rown size-1))
	 (iseq rown (+ rown size-1)))
	off
	)
       (setf
	(select
	 result
	 (iseq rown (+ rown size-1))
	 (iseq (+ rown size) (1- (+ rown (* 2 size)))))
	id
	)
       (setf
	(select
	 result
	 (iseq (+ rown size) (1- (+ rown (* 2 size))))
	 (iseq rown (+ rown size-1)))
	id
	)
       )
     )
    result
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun simulation(var buff-c &key (times 100) (type #'moran-I))
  "
;; Examples:
 (simulation population buff-c)
 (simulation population buff-c :type #'geary-ratio)
"
  (let* ((pop (funcall type var buff-c))
	 (n (length var))
	 (trial
	  (mapcar
	   (lambda (x) (funcall
			type
			(select var (random-permutation n))
			buff-c))
	   (iseq times)
	   )))
    (title 
     (format nil "Value: ~a; proportion>Value: ~a"
	     pop (/ (length (where trial pop #'>)) times)))
    (histo trial)))


;; add to manual: toronto
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				toronto
;;
(defun toronto(a b al be)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			toronto Doc:
;;
;; Author: Andy Long
;; Created: Mon Dec 7 00:06:56 1998
;;
;; Description/Arguments: toronto

	

;; Arguments: a b

;; Returns: 

;; Notes: 

;; See Also: 

;; Key Functions:

;; Examples:
 (toronto 26993.73 .0929 10919.66 .0989)
"
  (plot (lambda(d) (* a (exp (- (* b d))))) 0 100)
  (send my-plot :add-plot-item
	(list 'function 
	      (lambda(d) (* al (exp (- (* be d)))))
	      'red
	      'lines
	      )
	)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 SAgame
;;
(defun SAgame(file output)
  "
;; Used in a script for the web course in spatial epi
;; Examples:
 (SAgame-gen \"crap2\")
 (SAgame \"crap2\" \"stuff2\")
"
  (let* (
	 (a (read-matrix-data file))
	 (grif (pattern-game a (size a)))
	 (moran (moran-I (combine a) (matrix-neighbors (first (size a)))))
	 )
    (with-open-file
     (stream output :direction :output)
     (format stream "~8,6f~%~8,6f~%" grif moran)
     )
    (list grif moran)
    )
  )

(defun SAgame-gen(file)
  "
;; Used in a script for the web course in spatial epi
"
  (with-open-file
   (stream file :direction :output)
   (let ((a (matrix '(4 4) (sample (iseq 100) 16 t))))
     (dolist
      (row (row-lists a))
      (format stream "~{~d ~}~%" row)
      (format t "~{~d ~}~%" row)
      )
     )
   )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
