#| - Mon Mar 9 20:52:26 1998

	This is Strang's version of the simplex method, created because the
guys in the DESS program wanted to see it! Your wish is my command....

	Strang doesn't advocate this sloppy method of solution - he would be
horrified to see me actually calculating the inverse. But this is just to get
the job done and explain to the students. Optimizing the code is le cour
superieur.

	At the bottom of this file I've also created a tex file which
effectively contains the lectures I gave to the DESS gang.

functions:
simplex-phase-one
size
first
second
iseq
let*
do*
where
/= = - >= /
set-difference
if
length
combine
sample
select
inverse
matmult
min
setf
elt
remove
repeat	      

additional functions used in simplex-phase-one:
id
dotimes
aref

|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				simplex
;;
(defun simplex(A b c &key guess max)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			simplex Doc:
;;
;; Author: Andy Long
;; Created: Mon Mar 9 20:53:19 1998
;;
;; Description/Arguments: simplex
  
Simplex solves a linear program, given by the following:

	Minimize c'x where Ax=b and the elements of x are positive.

;; Arguments: A b c &key guess max

	A	- a matrix mxn, with m<n (we expect an overdetermined system,
		  with lots of solutions in the 'feasible set'.
	b	- a vector of length m, representing the rhs of the system of
		  constraints. 
	c	- a vector of length n, the cost function (to be minimized).
	guess	- a vertex
	max	- set max to t if you want to maximize instead of minimize.

;; Returns: a solution to the linear program, x, if one exists.

;; Notes: Taken from Gil Strang's mighty tome 'Introduction to Applied
          Mathematics'.

;; See Also: simplex-phase-one, for generating an initial guess automagically.

;; Examples:
 ;; Page 675:
 (simplex #2a((1 0 2 1)(0 1 1 -1)) '(4 2) '(3 1 9 1))
 ;; is the same as the next:
 (simplex #2a((1 0 2 1)(0 1 1 -1)) '(4 2) '(3 1 9 1) :guess '(0 0 2 0))
 ;; this guess also gets us the correct solution:
 (simplex #2a((1 0 2 1)(0 1 1 -1)) '(4 2) '(3 1 9 1) :guess '(4 2 0 0))

 ;; Class example:
 (simplex #2a((0 1 1 0 0)(1 1 0 -1 0)(5/2 -1 0 0 1)) '(5 3 10) '(1 3 0 0 0))
 ;; You can find a maximum, rather than a minimum, by using the key :max
 (simplex #2a((0 1 1 0 0)(1 1 0 -1 0)(5/2 -1 0 0 1)) '(5 3 10) '(1 3 0 0 0)
   :max t)

 ;; here are two more problems, with known solutions from Strang:
 ;; Page 689, exercise 8.2.1:
 (simplex #2a((1 2 3)) '(12) '(1 1 1) :guess '(12 0 0))
 ;; Page 689, exercise 8.2.2:
 (simplex #2a((1 1 1 0)(0 1 2 1)) '(6 1) '(0 7 9 0) :guess '(5 1 0 0))
 (simplex #2a((1 1 1 0)(0 1 2 1)) '(6 1) '(0 7 9 0) :guess '(5.5 0 .5 0))

 ;; If no guess is given, then simplex-phase-one solves a related simplex
 ;; problem to obtain an initial guess: for example, it solves this simplex
 ;; problem to get a starting value for the simplex problem in the first
 ;; example from above:
 (simplex #2A((1 0 2 1 1 0) (0 1 1 -1 0 1))
	  '(4 2) '(0 0 0 0 1 1) :guess '(0 0 0 0 4 2))
"
  (let* (
	 (c (if max (- c) c))
	 (x (if guess guess (simplex-phase-one A b c)))
	 (size (size A))
	 (m (first size))
	 (n (second size))
	 (nseq (iseq n))
	 (mseq (iseq m))
	 entering leaving pos ratios
	 )
    (do*
     (
      ;; We identify the non-zero components of x, as well as the non-zero
      ;; components:
      (where-non-zero (where x 0 #'/=) (where x 0 #'/=))
      (where-zero
       (set-difference nseq where-non-zero)
       (set-difference nseq where-non-zero))

      ;; In case we have a degeneracy, and there aren't enough non-zeros to
      ;; make up an invertible matrix, we tack on a few extras from the zero
      ;; group:
      (where-non-zero
       (if (= (length where-non-zero) m)
	   where-non-zero
	 (combine
	  where-non-zero
	  (sample where-zero (- m (length where-non-zero)))))
       (if (= (length where-non-zero) m)
	   where-non-zero
	 (combine
	  where-non-zero
	  (sample where-zero (- m (length where-non-zero))))))
      (where-zero
       (set-difference nseq where-non-zero)
       (set-difference nseq where-non-zero))

      ;; We split A into [A' N'], corresponding to the non-zero and zero
      ;; components of x respectively:
      (A-prime (select A mseq where-non-zero)
	       (select A mseq where-non-zero))
      (N-prime (select A mseq where-zero) (select A mseq where-zero))

      ;; We split x and c similarly:
      (xb (select x where-non-zero) (select x where-non-zero))
      (xn (select x where-zero) (select x where-zero))
      (cb (select c where-non-zero) (select c where-non-zero))
      (cn (select c where-zero) (select c where-zero))

      (a-inv (inverse A-prime) (inverse A-prime))
      (b-inv-n (matmult a-inv N-prime) (matmult a-inv N-prime))
      (b-inv-b (matmult a-inv b) (matmult a-inv b))
	       
      (r (- cn (matmult cb b-inv-n)) (- cn (matmult cb b-inv-n)))
      )

     ;; Here's our test: when r is non-negative, we can't improve, so exit with
     ;; the value of x:
     ((>= (min r) 0) x)
      
     ;; Otherwise, let's find out where to make our improvement. We seek one
     ;; variable to set to 0 (leaving) and one to draw into the very exclusive
     ;; non-zero club (entering):
     (setf
      entering (first (where r (min r)))
      v (combine (select b-inv-n mseq entering))

      ;; which of the zero components was that? We'll make it entering:
      entering (elt where-zero entering)

      ;; Now we're only interested in positive components of v in our search
      ;; for the leaving variable:
      pos (where v 0 #'>)
      ;; for which we form the ratios
      ratios (/ (select b-inv-b pos) (select v pos))
      ;; and we remove the element of the non-zero club which has the lowest
      ;; ratio (alpha) among the positives:
      alpha (min ratios)
      ;; first of all, leaving is an index of ratios, which we then associate
      ;; to the positive components of v, which we then associate with the
      ;; non-zero components of where-non-zero:
      leaving (elt where-non-zero (elt pos (first (where ratios alpha))))

      ;; Now we adjust our vector x:
      (select x where-non-zero) (- xb (* alpha v))
      ;; You'd think that previous line would be enough, but tiny little
      ;; numbers (negative as well as positive) get shoved in there instead of
      ;; zero:
      (select x leaving) 0
      (elt x entering) alpha
      )
     )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; add to manual: simplex-phase-one
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				simplex-phase-one
;;
(defun simplex-phase-one(A b c)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			simplex-phase-one Doc:
;;
;; Author: Andy Long
;; Created: Mon Mar 9 21:00:16 1998
;;
;; Description/Arguments: simplex-phase-one

	Carries out 'phase-one' for a simplex system: finding an initial guess
for the linear program.

;; Arguments: A b c

	A	- a matrix mxn, with m<n (we expect an overdetermined system,
		  with lots of solutions in the 'feasible set'.
	b	- a vector of length m, representing the rhs of the system of
		  constraints. 
	c	- a vector of length n, the cost function (to be minimized).

;; Returns: a starting guess for simplex.

;; Notes: written as a helper for simplex.

;; See Also: simplex

;; Examples:
 (simplex-phase-one #2a((1 0 2 1)(0 1 1 -1)) '(4 2) '(3 1 9 1))
 (simplex-phase-one #2a((1 2 3)) '(12) '(1 1 1))
 (simplex-phase-one #2a((1 1 1 0)(0 1 2 1)) '(6 1) '(0 7 9 0))

 ;; Class example:
 (simplex-phase-one #2a((0 1 1 0 0)(1 1 0 -1 0)(5/2 -1 0 0 1))
		    '(5 3 10) '(1 3 0 0 0))
"
  (let* (
	 (size (size A))
	 (m (first size))
	 (n (second size))
	 (signs (id m))
	 )
    (dotimes
     (i m)
     (if (< (elt b i) 0) (setf (aref signs i i) (- (aref signs i i))))
     )
    (setq
     A (matmult signs A)
     b (matmult signs b)
     )
    (select
     (simplex
      (bind-columns A (id m))
      b
      (combine (repeat 0 n) (repeat 1 m))
      :guess (combine (repeat 0 n) b))
     (iseq n))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#| - Mon Mar 16 09:25:44 1998
\documentstyle[12pt,epsf,rotate]{article}
\input usual_defs
\begin{document}

\title{\bf Linear Programming}
\author{Andy Long}
\maketitle

\begin{abstract}
These are the lecture notes for the brief introduction to linear programming
that I gave while at the IMSP in Benin. Thanks to the DESS gang for encouraging
me!
\end{abstract}

\section{Overview with sample problems:}

	Consider the following problem:

\centerline{\bf \rule{2mm}{2mm} Minimize $c(x_1,x_2)=x_1+3x_2$ subject to}
\centerline{$x_1+x_2=2$ with $x_1 \ge 0$ and $x_2 \ge 0$.}

[Draw the graph of the constraint, and show the feasible set - a segment.]

	One could solve for $x_2=f(x_1),$ $\implies x_2=2-x_1,$ plug that into
$c(x_1,2-x_1)=6-2x_1$, and now minimize this function. We try the old calculus
approach, and find that the derivative is $-2$ (and hence never zero), so we
have to look for our minimum on the \u{boundary}: in this case, either $x_1=0$
or $x_1=2$. Quick verification shows that the minimum occurs at $x_1=0$, so the
solution to our \u{linear program} is (0,2).

	We could easily have constraints other than equality constraints, and
we now show that we can transform such constraints into equality constraints by
the introduction of \u{slack variables}. Consider the linear program

\centerline{\bf \rule{2mm}{2mm} Minimize $c(x_1,x_2)=x_1+3x_2$ subject to}
\centerline{$x_1+x_2 \le 2$, $x_1 \le 1$, and $x_1 \ge 0$ and $x_2 \ge 0$.}

[Draw the graph and show the feasible set - a convex region. Discuss again the
fact that our function $c$ is linear, and hence can have a minimum only on the
boundary of the region. Furthermore, the minimum must be at a vertex, as the
cost function is linear in each variable: as we move along an edge the function
either increases linearly in one sense and decreases linearly in the other, or
stays unchanged. In either event, we can find a minimum at a vertex.]

	We create auxiliary variables which ``take up the slack'' in the
inequalities: in particular, we create
\[
	x_3 \suchthat x_1+x_3=1
\]
and
\[
	x_4 \suchthat x_1+x_2+x_4=2.
\]
Now we have a problem in four dimensions, where we can write the constraints as
a matrix equation
\[
	\left [
\begin{array}{cccc}
{1}&{0}&{1}&{0}\cr
{1}&{1}&{0}&{1}
\end{array}
\right ]
	\left [
\begin{array}{c}
{x_1}\cr
{x_2}\cr
{x_3}\cr
{x_4}
\end{array}
\right ] =
	\left [
\begin{array}{c}
{1}\cr
{2}
\end{array}
\right ]
\]
with a cost function which is unchanged except padded with zeros corresponding
to the slack variables:
\[
	c(x_1,x_2,x_3,x_4)=
	\left [
\begin{array}{cccc}
{1}&{3}&{0}&{0}
\end{array}
\right ]
	\left [
\begin{array}{c}
{x_1}\cr
{x_2}\cr
{x_3}\cr
{x_4}
\end{array}
\right ]
\]
This is now in the ``canonical form'' of a linear problem:

\bigskip
\centerline{\bf \rule{2mm}{2mm} Minimize $\u{c}^T\u{x}$ such that $A_{m \times n}\u{x}=\u{b}$ and $\u{x} \ge \u{0}.$}

	This defines a feasible region in $\u{x}$ space, and a minimum (if it
exists) must exist at a vertex. But it is certainly not necessary that a
minimum exist. $\exists$ three possibilities:
\begin{itemize}
\item
the feasible set is empty,
\item
the minimum is $-\infinity$, or
\item
$\exists$ a minimum at a vertex.
\end{itemize}

	Suppose for the moment that our program falls under case 3: there is,
in fact, a minimum at a vertex. George Dantzig is the father of the simplex
method\cite{strang-am}, which works as follows: we find a feasible vertex, then
march from neighboring vertex to neighboring vertex (always moving to one with
a lower cost) until we fall upon the minimum.

Now we did another problem, this one defined by a region in space. I drew the
figure described by the inequalities $\u{x} \ge \u{0}$, $x_1+x_2 \ge 3$, $x_2
\le 5$, and $-5x_1+2x_2 \ge -20$.

	We wrote down the matrix
\[
	\left [
\begin{array}{ccccc}
{0}&{1}&{1}&{0}&{0}\cr
{1}&{1}&{0}&{-1}&{0}\cr
{\frac{5}{2}}&{-1}&{0}&{0}&{1}
\end{array}
\right ]
\left [
\begin{array}{c}
{x_1}\cr
{x_2}\cr
{x_3}\cr
{x_4}\cr
{x_5}
\end{array}
\right ]
=
\left [
\begin{array}{c}
{5}\cr
{3}\cr
{10}
\end{array}
\right ],
\]
without worrying about the cost function for the moment.

	I had the students write down the vertices:
\[
	\left [
\begin{array}{ccccc}
{6}&{5}&{0}&{8}&{0}\cr
{4}&{0}&{5}&{1}&{0}\cr
{3}&{0}&{5}&{0}&{\frac{5}{2}}\cr
{0}&{3}&{2}&{0}&{13}\cr
{0}&{5}&{0}&{2}&{15}
\end{array}
\right ],
\]
and we discussed the pattern of zeros, noting that there are $n-m$ zeros in
each vertex, and that as we pass from vertex to vertex there is a single
exchange of variables: one that was non-zero becomes zero, and vice versa.

	We then assumed the same cost function, which allowed us to attach
costs to the vertices. Thus the students could plainly see the solution, and
that one could do exactly what Dantzig had proposed: from a given vertex walk
``downhill'' until arriving at a minimum.

	In general a linear program poses two problems: problem 1 (aka ``Phase
I''), ``how do we find a starting vertex?''; and problem 2 (``Phase II''),
``how do we move from vertex to vertex? It turns out that to solve Phase I we
use phase II, which seems contradictory but isn't. We modify our problem so
that it becomes a linear program with a known starting vertex and a solution
which is, in fact, a starting vertex for the original program. Too cool!

	I told the students a story at this point, that I'd heard from
somewhere: if you put a chicken in a three-sided pen and some food on the
other side of the fence on the side opposite the open side, then the chicken
will spend the rest of its life banging its head against the wall trying to get
at the food; whereas a dog will take a look at the food, take a look at the
fence, then zip around the back. Take home message: don't be bothered by the
fact that you have to solve Phase II before you can solve Phase I (mentioned
George Polya's ``How to Solve It'').

\section{Algorithm: Phase II}

	We assume that we already have a feasible vertex, $x$, which may or may
not be optimal. We split the problem into two parts, corresponding to the zero
and non-zero components of $x$:
\[
	\u{x}=\left [
\begin{array}{c}
{\u{x}_B}\cr
{\u{x}_N}
\end{array}
\right ]
\]
\[
	\u{c}=\left [
\begin{array}{c}
{\u{c}_B}\cr
{\u{c}_N}
\end{array}
\right ]
\]
\[
	\u{A}=\left [
\begin{array}{cc}
{A_B}&{A_N}
\end{array}
\right ]
\]

Note that, in the non-degenerate case, $A$ is square. If not, that is if there
are more than $n-m$ zeros in the vertex $\u{x}$, then we borrow some zero
components from $\u{x}_N$ to complete it until its square. Assuming that it is
also invertible, we can write
\[
	A_B\u{x}_B = \u{b} \implies \u{x}_B = A^{-1}_B\u{b}.
\]
The cost in this case is
\[
	\u{c}^T\u{x}=\u{c_B}^T\u{x_B}=\u{c_B}^TA^{-1}_B\u{b}.
\]

	We're now going to perturb $\u{x}$ a little, to see if we can reduce
the cost while still satisfying the constraints:
\[
	\u{A}=\left [
\begin{array}{cc}
{A_B}&{A_N}
\end{array}
\right ]
	\u{x}=\left [
\begin{array}{c}
{\u{x}_B + \u{\delta}}\cr
{\u{x}_N}
\end{array}
\right ]
= \u{b}
\]
$\u{x}_N$ is no longer constrained to be the vector 0.

	We can solve for the relationship between $\u{\delta}$ and $\u{x}_N$,
to get
\[
	\u{\delta} = -A_B^{-1}A_N\u{x}_N,
\]
and substituting this into the cost we find that
\[
	\u{c}^T\u{x}=\u{c_B}^T\u{x_B}
			+ (\u{c_N}^T - \u{c_B}^TA_B^{-1}A_N)\u{x}_N.
\]
Set $\u{r}=(\u{c_N}^T - \u{c_B}^TA_B^{-1}A_N)^T.$ Now if
\[
	\u{r} \ge \u{0},
\]
we're at the solution, as we cannot change $\u{x}_N$ so that it remains
positive and reduce the cost. In other words, any way that we move off of our
present vertex into the interior of the feasible set raises the cost. In this
case, we're done and can return the vector $\u{x}$.

	Otherwise, we select the variable corresponding to the most negative
value of $\u{r}$ (call it the \u{entering} variable, let's say $x_i$). Now we
need to find a \u{entering} variable, to keep the number of zeros at $n-m$.

Let $\u{v}$ be the $i^{th}$ column of the matrix $A_B^{-1}A_N$. Now we suppose
that $\u{x}_N$ is zero everywhere except in the $i^{th}$ component, where we
slowly raise its value by $\alpha$. $\u{\delta}$ in this case is given by
\[
	\u{\delta} = -\alpha \u{v},
\]
so
\[
	\u{x}_B = A_B^{-1}b - \alpha \u{v}.
\]

	As $\alpha$ rises from 0, one component (at least) of the vector
$A_B^{-1}b - \alpha \u{v}$ will finally equal zero. The \u{first} component to
do so corresponds to the variable that will be leaving.

	The leaving variable is the variable corresponding to the index of
the minimum of the ratio of the $j^{th}$ component of $A_B^{-1}b$ to the
$j^{th}$ component of $\u{v}$, for positive components of $\u{v}$.

	Now our new vertex vector is given by setting
\[
	\u{x}_B = A_B^{-1}b - \alpha \u{v}
\]
and
\[
	\u{x}_N = \alpha \u{e}_i.
\]

	Now loop to your heart's contentment.

\section{Algorithm: Phase I}

	As already mentioned, the trick to getting a starting vertex is to
rewrite the given program
\begin{itemize}
\item
by adding an additional $m$ variables, one for each row in the matrix $A$, and
\item
by rewriting the matrix $A$ to include these variables: $A$ becomes the
original $A$ augmented by the columns of a diagonal matrix filled with the
elements $signum(\u{b})$; and
\item
by creating a new cost vector which is a vector of $n$ zeros followed by $m$
ones.
\end{itemize}

	This problem has one vertex which is evident: the vector of $n$ zeros
combined with $|\u{b}|$, so the phase I problem doesn't exist. And the solution
of the problem is also clear, at least in part: the $m$ variables that we just
introduced must all be zero, to give a minimum cost of 0. That leaves the rest
of the $n$ variables to meet the constraints, which are exactly our original
$A\u{x}=\u{b}$.

	So we solve one phase II problem with simple known vertex, to get as a
solution a starting vertex so that we can begin at phase II for the original
problem. It's a miracle!

	At the end of explaining all of this to my students I reflected on the
discovery of this bit of knowledge. Imagine the state of excitement of the
person who discovered this! To possess such exciting knowledge, and to be alone
in having the solution. I told the story of the guy who first figured out that
the stars are powered by nuclear fusion, how he was out walking with his
sweetheart, pointed up to the stars, and said ``You know honey, at this moment
I'm the only guy in the whole world who knows why those stars twinkle.''

\bibliography{/home/ael/diss/paper/diss}
\bibliographystyle{abbrv}
\end{document}
|#
