#|
Susan is working on a boundary detection routine, and I wanted to have a look
at some analytic approaches (rather than 1000 monte carlo location
realizations, then funny triangles fit by planes, etc...).
|#

#|
This example only works for positive slope:
|#

(defun in(a b x &key (c1 #'>) (c2 #'<))
  "

	in(a b x &key (c1 #'>) (c2 #'<))

	Is x in (a,b)?

	Will check whether x is strictly larger than a and strictly smaller
than b. To check whether

	x is in (a,b]: (in a b x :c2 #'<=)
	x is in [a,b): (in a b x :c1 #'>=)
	x is in [a,b]: (in a b x :c1 #'>= :c2 #'<=)

;; Examples:
 (in 0 1 .5)
 (in 0 1 -.5)
 (in 0 1 2)

 (in 0 1 0 :c1 #'>=)
 (in 0 1 1 :c2 #'<=)
"
  (if (and (funcall c1 x a) (funcall c2 x b)) t)
  )

(let ()
  (setq
   a 1
   b 2
   c 2.5
   y1 -2
   y2 -4
   mc -3
   )
  (plot
   (lambda(y)
     (let* (
	    (x (if (> y b) (- b (- y b)) y))
	    (ux (+ x (/ (- y2 y1) mc)))
	    )
       (if (and (< y c) (> x a))
	   (* 
	    (heaviside (- ux b))
	    (- (min ux c) b))
	 0)
       )
     )
   a (+ b (- b a))
   )
  )

(let* (
       (xs (iseq 10))
       (ys '(4 5 2 2 2 3 10 3 1))
       (mc -3)
       (a (min xs))
       (b (max xs))
       )
  (plot
   (lambda(y)
     (sum
      (mapcar
       (lambda(a b c y1 y2)
	 (let* (
		(mc (if (> y2 y1) (abs mc) (- (abs mc))))
		(x (if (> y b) (- b (- y b)) y))
		(ux (+ x (/ (- y2 y1) mc)))
		)
	   (if (and (< y c) (> x a))
	       (* 
		(heaviside (- ux b))
		(- (min ux c) b))
	     0)
	   )
	 )
       (butlast (butlast xs))
       (butlast (rest xs))
       (rest (rest xs))
       (butlast ys) (rest ys)
       )
      )
     )
   a b
   )
  (send my-plot :add-plot-item (list 'lines
				     (combine
				      (transpose
				       (list (butlast xs) (rest xs))))
				     (combine
				      (transpose
				       (list ys ys)))
				     'red))
  (send my-plot :adjust-to-data :draw t)
  (send my-plot :range 0 a b)
  )
