#| - Sat Apr 18 18:34:23 1998

	I used Kaplan and Glass, Understanding Non-Linear Dynamics, as a text
in a math modelling course at Ripon College, in 1995-96. This file was created
a couple of years later, as a summary of our work in that class.

	All was originally in subdirectories, as follows:

Chapter_2:
boole_function.lsp
logic.lsp
netclock.lsp
networks.lsp
networktest.lsp
trial.lsp

Chapter_3:
fractal.lsp
recursive.lsp
simpletree.lsp

Chapter_4:
4.7.lsp
blowfly.lsp
bump.lsp
delay.lsp
delay_examples.lsp
euler-delay.lsp
euler-nd.lsp
euler.lsp
euler1.lsp
euler2.lsp
euler2d.lsp
garbage.lsp
logtrans.lsp

Chapter_5:
5.lsp
euler-nd.lsp
limpets.lsp
lorenz.lsp
lorenz2.lsp
lotka-volterra.lsp
one-d.lsp
orbitals.lsp
rk-4.lsp
rk-4.test.lsp
rossler.lsp
two-d.lsp
van-der-pol.lsp

Chapter_6:
drunk.lsp
fdl.lsp
fdl2.lsp
models.lsp
power-spectrum.lsp
project4.lsp
time-series.lsp

	I'm now going to prune the stuff, and leave out the chaff!
|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Chapter 1
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Nothing so far!

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;		Chapter 2: Boolean Networks and Cellular Automata
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun boole_function (fname b1 b2)
  (case fname
	('tautology 1)
	('contradiction 0)
	('and (if-else (= (* b1 b2) 1) 1 0))
	('nand (if-else (= (* b1 b2) 1) 0 1))
	('transfer1 (if-else (= b1 1) 1 0))
	('ntransfer1 (if-else (= b1 1) 0 1))
	('transfer2 (if-else (= b2 1) 1 0))
	('ntransfer2 (if-else (= b2 1) 0 1))
	('inhibition1 (if-else (and (= b1 1) (= b2 0)) 1 0))
	('ninhibition1 (if-else (and (= b1 1) (= b2 0)) 0 1))
	('inhibition2 (if-else (and (= b2 1) (= b1 0)) 1 0))
	('ninhibition2 (if-else (and (= b2 1) (= b1 0)) 0 1))
	('or (logior b1 b2))
	('nor (if-else (= (logior b1 b2) 0) 1 0))
	('xor (logxor b1 b2))
	('xnor (if-else (= (logxor b1 b2) 0) 1 0))
	(t -666)
	)
  )
;; ael-mark
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun boole_test (fname) 
  ;;
  ;; Within the following double quotes is the documentation
  ;; for the function boole_test.
  ;;
  "
This shows the result of a given function call to boole_function,
for the four legitimate paired inputs:
00 01 10 11.
;; Examples:
 (boole_test 'xor)
 (boole_test 'and)
"
  (mapcar (lambda(x y) (boole_function fname x y)) '(0 0 1 1) '(0 1 0 1))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Table 2.1, p. 67
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun TABLE2.1()
  "
Documentation: Demonstrates that the Boolean
binary functions are all working properly.
;; Examples:
 (def logicals
     (list
      'contradiction
      'and
      'inhibition1
      'transfer1
      'inhibition2
      'transfer2
      'xor
      'or
      'nor
      'xnor
      'ntransfer2
      'ninhibition2
      'ntransfer1
      'ninhibition1
      'nand
      'tautology
      )
     )
 (pprint (table2.1))
" 
  (mapcar #'boole_test logicals)
  )
(try-out table2.1)

#| - Sun Apr 19 21:32:40 1998
	These are some clocks, based on an oscillator (the last node in every
case) and a set of xor's and and's. The idea is basically
oscillator -> xor
"and" of those, with an xor of that "and" and itself;
"and" of all those, with an xor of that "and" and itself;
"and" of all those, with an xor of that "and" and itself;
iterate!

It would be better to work this sequentially: one network based on the
output of each additional network. I.e., make this recursive!

The idea is that, if everything lines up, add another to the next level.

		Andy Long, for modelling class, fall of 1995.
|#
(defun network (cell-types cell-states cell-inputs iterations)
  "
Documentation:
    Use this command to make a network of interconnected
boolean cells, each with an arbitrary number of input nodes
and one output.

Specify:
           cell-types cell-states cell-inputs iterations
where
      cell-types: a list of lists of binary numbers, giving the cell
            responses to all possible inputs (given as binary
            numbers starting from 0).
      cell-states: the initial state of the nodes.
      cell-inputs: the list of lists of which cells plug into
            each individual cell.
      iterations: number of iterations to carry out.

For example, for three cells, you might have try

;; Examples:
 (network
  '((0 1 1 0) (0 1) (0 0 1 1 1 1 0 0))
  '(1 0 0)
  '((1 2) (2) (0 1 2))
  6
  )
;; (Draw the network!)
"
  (let
      (
       (num-nodes (length cell-states))
       (states (copy-list cell-states))
       (lengths (copy-list cell-states))
       newstates binum numimps j_input j_input_jj jnodein
       )
    ;; loop to set lengths of cells
    (dotimes
     (j num-nodes)
     (setf (select lengths j) (length (select cell-inputs j)))
     )
    ;; iteration loop
    (dotimes
     (i iterations)
     (setq newstates (copy-list states))
     (dotimes
      (j num-nodes)
      (setq
       jnodein (select cell-inputs j)
       binum 0 ;; binum will tell which index to choose from cell-types for j
       numimps (select lengths j) ;; numimps is the number of inputs
       ;; j_input is the list of input states for cell j:
       j_input (select states (select cell-inputs j))
       )
      (dotimes
       (jj numimps)
       (setq j_input_jj (select j_input jj)
	     binum (+ binum (* j_input_jj (^ 2 (- numimps (+ jj 1))))))
       )
      (setf (select newstates j) (select (select cell-types j) binum))
      )
     (princ states)
     (terpri)
     (setq states (copy-list newstates))
     )                                   
    )
  (terpri)
  )

(network
 '((1 0))
 '(0)
 '((0))
 2
 )

(network
 '((0 1 1 0) (1 0))
 '(0 0)
 '((0 1) (1))
 4
 )

(network
 '((0 0 0 1 1 1 1 0) (0 1 1 0) (1 0))
 '(0 0 0)
 '((0 1 2) (1 2) (2))
 8
 )

(network
 '((0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 0) (0 0 0 1 1 1 1 0) (0 1 1 0) (1 0))
 '(0 0 0 0)
 '((0 1 2 3) (1 2 3) (2 3) (3))
 16
 )

(network
 '((0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 0) 
   (0 0 0 0 0 0 0 1 1 1 1 1 1 1 1 0) (0 0 0 1 1 1 1 0) (0 1 1 0) (1 0))
 '(0 0 0 0 0)
 '((0 1 2 3 4) (1 2 3 4) (2 3 4) (3 4) (4))
 32
 )
;; Note: second hand, 2^1 second hand, 2^2 second hand, etc.!
;; 2*(1), 2*(1+2), 2*(1+2+2^2), ..., previous + 2^(n+1)
;; (n measured from 0).;;;
;;;
(princ "

 This is the nor nor nor nor network of trial, using the new network function:

")
(network
 '((1 0 0 0) (1 0 0 0) (1 0 0 0) (1 0 0 0))
 '(1 0 0 0)
;;            a          b          c          d        
;;           b d        c d        a b        a c
 '((1 3) (2 3) (0 1) (0 2))
 16
 )

(princ "

 This is the xor xnor nor ninhibition1 network of trial, using the new network
 function:

")
(network
 '((0 1 1 0) (1 0 0 1) (1 0 0 0) (1 1 0 1))
 '(1 0 0 0)
;;            w          x          y          z
;;           y x        w z        x w        w y
 '((2 1) (0 3) (1 0) (0 2))
 16
 )

(princ "

 This is a random network, with various connections and initial conditions:

(network
 '((1 1 1 1) (1 0) (1 1 1 1 1 1 1 1))
 '(0 1 0)
 '((1 2) (2) (0 1 2))
 6
 )

")
(network
 '((1 1 1 1) (1 0) (1 1 1 1 1 1 1 1))
 '(0 1 0)
 '((1 2) (2) (0 1 2))
 6
 )

(princ "

 Here are a couple of others:

")
(network
 '((0 1 1 0) (0 1) (0 0 1 1 1 1 0 0))
 '(1 0 0)
 '((1 2) (2) (0 1 2))
 6
 )

(network
 '((0 1 1 0) (0 1) (1 1 1 1 1 1 1 1))
 '(0 1 0)
 '((1 2) (2) (0 1 2))
 6
 )

;; passes the Jentest
(network
 '((0 1 0 1 1 0 1 0) (1 0 0 1 0 1 1 0) (0 1 0 1 1 0 1 0) (1 0 0 1 0 1 1 0))
 '(1 0 1 1)
 '((1 2 3) (0 2 3) (0 1 3) (0 1 2))
 6
 )
(network
 '((0 1 0 1 1 0 1 0) (1 0 0 1 0 1 1 0) (0 1 0 1 1 0 1 0) (1 0 0 1 0 1 1 0))
 '(0 0 0 0)
 '((1 2 3) (0 2 3) (0 1 3) (0 1 2))
 6
 )

;; passes the Daventest
(network
 '((1 0) (1 1 0 0 1 0 0 1) (1 0 1 0) (0 1 0 1))
 '(0 0 0 0)
 '((0) (0 2 3) (1 3) (1 2))
 6
 )
(network
 '((1 0) (1 1 0 0 1 0 0 1) (1 0 1 0) (0 1 0 1))
 '(1 1 1 1)
 '((0) (0 2 3) (1 3) (1 2))
 6
 )
(network
 '((1 0) (1 1 0 0 1 0 0 1) (1 0 1 0) (0 1 0 1))
 '(0 0 0 1)
 '((0) (0 2 3) (1 3) (1 2))
 6
 )
(network
 '((1 0) (1 1 0 0 1 0 0 1) (1 0 1 0) (0 1 0 1))
 '(0 0 1 0)
 '((0) (0 2 3) (1 3) (1 2))
 6
 )
(network
 '((1 0) (1 1 0 0 1 0 0 1) (1 0 1 0) (0 1 0 1))
 '(0 1 0 0)
 '((0) (0 2 3) (1 3) (1 2))
 6
 )
(network
 '((1 0) (1 1 0 0 1 0 0 1) (1 0 1 0) (0 1 0 1))
 '(1 0 0 0)
 '((0) (0 2 3) (1 3) (1 2))
 6
 )

(def
 netclock4
 '((0 0 0 0)
   (0 0 0 1)
   (0 0 1 0)
   (0 0 1 1)
   (0 1 0 0)
   (0 1 0 1)
   (0 1 1 0)
   (0 1 1 1)
   (1 0 0 0)
   (1 0 0 1)
   (1 0 1 0)
   (1 0 1 1)
   (1 1 0 0)
   (1 1 0 1)
   (1 1 1 0)
   (1 1 1 1)
   ))

(def
 netclock4
 '((0 0 0 1)
   (0 0 1 0)
   (0 1 0 0)
   (0 1 1 0)
   (1 0 0 0)
   (1 0 1 0)
   (1 1 0 1)
   (1 1 1 1)
   ))

(defun daventest (list1)
  (dotimes (i (length list1))
	   (network
	    '((1 0) (1 1 0 0 1 0 0 1) (1 0 1 0) (0 1 0 1))
	    (select list1 i)
	    '((0) (0 2 3) (1 3) (1 2))
	    6
	    )
	   )
  )
(daventest netclock4)
;; five transients to one cycle, three transients to the other!

(princ 
"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This is for the problem with the network on page 72.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

This is the original, and it has a permutation problem:
            a          b          c          d        
           b d        c d        a b        a c

")

(network
 '((1 0 0 0) (1 0 0 0) (1 0 0 0) (1 0 0 0))
 ;; '(nor nor nor nor)
 '(1 0 0 0)
;;            a          b          c          d        
;;           b d        c d        a b        a c
 '((1 3) (2 3) (0 1) (0 2))
 16
 )

(princ 
"

This is corrected, but goes backwards!
            a          b          c          d        
           c d        d a        a b        b c

")
(network
 '((1 0 0 0) (1 0 0 0) (1 0 0 0) (1 0 0 0))
 ;; '(nor nor nor nor)
 '(1 0 0 0)
;;            a          b          c          d        
;;           c d        d a        a b        b c
 '((2 3) (3 0) (0 1) (1 2))
 16
 )

(princ 
"

This is it! Take the following....
            a          b          c          d        
           b c        c d        d a        a b

")
(network
 '((1 0 0 0) (1 0 0 0) (1 0 0 0) (1 0 0 0))
 ;; '(nor nor nor nor)
 '(1 0 0 0)
;;            a          b          c          d        
;;           b c        c d        d a        a b
 '((1 2) (2 3) (3 0) (0 1))
 16
 )
;; (Corresponds to merely changing the c and d in figure on page 72).

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This is for problem 2.8, page 100:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; w - xor, 
;; x - xnor, 
;; y - nor,
;; z - ninhibition1

;; Here's an example:

(network
 '((0 1 1 0) (1 0 0 1) (1 0 0 0) (1 1 0 1))
 ;; '(xor xnor nor ninhibition1)
 '(1 0 0 0)
;;            w          x          y          z
;;           y x        w z        x w        w y
 '((2 1) (0 3) (1 0) (0 2))
 16
 )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for ALL_COMBINATIONS_OF_STATES:
;;
(defun ALL_COMBINATIONS_OF_STATES()
"
Documentation:
" 
       ;; Below is the actual definition of ALL_COMBINATIONS_OF_STATES:
   (dotimes (j 2)
   (dotimes (jj 2)
   (dotimes (jjj 2)
   (dotimes (jjjj 2)
	    (def results
	    (network
	     '((0 1 1 0) (1 0 0 1) (1 0 0 0) (1 1 0 1))
	     ;; '(xor xnor nor ninhibition1)
	     (list j jj jjj jjjj)
	     ;;            w          x          y          z
	     ;;           y x        w z        x w        w y
	     '((2 1) (0 3) (1 0) (0 2))
	     16
	     )
	    )
	    )
   (princ results)
   (terpri)
   )
   )
   )
)

(princ 
"
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This is for problem 2.8, page 100:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; w - xor, 
;; x - xnor, 
;; y - nor,
;; z - ninhibition1

;; Here's an example:

(network
 '((0 1 1 0) (1 0 0 1) (1 0 0 0) (1 1 0 1))
 ;; '(xor xnor nor ninhibition1)
 '(1 0 0 0)
;;            w          x          y          z
;;           y x        w z        x w        w y
 '((2 1) (0 3) (1 0) (0 2))
 16
 )

")
(all_combinations_of_states)

(princ "

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; This is for problem 2.5, page 98:
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(network
 (mapcar (lambda(x) '(0 1 1 0))
	 '(xor xor xor xor xor xor xor xor xor xor))
 '(1 0 0 0 1 0 0 0 1 0)
 '((9 1) (0 2) (1 3) (2 4) (3 5) (4 6) (5 7) (6 8) (7 9) (8 0))
 60
 )

")

(network
 (mapcar (lambda(x) '(0 1 1 0))
	 '(xor xor xor xor xor xor xor xor xor xor))
 ;; '(xor xor xor xor xor xor xor xor xor xor)
 '(1 0 0 0 1 0 0 0 1 0)
 '((9 1) (0 2) (1 3) (2 4) (3 5) (4 6) (5 7) (6 8) (7 9) (8 0))
 60
 )

;; Sort of p. 143ish...

(defun pattern (n length lenmin divisions x y plot_name &optional (type 'dot))
  "
Documentation:
"
  (send plot_name :add-points (list x) (list y) :symbol type)
  (dotimes (i n)
	   (def arg (* (* 2 pi) (/ (float i) n)))
	   (def rotate (* length (exp (* #c(0 1) arg))))
	   (def xreal (+ x (realpart rotate)))
	   (def ximag (+ y (imagpart rotate)))
	   (send plot_name :add-points (list xreal) (list ximag) :symbol type)
	   (def newlength (/ length divisions))
	   (if (> length lenmin)
	       (pattern
		n newlength lenmin divisions xreal ximag plot_name type)
	     )
	   )
  (def newlength (/ length divisions))
  (if (> length lenmin)
      (pattern n newlength lenmin divisions x y plot_name type))
  ) 
(plot-points (rand.u 10) (rand.u 10) :color 'black :symbol 'dot)
(def my-plot (plot-points '(1 0) '(0 1) :color 'black :symbol 'dot))
(pattern 3 .3 .025 3 .5 .5 my-plot 'dot4)

(pattern 3 .4 .025 1 .5 .5
	 (plot-points '(1 0) '(0 1) :color 'black :symbol 'dot)
	 'dot4)

(def my-plot (plot-points '(1 0) '(0 1) :color 'black :symbol 'dot))
(pattern 5 .5 .1 4 .5 .5 my-plot 'diamond)

(def my-plot (plot-points '(1 0) '(0 1) :color 'black :symbol 'dot))
(pattern 10 .3 .05 3 .5 .5 my-plot 'disk)

;; A simple recursive function:
(defun this (x)
  (let ((newx (/ x 2)))
    (print newx)
    (if (> newx 1) (this newx))))
(this 10)

(defun simpletree (x y angle theta length twigsize plot_name)
  "
Documentation:
  Implements simple tree of modelling text, page 107: Understanding
  Nonlinear Dynamics, Kaplan and Glass.
"
  (let* (
	 (topx (+ x (* length (cos angle))))
	 (topy (+ y (* length (sin angle))))
	 (point1 (list x topx))
	 (point2 (list y topy))
	 newangle newtheta newlength
	 )
    (send plot_name :add-lines (list point1 point2))
    (if (> length twigsize)
	(dotimes (i 1)
	 (setq newangle (+ angle theta))
	 (setq newtheta (/ theta 2.0))
	 (setq newlength (/ length 2.0))
	 (simpletree topx topy newangle newtheta newlength twigsize plot_name)
	 (setq newangle (- angle theta))
	 (setq newtheta (/ theta 2.0))
	 (setq newlength (/ length 2.0))
	 (simpletree topx topy newangle newtheta newlength twigsize plot_name)
	 )
      )
    )
  )

(def my-plot (plot-points '(0 1) '(0 1)))
(def angle (/ pi 2))
(def theta (/ pi 16))
(def length .5)
(def twigsize .01)
(simpletree .5 0 angle theta length twigsize my-plot)

(def my-plot (plot-points '(0 1) '(0 1)))
(def angle (/ pi 2))
(def theta (/ pi 4))
(def length .4)
(def twigsize .01)
(simpletree .5 0 angle theta length twigsize my-plot)
(def angle (/ pi 2))
(def theta (/ pi 16))
(def length .5)
(def twigsize .01)
(simpletree .5 0 angle theta length twigsize my-plot)

					;
					; 4.7 page 194
					;
; g(time) = 
; (pow (diff (pow gkin .25) (prod (diff (pow gkin .25) (pow gknot .25))
; (exp (quo (- time) tao)))) 4)


(def tao .75)
(def gkinf .09)
(def gknot 7.06)
(defun g (time)
  (^ (- (^ gkinf .25)
	(* (- (^ gkinf .25)
	      (^ gknot .25))
	   (exp (/ (- time) tao)))) 4))

;;				Blowflies!

;; Putting together some blow-fly population models, to replicate
;; p. 186, of "Understanding Non-Linear Dynamics".  ael, Fall, 1995

;; delay contains the definitions of the euler-delay-plot functions.

;; From the exercises at the end of chapter 4: one-d diffeqs
;; Euler's method for delay equations
					;general first order de's
;; blowfly will give a continuous look at the euler equations, as
;; time carries on:
(defun blowfly (n)
  ;; Trying to put together some blow-fly populations (p. 186)
  (cond
   ((= n 1)
    (let* (
	   (num 500)(tmin 0)(tmax 200)
	   (d .15)(alpha 10)(tao 10)
	   (ynot
	    (*
	     10
	     (+
	      2
	      (* .5 (normal-rand (ceiling (* num (/  tao (- tmax tmin))))))))
	    )
	   (my-plot (plot-points (iseq 1 (length ynot)) ynot))
	   )
      (euler-delay-plot-continue
       #'(lambda(x time xd) (- (* xd (exp (- (^ (/ xd alpha) 2)))) (* d x)))
       ynot tmin tmax tao my-plot
       :title "Periodic Blow-flies")
      ))
   ((= n 2)
    (let* (
	   ;; we have periodic blowflies!
	   ;; Trying to put together some chaotic blow-fly populations now
	   ;; ....
	   (num 500)(tmin 0)(tmax 200)
	   (d .15)(alpha 10)(tao 20)
	   (ynot
	    (* 10 (+ 2 (* .5 (normal-rand (ceiling (* num (/  tao (- tmax tmin))))))))
	    )
	   (my-plot (plot-points (iseq 1 (length ynot)) ynot))
	   )
      (euler-delay-plot-continue
       #'(lambda(x time xd) (- (* xd (exp (- (^ (/ xd alpha) 2)))) (* d x)))
       ynot tmin tmax tao my-plot
       :title "Sorta Periodic Blow-flies")
      
      ;; wilder, yet still periodic?
      ))
   ((= n 3)
    (let* (
	   (num 500)(tmin 0)(tmax 200)
	   (d .15)(alpha 10)(tao 30)
	   (ynot
	    (* 10 (+ 2 (* .5 (normal-rand (ceiling (* num (/  tao (- tmax tmin))))))))
	    )
	   (my-plot (plot-points (iseq 1 (length ynot)) ynot))
	   )
      (euler-delay-plot-continue
       #'(lambda(x time xd) (- (* xd (exp (- (^ (/ xd alpha) 2)))) (* d x)))
       
       ynot tmin tmax tao my-plot :title "Blow-flies?")
      
      ;; chaotic?
      )
    )
   )
  )

;; blowfly-static will give a static look at the euler solutions.
(defun blowfly-static (n)
  ;; Trying to put together some blow-fly populations (p. 186)
  (cond
   ((= n 1)
    (let* (
	   (num 500)(tmin 0)(tmax 200)
	   (d .15)(alpha 10)(tao 10)
	   (ynot
	    (* 10 (+ 2 (* .5 (normal-rand (ceiling (* num (/  tao (- tmax tmin))))))))
	    )
	   )
      (euler-delay-plot
       #'(lambda(x time xd) (- (* xd (exp (- (^ (/ xd alpha) 2)))) (* d x)))
       ynot tmin tmax tao
       :title "Periodic Blow-flies")
      ))
   ((= n 2)
    (let* (
	   
	   ;; we have periodic blowflies!
	   
	   ;; Trying to put together some chaotic blow-fly populations now ....
	   
	   (num 500)(tmin 0)(tmax 200)
	   (d .15)(alpha 10)(tao 20)
	   (ynot
	    (* 10 (+ 2 (* .5 (normal-rand (ceiling (* num (/  tao (- tmax tmin))))))))
	    )
	   )
      (euler-delay-plot
       #'(lambda(x time xd) (- (* xd (exp (- (^ (/ xd alpha) 2)))) (* d x)))
       ynot tmin tmax tao
       :title "Sorta Periodic Blow-flies")
      
      ;; wilder, yet still periodic?
      ))
   ((= n 3)
    (let* (
	   
	   (num 500)(tmin 0)(tmax 200)
	   (d .15)(alpha 10)(tao 30)
	   (ynot
	    (* 10 (+ 2 (* .5 (normal-rand (ceiling (* num (/  tao (- tmax tmin))))))))
	    )
	   )
      
      (euler-delay-plot
       #'(lambda(x time xd) (- (* xd (exp (- (^ (/ xd alpha) 2)))) (* d x)))
       ynot tmin tmax tao :title "Chaotic Blow-flies?")
      ;; chaotic?
      )
    )
   )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for HEAVISIDE:
;;
(defun HEAVISIDE(x)
  "
;; Documentation:

(plot-function #'heaviside -2 2)

;; Examples:

" 
  ;; Below is the actual definition of HEAVISIDE:
  (if (< x 0) 0 1)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for BUMP:
;;
(defun BUMP(x a b)
  "
;; Documentation:

;; Examples:

(bump 1 2 4)
(bump 3 2 4)
(bump 5 2 4)

" 
  (- (heaviside (- x a)) (heaviside (- x b)))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for OLD-HOUSE-TEMPERATURE:
;;
(defun OLD-HOUSE-TEMPERATURE()
  "
;; Documentation:

 The temperature distribution in an old house is a function of the
furnace! And as we all know, the furnace is on, then off. We model
the behavior of the heat distribution with a linear DE

			T' = -aT + H(-sin(pi*x))

The second part is really a forcing function (heaviside of a sine)
that is a periodic bump function.

We compare the true solution with the solution obtained using
Euler's method.
;; Examples:
 (OLD-HOUSE-TEMPERATURE)
" 
			; The differential equation:
  ;; some peripherals:
  (let* (
	 (t_hi 100)
	 (t_lo 20)
	 (tmin 0)
	 (tmax 4)
	 (alpha (log (/ t_hi t_lo)))
	 (kappa
	  (/ (* t_hi (- 1 (exp (* -2 alpha))) alpha) (- 1 (exp (- alpha)))))
	 (my-plot
	  (euler-delay-plot 
	   (lambda(x time xd)
	     (+ (* (- alpha) x) (* kappa (heaviside (- (sin (* pi time)))))))
	   (list t_hi) tmin tmax .01
	   :title "Old House Temperatures"))
	 )
    (flet ((BUMP-BIT
	    (x i)
	    ;; A useful function in the scheme of things: solution of
	    ;; one bump.
	    (let (
		  (2i (* 2 i))
		  (2i-1 (1- (* 2 i)))
		  )
	      (if (< x 2i-1) 0
		(* (/ kappa alpha)
		   (+
		    (* (heaviside (- x 2i-1))
		       (- 1 (exp (* alpha (- 2i-1 x)))))
		    (* (heaviside (- x 2i))
		       (- (exp (* alpha (- 2i x))) 1)))))))
	   )
					; The true solution
	  (send my-plot :use-color 't)
	  (send my-plot :draw-color 'blue)
	  (send my-plot :add-plot-item 'function 
		(lambda(x)
		  (let* (
			 (enough (ceiling (/ x 2)))
			 (value (* t_hi (exp (* (- alpha) x))))
			 )
		    (dotimes
		     (i enough)
		     (setq value (+ value (bump-bit x (1+ i)))))
		    value
		    )
		  )
		'black t)
	  )
    (send my-plot :reverse-colors)
    (let* (
	   (graph-vals (send my-plot :view-rect))
	   (LEFT (first graph-vals))
	   (TOP (second graph-vals))
	   (WIDTH (third graph-vals))
	   (HEIGHT (fourth graph-vals))
	   )
      (send my-plot
	    :draw-text "True and Euler-Computed"
	    (+ left (/ width 2))
	    (+ top (/ width 10))
	    1 0 )
      )
    )
  )
(OLD-HOUSE-TEMPERATURE)
;;
;; This is just a quick example that I thought might be interesting:
;;
(def yknot 1)
(def num 200)
(def tao 2)
(def ynot 1)
(def tmin 0)
(def tmax 16)
(def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))

;; This is not a delay equation: it should settle down to sinusoidal variation:
(def alpha -.5)
(defun f (x time xd)
  (+ (* alpha x) (sin (* 2 time)))
  )

(euler-delay-plot #'f ynot tmin tmax tao :title "No Delay")

(defun f (x time xd)
  (+ (* alpha xd) (sin (* 2 time)))
  )

(euler-delay-plot #'f ynot tmin tmax tao :title "Delay")

;;
;; This example is suggested by the authors, in computer project 3:
;;
(def num 400)
(def tao 2)
(def tmin 0)
(def tmax 32)

(defun f (x time xd) (- (/ (* 2 xd) (+ 1 (^ xd alpha))) x))

(def alpha .25)

(def yknot 1.3)
(def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
(euler-delay-plot #'f ynot tmin tmax tao :title "Repeated high: .25")

(def yknot .3)
(def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
(euler-delay-plot #'f ynot tmin tmax tao :title "Repeated low: .25")

(def ynot (uniform-rand (ceiling (* num (/  tao (- tmax tmin))))))
(euler-delay-plot #'f ynot tmin tmax tao :title "Uniform Rand: .25")

(def alpha 1)

(def yknot .3)
(def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
(euler-delay-plot #'f ynot tmin tmax tao :title "Repeated low: 1")

(def yknot 1.3)
(def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
(euler-delay-plot #'f ynot tmin tmax tao :title "Repeated high: 1")

(def ynot (normal-rand (ceiling (* num (/  tao (- tmax tmin))))))
(euler-delay-plot #'f ynot tmin tmax tao :title "Normal: 1")

(def ynot (uniform-rand (ceiling (* num (/  tao (- tmax tmin))))))
(euler-delay-plot #'f ynot tmin tmax tao :title "Uniform: 1")

(def alpha 2)

(def yknot 1.3)
(def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
(euler-delay-plot #'f ynot tmin tmax tao :title "Repeated high: 2")

(def ynot (normal-rand (ceiling (* num (/  tao (- tmax tmin))))))
(euler-delay-plot #'f ynot tmin tmax tao :title "Normal: 2")

(def ynot (uniform-rand (ceiling (* num (/  tao (- tmax tmin))))))
(euler-delay-plot #'f ynot tmin tmax tao :title "Uniform: 2")

(def alpha 4)

(def yknot 1.3)
(def ynot (repeat yknot (ceiling (* num (/  tao (- tmax tmin))))))
(euler-delay-plot #'f ynot tmin tmax tao :title "Repeated high: 4")

(def ynot (normal-rand (ceiling (* num (/  tao (- tmax tmin))))))
(euler-delay-plot #'f ynot tmin tmax tao :title "Normal: 4")

(def ynot (uniform-rand (ceiling (* num (/  tao (- tmax tmin))))))
(euler-delay-plot #'f ynot tmin tmax tao :title "Uniform: 4")


;; This is a pretty damned good fake of figure 4.25, page 185:

(def num 1000)
(def tao 2)
(def tmin 0)
(def tmax 60)
(def alpha 10)

(def ynot (rseq .2 .3 (ceiling (* num (/  tao (- tmax tmin))))))
(euler-delay-plot #'f ynot tmin tmax tao :title "Figure 4.25: model?")

(def ynot (rseq 1 .5 (ceiling (* num (/  tao (- tmax tmin))))))
(euler-delay-plot #'f ynot tmin tmax tao :title "Figure 4.25: data?")

(def ynot (1+ (* .5 (normal-rand (ceiling (* num (/  tao (- tmax tmin))))))))
(euler-delay-plot #'f ynot tmin tmax tao :title "Figure 4.25: normalesque")


(defun euler-delay (f ynot start end num k)
  "
;; Description: euler-delay

	This is a modified euler-nd, just for this problem. It's goal is to
take into account the delay for this particular case.
"
  (let (
	(hsize (/ (- end start) num))
	(timesteps (rseq start end num))
	(yvalues (list ynot))
	)
    (dotimes (i num)
	     ;; nextvals starts off as just the populations and the times: that
	     ;; is, we knock off the two sum values at the end:
	     (setq nextvals (list (butlast (butlast (select yvalues i)))))
	     ;; if we've hit the time to do the delay, then use the sum of the
	     ;; previous values: otherwise, go with zeros:
	     (if (>= i k) (list 
			   (setq oldvals (select yvalues (- i k)))
			   (setq light1 (sum (select oldvals '(0 1 2 3))))
			   (setq light2 (sum (select oldvals '(4 5 6 7))))
			   )
	       (setq oldvals (repeat 0 (length (first yvalues)))
		     light1 0
		     light2 0
		     )
	       )
	     (setq
	      nextvals (combine nextvals light1 light2)
	      yvalues (append
		       yvalues
		       (+ (last yvalues)
			  (* hsize (mapcar f (list nextvals))))))
	     )
    yvalues
    )
  )

(defun f (x y) (- x (^ x 2)))
(eulerplot #'f .1 0 8 100 :axes '(0 8 0 2))
(eulerplot #'f .1 0 8 100 :axes '(-2 10 -1 3))
;
; f(x,t) = k e^(alpha t) x   Gompertz growth (decreasing exponential constant)
;
(def alpha -.7)
(def kappa 2)
(defun f (x tt) (* kappa (exp (* alpha tt)) x))
(eulerplot #'f .1 0 8 100 :axes '(0 8 0 2))
(eulerplot #'f .1 0 8 100 :axes '(0 8 0 2)
	   :title "Gompertz growth (decreasing exponential constant)")
;;
(defun eulerplot2 (f ynot start end num g)
  (let* (
	 (hsize (/ (- end start) num))
	 (timesteps (rseq start end num))
	 (yvalues (list ynot))
	 )
    (dotimes
     (i (- num 1))
     (setq yvalues
	   (append yvalues 
		   (+ (last yvalues)
		      (* hsize (mapcar f (list (select yvalues i))))
		      )
		   )
	   )
     )
    (def my-plot (plot-points timesteps yvalues))
    (send my-plot :add-points timesteps (mapcar g timesteps) :symbol 'diamond)
    )
  )

;;
			;non-autonomous first order de's
;;

#| - ael - Sat Jul 25 09:47:57 1998
I moved the following two euler commands, and changed them from eulerplot to
eulerplot2: eulerplot caused an error. I wonder if eulerplot used to handle
non-autonomous equations. I need a function which handles them, but don't think
that eulerplot2 is it.
 - lea - |#
(eulerplot2 #'cos 0 0 (* 2 pi) 100 #'sin)

(defun f (x) (- x (^ x 2)))
(defun g (x) 0)
(eulerplot2 #'f 1 0 (* 2 pi) 100 #'g)
;;

			;autonomous first order des

(defun f (x) (- x (^ x 2)))
(defun g (x) 0)
(eulerplot2 #'f .2 0 8 100 #'g)
(eulerplot2 #'f 1.8 0 8 100 #'g)

(def alpha -3)
(defun f (x) (* x (+ 1 (* alpha (^ x 2)))))
(defun g (x) 0)
(eulerplot2 #'f .2 0 8 100 #'g)
(eulerplot2 #'f 1.2 0 8 100 #'g)
(eulerplot2 #'f -1.2 0 8 100 #'g)

(eulerplot2 #'f -2 0 8 100 #'g)
(eulerplot2 #'f -3 0 8 3 #'g)
;;
			;general n-dimensional first order de's
;;
(defun euler2d (f ynot start end num &key axes title)
  "
f should take a list and return a same-sized list.
"
  (let (
	(hsize (/ (- end start) num))
	(timesteps (rseq start end num))
	(yvalues (list ynot))
	)
    (dotimes
     (i (- num 1))
     (setq nextvals (list (select yvalues i)))
     (def yvalues
	  (append yvalues 
		  (+ (last yvalues)
		     (* hsize
			(mapcar f
				nextvals
				)
			)
		     )
		  )
	  )
     )
    yvalues
    )
  )

(setq alpha .1)
(setq beta .1)
(setq gamma .1)
(setq delta .1)
(defun f (vals)
  (let* (
	 (x (first vals))
	 (y (second vals))
	 )
    (list (- (* alpha x) (* beta x y)) (- (* gamma x y) (* delta y)))
    )
  )

(def points (euler2d #'f '(.1 .2) 0 100 200))
(let (xvals yvals)
  (dotimes (i (length points))
	   (setq xvals (append xvals (list (first (select points i)))))
	   (setq yvals (append yvals (list (second (select points i)))))
	   )
  (def orbit1 (list xvals yvals))
  (plot-lines orbit1)
  )

(defun orbits (xin yin)
  (let* (
	 (points (euler2d #'f (list xin yin) 0 100 200))
	 xvals yvals
	 )
    (dotimes (i (length points))
	     (setq xvals (append xvals (list (first (select points i)))))
	     (setq yvals (append yvals (list (second (select points i)))))
	     )
    (list xvals yvals)
    )
  )
(def orbit2 (orbits 2 3))
(def myplot (plot-lines orbit1))
(send myplot :add-lines orbit2)

;; Project 2
(def alpha -.5)
(defun f (x time xd)
  (* alpha xd)
  )
(def tmax 30)
(euler-delay-plot #'f ynot tmin tmax tao :title "No Delay")

(defun f (x time xd)
  (if (> x 0) -1 1)
  )
(def my-plot (euler-delay-plot #'f ynot tmin tmax tao :title "No Delay"))
(send my-plot :range 1 -.1 .1)
(send my-plot :range 0 0 5)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Problem 4.2 of the modelling text.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Data (note: used integers instead of decimals for times)
;;
(def times (iseq 5 25))
(def x (list 1.27 6.58 7 8.83 8.66 5.53 9.33 14.57 8.51 17.61 12.94 18.45 19.85
	     25.03 28.14 28.31 33.41 41.43 40.87 56.71 59.32))
(def my-plot (plot-points times x))
(def num4.2 (regression-model times x))
(print (send num4.2 :coef-estimates))
(def linelist (send num4.2 :coef-estimates))
(send my-plot :abline (first linelist) (second linelist) )
(print (send num4.2 :coef-standard-errors))
(send num4.2 :plot-residuals times)
(send num4.2 :plot-bayes-residuals)

(def logs_of_x (log x))
(def my-plot (plot-points times logs_of_x))
(def num4.2
(regression-model times logs_of_x))
(print (send num4.2 :coef-estimates))
(def linelist (send num4.2 :coef-estimates))
(send my-plot :abline (first linelist) (second linelist) )
(print (send num4.2 :coef-standard-errors))
(send num4.2 :plot-residuals times)
(send num4.2 :plot-bayes-residuals)
(terpri)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Problem 4.7 of the modelling text.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Data
;;
(def voltage '(109 100 88 76 63 51 38 32 26 19 10 6))
(def voltage (- voltage))
(def beta '(37 43 52 57 64 69 75 71 72 72 96 105))
(def beta (* beta .001))
(def logs_of_beta (log beta))
(def my-plot (plot-points logs_of_beta voltage))
(def num4.7 (regression-model logs_of_beta voltage))
(print (send num4.7 :coef-estimates))
(def linelist (send num4.7 :coef-estimates))
(send my-plot :abline (first linelist) (second linelist) )
(print (send num4.7 :coef-standard-errors))
(def c2 (second linelist))
(def c1 (exp (- (/ (first linelist) C2))))
(print (list c1 c2))
(send num4.7 :plot-residuals voltage)
(send num4.7 :plot-bayes-residuals)
(def my-plot (plot-points voltage beta :symbol 'cross))
(send my-plot :add-points voltage (* c1 (exp (/ voltage c2))))

(def my-plot (plot-points voltage logs_of_beta))
(def num4.7 (regression-model voltage logs_of_beta))
(print (send num4.7 :coef-estimates))
(def linelist (send num4.7 :coef-estimates))
(send my-plot :abline (first linelist) (second linelist) )
(print (send num4.7 :coef-standard-errors))
(def c1 (exp (first linelist)))
(def c2 (/ 1 (second linelist)))
(print (list c1 c2))
(send num4.7 :plot-residuals voltage)
(send num4.7 :plot-bayes-residuals)
(def my-plot (plot-points voltage beta :symbol 'cross))
(send my-plot :add-points voltage (* c1 (exp (/ voltage c2))))
(terpri)

#|
	This file demonstrates the use of euler-nd with some three-d problems!
In particular, I look at a couple of examples from the Book "Understanding
Non-Linear Dynamics".

	We start with one-dimensional examples:

	These are standard one-d examples: Note that these problems are written
as two-d: time will then be plotted on the x axis.

|#
;; Exponential Decay:
(defun f (vals)
  (let (
	(x (first vals))
	(y (second vals))
	)
    (setq xval 1)
    (setq yval (- (* alpha y)))
    (list xval yval)
    )
  )
(setq alpha 1)
(setq xin 0)
(setq yin .2)
(setq end 1)
(setq nsteps 10)
(setq ntimes 1)
(phase-plane #'f (list xin yin) end nsteps ntimes :title "Exponential Decay")

(setq ntimes 100)
(phase-plane #'f (list xin yin) end nsteps ntimes
	     :randomize 3 :title "Exponential Decay")

;; Logistic Equation:
(defun f (vals)
  (let (
	(x (first vals))
	(y (second vals))
	)
    (setq xval 1)
    (setq yval (* y (- 1 y)))
    (list xval yval)
    )
  )
(setq xin 0)
(setq yin .2)
(setq end 5)
(setq nsteps 100)
(setq ntimes 1)
(phase-plane #'f (list xin yin) end nsteps ntimes :title "Logistic Growth/Decay")

(setq ntimes 20)
(phase-plane #'f (list xin yin) end nsteps ntimes
	     :randomize 3
	     :title "Logistic Growth/Decay")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
	then some two-dimensional examples:
|#
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; These are the Lotka-Volterra equations:
(defun f (vals)
  (let (
	(x (first vals))
	(y (second vals))
	)
    (setq xval (- (* alpha x) (* beta x y)))
    (setq yval (- (* gamma x y) (* delta y)))
    (list xval yval)
    )
  )
;; choose some parameter values:
(setq alpha 1)
(setq beta 1)
(setq gamma 1)
(setq delta 1)

(setq xin .2)
(setq yin .2)
(setq end 1)
(setq nsteps 10)
(setq ntimes 10)
(phase-plane #'f (list xin yin) end nsteps ntimes :title "Lotka-Volterra")

(setq xin .2)
(setq yin .2)
(setq end .2)
(setq nsteps 10)
(setq ntimes 100)
(phase-plane #'f (list xin yin) end nsteps ntimes
	     :randomize 3
	     :title "Lotka-Volterra")
(setq end .5)
(phase-plane #'f (list xin yin) end nsteps ntimes
	     :randomize 2
	     :title "Lotka-Volterra")

(setq xin (first (uniform-rand 1)))
(setq yin (first (uniform-rand 1)))
(setq ntimes 100)
(phase-plane #'f (list xin yin) end nsteps ntimes :title "Lotka-Volterra")

(setq xin (1+ (* .05 (first (uniform-rand 1)))))
(setq yin (1+ (* .05 (first (uniform-rand 1)))))
(setq ntimes 1000)
(phase-plane #'f (list xin yin) end nsteps ntimes :title "Lotka-Volterra")

;; Here's another example: p. 270
;;
;;		Problem 5.16: Limpets and Seaweed
;;
(defun f (vals)
  (let (
	(x (first vals))
	(y (second vals))
	)
    (setq xval (- x (^ x 2) (* x y)))
    (setq yval (- (* x y) (/ y 2) (^ y 2)))
    (list xval yval)
    )
  )

(setq xin .1)
(setq yin .2)
(setq end 100)
(setq nsteps 200)
(setq ntimes 1)
(phase-plane #'f (list xin yin) end nsteps ntimes
	     :title "5.16: Limpets and Seaweed")

(setq end 2)
(setq nsteps 10)
(setq ntimes 100)
(phase-plane #'f (list xin yin) end nsteps ntimes
	     :randomize 1.2
	     :title "5.16: Limpets and Seaweed")
(setq end 1)
(setq nsteps 10)
(setq ntimes 100)
(phase-plane #'f (list xin yin) end nsteps ntimes
	     :randomize 2
	     :title "5.16: Limpets and Seaweed")
;;
;;		Van der Pol Equations
;;
(defun f (vals)
  (let (
	(x (first vals))
	(y (second vals))
	)
    (setq xval (/ (- y (/ (^ x 3) 3) (- x)) epsilon))
    (setq yval (- (* epsilon x)))
    (list xval yval)
    )
  )
(setq epsilon .1)

(setq xin .1)
(setq yin .2)
(setq end 10)
(setq nsteps 200)
(setq ntimes 1)
(phase-plane #'f (list xin yin) end nsteps ntimes :title "Van der Pol Equations")

(setq xin 1)
(setq yin 3)
(setq end 10)
(setq nsteps 200)
(setq ntimes 1)
(phase-plane #'f (list xin yin) end nsteps ntimes :title "Van der Pol Equations")

(setq ntimes 10)
(phase-plane #'f (list xin yin) end nsteps ntimes
	     :randomize 3
	     :title "Van der Pol Equations")

(setq end 1)
(setq nsteps 10)
(setq ntimes 100)
(phase-plane #'f (list xin yin) end nsteps ntimes
	     :randomize 2
	     :title "5.16: Limpets and Seaweed")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
	And now look at some 3d examples:

	Check out the Rossler equations! Those are some COOL orbits.

|#
(defun f (vals)
  (let (
	(x (first vals))
	(y (second vals))
	(z (third vals))
	)
    (setq xval (- (+ y z)))
    (setq yval (+ x (* .2 y)))
    (setq zval (+ .2 (* z (- x 5.7))))
    (list xval yval zval)
    )
  )

(def nextpoints (* 10 (uniform-rand 3)))
(setq xin (first nextpoints))
(setq yin (second nextpoints))
(setq zin (third nextpoints))
(setq nextpoints (orbitals #'f xin yin zin 1 50 50))
(setq nextpoints (orbitals #'f xin yin zin 4 200 10 :randomize 10))
(setq nextpoints (orbitals #'f xin yin zin .5 10 100 :randomize 10))

;;
;; 			Lorenz equations: beautiful!
;;
(defun f (vals) ;; These are the equations:
  (let* (
	 (x (first vals))
	 (y (second vals))
	 (z (third vals))
	 )
    (list (* 10 (- y x))
	  (- (* x (- 28 z)) y)
	  (- (* x y) (* z 2.6666666666666665)))
    )
  )

;; These are some decent parameter values:
(defun orbits (xin yin zin)
  (def points (euler-nd #'f (list xin yin zin) 0 4 500))
  (setq xvals '())
  (setq yvals '())
  (setq zvals '())
  (dotimes (i (length points))
	   (setq xvals (append xvals (list (first (select points i)))))
	   (setq yvals (append yvals (list (second (select points i)))))
	   (setq zvals (append zvals (list (third (select points i)))))
	   )
  (list xvals yvals zvals)
  )

;; This will show a certain number ntimes of orbits:
(defun lorenz-show (ntimes)
  (let ()
    (setq starting (* 10 (uniform-rand 3)))
    (def orbit (orbits (first starting) (second starting) (third starting)))
    (def myplot (spin-plot orbit))
    (send myplot :depth-cuing 'nil)
    (send myplot :showing-axes 'nil)
    (send myplot :title "Lorenz Equations")
    (send myplot :add-lines orbit)
    (dotimes (i ntimes)
	     (setq starting (* 10 (uniform-rand 3)))
	     (setq orbit
		   (orbits (first starting) (second starting) (third starting)))
	     (send myplot :add-lines orbit)
	     )
    myplot
    )
  )

(lorenz-show 3)

(def nextpoints (* 10 (uniform-rand 3)))
(setq xin (first nextpoints))
(setq yin (second nextpoints))
(setq zin (third nextpoints))
(setq nextpoints (orbitals #'f xin yin zin 1 50 50))
(setq nextpoints (orbitals #'f xin yin zin 4 200 10 :randomize 10))

(dotimes (i 5)
	 (def step-size (+ .4 (* i .4)))
	 (def end-size (+ 40 (* i 40)))
	 (orbitals #'f xin yin zin step-size end-size 20 :randomize 10)
	 )


#|
	This file demonstrates the use of euler-nd with some three-d problems!
In particular, I look at a couple of examples from the Book "Understanding
Non-Linear Dynamics".

|#


(setq alpha .1)
(setq beta .1)
(setq gamma .1)
(setq delta .1)
(defun f (vals)
  (let (
	(x (first vals))
	(y (second vals))
	)
    (setq xval (- (* alpha x) (* beta x y)))
    (setq yval (- (* gamma x y) (* delta y)))
    (list xval yval)
    )
  )

(defun f (vals)
  (let (
	(x (first vals))
	(y (second vals))
	)
    (setq xval (- x (^ x 2) (* x y)))
    (setq yval (- (* x y) (/ y 2) (^ y 2)))
    (list xval yval)
    )
  )

(def points (euler-nd #'f '(.1 .2) 0 100 200))
(setq xvals '())
(setq yvals '())
(dotimes (i (length points))
	 (setq xvals (append xvals (list (first (select points i)))))
	 (setq yvals (append yvals (list (second (select points i)))))
	 )
(def orbit1 (list xvals yvals))
(plot-lines orbit1)

(defun orbits (xin yin)
  (def points (euler-nd #'f (list xin yin) 0 100 200))
  (setq xvals '())
  (setq yvals '())
  (dotimes (i (length points))
	   (setq xvals (append xvals (list (first (select points i)))))
	   (setq yvals (append yvals (list (second (select points i)))))
	   )
  (list xvals yvals)
  )
(def orbit2 (orbits .01 .01))
(def myplot (plot-lines orbit1))
(send myplot :add-lines orbit2)
(def orbit2 (orbits 2 3))
(def myplot (plot-lines orbit2))
(def orbit2 (orbits 1 1))
(def myplot (plot-lines orbit2))

;; Check out the Rossler equations! These are some cool pictures!
(defun f (vals)
  (let (
	(x (first vals))
	(y (second vals))
	(z (third vals))
	)
    (setq xval (- (+ y z)))
    (setq yval (+ x (* .2 y)))
    (setq zval (+ .2 (* z (- x 5.7))))
    (list xval yval zval)
    )
  )
(defun orbits (xin yin zin)
  (def points (euler-nd #'f (list xin yin zin) 0 10 200))
  (setq xvals '())
  (setq yvals '())
  (setq zvals '())
  (dotimes (i (length points))
	   (setq xvals (append xvals (list (first (select points i)))))
	   (setq yvals (append yvals (list (second (select points i)))))
	   (setq zvals (append zvals (list (third (select points i)))))
	   )
  (list xvals yvals zvals)
  )
(def orbit1 (orbits .01 .01 .01))
(def myplot (spin-plot orbit1))
(send myplot :title "Rossler Equations")
(send myplot :depth-cuing 'nil)
(send myplot :showing-axes 'nil)
(send myplot :add-lines orbit1)
(def orbit2 (orbits .1 .1 .1))
(spin-plot orbit2)
(send myplot :add-lines orbit2)
(def orbit3 (orbits 1 1 1))
(spin-plot orbit3)
(send myplot :add-lines orbit3)
(def orbit4 (orbits 10 10 10))
(spin-plot orbit4)
(send myplot :add-lines orbit4)


;; 			Lorenz equations: beautiful!
;; These are the equations:
(defun f (vals) ;; These are the equations:
  (let* (
	 (x (first vals))
	 (y (second vals))
	 (z (third vals))
	 )
    (list (* 10 (- y x))
	  (- (* x (- 28 z)) y)
	  (- (* x y) (* z 2.6666666666666665)))
    )
  )

;; These are some decent parameter values:
(defun orbits (xin yin zin)
  (def points (euler-nd #'f (list xin yin zin) 0 4 500))
  (setq xvals '())
  (setq yvals '())
  (setq zvals '())
  (dotimes (i (length points))
	   (setq xvals (append xvals (list (first (select points i)))))
	   (setq yvals (append yvals (list (second (select points i)))))
	   (setq zvals (append zvals (list (third (select points i)))))
	   )
  (list xvals yvals zvals)
  )

;; This will show a certain number ntimes of orbits:
(defun lorenz-show (ntimes)
  (let ()
    (setq starting (* 10 (uniform-rand 3)))
    (def orbit (orbits (first starting) (second starting) (third starting)))
    (def myplot (spin-plot orbit))
    (send myplot :depth-cuing 'nil)
    (send myplot :showing-axes 'nil)
    (send myplot :title "Lorenz Equations")
    (send myplot :add-lines orbit)
    (dotimes (i ntimes)
	     (setq starting (* 10 (uniform-rand 3)))
	     (setq orbit
		   (orbits (first starting) (second starting) (third starting)))
	     (send myplot :add-lines orbit)
	     )
    myplot
    )
  )

(lorenz-show 3)
;;
(defun HEAVISIDE(x &key zero)
  "
;; Documentation:

Heaviside function is zero for negatives, 1 for positives. It's currently going to give 0 for zero. According to Friedman, not defined for x=0, but I need it to give zero for the following example, so....

;; Examples:

(plot-function #'heaviside -2 2)

" 
  (if (> x 0) 1 (if (< x 0) 0 (if zero zero 0)))
  )

(def alpha .00001)
;(def alpha (/ .25 (* 30 30)))
;(def vn 30)(def vs 30)(def ve 25)(def vw 25)		;; speeds
(def vn 30)(def vs 30)(def ve 30)(def vw 30)		;; speeds
(def pnl .25)(def psl .25)(def pel .25)(def pwl .25)	;; left-turn probs
(def ewtime 20)
(def endtime 60)
(defun f (vals)
  (let (
	(n (first vals))
	(s (second vals))
	(e (third vals))
	(w (fourth vals))
	(times (mod (fifth vals) endtime))
	)
    (setq gl (if (< times ewtime) 0 1))
    (if (> n 1) (setq n 1))
    (if (> s 1) (setq s 1))
    (if (> e 1) (setq e 1))
    (if (> w 1) (setq w 1))

;    (if (= times 0) (list 
		     (setq dn (first (poisson-rand 1 30)))
		     (setq ds dn)
		     (setq de (first (poisson-rand 1 5)))
		     (setq dw de)
;		     )
;      )

;    (setq nvalin (* alpha dn vn (heaviside (- 1 n))))
    (setq nvalin (* alpha dn vn))
    (setq nvalout (* gl n (+ (* pnl (- 1 s)) (- 1 pnl))))
    (setq nval (- nvalin nvalout))

;    (setq svalin (* alpha ds vs (heaviside (- 1 s))))
    (setq svalin (* alpha ds vs))
    (setq svalout (* gl s (+ (* psl (- 1 n)) (- 1 psl))))
    (setq sval (- svalin svalout))

;    (setq evalin (* alpha de ve (heaviside (- 1 e))))
    (setq evalin (* alpha de ve))
    (setq evalout (* (- 1 gl) e (+ (* pel (- 1 w)) (- 1 pel))))
    (setq evalue (- evalin evalout))

;    (setq wvalin (* alpha dw vw (heaviside (- 1 w))))
    (setq wvalin (* alpha dw vw))
    (setq wvalout (* (- 1 gl) w (+ (* pwl (- 1 e)) (- 1 pwl))))
    (setq wval (- wvalin wvalout))

    (list nval sval evalue wval 1)
    )
  )
;; (f '(.3 .2 .3 .4 5 ))


(def lasttime 120)
(def numpnts 400)
(defun orbits (nin sin ein win)
  (def points (rk-4 #'f (list nin sin ein win 0) 0 lasttime numpnts))
  (setq nvals '())
  (setq svals '())
  (setq evals '())
  (setq wvals '())
  (setq times '())
  (dotimes (i (length points))
	   (setq point (select points i))
	   (setq nvals (append nvals (list (first point))))
	   (setq svals (append svals (list (second point))))
	   (setq evals (append evals (list (third point))))
	   (setq wvals (append wvals (list (fourth point))))
	   (setq times (append times (list (fifth point))))
	   )
  (list nvals svals evals wvals times)
  )
(def orbit (orbits .01 .01 .1 .1))
(def times (fifth orbit))
(plot-points (list times (first orbit)) :title "first")
(plot-points (list times (third orbit)) :title "third")
(plot-points (list times (+ (first orbit) (second orbit) (third orbit)
		     (fourth orbit))) :title "Total Number")

(/ (sum (select orbit (iseq 0 3))) (last times))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun traffic-simulation (a b n)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  (let (
	(nstep (/ (- b a) (float n)))
	(ewtime (float a))
	(ewtimes '())
	(throughput '())
	)
    (dotimes (i (1+ n))

(defun f (vals)
  (let (
	(n (first vals))
	(s (second vals))
	(e (third vals))
	(w (fourth vals))
	(times (mod (fifth vals) endtime))
	)
    (setq gl (heaviside (- times ewtime) :zero .5))
    (if (= times ewtime) (print "equal")
;      (if (< times ewtime) (setq lcount (1+ lcount))
;	(setq rcount (1+ rcount)))
      )
    
    (if (> n 1) (setq n 1))
    (if (> s 1) (setq s 1))
    (if (> e 1) (setq e 1))
    (if (> w 1) (setq w 1))

#|
    (if (= times 0) (list 
		     (setq dn (first (poisson-rand 1 30)))
		     (setq ds dn)
		     (setq de (first (poisson-rand 1 5)))
		     (setq dw de)
		     )
      )
|#
    (setq nvalin (* alpha dn vn (heaviside (- 1 n))))
    (setq nvalout (* gl n (+ (* pnl (- 1 s)) (- 1 pnl))))
    (setq nval (- nvalin nvalout))
    (setq svalin (* alpha ds vs (heaviside (- 1 s))))
    (setq svalout (* gl s (+ (* psl (- 1 n)) (- 1 psl))))
    (setq sval (- svalin svalout))

    (setq evalin (* alpha de ve (heaviside (- 1 e))))
    (setq evalout (* (- 1 gl) e (+ (* pel (- 1 w)) (- 1 pel))))
    (setq evalue (- evalin evalout))
    (setq wvalin (* alpha dw vw (heaviside (- 1 w))))
    (setq wvalout (* (- 1 gl) w (+ (* pwl (- 1 e)) (- 1 pwl))))
    (setq wval (- wvalin wvalout))

    (list nval sval evalue wval 1)
    )
  )
(def lasttime 120)
(def numpnts 120)
(defun orbits (nin sin ein win)
  (def points (rk-4 #'f (list nin sin ein win 0) 0 lasttime numpnts))
  (setq nvals '())
  (setq svals '())
  (setq evals '())
  (setq wvals '())
  (setq times '())
  (dotimes (i (length points))
	   (setq point (select points i))
	   (setq nvals (append nvals (list (first point))))
	   (setq svals (append svals (list (second point))))
	   (setq evals (append evals (list (third point))))
	   (setq wvals (append wvals (list (fourth point))))
	   (setq times (append times (list (fifth point))))
	   )
  (list nvals svals evals wvals times)
  )
	     (def orbit (orbits .01 .01 .01 .01))
	     (def times (fifth orbit))
             (princ ewtime)
             (princ ": ")
          (princ (/ (sum (select orbit (iseq 0 3))) (list (length times))))
	     (terpri)
	     (setq ewtimes (append ewtimes (list ewtime)))
	     (setq throughput (append throughput 
		(/ (sum (select orbit (iseq 0 3))) (list (length times)))))
	     (def ewtime (+ ewtime nstep))
	     )
    (list ewtimes throughput)
    )
  )
(def vn 30)(def vs 30)(def ve 30)(def vw 30)		;; speeds
(setq dn 25)
(setq ds dn)
(setq de 25)
(setq dw de)
(setq traffic.out (traffic-simulation 20 40 2))
(plot-points traffic.out :title "Intersection capacity")

(def vn 30)(def vs 30)(def ve 25)(def vw 25)
(setq dn 25)
(setq ds dn)
(setq de 5)
(setq dw de)
(setq traffic.out (traffic-simulation 0 20 10))
(plot-points traffic.out :title "Intersection capacity")

(def vn 30)(def vs 30)(def ve 15)(def vw 15)
(setq dn 25)
(setq ds dn)
(setq de 5)
(setq dw de)
(setq traffic.out (traffic-simulation 1 10 8))
(plot-points traffic.out :title "Intersection capacity")
(setq traffic.out (traffic-simulation .5 1.5 8))
(plot-points traffic.out :title "Intersection capacity")

#|

	Here are some standard two-d examples, discussed in our text or in the
problems which follow.

|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun corcoef (tseries &optional (k 1))
  "
Description: (corcoef tseries &optional (k 1))

	Computes the correlation coefficient, as defined by Kaplan and
Glass in Understanding Non-Linear Dynamics, p. 292.

Returns: a number (the correlation coefficient)
"
  (let* (
	 (n (length tseries))
	 (summa 0)
	 (ttseries (select tseries (iseq 0 (- n (1+ k)))))
	 )
    (dotimes (i (- n k))
	     (setq summa (+ summa (*
				   (select tseries i)
				   (select tseries (+ k i))
				   )
			    )
		   ) 
	     )
    (/ summa (sum (^ ttseries 2)))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun auto-correlate (tseries &optional nlength)
  "
Description: (auto-correlate tseries &optional nlength)

Creates the auto-correlation for time-series tseries, up to a lag of
nlength (or (length/2), default). Defined on page 296 of Understanding
Non-Linear Dynamics, Kaplan and Glass.

Returns: the auto-correlation list.
"
  (let (
	(n (floor (/ (length tseries) 2.0)))
	)
    (if nlength (setq n nlength))
    (setq corcoefs (make-array n))
    (def vs (- tseries (mean tseries)))
    (loop for i from 0 to (1- n) do (setf (aref corcoefs i) (corcoef vs i)))
    corcoefs
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun variogram (x y)
  "
Description: (variogram x y)

	Creates and returns the variogram of y against lags from x.

Example:
(def x (iseq 1 10))
(def y (+ (* .1 x) (normal-rand 10)))
(def vario (variogram x y))
(plot-points (first vario) (second vario))

"
  (def h (abs (diffs x)))
  (def var (diffs y))
  (def var (* var var))
  (def var (* .5 var))
  (list h var)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Power spectrum is the fourier transform of the auto-correlation function
;;
(defun f (vals)
  (let (
	(x (first vals))
	(y (second vals))
	)
    (setq xval (+ y (* randomness (first (normal-rand 1)))))
    (setq yval (- (* (- a) y) (* b x)))
    (list xval yval)
    )
  )

(setq a .5)
(setq b 3)
(setq xin 2)
(setq yin .1)
(setq end 10)
(setq nsteps 100)
(setq ntimes 0)
(setq randomness 3)
(phase-plane #'f (list xin yin) end nsteps ntimes :title "Drunk?")

(def ac (auto-correlate xvals))
(def my-plot (plot ac))
(send my-plot :title "Auto-correlation function")

(def fftac (fft ac))
(def my-plot (plot-lines (iseq 1 (length (realpart fftac))) (realpart fftac)))
(send my-plot :title "real part of fft")
(def my-plot (plot-lines (iseq 1 (length (imagpart fftac))) (imagpart fftac)))
(send my-plot :title "imag part of fft")

(def invfftac (fft fftac 't))
(def realinvfftac (coerce (realpart invfftac) 'list))
(def my-plot (plot (list ac realinvfftac)))
(send my-plot :title "auto versus reconstituted")

(def r (+ (^ (realpart fftac) 2) (^ (imagpart fftac) 2)))
(def my-plot (plot r))
(send my-plot :title "power spectrum")

(project4 xvals)
(defun fdl ()
"
INTERSECTION OF FOND DU LAC AND SIDE
NAME OF COUNTER: CITY
COUNT DAY: THURSDAY	DATE: 8/09/84
"
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def broad (list "PER." "NORTH BOUND" "SOUTH BOUND" "EAST  BOUND" "WEST BOUND" "VEH" "PEDS"))

(def lanes
     '("North Bound" "South Bound" "East Bound" "West Bound")
     )
(def titles
     '("PerEnd Hour" "PerEnd Minute" "Peds" "Lt" "Thru" "Rt" "Peds" "Lt" "Thru" "Rt" "Peds" "Lt" "Thru" "Rt" "Peds" "Lt" "Thru" "Rt" "Veh Total" "Peds Total")
     )

(def traffic_data
     (list
7 15 0 12 3 1 0 0 5 14 0 10 29 3 1 0 36 0 113 1
7 30 0 5 1 2 0 4 4 18 0 15 43 10 0 3 38 1 144 0
7 45 2 5 3 0 1 3 2 22 0 5 59 8 0 2 48 1 158 3
8 00 1 3 7 1 0 4 1 10 1 18 68 11 0 2 42 3 170 2
8 15 0 5 2 0 1 0 2 12 0 10 53 1 0 1 47 2 137 1
8 30 0 7 6 0 1 1 6 17 0 12 43 5 0 1 39 3 140 1
8 45 0 9 3 1 1 5 5 14 0 17 50 10 1 5 61 5 185 2
9 00 1 4 4 1 7 6 6 27 9 13 63 9 3 2 62 3 200 20
9 15 2 8 5 0 1 1 6 21 0 16 49 12 0 1 66 11 194 3
9 30 0 12 2 1 0 5 4 20 0 18 69 6 0 4 68 7 214 0
9 45 0 10 0 1 0 0 0 11 1 20 61 1 0 0 70 0 174 1
10 00 1 10 0 1 1 0 0 22 1 10 80 11 0 0 70 2 206 3
10 15 1 9 5 0 1 0 5 22 0 15 67 6 0 1 98 7 235 2
10 30 0 12 3 1 2 1 6 22 1 9 74 11 0 0 83 4 226 3
10 45 0 8 5 2 2 5 4 25 2 16 72 19 0 1 76 3 234 4
11 00 0 0 1 1 0 0 6 21 1 10 76 11 0 0 85 2 215 1
11 15 0 5 6 4 0 5 7 16 2 14 67 14 0 1 86 5 228 2
11 30 0 12 6 2 5 5 5 21 5 26 75 14 0 4 88 4 260 6
11 45 0 9 3 1 0 3 5 15 0 15 77 10 0 0 91 5 234 0
12 00 0 9 6 1 0 2 5 34 0 20 69 12 0 5 104 8 275 0
12 15 2 11 2 8 1 5 4 29 2 19 98 14 0 4 110 9 313 5
12 30 2 19 10 3 0 4 7 22 0 22 85 11 0 4 103 10 300 2
12 45 0 11 4 1 0 1 6 25 0 24 86 16 0 1 98 4 277 0
13 00 1 10 6 3 0 4 8 26 0 18 87 15 0 5 83 6 267 1
13 15 0 12 7 3 0 2 7 16 2 18 91 15 1 4 76 10 261 5
13 30 0 13 2 2 0 1 4 8 0 18 69 7 0 2 55 3 182 0
13 45 0 5 3 2 1 2 6 18 0 17 85 9 0 1 82 2 232 1
14 00 1 11 4 2 1 1 5 22 0 26 81 12 0 0 79 3 246 2
14 15 1 7 4 2 0 2 1 22 0 20 76 12 1 3 76 5 230 2
14 30 1 11 6 4 0 4 5 21 0 20 80 13 2 0 90 6 258 3
14 45 0 14 8 2 0 8 5 25 1 30 85 10 0 1 68 1 257 1
15 00 0 4 3 1 0 5 7 31 0 0 86 14 1 0 100 8 259 1
15 15 0 12 3 3 0 4 9 61 0 24 80 16 1 4 134 4 354 1
15 30 0 7 7 1 0 2 4 28 1 25 108 12 2 3 85 2 282 3
15 45 1 8 8 0 0 4 10 18 1 22 99 23 1 3 88 6 289 3
16 00 0 14 7 0 0 1 8 29 0 23 92 19 3 3 100 2 298 3
16 15 0 13 3 2 0 5 8 27 2 26 99 15 4 5 98 8 307 6
16 30 0 10 5 4 2 2 10 18 0 22 82 15 2 3 90 6 267 4
16 45 0 14 6 5 2 2 4 37 2 21 97 16 0 1 114 14 331 4
17 00 0 9 4 2 1 0 6 51 0 20 86 24 2 7 103 6 518 3
17 15 0 10 12 0 0 3 9 33 0 36 104 20 0 1 108 9 345 0
17 30 0 12 8 2 0 5 9 37 1 25 76 14 0 2 59 4 263 1
17 45 0 10 5 2 0 0 9 32 6 20 71 13 0 0 73 2 237 6
18 00 2 10 7 2 0 0 12 22 0 10 63 3 0 0 68 2 199 2
18 15 0 6 5 1 0 6 1 16. 1 16 60 9 1 2 65 2 187 4
18 30 0 5 4 5 0 7 9 18 2 16 58 7 4 1 57 2 187 6
18 45 0 14 8 2 1 1 4 20 1 25 61 17 2 2 71 3 226 4
19 00 0 10 2 2 2 8 5 22 1 10 61 13 4 8 65 2 208 7)
)
(def totals
     '(19 446 220 85 32 129 264 1118 44 860 3550 568 38 101 3762 215 11318 133))

(def tmat (matrix '(48 20) traffic_data))

(def hours (column 1 tmat))
(def minutes (column 2 tmat))
(def times (/ (+ (* 60 hours) minutes) 60))

(def northb (+ (column 4 tmat) (column 5 tmat) (column 6 tmat)))
(def southb (+ (column 8 tmat) (column 9 tmat) (column 10 tmat)))
(def eastb (+ (column 12 tmat) (column 13 tmat) (column 14 tmat)))
(def westb (+ (column 16 tmat) (column 17 tmat) (column 18 tmat)))

;; these are just the auto traffic values:
(def usage-mat (select tmat
		    (iseq 0 (1- (length times)))
		    '(4 5 6 8 9 10 12 13 14 16 17 18)
		    )
     )
(def cormat (correlation-matrix usage-mat))


(def directions (list northb southb eastb westb))
(def titles (list "north bound" "south bound" "east bound" "west bound"))

;; plots of the variables themselves:
(dotimes (i 4)
	 (def my-plot (plot-points times (select directions i)))
	 (send my-plot :title (select titles i))
	 )

;; histograms: poisson? Compare a same draw from a poisson of the
;; sample mean
(defun histos ()
(dotimes (i 4)
	 (def my-plot (histogram (select directions i)))
	 (send my-plot :title (select titles i))
	 (setq dmean (mean (select directions i)))
	 (def my-plot (histogram (poisson-rand (length times) dmean)))
	 (setq dtitle (strcat "Poisson draw:" (select titles i)))
	 (send my-plot :title dtitle)
	 )
)
#|
(dotimes (i 4)
	 (def errors (- (select directions i) (mean (select directions i))))
	 (def my-plot (histogram errors))
	 (send my-plot :title (select titles i))
	 (def my-plot (plot-points times errors))
	 (send my-plot :title (select titles i))
	 )
;; variograms
(dotimes (i 4)
	 (def vario (variogram times (select directions i)))
	 (def my-plot (kernel-plot (first vario) (second vario)))
	 (send my-plot :title (select titles i))
	 )
(dotimes (i 4)
	 (def my-plot (kernel-plot times (select directions i)))
	 (send my-plot :title (select titles i))
	 )

(dotimes (i 4)
	 (def my-plot (plot (row (1+ (* 3 i)) cormat)))
	 (send my-plot :title (strcat "correlations: " (select titles i)))
	 )
|#

(defun fftstuff()

(def northbfft (fft northb))
(plot (realpart northbfft))
(plot (imagpart northbfft))
(plot (abs northbfft))
(def my-plot (plot (^ (realpart northbfft) 2)))
(send my-plot :range 1 0 600)
(imagpart northbfft)
(plot (imagpart northbfft))

(def usage-mat-svd (svd usage-mat))

(plot (realpart (fft (rseq 0 2 10))))
(setq weights (realpart (fft (rseq 0 2 10))))
(setq phases (imagpart (fft (rseq 0 2 10))))

(setq recon (rseq 0 0 10))
(dotimes (i 10)
	 (setq recon
	       (+ recon
		  (* (select weights i) (sin (- (* i (/ pi 2) (rseq 0 2 10))
						(select phases i))))))
	 )
(plot recon)

(setq recon (rseq 0 0 10))
(dotimes (i 10)
	 (setq recon
	       (+ recon
		  (* (select weights i) (sin (- (* i (/ pi 2) (rseq 0 2 10))
						0)))))
	 )
(plot recon)

(corcoef northb)
(corcoef northb 2)

(def my-plot (kernel-plot times northb))
(send my-plot :add-points (list times northb))
(def my-plot (kernel-plot times southb))
(send my-plot :add-points (list times southb))

(loop for i from 1 to 10 do (print (corcoef (- northb (mean northb)) i)))
(auto-correlate northb)
(def song (matrix '(48 4) (combine northb southb eastb westb)))
(def song-svd (svd song))

(setq corcoefs '())
(loop for i from 1 to 24 do (setq corcoefs (combine corcoefs (corcoef northb i))))
(setq corcoefs (rest corcoefs))
corcoefs

(setq n 43)
(setq corcoefs (make-array n))
(def vs (- northb (mean northb)))
(loop for i from 0 to (1- n) do (setf (aref corcoefs i) (corcoef vs i)))
(plot corcoefs)
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun corcoef (tseries &optional (k 1))
  "
Description: (corcoef tseries &optional (k 1))

	Computes the correlation coefficient, as defined by Kaplan and
Glass in Understanding Non-Linear Dynamics, p. 292.

Returns: a number (the correlation coefficient)
"
  (let* (
	 (n (length tseries))
	 (summa 0)
	 (ttseries (select tseries (iseq 0 (- n (1+ k)))))
	 )
    (dotimes (i (- n k))
	     (setq summa (+ summa (*
				   (select tseries i)
				   (select tseries (+ k i))
				   )
			    )
		   ) 
	     )
    (/ summa (sum (^ ttseries 2)))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun auto-correlate (tseries &optional nlength)
  "
Description: (auto-correlate tseries &optional nlength)

Creates the auto-correlation for time-series tseries, up to a lag of
nlength (or (length/2), default). Defined on page 296 of Understanding
Non-Linear Dynamics, Kaplan and Glass.

Returns: the auto-correlation list.
"
  (let (
	(n (floor (/ (length tseries) 2.0)))
	)
    (if nlength (setq n nlength))
    (setq corcoefs (make-array n))
    (def vs (- tseries (mean tseries)))
    (loop for i from 0 to (1- n) do (setf (aref corcoefs i) (corcoef vs i)))
    corcoefs
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun variogram (x y)
  "
Description: (variogram x y)

	Creates and returns the variogram of y against lags from x.

Example:
(def x (iseq 1 10))
(def y (+ (* .1 x) (normal-rand 10)))
(def vario (variogram x y))
(plot-points (first vario) (second vario))

"
  (def h (abs (diffs x)))
  (def var (diffs y))
  (def var (* var var))
  (def var (* .5 var))
  (list h var)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun fdl ()
"
INTERSECTION OF FOND DU LAC AND SIDE
NAME OF COUNTER: CITY
COUNT DAY: THURSDAY	DATE: 8/09/84
"
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def broad (list "PER." "NORTH BOUND" "SOUTH BOUND" "EAST  BOUND" "WEST BOUND" "VEH" "PEDS"))

(def lanes
     '("North Bound" "South Bound" "East Bound" "West Bound")
     )
(def titles
     '("PerEnd Hour" "PerEnd Minute" "Peds" "Lt" "Thru" "Rt" "Peds" "Lt" "Thru" "Rt" "Peds" "Lt" "Thru" "Rt" "Peds" "Lt" "Thru" "Rt" "Veh Total" "Peds Total")
     )

(def traffic_data
     (list
7 15 0 12 3 1 0 0 5 14 0 10 29 3 1 0 36 0 113 1
7 30 0 5 1 2 0 4 4 18 0 15 43 10 0 3 38 1 144 0
7 45 2 5 3 0 1 3 2 22 0 5 59 8 0 2 48 1 158 3
8 00 1 3 7 1 0 4 1 10 1 18 68 11 0 2 42 3 170 2
8 15 0 5 2 0 1 0 2 12 0 10 53 1 0 1 47 2 137 1
8 30 0 7 6 0 1 1 6 17 0 12 43 5 0 1 39 3 140 1
8 45 0 9 3 1 1 5 5 14 0 17 50 10 1 5 61 5 185 2
9 00 1 4 4 1 7 6 6 27 9 13 63 9 3 2 62 3 200 20
9 15 2 8 5 0 1 1 6 21 0 16 49 12 0 1 66 11 194 3
9 30 0 12 2 1 0 5 4 20 0 18 69 6 0 4 68 7 214 0
9 45 0 10 0 1 0 0 0 11 1 20 61 1 0 0 70 0 174 1
10 00 1 10 0 1 1 0 0 22 1 10 80 11 0 0 70 2 206 3
10 15 1 9 5 0 1 0 5 22 0 15 67 6 0 1 98 7 235 2
10 30 0 12 3 1 2 1 6 22 1 9 74 11 0 0 83 4 226 3
10 45 0 8 5 2 2 5 4 25 2 16 72 19 0 1 76 3 234 4
11 00 0 0 1 1 0 0 6 21 1 10 76 11 0 0 85 2 215 1
11 15 0 5 6 4 0 5 7 16 2 14 67 14 0 1 86 5 228 2
11 30 0 12 6 2 5 5 5 21 5 26 75 14 0 4 88 4 260 6
11 45 0 9 3 1 0 3 5 15 0 15 77 10 0 0 91 5 234 0
12 00 0 9 6 1 0 2 5 34 0 20 69 12 0 5 104 8 275 0
12 15 2 11 2 8 1 5 4 29 2 19 98 14 0 4 110 9 313 5
12 30 2 19 10 3 0 4 7 22 0 22 85 11 0 4 103 10 300 2
12 45 0 11 4 1 0 1 6 25 0 24 86 16 0 1 98 4 277 0
13 00 1 10 6 3 0 4 8 26 0 18 87 15 0 5 83 6 267 1
13 15 0 12 7 3 0 2 7 16 2 18 91 15 1 4 76 10 261 5
13 30 0 13 2 2 0 1 4 8 0 18 69 7 0 2 55 3 182 0
13 45 0 5 3 2 1 2 6 18 0 17 85 9 0 1 82 2 232 1
14 00 1 11 4 2 1 1 5 22 0 26 81 12 0 0 79 3 246 2
14 15 1 7 4 2 0 2 1 22 0 20 76 12 1 3 76 5 230 2
14 30 1 11 6 4 0 4 5 21 0 20 80 13 2 0 90 6 258 3
14 45 0 14 8 2 0 8 5 25 1 30 85 10 0 1 68 1 257 1
15 00 0 4 3 1 0 5 7 31 0 0 86 14 1 0 100 8 259 1
15 15 0 12 3 3 0 4 9 61 0 24 80 16 1 4 134 4 354 1
15 30 0 7 7 1 0 2 4 28 1 25 108 12 2 3 85 2 282 3
15 45 1 8 8 0 0 4 10 18 1 22 99 23 1 3 88 6 289 3
16 00 0 14 7 0 0 1 8 29 0 23 92 19 3 3 100 2 298 3
16 15 0 13 3 2 0 5 8 27 2 26 99 15 4 5 98 8 307 6
16 30 0 10 5 4 2 2 10 18 0 22 82 15 2 3 90 6 267 4
16 45 0 14 6 5 2 2 4 37 2 21 97 16 0 1 114 14 331 4
17 00 0 9 4 2 1 0 6 51 0 20 86 24 2 7 103 6 518 3
17 15 0 10 12 0 0 3 9 33 0 36 104 20 0 1 108 9 345 0
17 30 0 12 8 2 0 5 9 37 1 25 76 14 0 2 59 4 263 1
17 45 0 10 5 2 0 0 9 32 6 20 71 13 0 0 73 2 237 6
18 00 2 10 7 2 0 0 12 22 0 10 63 3 0 0 68 2 199 2
18 15 0 6 5 1 0 6 1 16. 1 16 60 9 1 2 65 2 187 4
18 30 0 5 4 5 0 7 9 18 2 16 58 7 4 1 57 2 187 6
18 45 0 14 8 2 1 1 4 20 1 25 61 17 2 2 71 3 226 4
19 00 0 10 2 2 2 8 5 22 1 10 61 13 4 8 65 2 208 7)
)
(def totals
     '(19 446 220 85 32 129 264 1118 44 860 3550 568 38 101 3762 215 11318 133))

(def tmat (matrix '(48 20) traffic_data))

(def hours (column 1 tmat))
(def minutes (column 2 tmat))
(def times (/ (+ (* 60 hours) minutes) 60))

(def northb (+ (column 4 tmat) (column 5 tmat) (column 6 tmat)))
(def southb (+ (column 8 tmat) (column 9 tmat) (column 10 tmat)))
(def eastb (+ (column 12 tmat) (column 13 tmat) (column 14 tmat)))
(def westb (+ (column 16 tmat) (column 17 tmat) (column 18 tmat)))

;; these are just the auto traffic values:
(def usage-mat (select tmat
		    (iseq 0 (1- (length times)))
		    '(4 5 6 8 9 10 12 13 14 16 17 18)
		    )
     )
(def cormat (correlation-matrix usage-mat))


(def directions (list northb southb eastb westb))
(def titles (list "north bound" "south bound" "east bound" "west bound"))

;; plots of the variables themselves:
(dotimes (i 4)
	 (def my-plot (plot-points times (select directions i)))
	 (send my-plot :title (select titles i))
	 )

;; histograms: poisson? Compare a same draw from a poisson of the
;; sample mean
(defun histos ()
(dotimes (i 4)
	 (def my-plot (histogram (select directions i)))
	 (send my-plot :title (select titles i))
	 (setq dmean (mean (select directions i)))
	 (def my-plot (histogram (poisson-rand (length times) dmean)))
	 (setq dtitle (strcat "Poisson draw:" (select titles i)))
	 (send my-plot :title dtitle)
	 )
)
#|
(dotimes (i 4)
	 (def errors (- (select directions i) (mean (select directions i))))
	 (def my-plot (histogram errors))
	 (send my-plot :title (select titles i))
	 (def my-plot (plot-points times errors))
	 (send my-plot :title (select titles i))
	 )
;; variograms
(dotimes (i 4)
	 (def vario (variogram times (select directions i)))
	 (def my-plot (kernel-plot (first vario) (second vario)))
	 (send my-plot :title (select titles i))
	 )
(dotimes (i 4)
	 (def my-plot (kernel-plot times (select directions i)))
	 (send my-plot :title (select titles i))
	 )

(dotimes (i 4)
	 (def my-plot (plot (row (1+ (* 3 i)) cormat)))
	 (send my-plot :title (strcat "correlations: " (select titles i)))
	 )
|#

(defun fftstuff()

(def northbfft (fft northb))
(plot (realpart northbfft))
(plot (imagpart northbfft))
(plot (abs northbfft))
(def my-plot (plot (^ (realpart northbfft) 2)))
(send my-plot :range 1 0 600)
(imagpart northbfft)
(plot (imagpart northbfft))

(def usage-mat-svd (svd usage-mat))



(plot (realpart (fft (rseq 0 2 10))))
(setq weights (realpart (fft (rseq 0 2 10))))
(setq phases (imagpart (fft (rseq 0 2 10))))

(setq recon (rseq 0 0 10))
(dotimes (i 10)
	 (setq recon
	       (+ recon
		  (* (select weights i) (sin (- (* i (/ pi 2) (rseq 0 2 10))
						(select phases i))))))
	 )
(plot recon)

(setq recon (rseq 0 0 10))
(dotimes (i 10)
	 (setq recon
	       (+ recon
		  (* (select weights i) (sin (- (* i (/ pi 2) (rseq 0 2 10))
						0)))))
	 )
(plot recon)

(corcoef northb)
(corcoef northb 2)

(def my-plot (kernel-plot times northb))
(send my-plot :add-points (list times northb))
(def my-plot (kernel-plot times southb))
(send my-plot :add-points (list times southb))

(loop for i from 1 to 10 do (print (corcoef (- northb (mean northb)) i)))
(auto-correlate northb)
(def song (matrix '(48 4) (combine northb southb eastb westb)))
(def song-svd (svd song))

(setq corcoefs '())
(loop for i from 1 to 24 do (setq corcoefs (combine corcoefs (corcoef northb i))))
(setq corcoefs (rest corcoefs))
corcoefs

(setq n 43)
(setq corcoefs (make-array n))
(def vs (- northb (mean northb)))
(loop for i from 0 to (1- n) do (setf (aref corcoefs i) (corcoef vs i)))
(plot corcoefs)
)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Model 1
(defun model1()
(def n 100)
(def A 4)
(def rho .95)
(def xnot 77)
(def model1 (make-array n :initial-element xnot))
(loop for i from 1 to (1- n) do
      (setf
       (aref model1 i)
       (+ A (* rho (aref model1 (1- i))))
       )
      )
(def model1 (+ model1 (* 2 (normal-rand n))))
(def my-plot (plot-lines (iseq 0 (1- n)) model1))
(send my-plot :add-points (list (iseq 0 (1- n)) model1))

(def model1 (coerce model1 'list))
(def vs (- model1 (mean model1)))
(plot (list (rest vs) (butlast vs)))

(list (mean model1) (standard-deviation model1) (/ (standard-deviation
							model1) (sqrt n)))

(def my-plot (plot-points (iseq 0 20) (auto-correlate model1 21)))
(send my-plot :range 0 0 20)
(send my-plot :range 1 -.2 1)
(send my-plot :x-axis 't 't 5)
(send my-plot :y-axis 't 't 7)
(send my-plot :add-function #'(lambda (x) 0) 1 20)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Model 2
(defun model2 (&optional (r1 3) (r2 2))
(def n 100)
(def A 4)
(def rho .95)
(def xnot 77)
(def model2 (make-array n :initial-element xnot))
(loop for i from 1 to (1- n) do
      (setf
       (aref model2 i)
       (+ A (first (* r1 (normal-rand 1))) (* rho (aref model2 (1- i))))
       )
      )
(def model2 (+ model2 (* r2 (normal-rand n))))
(def my-plot (plot-lines (iseq 0 (1- n)) model2))
(send my-plot :add-points (list (iseq 0 (1- n)) model2))

(def vs (coerce model2 'list))
(def vs (- vs (mean vs)))
(plot (list (rest vs) (butlast vs)))

(def my-plot (plot-points (iseq 0 20) (auto-correlate model2 21)))
(send my-plot :range 0 0 20)
(send my-plot :range 1 0 1)
(send my-plot :x-axis 't 't 5)
(send my-plot :y-axis 't 't 6)
(send my-plot :add-function #'(lambda (x) (^ rho x)) 1 21)

(list (mean model2) (standard-deviation model2) (/ (standard-deviation model2) (sqrt n)))
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun model3(&optional (r1 3) (r2 1))
;;
;; Model 3: p. 294
;;
;; this one requires Euler-nd and orbitals.
;;
;;
(defun f (vals)
  (let (
	(x (first vals))
	(y (second vals))
	)
    (setq xval (+ y (* randomness (first (normal-rand 1)))))
    (setq yval (- (* (- a) y) (* b x)))
    (list xval yval)
    )
  )

(setq a .5)
(setq b 3)
(setq xin 20)
(setq yin .10)
(setq end 50)
(setq nsteps 500)
(setq ntimes 0)
(setq randomness 0)
(phase-plane #'f (list xin yin) end nsteps ntimes :title "Model 3, p. 294")
(setq randomness r1)
(phase-plane #'f (list xin yin) end nsteps ntimes :title "Model 3, p. 294")

(def points (euler-nd #'f (list xin yin) 0 end nsteps))
(def n (length points))
(def model3 (make-array n))
(loop for i from 0 to (1- n) do (setf (aref model3 i) (first (nth i points))))

(def model3 (+ model3 (* r2 (normal-rand n))))
(def my-plot (plot-lines (iseq 1 n) model3))
(send my-plot :add-points (list (iseq 1 n) model3))

(def my-plot (plot-points (iseq 0 100) (auto-correlate model3 101)))
(send my-plot :range 0 0 100)
(send my-plot :x-axis 't 't 11)

(def tc (/ a 2))
(def omega (/ (sqrt (- (^ a 2) (* 4 b))) 2))

(send my-plot
      :add-function
      #'(lambda (x) (exp (* (- tc) .1 (imagpart omega) (/ x (* 2 pi)))))
      0 100)
(send my-plot
      :add-function
      #'(lambda (x) (- (exp (* (- tc) .1 (imagpart omega) (/ x (* 2 pi))))))
      0 100)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun model4()
;;
;; Model 4: p. 301
;;
(def n 100)
(setq mu 4)
(setq xnot (first (uniform-rand 1)))
(def model4 (make-array n :initial-element xnot))
(loop for i from 1 to (1- n) do
      (setf
       (aref model4 i)
       (* mu (aref model4 (1- i)) (- 1 (aref model4 (1- i))))
       )
      )
(def my-plot (plot-lines (iseq 0 (1- n)) model4))
(send my-plot :add-points (list (iseq 0 (1- n)) model4))

(def vs (coerce model4 'list))
(def vs (- vs (mean vs)))
(plot (list  (butlast vs) (rest vs)))

(def my-plot (plot-points (iseq 0 20) (auto-correlate model4 21)))
(send my-plot :range 0 0 20)
(send my-plot :range 1 -.2 1)
(send my-plot :x-axis 't 't 5)
(send my-plot :y-axis 't 't 7)
(send my-plot :add-function #'(lambda (x) 0) 1 20)

(list (mean model4) (standard-deviation model4) (/ (standard-deviation model4) (sqrt n)))
)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Model 1
(defun model1()
(def n 100)
(def A 4)
(def rho .95)
(def xnot 77)
(def model1 (make-array n :initial-element xnot))
(loop for i from 1 to (1- n) do
      (setf
       (aref model1 i)
       (+ A (* rho (aref model1 (1- i))))
       )
      )
(def model1 (+ model1 (* 2 (normal-rand n))))
(def my-plot (plot-lines (iseq 0 (1- n)) model1))
(send my-plot :add-points (list (iseq 0 (1- n)) model1))

(def model1 (coerce model1 'list))
(def vs (- model1 (mean model1)))
(plot (list (rest vs) (butlast vs)))

(list (mean model1) (standard-deviation model1) (/ (standard-deviation
							model1) (sqrt n)))

(def my-plot (plot-points (iseq 0 20) (auto-correlate model1 21)))
(send my-plot :range 0 0 20)
(send my-plot :range 1 -.2 1)
(send my-plot :x-axis 't 't 5)
(send my-plot :y-axis 't 't 7)
(send my-plot :add-function #'(lambda (x) 0) 1 20)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Model 2
(defun model2 (&optional (r1 3) (r2 2))
(def n 100)
(def A 4)
(def rho .95)
(def xnot 77)
(def model2 (make-array n :initial-element xnot))
(loop for i from 1 to (1- n) do
      (setf
       (aref model2 i)
       (+ A (first (* r1 (normal-rand 1))) (* rho (aref model2 (1- i))))
       )
      )
(def model2 (+ model2 (* r2 (normal-rand n))))
(def my-plot (plot-lines (iseq 0 (1- n)) model2))
(send my-plot :add-points (list (iseq 0 (1- n)) model2))

(def vs (coerce model2 'list))
(def vs (- vs (mean vs)))
(plot (list (rest vs) (butlast vs)))

(def my-plot (plot-points (iseq 0 20) (auto-correlate model2 21)))
(send my-plot :range 0 0 20)
(send my-plot :range 1 0 1)
(send my-plot :x-axis 't 't 5)
(send my-plot :y-axis 't 't 6)
(send my-plot :add-function #'(lambda (x) (^ rho x)) 1 21)

(list (mean model2) (standard-deviation model2) (/ (standard-deviation model2) (sqrt n)))
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun model3(&optional (r1 3) (r2 1))
;;
;; Model 3: p. 294
;;
;; this one requires Euler-nd and orbitals.
;;
;;
(defun f (vals)
  (let (
	(x (first vals))
	(y (second vals))
	)
    (setq xval (+ y (* randomness (first (normal-rand 1)))))
    (setq yval (- (* (- a) y) (* b x)))
    (list xval yval)
    )
  )

(setq a .5)
(setq b 3)
(setq xin 20)
(setq yin .10)
(setq end 50)
(setq nsteps 500)
(setq ntimes 0)
(setq randomness 0)
(phase-plane #'f (list xin yin) end nsteps ntimes :title "Model 3, p. 294")
(setq randomness r1)
(phase-plane #'f (list xin yin) end nsteps ntimes :title "Model 3, p. 294")

(def points (euler-nd #'f (list xin yin) 0 end nsteps))
(def n (length points))
(def model3 (make-array n))
(loop for i from 0 to (1- n) do (setf (aref model3 i) (first (nth i points))))

(def model3 (+ model3 (* r2 (normal-rand n))))
(def my-plot (plot-lines (iseq 1 n) model3))
(send my-plot :add-points (list (iseq 1 n) model3))

(def my-plot (plot-points (iseq 0 100) (auto-correlate model3 101)))
(send my-plot :range 0 0 100)
(send my-plot :x-axis 't 't 11)

(def tc (/ a 2))
(def omega (/ (sqrt (- (^ a 2) (* 4 b))) 2))

(send my-plot
      :add-function
      #'(lambda (x) (exp (* (- tc) .1 (imagpart omega) (/ x (* 2 pi)))))
      0 100)
(send my-plot
      :add-function
      #'(lambda (x) (- (exp (* (- tc) .1 (imagpart omega) (/ x (* 2 pi))))))
      0 100)
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun model4()
  ;;
  ;; Model 4: p. 301
  ;;
  (def n 100)
  (setq mu 4)
  (setq xnot (first (uniform-rand 1)))
  (def model4 (make-array n :initial-element xnot))
  (loop for i from 1 to (1- n) do
	(setf
	 (aref model4 i)
	 (* mu (aref model4 (1- i)) (- 1 (aref model4 (1- i))))
	 )
	)
  (def my-plot (plot-lines (iseq 0 (1- n)) model4))
  (send my-plot :add-points (list (iseq 0 (1- n)) model4))
  
  (def vs (coerce model4 'list))
  (def vs (- vs (mean vs)))
  (plot (list  (butlast vs) (rest vs)))
  
  (def my-plot (plot-points (iseq 0 20) (auto-correlate model4 21)))
  (send my-plot :range 0 0 20)
  (send my-plot :range 1 -.2 1)
  (send my-plot :x-axis 't 't 5)
  (send my-plot :y-axis 't 't 7)
  (send my-plot :add-function #'(lambda (x) 0) 1 20)
  
  (list
   (mean model4)
   (standard-deviation model4) (/ (standard-deviation model4) (sqrt n)))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun model5()
  ;;
  ;; Model 5: p. 311
  ;;
  #|
	I'm going to use Euler in an "unusual" manner to calculate a difference
equation:

The idea is to use the fact that

	f_{i+1} = f_{i} + h*f'_{i}

to write a difference equation such as

	x_{i+1} = f(x_{i}) + h*f'_{i}
as
	x_{i+1} = x_{i} + 1*(f(x_{i}) - x_{i})

I.e., choose step-size 1, and use the original f - x as the derivative
function.

|#
  
  (defun ikeda (vals) ; Ikeda map, page 311
    (let* (
	   (x (first vals))
	   (y (second vals))
	   (m (- .4 (/ 6.0 (+ 1 (^ y 2) (^ x 2)))))
	   )
      (setq xval (+ 1 (* mu (- (* x (cos m)) (* y (sin m)))) (- x)))
      (setq yval (-   (* mu (- (* x (sin m)) (* y (cos m)))) y))
      (list xval yval)
      )
    )
  (defun henon (vals) ; henon map, page 312
    (let* (
	   (x (first vals))
	   (y (second vals))
	   )
      (setq xval (- 1.4 (* -.3 y) (^ x 2) x))
      (setq yval (- x y))
      (list xval yval)
      )
    )
  (setq mu .95) ; for ikeda
  (setq xin .5)
  (setq yin -.8)
  (setq end 100)
  (setq nsteps 100)
  (setq ntimes 0)
  (phase-plane #'ikeda (list xin yin) end nsteps ntimes :title "Ikeda map")
  (plot (list xvals yvals))
  (def my-plot (plot-points (iseq 1 (length xvals)) xvals))
  (send my-plot :title "Ikeda map")
  (send my-plot :add-points (iseq 1 (length xvals)) yvals :draw-symbol 'cross)
  (send my-plot :range 1 (min (min yvals) (min xvals)) (max (max yvals) (max xvals)))
  
  (setq xin .2)
  (setq yin .1)
  (setq end 100)
  (setq nsteps 100)
  (setq ntimes 0)
  (phase-plane #'henon (list xin yin) end nsteps ntimes :title "Henon map")
  (plot (list xvals yvals))
  (def my-plot (plot-points (iseq 1 (length xvals)) xvals))
  (send my-plot :title "Henon map")
  (send my-plot :add-points (iseq 1 (length xvals)) yvals :draw-symbol 'cross)
  (send my-plot :range 1 (min (min yvals) (min xvals)) (max (max yvals) (max xvals)))
  
  )
(defun power-spectrum (xvals)
  (def ac (auto-correlate xvals))
  (def my-plot (plot ac))
  (send my-plot :title "Auto-correlation function")
  
  (def ac xvals)
;  (def fftac (/ (fft ac) (length xvals)))
  (def fftac (fft ac))
  (def my-plot
       (plot-lines (iseq 1 (length (realpart fftac))) (realpart fftac)))
  (send my-plot :title "real part of fft")

  (def my-plot
       (plot-lines (iseq 1 (length (imagpart fftac))) (imagpart fftac)))
  (send my-plot :title "imag part of fft")
  
  (def invfftac (fft fftac 't))
  (def realinvfftac (coerce (realpart invfftac) 'list))
  (def my-plot (plot (list ac realinvfftac)))
  (send my-plot :title "auto versus reconstituted")
  
  (def r (+ (^ (realpart fftac) 2) (^ (imagpart fftac) 2)))
  (def my-plot (plot r))
  (send my-plot :title "power spectrum")
  )
(power-spectrum (sin (* 2 pi (rseq 0 1 101))))
(defun project4 (xvals)
  (let (
	(len (length xvals))
	)
    (print (length xvals))
    (print len)
    (print (mean (realpart xvals)))
    (setq halflen (floor (/ len 2.0)))

    (print 1)
    (def my-plot (plot-points (iseq 1 len) xvals))
    (send my-plot :title "1. Original time series")
    
    (def fftxvals (fft xvals))
    (def invfftxvals (fft fftxvals 't))
    
    (print 2)
    (def my-plot (plot (list (realpart fftxvals) (imagpart fftxvals))))
    (send my-plot :title "2. FFT of the Time Series")
    
    (print 3)
    (def dc (first fftxvals))
    (def nyquist (nth halflen fftxvals))
    (def others (select fftxvals (iseq 1 (1- halflen))))
    (def r (sqrt (+ (^ (realpart others) 2) (^ (imagpart others) 2))))
    (def newothers (* 2 pi (uniform-rand (length r))))
    (def rcos (* r (cos newothers)))
    (def rsin (* r (sin newothers)))
    (def newothers (+ rcos (* #c(0 1) rsin)))
    (def fftsurrogate
	 (combine
	  dc newothers nyquist (conjugate nyquist) (reverse (conjugate newothers))
	  )
	 )
    (print 4)
    (def my-plot (plot (list (realpart fftsurrogate) (imagpart fftsurrogate))))
    (send my-plot :title "4. FFT of the Surrogate Series")
    
    (def surrogate (/ (fft fftsurrogate 't) len))
    
    (def my-plot (plot-points (iseq 1 (length surrogate)) (realpart surrogate)))
    (send my-plot :title "5. Surrogate Time Series")
    (print (mean (realpart surrogate)))
    
    (def my-plot (plot (imagpart surrogate)))
    (send my-plot :title "6. tiny imaginary parts!")
    )
  )

(project4 (rest xvals))
(project4 (sin (* 2 pi (rseq 0 1 401) 4)))
