#| - ael - Tue Feb 23 21:27:22 1999

Webster, R. et al. Kriging the local risk of a rare disease from a register of
diagnoses. Geographical Analysis. 26(2), pp. 168-185. 1994.

 - lea - |#


(def cases 595)
(def wards 838)
(def mean (/ cases wards)) ;; cases per ward

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				poisson
;;
;; Poisson expected cell values - p. 173
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(def poisson-expected (* wards (poisson-pmf (iseq 10) mean)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				binomial
;;
;; Binomial expected cell values
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#| - ael - Tue Feb 23 22:48:44 1999
(plot
 (mapcar 
  (lambda(k)
    (chis-square-stat
     F (* wards (binomial-pmf (iseq 10) k (/ mean k)))
     :guess (smoosh
	     (list
	      (mapcar #'list (iseq 5))
	      (list (iseq 5 9))))
     ))
  (iseq 9 200)
  ))
(plot
 (lambda(x) (sum (* F (ln (binomial-pmf (iseq 10) 9 (* x (/ mean 9)))))))
 .9 1.1)
 - lea - |#
(setq
 k 9
 mle-p (/ mean k)
 binomial-expected (* wards (binomial-pmf (iseq 10) k mle-p))
 )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				negative-binomial
;;
;; Negative-binomial expected cell values
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#| - ael - Tue Feb 23 22:40:36 1999
;; shows a maximum at nk=1; mle for n is thus 1.
(plot
 (mapcar 
  (lambda(k)
    (sum (* F (negative-binomial-pmf (iseq 10) k (/ k (+ k mean))))))
  (iseq 1 10)))
 - lea - |#
(def nk 1)
(setq nmle-p (/ nk (+ nk mean)))
(def nbinomial-expected (* wards (negative-binomial-pmf (iseq 10) nk nmle-p)))

(defun chis-square-stat (obs exp &key guess verbose)
  "
	chis-square-stat (obs exp &key guess verbose)

;; Examples: 
 (def mat
 #2a(
    (  1.105547E-2         0             0    )
    ( 0.100715             0             1    )
    ( 0.458758             0             2    )
    (  1.39310             5             3    )
    (  3.17278             3             4    )
    (  5.78080             8             5    )
    (  8.77718            12             6    )
    (  11.4229            15             7    )
    (  13.0078            11             8    )
    (  13.1668             8             9    )
    (  11.9949             5            10    )
    (  9.93399             7            11    )
    (  7.54155             6            12    )
    (  5.28489             6            13    )
    (  3.43895             2            14    )
    (  2.08859             5            15    )
    (  1.18919             3            16    )
    ( 0.637267             1            17    )
    ( 0.322528             0            18    )
    ( 0.154644             2            19    )
    (  7.044014E-2         0            20    )
    (  3.055760E-2         0            21    )
    (  1.265363E-2         0            22    )
    (  5.011936E-3         0            23    )
    (  1.902447E-3         1            24    )
   ))
 (chis-square-stat
  (column 2 mat) (column 1 mat)
  :guess (smoosh
	 (list
	  (list (iseq 5))
	  (mapcar #'list (iseq 5 12))
	  (list (iseq 13 14) (iseq 15 23)))))
 (def mat
  #2a(
    (  561.668           584             0    )
    (  426.399           398             1    )
    (  161.854           168             2    )
    (  40.9581            35             3    )
    (  7.77350             9             4    )
    (  1.18028             4             5    )
    ( 0.149338             0             6    )
    (  1.619604E-2         1             7    )
    (  1.536936E-3         0             8    )
    (  1.296434E-4         1             9    )
   )
   )
 (chis-square-stat
  (column 2 mat) (column 1 mat)
  :guess (smoosh
	 (list
	  (mapcar #'list (iseq 4))
	  (list (iseq 4 9)))))
"
  (if guess
      (sum
       (mapcar
	(lambda(set)
	  (let (
		(o (sum (select obs set)))
		(e (sum (select exp set)))
		)
	    (if verbose (print (list o e (/ (square (- o e)) e))))
	    (/ (square (- o e)) e)))
	guess
	)
       )
    (sum (/ (square (- obs exp)) exp))
    )
  )  

(- 1 (chisq-cdf
      (chis-square-stat
       F poisson-expected
       :guess (smoosh
	       (list
		(mapcar #'list (iseq 4))
		(list (iseq 4 9))))
       :verbose t
       )
      3))

(- 1 (chisq-cdf
      (chis-square-stat
       F binomial-expected
       :guess (smoosh
	       (list
		(mapcar #'list (iseq 5))
		(list (iseq 5 9))))
       :verbose t
       )
      4))

;; This is what they get in their paper:
(- 1 (chisq-cdf 3.73 4))

(- 1 (chisq-cdf
      (chis-square-stat
       F nbinomial-expected
       :guess (smoosh
	       (list
		(mapcar #'list (iseq 5))
		(list (iseq 5 9))))
       :verbose t
       )
      3))
