;;;; -*- Mode: Lisp; Syntax: Common-Lisp -*-
;;;; Code from Paradigms of AI Programming
;;;; Copyright (c) 1991 Peter Norvig

;;;; File macsymar.lisp: The rewrite rules for MACSYMA in Chapter 8

(setf
 *simplification-rules*
 (mapcar
  #'infix->prefix
  '(
    (x + 0  = x)
    (0 + x  = x)
    (x + x  = 2 * x)
    (x + x + x  = 3 * x) ;; ael
    (x - 0  = x)
    (0 - x  = - x)
    (x - x  = 0)
    (- - x  = x)
    (x * 1  = x)
    (1 * x  = x)
    (x * 0  = 0)
    (0 * x  = 0)
    (x * x  = x ^ 2)
    (x / 0  = undefined)
    (0 / x  = 0)
    (x / 1  = x)
    (x / x  = 1)
    (0 ^ 0  = undefined)
    (x ^ 0  = 1)
    (0 ^ x  = 0)
    (1 ^ x  = 1)
    (x ^ 1  = x)
    (x ^ -1 = 1 / x)
    (x * (y / x) = y)
    ((y / x) * x = y)
    ((y * x) / x = y)
    ((x * y) / x = y)
    (x + - x = 0)
    ((- x) + x = 0)
    (x + y - x = y)

    ;; ael: "realize" everything!
    (x + 0.0  = x)
    (0.0 + x  = x)
    (x - 0.0  = x)
    (0.0 - x  = - x)
    (x * 1.0  = x)
    (1.0 * x  = x)
    (x * 0.0  = 0)
    (0.0 * x  = 0)

    (x / 0.0  = undefined)
    (0.0 / x  = 0)
    (x / 1.0  = x)
    (0.0 ^ 0.0  = undefined)
    (0 ^ 0.0  = undefined)
    (0.0 ^ 0  = undefined)
    (x ^ 0.0  = 1)
    (0.0 ^ x  = 0)
    (1.0 ^ x  = 1)
    (x ^ 1.0  = x)
    (x ^ -1.0 = 1 / x)
    ;; ael

    ;; ael: now add a few of my own:
    (-1.0 * x  = - x)
    (0.5 * x  = x / 2)
    (-0.5 * x  = - x / 2)
    ;; ael
    )))

(setf
 *simplification-rules* 
 (append
  *simplification-rules*
  (mapcar
   #'simp-rule
   '(
     ;; ael: noticed that numbers at the front for products,
     (s * n = n * s)
     (n * (m * x) = (n * m) * x)
     (x * (n * y) = n * (x * y))
     ((n * x) * y = n * (x * y))
     ;; ael: noticed that numbers at the rear for sums.
     (n + s = s + n)
     ((x + m) + n = x + n + m)
     (x + (y + n) = (x + y) + n)
     ((x + n) + y = (x + y) + n)

;;; trial: remove parentheses when unnecessary
;;;     ((x) + (y) = x + y)
;;; leads to stack overflow.

     ;; ael: I added the following
     ;; distributive laws:
     ((n * x) + (n * y) = n * (x + y))
     ((n * x) + (m * x) = (n + m) * x)
     ((z * x) + (z * y) = z * (x + y))
     (z * x + z * y = z * (x + y))
     (z * x + y * z = z * (x + y))
     (x * z + z * y = z * (x + y))
     (x * z + y * z = z * (x + y))

     (x + (n * x) = (1 + n) * x)
     ((n * x) + x = (1 + n) * x)

     (x + (y * x) = (1 + y) * x)
     (x + (x * y) = (1 + y) * x)
     ((y * x) + x = (1 + y) * x)
     ((x * y) + x = (1 + y) * x)

     (x + (y + x) = (2 * x) + y)
     (x + (x + y) = (2 * x) + y)
     ((y + x) + x = (2 * x) + y)
     ((x + y) + x = (2 * x) + y)
     ;; ael
     )
   )))

(setf
 *simplification-rules* 
 (append
  *simplification-rules*
  (mapcar
   #'simp-rule
   '(
     (log 1         = 0)
     (log 0         = undefined)
     (log e         = 1)
     (sin 0         = 0)
     (sin pi        = 0)
     (cos 0         = 1)
     (cos pi        = -1)
     (sin(pi / 2)   = 1)
     (cos(pi / 2)   = 0)
     (exp 0         = 1)
     (exp 1         = e)
     (log (x ^ y)   = y * log(x)) ;; ael
     (log (e ^ x)   = x)
     (log (exp x)   = x)
     (e ^ (log x)   = x)
     (exp (log x)   = x) ;; Note: implicit domain restriction
     ((x ^ y) * (x ^ z) = x ^ (y + z))
     ((x ^ y) / (x ^ z) = x ^ (y - z))
     (log x + log y = log(x * y))
     (log x - log y = log(x / y))
     ((sin x) ^ 2 + (cos x) ^ 2 = 1)
     ((cos x) ^ 2 + (sin x) ^ 2 = 1) ;; ael - symmetric case

     ;; ael: "realize" everything:
     (log 1.0 = 0)
     (log 0.0 = undefined)
     (sin 0.0 = 0)
     (cos 0.0 = 1)
     (exp 0.0 = 1)
     ;; ael

     ;; ael: a few of my own:
     (ln 1         = 0)
     (ln 0         = undefined)
     (ln e         = 1)
     (ln 1.0       = 0)
     (ln 0.0       = undefined)
     (ln (x ^ y)   = y * ln(x))
     (ln (e ^ x)   = x)
     (ln (exp x)   = x)
     (e ^ (ln x)   = x)
     (exp (ln x)   = x) ;; Note: implicit domain restriction
     (ln x + ln y = ln(x * y))
     (ln x - ln y = ln(x / y))
     ;; ael

     ))))

(setf
 *simplification-rules* 
 (append
  *simplification-rules*
  (mapcar
   #'simp-rule
   '(
     (d x / d x       = 1)
     (d (u + v) / d x = (d u / d x) + (d v / d x))
     (d (u - v) / d x = (d u / d x) - (d v / d x))
     (d (- u) / d x   = - (d u / d x))
     (d (u * v) / d x = u * (d v / d x) + v * (d u / d x))
     (d (u / v) / d x = (u * (d v / d x) - v * (d u / d x)) / v ^ 2)
     (d (u ^ n) / d x = n * u ^ (n - 1) * (d u / d x))
     (d (u ^ v) / d x =   v * u ^ (v - 1) * (d u / d x)
	                + u ^ v * (log u) * (d v / d x))
     (d (log u) / d x = (d u / d x) / u)
     (d (ln u) / d x = (d u / d x) / u) ;; ael
     (d (sin u) / d x = (cos u) * (d u / d x))
     (d (cos u) / d x = - (sin u) * (d u / d x))
     (d (exp u) / d x = (exp u) * (d u / d x))
     (d u / d x       = 0)
     ))))

(integration-table
 '(
   (Int log(x) d x = x * log(x) - x)
   (Int exp(x) d x = exp(x))
   (Int sin(x) d x = - cos(x))
   (Int cos(x) d x = sin(x))
   (Int tan(x) d x = - log(cos(x)))
   (Int sinh(x) d x = cosh(x))
   (Int cosh(x) d x = sinh(x))
   (Int tanh(x) d x = log(cosh(x)))
   (Int ln(x) d x = x * ln(x) - x) ;; ael
   ))
