;;;; -*- Mode: Lisp; Syntax: Common-Lisp -*-
;;;; Code from Paradigms of AI Programming
;;;; Copyright (c) 1991 Peter Norvig

;;;; File macsyma.lisp: The implementation of MACSYMA in Chapter 8
#|
	I usually need to load this twice - don't ask me why! Then (simplifier)
works. I can nearly reproduce (slight, non-critical differences) all the
examples in the example file macsyma.dat, which are now part of the
documentation of simplifier.

	I added the 
(require "macsyma-rules")
at the bottom of this file.

	Now you're ready to simplify, and integrate. Cool! Peter, thanks
bunches....

						Andy Long
						November, 1997

|#
;;ael: Need to turn off the special use of the question mark, etc.:
(unset-question-mark)

;;ael: requires patmatch, but I load it first:
;; (require "patmatch")

;; Some things that pattern matching will require:
(pat-match-abbrev 'x+ '(?+ x))
(pat-match-abbrev 'y+ '(?+ y))

;; Moved these up, but don't think it made any difference...
;; Define n and m as numbers; s as a non-number:
(pat-match-abbrev 'n '(?is n numberp))
(pat-match-abbrev 'm '(?is m numberp))
(pat-match-abbrev 's '(?is s not-numberp))

;;; From student.lisp:
(defstruct (rule (:type list)) pattern response)
(defstruct (exp (:type list) (:constructor mkexp (lhs op rhs)))
  op lhs rhs)

(defun exp-p (x) (consp x))
(defun exp-args (x) (rest x))

(defun binary-exp-p (x)
  (and (exp-p x) (= (length (exp-args x)) 2)))

(defun prefix->infix (exp)
  "Translate prefix to infix expressions."
  (if (atom exp) exp
      (mapcar #'prefix->infix
              (if (binary-exp-p exp)
                  (list (exp-lhs exp) (exp-op exp) (exp-rhs exp))
                  exp))))

(defparameter *infix->prefix-rules*
  (mapcar #'expand-pat-match-abbrev
    '(((x+ = y+) (= x y))
      ((- x+)    (- x))
      ((+ x+)    (+ x))
      ((x+ + y+) (+ x y))
      ((x+ - y+) (- x y))
      ((d y+ / d x) (d y x))        ;*** New rule
      ((Int y+ d x) (int y x))      ;*** New rule
      ((x+ * y+) (* x y))
      ((x+ / y+) (/ x y))
      ((x+ ^ y+) (^ x y)))))

(defun infix->prefix (exp)
  "Translate an infix expression into prefix notation."
  ;; Note we cannot do implicit multiplication in this system
  (cond ((atom exp) exp)
        ((= (length exp) 1) (infix->prefix (first exp)))
        ((rule-based-translator exp *infix->prefix-rules*
           :rule-if #'rule-pattern :rule-then #'rule-response
           :action
           #'(lambda (bindings response)
               (sublis (mapcar
                         #'(lambda (pair)
                             (cons (first pair)
                                   (infix->prefix (rest pair))))
                         bindings)
                       response))))
        ((symbolp (first exp))
         (list (first exp) (infix->prefix (rest exp))))
        (t (error "Illegal exp"))))

(defun variable-p (exp)
  "Variables are the symbols M through Z."
  ;; put x,y,z first to find them a little faster
  (member exp '(x y z m n o p q r s t u v w)))

#|
	ael: I had to move these up - it was requiring two go-rounds of
loading before things worked. ael

(pat-match-abbrev 'x+ '(?+ x))
(pat-match-abbrev 'y+ '(?+ y))
|#
(defparameter *simplification-rules* nil) ; Rules are in file macsyma-rules.lisp

#|
(defun ^ (x y) "Exponentiation" (expt x y))
|#

(defun simplifier (&optional exp)
  "
;; Description: function simplifier (&optional exp)

	Reads a mathematical expression, simplifies it, and prints the
result. Without exp, works interactively until the word quit or exit is typed
at the command line.

;; Examples:
 (simplifier '(2 + 2))
 (simplifier '(5 * 20 + 30 + 7))
 (simplifier '(5 * x - (4 + 1) * x))
 (simplifier '(y / z * (5 * x - (4 + 1) * x)))
 (simplifier '((4 - 3) * x + (y / y - 1) * z))
 (simplifier '(1 * f(x) + 0))
 (simplifier '(3 * 2 * x))
 (simplifier '(3 * 2 * x))
 (simplifier '(2 * x * x * 3))
 (simplifier '(2 * x * 3 * y * 4 * z * 5 * 6))
 (simplifier '(3 + x + 4 + x))
 (simplifier '(2 * x * 3 * x * 4 * (1 / x) * 5 * 6))
 (simplifier '(3 + x + 4 - x))
 (simplifier '(x + y + y + x))
 (simplifier '(3 * x + 4 * x))
 (simplifier '(x ^ 2 * x ^ 3))
 (simplifier '(x * 2 + x * y))
 (simplifier '(d (x + x) / d x))
 (simplifier '(d (a * x ^ 2 + b * x + c) / d x))
 (simplifier '(d ((a * x ^ 2 + b * x + c) / x) / d x))
 (simplifier '(log ((d (x + x) / d x) / 2)))
 (simplifier '(log(x + x) - log x))
 (simplifier '(x ^ cos pi))
 (simplifier '(d (3 * x + (cos x) / x) / d x))
 (simplifier '(d ((cos x) / x) / d x))
 (simplifier '(d (3 * x ^ 2 + 2 * x + 1) / d x))
 (simplifier '(sin(x + x) ^ 2 + cos(d x ^ 2 / d x) ^ 2))
 (simplifier '(sin(x + x) * sin(d x ^ 2 / d x)))
 (simplifier '(x + y + y + x))
 (simplifier '(3 * x + 4 * x))
 (simplifier '(3 * x + y + x + 4 * x) )
 (simplifier '(int x * sin(x ^ 2) d x))
 (simplifier '(int ((3 * x ^ 3) - 1 / (3 * x ^ 3)) d x))
 (simplifier '(int (3 * x + 2) ^ -2/3 d x))
 (simplifier '(int (3 * x + 2) ^ -.6 d x))
 (simplifier '(int sin(x) ^ 2 * cos(x) d x))
 (simplifier '(int sin(x) / (1 + cos(x)) d x))
 (simplifier '(int (2 * x + 1) / (x ^ 2 + x - 1) d x))
 (simplifier '(int 8 * x ^ 2 / (x ^ 3 + 2) ^ 3 d x))

 ;;	We can pass literal expressions to the simplifier too, so that special
 ;; rules are operative in special contexts. For example, we might know that x=0;
 ;; thus, it would be nice if we could add a special rule, temporarily: x=0.
 ;; This can be done as follows:

 (func m(c) (a*(exp c)+b)^n) ;; a is p, b is q, and x is t.
 ;; note: the variable names have to be different than the following,
 ;;    x y z m n o p q r s t u v w,
 ;; because of the definition of variable-p. This m function is the generating
 ;; function of a binomial with n trials and probability p: I've replaced p
 ;; and q by a and b, as I wanted to add the special rules to the general
 ;; simplification rules (defined in file macrules.lsp).
 (def
  *simplification-rules*
  (append
   (mapcar
    #'simp-rule
    '((a + b = 1)
      ((- b) + 1 = a)
      ((- a) + 1 = b)))
   *simplification-rules*))

 ;; derivatives of the generating function, evaluated at 0, give us the
 ;; moments of the binomial distribution:
 (def *variable-substitution* '((c 0)))
 (simplifier (diff m c)) ;; (N * A)
 (simplifier (diff-n m c 2)) ;; ((N * (B * A)) + ((N * A) ^ 2))

 (func m(x) (x+1)^n)
 (def *variable-substitution* '((x 0)))
 (simplifier (diff m x))	;; N
 (simplifier (diff-n m x 2))	;; ((-1 * N) + (N ^ 2))

 ;; reset variables to normal values:
 (def *variable-substitution* nil)
 (load-calc macrules)
 "
  ;; if you just want to simplify an expression, then give it: 
  (if exp (simp-norvig exp) 
    ;; otherwise, simplify interactively:
    (loop
     (print 'simplifier>)
     (setq exp (read))
     (if (or (eq exp 'exit) (eq exp 'quit)) (return 'done)
       (print (simp-norvig exp))))))

(defun simp-norvig (inf) (prefix->infix (simplify (infix->prefix inf))))

(defun simplify (exp) 
  "Simplify an expression by first simplifying its components."
  (if (atom exp) exp
      (simplify-exp (mapcar #'simplify exp))))

#| - ael - Sun May 24 21:08:50 1998
Modified simplify to allow for variable substitution (e.g., setting x=0).
Added the function check-variable-substitution and the variable
*variable-substitution* which is a list of pairs, of the form
'((x 0) (y pi)), etc.
							- lea - |#
(defun simplify (exp) 
  "Simplify an expression by first simplifying its components."
  (if (atom exp) (check-variable-substitution exp)
      (simplify-exp (mapcar #'simplify exp))))

(defparameter *variable-substitution* nil)
(defun check-variable-substitution (exp)
  (let ((found (search (list exp) (mapcar #'first *variable-substitution*))))
    (if found (elt (mapcar #'second *variable-substitution*) (final found))
      exp)))
#| - ael - Sun May 24 21:10:46 1998 - lea - |#

(defun simplify-exp (exp)
  "Simplify using a rule, or by doing arithmetic."
  (cond ((rule-based-translator exp *simplification-rules*
           :rule-if #'exp-lhs :rule-then #'exp-rhs
           :action #'(lambda (bindings response)
                       (simplify (sublis bindings response)))))
        ((evaluable exp) (eval exp))
        (t exp)))

(defun evaluable (exp)
  "Is this an arithmetic expression that can be evaluated?"
  (and (every #'numberp (exp-args exp))
       (or (member (exp-op exp) '(+ - * /))
           (and (eq (exp-op exp) '^)
                (integerp (second (exp-args exp)))))))

(defun not-numberp (x) (not (numberp x)))

(defun simp-rule (rule)
  "Transform a rule into proper format."
  (let ((exp (infix->prefix rule)))
    (mkexp (expand-pat-match-abbrev (exp-lhs exp))
           (exp-op exp) (exp-rhs exp))))

(defun simp-fn (op) (get op 'simp-fn))
(defun set-simp-fn (op fn) (setf (get op 'simp-fn) fn))

(set-simp-fn
 'Int
 #'(lambda (exp)
     (unfactorize (factorize (integrate-norvig (exp-lhs exp) (exp-rhs exp))))))
 
(defun simplify-exp (exp)
  "Simplify using a rule, or by doing arithmetic,
  or by using the simp function supplied for this operator."
  (cond ((simplify-by-fn exp))                             ;***
        ((rule-based-translator exp *simplification-rules*
           :rule-if #'exp-lhs :rule-then #'exp-rhs
           :action #'(lambda (bindings response)
                       (simplify (sublis bindings response)))))
        ((evaluable exp) (eval exp))
        (t exp)))

(defun simplify-by-fn (exp)
  "If there is a simplification fn for this exp,
  and if applying it gives a non-null result,
  then simplify the result and return that."
  (let* ((fn (simp-fn (exp-op exp)))
         (result (if fn (funcall fn exp))))
    (if (null result)
        nil
        (simplify result))))

(defun factorize (exp)
  "Return a list of the factors of exp^n,
  where each factor is of the form (^ y n)."
  (let ((factors nil)
        (constant 1))
    (labels
      ((fac (x n)
         (cond
           ((numberp x)
            (setf constant (* constant (expt x n))))
           ((starts-with x '*)
            (fac (exp-lhs x) n)
            (fac (exp-rhs x) n))
           ((starts-with x '/)
            (fac (exp-lhs x) n)
            (fac (exp-rhs x) (- n)))
           ((and (starts-with x '-) (length=1 (exp-args x)))
            (setf constant (- constant))
            (fac (exp-lhs x) n))
           ((and (starts-with x '^) (numberp (exp-rhs x)))
            (fac (exp-lhs x) (* n (exp-rhs x))))
           (t (let ((factor (find x factors :key #'exp-lhs
                                  :test #'equal)))
                (if factor
                    (incf (exp-rhs factor) n)
                    (push `(^ ,x ,n) factors)))))))
      ;; Body of factorize:
      (fac exp 1)
      (case constant
        (0 '((^ 0 1)))
        (1 factors)
        (t `((^ ,constant 1) .,factors))))))

(defun unfactorize (factors)
  "Convert a list of factors back into prefix form."
  (cond ((null factors) 1)
        ((length=1 factors) (first factors))
        (t `(* ,(first factors) ,(unfactorize (rest factors))))))

(defun divide-factors (numer denom)
  "Divide a list of factors by another, producing a third."
  (let ((result (mapcar #'copy-list numer)))
    (dolist (d denom)
      (let ((factor (find (exp-lhs d) result :key #'exp-lhs
                          :test #'equal)))
        (if factor
            (decf (exp-rhs factor) (exp-rhs d))
            (push `(^ ,(exp-lhs d) ,(- (exp-rhs d))) result))))
    
    ;; ael: my kludgy way of making this work is to add this second level of
    ;; delete to the previous. 3-50 was giving a (^ x 0.0) for expressions,
    ;; rather than (^ x 0), so now that seems to have taken care of it.
    (delete 0.0
	    (delete 0 result :key #'exp-rhs)
	    :key #'exp-rhs)
    )
  )

(defun free-of (exp var)
  "True if expression has no occurrence of var."
  (not (find-anywhere var exp)))

(defun find-anywhere (item tree)
  "Does item occur anywhere in tree?  If so, return it."
  (cond ((eql item tree) tree)
        ((atom tree) nil)
        ((find-anywhere item (first tree)))
        ((find-anywhere item (rest tree)))))

(defun integrate-norvig (exp x)
  ;; First try some trivial cases
  (cond
   ;; ael: replacing x with ,x so that I can use other variables:
   ((free-of exp x) `(* ,exp ,x))          ; Int c dx = c*x
   ((starts-with exp '+)                  ; Int(f + g)  = Int f + Int g
    `(+ ,(integrate-norvig (exp-lhs exp) x)      
	,(integrate-norvig (exp-rhs exp) x)))
   ((starts-with exp '-)              
    (ecase (length (exp-args exp))
	   (1 (integrate-norvig (exp-lhs exp) ,x))     ; Int - f = - Int f
	   (2 `(- ,(integrate-norvig (exp-lhs exp) x) ; Int f - g  =
		  ,(integrate-norvig (exp-rhs exp) x)))))  ; Int f - Int g
   ;; Now move the constant factors to the left of the integral
   ((multiple-value-bind
     (const-factors x-factors)
     (partition-if #'(lambda (factor) (free-of factor x)) (factorize exp))
     (identity ;simplify
      `(* ,(unfactorize const-factors)
	  ;; And try to integrate:
	  ,(cond
	    ((null x-factors) x)
	    ((some #'(lambda (factor) (deriv-divides factor x-factors x))
		   x-factors))
	    ;; <other methods here>
	    (t `(int? ,(unfactorize x-factors) ,x)))))))))

(defun partition-if (pred list)
  "Return 2 values: elements of list that satisfy pred,
  and elements that don't."
  (let ((yes-list nil)
        (no-list nil))
    (dolist (item list)
      (if (funcall pred item)
          (push item yes-list)
          (push item no-list)))
    (values (nreverse yes-list) (nreverse no-list))))

(defun deriv-divides (factor factors x)
  (assert (starts-with factor '^))
  (let* ((u (exp-lhs factor))              ; factor = u^n
         (n (exp-rhs factor))
         (k (divide-factors 
              factors (factorize `(* ,factor ,(deriv-norvig u x))))))
    (cond
     ((free-of k x)
      ;; Int k*u^n*du/dx dx = k*Int u^n du
      ;;                    = k*u^(n+1)/(n+1) for n/=-1
      ;;                    = k*log(u) for n=1
      (if (= n -1)
	  `(* ,(unfactorize k) (log ,u))
	`(/ (* ,(unfactorize k) (^ ,u ,(+ n 1))) ,(+ n 1))))
     ((and (= n 1) (in-integral-table? u))
      ;; Int y'*f(y) dx = Int f(y) dy
      (let ((k2 (divide-factors
		 factors
		 (factorize `(* ,u ,(deriv-norvig (exp-lhs u) x))))))
	(if (free-of k2 x)
	    `(* ,(integrate-from-table (exp-op u) (exp-lhs u))
		,(unfactorize k2))))))))

(defun deriv-norvig (y x) (simplify `(d ,y ,x)))

(defun integration-table (rules)
  (dolist (i-rule rules)
	  (let ((rule (infix->prefix i-rule)))
	    (setf (get (exp-op (exp-lhs (exp-lhs rule))) 'int)
		  rule))))

(defun in-integral-table? (exp)
  (and (exp-p exp) (get (exp-op exp) 'int)))

(defun integrate-from-table (op arg)
  (let ((rule (get op 'int)))
    (subst arg (exp-lhs (exp-lhs (exp-lhs rule))) (exp-rhs rule))))

;;ael: requires macsyma-rules, but I load it last:
;; (require "macsyma-rules"); now called macrules.lsp

;;ael: Need to turn the special use of the question mark back on:
(reset-question-mark)
