;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                           Udina thefts
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;              andyplot do-key, and some other methods
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth andyplot-proto :do-key (c m1 m2)
  "
  The following keys are currently defined: typing them while a plot is
active will give the following results:

                        h - help (this message)

d - change domain       r - change range        b - change both

p - replot the item     q - remove the plot     a - adjust-to-data
P - print a postscript file                     u - update the plot
v - shrink the plot
V - grow the plot

n - move right          N - move down           z - zoom in
m - move left           M - move up             Z - zoom out

l - make the window large (of full screen-size)
s - make the window small (250 by 250)
C-m - choose-mouse-mode ('C-m' denotes control-m)
C-z - mouse-mode zoom-in
C-s - mouse-mode selecting
C-b - mouse-mode brushing
"
  (case c
	(#\a (send self :adjust-to-data))
	(#\b (send self :change-domain-and-range))
	(#\d (send self :change-domain))
	(#\h (princ
"
  The following keys are currently defined: typing them while a plot is
active will give the following results:

                        h - help (this)         i - info (send :help command)

d - change domain       r - change range        b - change both

			P - print the plot to a postscript file

p - replot the item     q - remove the plot     a - adjust-to-data
u - update the plot     v - shrink the plot     V - grow the plot

n - move right          N - move down           z - zoom in
m - move left           M - move up             Z - zoom out

l - make the window large (of full screen-size)
s - make the window small (250 by 250)
C-m - choose-mouse-mode ('C-m' denotes control-m)
C-z - mouse-mode zoom-in
C-s - mouse-mode selecting
C-b - mouse-mode brushing
")
	 )
	(#\i (send self :help))
	(#\l
 	 (let (
	       (screen-size (screen-size))
	       )
	   (send self :location 0 20)
	   (send self :size (first screen-size) (- (second screen-size) 20))
	   (send self :plot-item)
	   )
	 )
	(#\m (send self :shift 0 0 0))
	(#\n (send self :shift 1 0 0))
	(#\M (send self :shift 3 0 0))
	(#\N (send self :shift 2 0 0))
	(#\p (send self :plot-item))
	(#\P (send self :postscript-image))
	(#\q (send self :remove))
	(#\r (send self :change-range))
	(#\s
	 (send self :location 0 20)
	 (send self :size 250 250)
	 (send self :plot-item)
	 )
	(#\u (send self :update-plot))
	(#\v (let((size (send self :size)))
	       (send self :size
		     (floor (* .9 (first size)))
		     (floor (* .9 (second size))))))
	(#\V (let((size (send self :size)))
	       (send self :size
		     (floor (* 1.1 (first size)))
		     (floor (* 1.1 (second size))))))

	(#\z (send self :shift 5 0 0)) ;; zoom in
	(#\Z (send self :shift 4 0 0)) ;; zoom out

	;; These are for keypad use:
	(#\C-m (send self :choose-mouse-mode))
	(#\C-z (send self :mouse-mode 'zoom-in))
	(#\C-s (send self :mouse-mode 'selecting))
	(#\C-b (send self :mouse-mode 'brushing))
	(#\4 (send self :shift 0 0 0))
	(#\6 (send self :shift 1 0 0))
	(#\2 (send self :shift 2 0 0))
	(#\8 (send self :shift 3 0 0))
	(#\5 (send self :shift 5 0 0)) ;; zoom in
	(#\- (send self :shift 4 0 0)) ;; zoom out
	(#\+ (send self :plot-item))
	(#\= (send self :remove))
	(#\C-q (send self :remove)) ;; control characters work too!

	(#\7
	 (send self :location 0 20)
	 (send self :size 250 250)
	 )
	(#\9
	 (let ((screen-size (screen-size)))
	   (send self :location 0 20)
	   (send self :size (first screen-size) (- (second screen-size) 20))
	   )
	 )
	)
  (if (member self (active-graph-windows)) (send self :plot-item))
  nil
  )

(defmeth andyplot-proto :do-show-coordinates (x y m1 m2)
  (let* ((xy (cond (m1 (list x y))
                   (m2 (send self :canvas-to-scaled x y))
                   (t (send self :canvas-to-real x y))
                   ))
	 (s (format nil "~s" xy))
	 (str-size (send self :text-width s))
	 (left (> (+ x str-size) (send self :canvas-width)))
	 (horz (if left 2 0))
	 (vert 0)
	 (mode (send self :draw-mode)))
    (send self :draw-mode 'xor)
    (send self :draw-text s x y horz vert)
    (send self :while-button-down #'(lambda (x y) nil))
    (send self :draw-text s x y horz vert)
    (send self :draw-mode mode)))

(defmeth andyplot-proto :draw-string-while-button (s x y h v)
  (let* ((oldx x)
         (oldy y)
         (origin (first (transpose (send self :scaled-range '(0 1)))))
         (origin (apply #'send self :scaled-to-canvas origin))
         (mode (send self :draw-mode)))
    (send self :draw-mode 'xor)
    (send self :draw-line x y (first origin) y)
    (send self :draw-line x y x (second origin))
    (send self :draw-text s x y h v)
    (send self :while-button-down
          #'(lambda (nx ny)
              (send self :draw-text s oldx oldy h v)
              (send self :draw-text s nx ny h v)
              (setq oldx nx oldy ny)))
    ;redraw things for erasing
    (send self :draw-text s oldx oldy h v)
    (send self :draw-line x y (first origin) y)
    (send self :draw-line x y x (second origin))
    (send self :draw-mode mode)))

(defmeth andyplot-proto :frame-rect (l2 t2 w2 h2)
  (when (< w2 0) (setq w2 (- w2)) (setq l2 (- l2 w2)))
  (when (< h2 0) (setq h2 (- h2)) (setq t2 (- t2 h2)))
  (call-next-method l2 t2 w2 h2))

(defmeth andyplot-proto :do-zoom-in (x y m1 m2)
  (if (or m1 m2)
      (send self :adjust-to-data)
    (let* ((oldx x)
	   (oldy y)
	   (old-draw-mode (send self :draw-mode))
	   (rect (progn (send self 
			      :while-button-down
			      #'(lambda (nx ny)
				  (send self :draw-mode 'xor)
				  (send self :frame-rect x y
					(- oldx x) (- oldy y))
				  (send self :frame-rect x y
					(- nx x) (- ny y))
				  (setf oldx nx)
				  (setf oldy ny)))
			(list x y (- oldx x) (- oldy y))))
	   (l (nth 0 rect))
	   (to (nth 1 rect))
	   (w (nth 2 rect))
	   (h (nth 3 rect))
	   (lb (send self :canvas-to-scaled l (+ to h)))
	   (tr (send self :canvas-to-scaled (+ l w) to))
	   (xmin (nth 0 lb))
	   (xmax (nth 0 tr))
	   (ymin (nth 1 lb))
	   (ymax (nth 1 tr))
	   )
      (if (= 0 (* w h))
	  (let* (
		 (current-domain (send self :scaled-range 0))
		 (current-range (send self :scaled-range 1))
		 (xmin (nth 0 current-domain))
		 (xmax (nth 1 current-domain))
		 (ymin (nth 0 current-range))
		 (ymax (nth 1 current-range))
		 (xcenter (mean (list xmax xmin)))
		 (ycenter (mean (list ymax ymin)))
		 (xdiff (/ (* *zoom-out-factor* (- xmax xmin)) 2))
		 (ydiff (/ (* *zoom-out-factor* (- ymax ymin)) 2))
		 (xmin (- xcenter xdiff))
		 (xmax (+ xcenter xdiff))
		 (ymin (- ycenter ydiff))
		 (ymax (+ ycenter ydiff))
		 )
	    ;; it would be better to do this using the :scale option (see
	    ;; vista.lsp)
	    (send self :set-ranges
		  (min xmin xmax) (max xmin xmax)
		  (min ymin ymax) (max ymin ymax)
		  )
	    (send self :draw-mode old-draw-mode)
	    )
	(list
	 (send self :frame-rect l to w h)
	 (send self :draw-mode old-draw-mode)
	 (send self :set-ranges
	       (min xmin xmax) (max xmin xmax)
	       (min ymin ymax) (max ymin ymax)
	       )
	 )
	)
      )
    )
;;   (send self :redraw)
  (send self :plot-item)
  )

(defmeth andyplot-proto :set-ranges (xmin xmax ymin ymax &key (recalc nil))
  (send self :scaled-range 0 xmin xmax :draw nil)
  (send self :scaled-range 1 ymin ymax :draw nil)
  (when recalc ()) ;this is to be done...................
  (unless (featurep :macintosh) (send self :redraw))
  (send self :update nil))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;		Use do-key to set the domain and range on the fly
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth andyplot-proto :change-domain ()
  "
Changes the domain, using the get-value-dialog.
"
  (let* (
         (location (send self :location))
	 (domain (get-values-dialog "Domain (e.g. -1 4)" :location location))
	 )
    (if (= 2 (length domain))
	(let()
	 (send self :range 0 (first domain) (second domain))
	 (send self :plot-item)
	 )
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                change-range
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth andyplot-proto :change-range ()
  "
Changes the range, using the get-value-dialog.
"
  (let* (
         (location (send self :location))
	 (range (get-values-dialog "Range (e.g. -1 4)" :location location))
	 )
    (if (= (length range) 2)
	(let()
	 (send self :range 1 (first range) (second range))
	 (send self :plot-item)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        change-domain-and-range
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth andyplot-proto :change-domain-and-range ()
  "
Changes the domain and range, using the :change-domain and :change-range
methods.
"
  (let* (
         (location (send self :location))
	 (domain (get-values-dialog
		  "Domain and Range (e.g. -1 4 -2 3):"
		  :location location))
	 )
    (if (= 4 (length domain))
	(let()
	 (send self :range 0 (first domain) (second domain))
	 (send self :range 1 (third domain) (fourth domain))
	 (send self :plot-item)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			paint-real-rect
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth andyplot-proto :paint-real-rect
  (LEFT TOP WIDTH HEIGHT &optional (col 'blue) (line-col 'black))
  "
Method args: (left top width height)
Fills the rectangle in graph coordinates (LEFT TOP WIDTH HEIGHT) with the
drawing color.
"
  (let* (
	 (xl left)
	 (xr (+ left width))
	 (yl top)
	 (yr (- top height))
	 )
    (send self :add-plot-item
	  (list 'patch
		(list (list xl yl) (list xl yr) (list xr yr) (list xr yl)
		      (list xl yl))
		col line-col))
    )
  nil
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			paint-real-poly
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth andyplot-proto :paint-real-poly
  (poly &optional (col 'blue) (line-col 'black))
  "
Method args: (poly &optional (col 'blue) (line-col 'black))
Paints content of polygon in current draw color. POLY is a list of lists of two
numbers, the first and last elements being joined together.
"
  (send self :add-plot-item (list 'patch poly col line-col))
  nil
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			:clear-content
;;
;; Sun May 24 10:05:38 1998
;; Yesterday I was glancing through Lukes's changes.tex file, and discovered
;; the clear-content method. I have now replaced redraw modifications
;; (previously in the file labels.lsp) with clear-content modifications.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth andyplot-proto :clear-content()
  (call-next-method)
  (if (send self :has-slot 'jason-horiz-vert)
      (let* (
	     (v-h (send self :jason-horiz-vert))
	     (horiz (send self :jason-horiz))
	     (vert (send self :jason-vert))
	     (method
	      (mapcar
	       #'(lambda (x) (if (= x 0) :draw-text :draw-text-up))
	       v-h))
	     (coords (send self :jason-label-coordinates))
	     (labels (send self :jason-plot-labels))
	     )
	(mapcar
	 #'(lambda (x y z h v) (send self z x (first y) (second y) h v))
	 labels coords method horiz vert)
	))
  (if (send self :slot-value 'text)
      (apply #'send self :text (send self :slot-value 'text))))


(defmeth andyspin-proto :clear-content ()
  (call-next-method)
  (if (send self :has-slot 'jason-horiz-vert)
      (let* (
	     (v-h (send self :jason-horiz-vert))
	     (horiz (send self :jason-horiz))
	     (vert (send self :jason-vert))
	     (method
	      (mapcar
	       #'(lambda (x) (if (= x 0) :draw-text :draw-text-up))
	       v-h))
	     (coords (send self :jason-label-coordinates))
	     (labels (send self :jason-plot-labels))
	     )
	(mapcar
	 #'(lambda (x y z h v) (send self z x (first y) (second y) h v))
	 labels coords method horiz vert))))

