;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			Initialize the plot-variables:
#|
	This is a tough one for the compiler - all those defs must be
trouble-some.
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun new-plot ()
  "
New-plot reinitializes the plotting variables (plot-variables). These can be
individually set using the functions

        mouse-mode      ; to 'zoom-in
	zoom 		; sets the zoom factor
	domain		; set the domain for a 2-D plot
	range		; set the range for a 2-D plot
	polar-domain	; set the domain for a polar plot
	polar-range	; set the range for a polar plot
	title		; create a title for a plot
	x-axis		; include (or remove) the x-axis
	y-axis		; include (or remove) the y-axis
	x-tics		; number of tics on the x-axis (default: 5)
	y-tics		; number of tics on the y-axis (default: 5)
	lines		; use lines or not (points will be used instead)
	points		; the number of points in a plot
	plot-symbol	; sets the symbol for plot in the plane
	point-labels	; sets the labels for plots in the plane
	variable-labels	; sets the variable labels for plots in the 2 and 3-D
        use-color	; toggles the use of color for a plot
        line-colors	; sets the colors available for lines
        draw-color	; sets the default line color

	spin-depth	; include the depth-showing feature of a 3-D function
	spin-axes	; to have axes showing or not
	real-axes	; to have true origin-based axes showing or not
	spin-domain	; for the actual domain of the function
	spin-points	; number of spin points (matrix will be square)
	spin-spline	; the number of spline terms for a plot
	spin-symbol	; sets the symbol for plotting points in spin

        contour-points	; the number of contour points
        contour-levels	; the number of contour levels

        verbose         ; toggles verbosity (some things will talk to you!)
        andy-mode       ; makes plots have overlays that andy likes
"
  (let()
   (def *andy-mode* t)
   (def *show-plots* t)
   (def *mouse-mode* 'zoom-in)
   (def *verbose* nil)

   (def *zoom-out-factor* 1.1)
   (def *zoom-in-factor* .9)
   (def *shift-factor* .1)
   (def *domain* '(-1 1))
   (def *range* nil)
   (def *title* "Plot Command")
   (def *x-axis* 't)
   (def *y-axis* 't)
   (def *lines* 't)
   (def *points* 100)
   (def *plot-symbol* 'disk)
   (def *point-labels* nil)
   (def *variable-labels* nil)

   (def *use-color* t)
   (def *line-colors* '(black blue red yellow cyan magenta green))
   (def *line-type* 'solid)
   (def *line-width* 1)
   (def *draw-color* 'blue)

   (def *polar-domain* '(0 1))
   (def *polar-range* (list 0 (* 2 pi)))
   (def *polar-points* 10)

   (def *x-tics* 5)
   (def *y-tics* 5)

   (def *spin-depth* nil)
   (def *spin-axes* t)
   (def *real-axes* t)
   (def *spin-domain* '(-1 1 -1 1))
   (def *spin-points* 6)
   (def *spin-spline* nil)
   (def *spin-symbol* 'cross)
   (def *true-axes-labels* '("O" "X" "Y" "Z"))

   (def *contour-points* 30)
   (def *contour-levels* 10)
   )
  "New plot environment."
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          remove-missing
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun remove-missing (data &rest value)
  "
;; Description: (remove-missing data &optional value &key test)

	Eliminates missing values (rows) from a matrix. It assumes that the
missing values are '.' (that's what sas gives, which inspired this function).

	The test should be TRUE for rows you want removed, and it should be a
function (see the examples below).

;; Notes: When using not-a-number, it's best to use the not-a-numberp test (as
;; in the last example below): not-a-number has peculiar properties!

;; Examples:

 (remove-missing (matrix '(2 2) (list #.NOT-A-NUMBER 3 2 '.)))
 ;; #2A((#.NOT-A-NUMBER 3))

 (remove-missing
   (matrix '(2 2) (list #.NOT-A-NUMBER 3 2 '.))
   #.NOT-A-NUMBER
   )
 ;; #2A((2 .))

 (remove-missing
   (matrix '(2 2) (list #.NOT-A-NUMBER 3 2 '.))
   3
   )
 ;; #2A((2 .))

 (remove-missing #2A((#.NOT-A-NUMBER 3) (2 '.)))
 ;; #2A((#.NOT-A-NUMBER 3))

 (remove-missing #2A((nil 3) (2 '.)) :test #'(lambda(x) (equalp x nil)))
 ;; #2A((2 (QUOTE .)))
 ;; An easier way to do the same thing:
 (remove-missing #2A((nil 3) (2 '.)) 'nil)
 ;; #2A((2 (QUOTE .)))
   
 (remove-missing #2A((1 #.NOT-A-NUMBER 3) (1 #.NOT-A-NUMBER 4) (4 2 '.)))
 ;; #2A((1 #.NOT-A-NUMBER 3) (1 #.NOT-A-NUMBER 4))

 (remove-missing
   #2A((1 #.NOT-A-NUMBER 3) (1 #.NOT-A-NUMBER 4) (4 2 5) (#.NOT-A-NUMBER 4 1))
   :test #'not-a-numberp
   )
 ;; #2A((4 2 5))

 (remove-missing
   #2A((1 #.NOT-A-NUMBER 3) (1 #.NOT-A-NUMBER 4) (4 2 5) (#.NOT-A-NUMBER 4 1))
   #.not-a-number
   )
 ;; #2A((4 2 5))
"
  (andy-print "in remove-missing")
  (let* (
	 (test (first (catch-keys '(:test) value)))
	 (value (if test (remove-keys '(:test) value) value))
	 (value (if value (first value)))
	 (test
	  (cond
	   ((equalp value 'nil) (lambda(x) (equalp x nil)))
	   (test test)
	   (value (if (not-a-numberp value) #'not-a-numberp #'equalp))
	   (t #'(lambda(x) (or (equalp x (quote '.)) (equalp x '.)))))
	  )
	 (value (if (not-a-numberp value) nil value))
	 (coerce
	  (cond
	   ((listp data) 'list)
	   ((vectorp data) 'vector)
	   (t nil)))
	 (data (if coerce (matrix (list (length data) 1) data) data))
	 (size (size data))
	 (nrows (first size))
	 (ncols (second size))
	 (choose (iseq 0 (1- nrows)))
	 result
	 )
    (if value
	(dotimes
	 (i nrows)
	 (dotimes
	  (j ncols)
	  (if (funcall test value (aref data i j)) (setf (elt choose i) nil))
	  ))
      (dotimes
       (i nrows)
       (dotimes
	(j ncols)
	(if (funcall test (aref data i j)) (setf (elt choose i) nil))
	)
       )
      )
    (setf choose (where choose nil #'neq))
    (if (= 0 (length choose)) nil
      (let()
	(setq result (select-rows data choose))
	(values
	 (if coerce (coerce (column 1 result) coerce) result)
	 choose
	 )
	)
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                physical constants
;;
;; see clmath in the contrib directory
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defcon velocity-of-light	c	2.9979250e8 meters per second)
(defcon electronic-charge	q	1.6021917e-19 coulomb)
(defcon Plancks-constant	h	6.626196e-34 Joule second)
(defcon Avagadros-number	N	6.022169e26 per K mole)
(defcon atomic-mass-unit	amu	1.660531e-27 K gram)
(defcon electron-rest-mass	me	9.109558e-31 K gram)
(defcon proton-rest-mass	mp	1.672614e-27 K gram)
(defcon magnetic-flux-quantum	phi	2.067854e-15 Tesla meter meter)
(defcon Boltzmanns-constant	k	1.380622e-23 Joule per Kelvin)
(defcon gravitational-constant	gamma	6.6732e-11 Newton (meter 2) per (K gram 2))

(defcon permeability-free-space mu0     (* 4.0 pi 1.0e-7) henry per meter)
(defcon permittivity-free-space epsilon0 8.854e-12 farads per meter)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        Aliases - functions
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; misc
(alias find-descendents-of find-descendants-of)
(alias xlisp::get-lambda-expression get-lambda-expression)
(alias tut tutorials)
(alias cwd get-working-directory)
(alias pwd get-working-directory)
(alias where-am-i nilly)
(alias andy-print nilly)

;; misc math
(alias real realpart)
(alias imag imagpart)
(alias xs abscissa)
(alias ys ordinates)
(alias fixnump numberp)

;; matrix
(alias XtX cross-product)
(alias matprod matmult)
(alias changem change-matrix-value)
(alias lap link-all-plots)
(alias zeromat zeros)
(alias diagonal-to-full diagonal-to-matrix)
(alias cholesky chol-decomp)
(alias matrix-bind combine-matrices)
(alias reduce-to-basis span-vectors)

;; string
(alias substring subseq)

;; stats:
(alias rand.cauchy cauchy-rand)
(alias rand.normal normal-rand)
(alias rand.uniform uniform-rand)
(alias rand.n normal-rand)
(alias rand.u uniform-rand)
(alias rand.chisq chisq-rand)
(alias rand.gamma gamma-rand)
(alias rand.poisson poisson-rand)
(alias rand.t t-rand)
(alias rand.beta beta-rand)
(alias rand.binomial binomial-rand)
(alias rand.f f-rand)
(alias uniform-dens rectangular-dens)
(alias uniform-cdf rectangular-cdf)
(alias uniform-quant rectangular-quant)
(alias correlation-ci fishers-z)
(alias paired-t-test paired-t)
(alias sd standard-deviation)
(alias tosses repeated-sampling-subsets)
(alias odds odds-ratio)
(alias rr relative-risk)
(alias inverse-normal normal-quant)

;; plotting:
(alias cap close-all-plots)
(alias caw close-all-windows)
(alias agw active-graph-windows)

(alias hidden hid)
(alias dos-display-help X11-display-help)
(alias mac-display-help X11-display-help)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                        Aliases - macros
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; misc:
(macro-alias cmv catch-multiple-values)
(macro-alias try try-out)
(macro-alias new-accessor-method new-assessor-method)

;; math
(macro-alias derivative diff)
(macro-alias nth-diff diff-n)
