#|

Some potential problems:

	In hidden-line, if a point is not visible, it is currently marked as
1e31. These should be handled better!

	I don't like that r11 stuff.

	I have to laugh: it suddenly occurs to me that I know exactly the
profile of the figure! That is, I know the horizons. Hidden-line calculates the
projections, for all points, and those I can use to find the base-line and
top-line.
|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; insert-y-in-x  Doc:    Creation Date:5/21/96        Author: Andy Long
;;
;;
(defun insert-y-in-x (x y &optional xx yy)
  "
;; Description/Arguments:

       (insert-y-in-x x y &optional xx yy)
  
;; Examples:
 (insert-y-in-x '(0 2 5) '(3 4)) ;; should get (0 2 3 4 5)
 (insert-y-in-x '(0 2 3.5 5) '(3 4)) ;; should still get (0 2 3 4 5)
 ;; Notice that the segment (3 4) replaces the point 3.5 (which was in that
 ;; segment). The segment (3 4) 'over-writes' 3.5.
 (insert-y-in-x '(0 2) '(3 4)) ;; should get (0 2 3 4)
 (insert-y-in-x '(5 6) '(3 4)) ;; should get (3 4 5 6)
 (insert-y-in-x '(5 6) '(3 4) '(0 2) '(3 4)) ;; can insert an associated vector
"
  (where-am-i 'insert-y-in-x)
  (let (
	val
	val2
	(y1 (first y))
	(ylast (first (last y)))
	i
	)
    (do
     ((i 0 (1+ i)))
     ((or (>= i (length x)) (>= (elt x i) y1)))
     (let()
       (setq val (append val (list (elt x i))))
       (if xx (setq val2 (append val2 (list (elt xx i)))))
       )
     )
    (setq val (append val y))
    (if xx (setq val2 (append val2 yy)))
    (setq i (1- (length x)))
    (do
     ()
     ((or (< i 0) (<= (elt x i) ylast)))
     (setq i (1- i))
     )
    (if (< i (1- (length x)))
	(let()
	  (setq val
		(append val
			(select x
				(remove-duplicates
				 (iseq (1+ i) (1- (length x)))))))
	  
	  (if xx (setq val2
		       (append val2
			       (select xx
				       (remove-duplicates
					(iseq (1+ i) (1- (length xx)))))))
	    ))
      )
    (where-am-i 'leaving-insert-y-in-x)
    (if xx (list val val2) val)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; pmin-interp Doc:    Creation Date:5/21/96        Author: Andy Long
;;
;;
(defun pmin-interp(x a b &optional (tol 1e-6))
  "
;; Description/Arguments:

       (pmin-interp x a b &optional (tol 1e-6))

Takes two arrays for the same x values. If there is a crossing, then adds a
point for the crossing. Returns the x values (possibly augmented) and the
min values.

;; Notes:

;; See Also:

;; Examples:
 (pmin-interp (iseq 4) '(1 -1 2 3) '(-1 3 2 1))
 (pmin-interp (iseq 4) '(1 -1 2 3) '(-1 3 2.5 1))
 (pmin-interp (iseq 4) '(1 -1 2 3) '(-1 3 9 1))
 (def x (sort-data (rand.u 10)))
 (def a (rand.n 10))
 (def b (rand.n 10))
 (lines off)
 (plot x a)
 (send my-plot :add-points x b)
 (def c (pmin-interp x a b))
 (send my-plot :add-lines (first c) (second c))
"
  (let (
	vals
	xvals
	denom
	newvals
	newxvals
	(lenx (length x))
	switch
	(a1 (first a))
	(b1 (first b))
	(x1 (first x))
	)
    (if (< a1 b1)
	(setq vals (list a1))
      (let()
	(setq vals (list b1))
	(setq switch t))
      )
    (setq xvals (list x1))
    
    (dolist
     (i (iseq 1 (1- lenx)))
     (let (
	   (ai (elt a i))
	   (bi (elt b i))
	   (xi (elt x i))
	   )
       (cond
	((and switch (< ai bi))
	 (setq switch nil) ;; there's a switch
	 (setq denom (- (+ b1 ai) (+ bi a1)))
	 (andy-print 'denom1)
	 (andy-print denom)
	 (setq newxval
	       (if (< denom tol) 
		   x1
		 (+ x1 (/ (* (- b1 a1) (- xi x1)) denom))
		 ))
	 (setq newxvals (list newxval xi))
	 (setq denom (- xi x1))
	 (andy-print 'denom2)
	 (andy-print denom)
	 (setq newvals (list
			(+ a1 (* (/ (- ai a1) denom) (- newxval x1)))
			ai
			))
	 )
	((and switch (>= ai bi))
	 (setq newxvals (list xi))
	 (setq newvals (list bi))
	 )
	((and (not switch) (< ai bi))
	 (setq newxvals (list xi))
	 (setq newvals (list ai))
	 )
	(t
	 (setq switch t) ;; there's a switch
	 (setq denom (- (+ b1 ai) (+ bi a1)))
	 (andy-print 'denom1)
	 (andy-print denom)
	 (setq newxval
	       (if (< denom tol) 
		   x1
		 (+ x1 (/ (* (- b1 a1) (- xi x1)) denom))
		 ))
	 (setq newxvals (list newxval xi))
	 (setq denom (- xi x1))
	 (andy-print 'denom2)
	 (andy-print denom)
	 (setq newvals (list
			(+ a1 (* (/ (- ai a1) denom) (- newxval x1)))
			bi
			))
	 ))
       (setq vals (append vals newvals))
       (setq xvals (append xvals newxvals))
       (setq a1 ai)
       (setq b1 bi)
       (setq x1 xi)
       )
     )
    (list xvals vals)
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; pmax-interp Doc:    Creation Date:5/21/96        Author: Andy Long
;;
;;
(defun pmax-interp(x a b &optional (tol 1e-6))
  "
;; Description/Arguments:

       (pmax-interp x a b &optional (tol 1e-6))

Takes two arrays for the same x values. If there is a crossing, then adds a
point for the crossing. Returns the x values (possibly augmented) and the
min values.

;; Notes:

;; See Also:

;; Examples:
 (pmax-interp (iseq 4) '(1 -1 2 3) '(-1 3 2 1))
 (pmax-interp (iseq 4) '(1 -1 2 3) '(-1 3 2.5 1))
 (pmax-interp (iseq 4) '(1 -1 2 3) '(-1 3 9 1))
 (def x (sort-data (rand.u 10)))
 (def a (rand.n 10))
 (def b (rand.n 10))
 (lines off)
 (plot x a)
 (send my-plot :add-points x b)
 (def c (pmax-interp x a b))
 (send my-plot :add-lines (first c) (second c))
"
  (let (
	vals
	xvals
	denom
	newvals
	newxvals
	(lenx (length x))
	switch
	(a1 (first a))
	(b1 (first b))
	(x1 (first x))
	)
    (if (> a1 b1)
	(setq vals (list a1))
      (let()
	(setq vals (list b1))
	(setq switch t))
      )
    (setq xvals (list x1))
    
    (dolist
     (i (iseq 1 (1- lenx)))
     (let (
	   (ai (elt a i))
	   (bi (elt b i))
	   (xi (elt x i))
	   )
       (cond
	((and switch (> ai bi))
	 (setq switch nil) ;; there's a switch
	 (setq denom (- (+ b1 ai) (+ bi a1)))
	 (andy-print 'denom1)
	 (andy-print denom)
	 (setq newxval
	       (if (< denom tol) 
		   x1
		 (+ x1 (/ (* (- b1 a1) (- xi x1)) denom))
		 ))
	 (setq newxvals (list newxval xi))
	 (setq denom (- xi x1))
	 (andy-print 'denom2)
	 (andy-print denom)
	 (setq newvals (list
			(+ a1 (* (/ (- ai a1) denom) (- newxval x1)))
			ai
			))
	 )
	((and switch (<= ai bi))
	 (setq newxvals (list xi))
	 (setq newvals (list bi))
	 )
	((and (not switch) (> ai bi))
	 (setq newxvals (list xi))
	 (setq newvals (list ai))
	 )
	(t
	 (setq switch t) ;; there's a switch
	 (setq denom (- (+ b1 ai) (+ bi a1)))
	 (andy-print 'denom1)
	 (andy-print denom)
	 (setq newxval
	       (if (< denom tol) 
		   x1
		 (+ x1 (/ (* (- b1 a1) (- xi x1)) denom))
		 ))
	 (setq newxvals (list newxval xi))
	 (setq denom (- xi x1))
	 (andy-print 'denom2)
	 (andy-print denom)
	 (setq newvals (list
			(+ a1 (* (/ (- ai a1) denom) (- newxval x1)))
			bi
			))
	 ))
       (setq vals (append vals newvals))
       (setq xvals (append xvals newxvals))
       (setq a1 ai)
       (setq b1 bi)
       (setq x1 xi)
       )
     )
    (list xvals vals)
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			remove-close-xvals
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun remove-close-xvals (xx yy &optional (eps 1e-6) (max-min #'max))
  "
;; Description: (remove-close-xvals xx yy &optional (eps 1e-6) (max-min #'max))

;; Examples:
 (def a (list 0 1e-12 4))
 (def b (list 0.87654677 0.89631069 0.70432941))
 (remove-close-xvals a b)
 (remove-close-xvals a b 1e-6 #'min)
"
  (let* (
	 (x (simul-sort xx yy))
	 (y (second x))
	 (x (first x))
	 )
    (dotimes
     (i (- (length x) 2))
     (if (< (abs (- (elt x (1+ i)) (elt x i))) eps)
	 (list 
	  (setf (elt x (1+ i)) (mean (list (elt x (1+ i)) (elt x i))))
	  (setf (elt y (1+ i))
		(funcall max-min (list (elt y (1+ i)) (elt y i))))
	  (setf (elt x i) nil)
	  (setf (elt y i) nil)
	  )
       )
     )
    (list (remove-if 'null x) (remove-if 'null y))
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                hidden-line
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun hidden-line
  (f eye-position &optional view-point up angle angle-divy near-sight)
  "

		hidden-line routine (after my dad's algorithm)

	1) set an eye position (view position is the origin).
	2) get the data (or form them from a function)
	3) determine the perspective view on the viewing plane, perpendicular
to the eye-position vector and through the origin.

	This is only a helper function for hidsurf, which doesn't work
properly. It involves a method of hidden line generation with which I'm
experimenting.

;; Description:  (hidden-line f eye-position)

;; See RATHER: hid, hid-quad. These functions are accessible from a 2-d plot,
;; using single keystroke 'b'.

;; Examples: 
 (func f(x y) x * y)
 (data eye-position 3 3 3)
 (hidden-line #'f eye-position)
"
  (let* (
	 (eyex (first eye-position))
	 (eyey (second eye-position))
	 (eyez (third eye-position))

	 (r (norm eye-position))
	 (phi (acos (/ eyez r)))
	 (th (argument (complex eyex eyey)))

	 (e (/ eye-position r))
	 (v1 (list (cos (- th pi/2)) (sin (- th pi/2)) 0))
	 (v2 (cross v1 e))

	 (xmin (first *spin-domain*))
	 (xmax (second *spin-domain*))
	 (ymin (third *spin-domain*))
	 (ymax (fourth *spin-domain*))
	 (xvals (rseq xmin xmax *spin-points*))
	 (yvals (rseq ymin ymax *spin-points*))

	 (matvals (outer-product xvals yvals f))

	 (xs-to-plot
	  (make-array (list *spin-points* *spin-points*)
		      :initial-element 1e31
		      ))
	 (ys-to-plot
	  (make-array (list *spin-points* *spin-points*)
		      :initial-element 1e31
		      ))

	 (closest-x 0)
	 (closest-y 0)
	 (close (distance eye-position
			  (list (elt xvals 0) (elt yvals 0)
				(aref matvals 0 0))))
	 )
    (dolist
     (i (list 0 (1- *spin-points*)))
     (dolist
      (j (list 0 (1- *spin-points*)))
      (let* (
	     (x (elt xvals i))
	     (y (elt yvals j))
	     (z (aref matvals i j))
	     (v (list x y z))
	     (closer (distance eye-position v))
	     )
	(if (> close closer)
	    (let()
	      (setf close closer)
	      (setf closest-x i)
	      (setf closest-y j)
	      )
	  )
	)
      )
     )
    (dotimes
     (i *spin-points*)
     (let ((x (elt xvals i)))
       (dotimes
	(j *spin-points*)
	(let* (
	       (y (elt yvals j))
	       (z (aref matvals i j))
	       (v (list x y z))
	       (tt (/ r (- r (dot v e))))
	       )
	  (if (> tt 0)
	      (let()
		(setf (aref xs-to-plot i j) (* tt (dot v v1)))
		(setf (aref ys-to-plot i j) (* tt (dot v v2)))
		)
	    )
	  )
	)
       )
     )
    (list xs-to-plot ys-to-plot closest-x closest-y)
    )
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                hidsurf - 2
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun hidsurf (f eye-position
		  &key
		  (fineness *spin-points*)
		  (mf 100)
		  (ptol 1e-2)
		  (ctol 1e-2)
		  (stol 1e-2)
		  plot
		  )
  "
;; Description: (hidsurf f eye-position
		  &key
		  (fineness *spin-points*)
		  (mf 100)
		  (ptol 1e-2) ;; pmax-interp tolerance
		  (ctol 1e-2) ;; removing close values tolerance
		  (stol 1e-2) ;; spline removing tolerance
		  plot
		  )

				hidden-line routine

 (after my dad's zig-zag algorithm, that I once made into hidsurf.f).

	1) set an eye position (view position is the origin).
	2) get the data (or form them from a function)
	3) determine the perspective view on the viewing plane, perpendicular
	   to the eye-position vector and through the origin.

;; See RATHER: hid, hid-quad. These functions are accessible from a 2-d plot,
;; using single keystroke 'b'.

;; Examples:
 (func f(x y) x*y)
 (def eye-position '(10 9 10))
 (hidsurf #'f eye-position)
 (def eye-position '(10 2 10))
 (hidsurf #'f eye-position)

 (spin-domain -1 1 -1 1)
 (func f(x y) x*y)
 (def eye-position '(10 6 5))
 (hidsurf #'f eye-position)
 (def eye-position '(10 9 5))
 (hidsurf #'f eye-position)
 (def eye-position '(10 2 5))
 (hidsurf #'f eye-position)

 (spin-domain -pi pi -pi pi)
 (func g(x y) (cos x) * (sin y))
 (def eye-position '(10 9 5))
 (hidsurf #'g eye-position)
 (def eye-position '(10 9 -5))
 (hidsurf #'g eye-position)
 (def eye-position '(4 -10 10))
 (hidsurf #'g eye-position)
"
  (let* (
	 (n1 *spin-points*)
	 (n2 *spin-points*)
	 ;; hidden-line gets the perspective views, and
	 (q (hidden-line f eye-position))
	 (p (first q))
	 ;; determines which corner is closest to the viewer.
	 (closest-x (1+ (third q)))
	 (closest-y (1+ (fourth q)))
	 (q (second q))

	 ;; fire up a two-d scatterplot:
	 (myplot
	  (if plot plot
	    (send andyplot-proto :new 2 :title *title* :show nil)
	    )
	  )

	 ;; get the rows and columns that we know are visible (up front):
	 (x1 (row closest-x p))
	 (x2 (column closest-y p))
	 (y1 (row closest-x q))
	 (y2 (column closest-y q))
	 x3 y3
	 
	 ;; watch the order here!!!!
	 (y1 (if (> (first x1) (first (last x1))) (reverse y1) y1))
	 (x1 (if (> (first x1) (first (last x1))) (reverse x1) x1))
	 (y2 (if (> (first x2) (first (last x2))) (reverse y2) y2))
	 (x2 (if (> (first x2) (first (last x2))) (reverse x2) x2))

	 ;; decide if we need to swap the rows and columns (they're backwards):
	 (r11 (if (> (min x1) (min x2)) t nil))
	 (r22 (if (or
		   (and
		    (<= (min x1) (min x2))
		    (>= (max x1) (max x2))
		    )
		   (and
		    (<= (min x2) (min x1))
		    (>= (max x2) (max x1))
		    )
		   )
		  t nil))
	 ;; if r22, then one is entirely to one side of the other

	 ;; these are some local variables:
	 base-line-new
	 top-line-new

	 base-line-x
	 base-line-y
	 top-line-x
	 top-line-y
	 rowlist
	 collist
	 newy newx newyy
	 )

    (if r11 
	(setq x3 x1
	      x1 x2
	      x2 x3
	      y3 y1
	      y1 y2
	      y2 y3
	      )
      )
    
    ;; x values are going to run from min x1 to max x2: we establish a pair of
    ;; vectors which are going to represent horizons, top and bottom.
    (if r22
	(if (<= (min x1) (min x2))
	    (setq base-line-x x1 base-line-y y1)
	  (setq base-line-x x2 base-line-y y2)
	  )
      (setq base-line-x (combine x1 (rest x2)) 
	    base-line-y (combine y1 (rest y2)))
      )

    (setq top-line-x base-line-x top-line-y base-line-y)
      
    ;; We can plot those for sure!
    (send myplot :show-window)
    (send myplot :add-lines base-line-x base-line-y)
    (send myplot :title (strcat (num-to-string closest-x) " "
				 (num-to-string closest-y)))
    (send myplot :adjust-to-data)
    
    ;; Now we want to rove over the rows and columns in the appropriate order:
    (setq rowlist (if (< 1 closest-x) (iseq (1+ closest-x) 1) (iseq 0 n1)))

    (dotimes
     (k (- (min (list n1 n2)) 0))
     (setq rowlist (rest rowlist))
     (let ((i (first rowlist)))
       (setq x3 (row i p) y3 (row i q))
       (if (> (first x3) (first (last x3))) 
	   (setq x3 (reverse x3) y3 (reverse y3)))
       
       (if (> fineness *spin-points*)
	   (setq tmp (lin-spline x3 y3 (min mf (* fineness *spin-points*)))
		 x3 (first tmp)
		 y3 (second tmp)))
       
       (setq newy (second (lin-spline top-line-x top-line-y x3 stol 0))
	     top-line-new (pmax newy y3))
       (if (/= 0 (norm (heaviside (- top-line-new newy))))
	   (let()
	     (setq
	      top-line-new (pmax-interp x3 newy y3 ptol)
	      newx (first top-line-new)
	      top-line-new (second top-line-new)
	      tmp (insert-y-in-x top-line-x newx top-line-y top-line-new)
	      top-line-x
	      (remove-close-xvals (first tmp) (second tmp) ctol #'max)
	      top-line-y (second top-line-x)
	      top-line-x (first top-line-x)
	      )
	     (send myplot :add-lines newx top-line-new)))
       
       (setq
	newy (second (lin-spline base-line-x base-line-y x3 stol 1))
	base-line-new (pmin newy y3))

       (if (/= 0 (norm (heaviside (- newy base-line-new)))) ;; note switch
	   (let()
	     (setq
	      base-line-new (pmin-interp x3 newy y3 ptol)
	      newx (first base-line-new)
	      base-line-new (second base-line-new)
	      tmp (insert-y-in-x base-line-x newx base-line-y base-line-new)
	      base-line-x
	      (remove-close-xvals (first tmp) (second tmp) ctol #'min)
	      base-line-y (second base-line-x)
	      base-line-x (first base-line-x))
	     (send myplot :add-lines newx base-line-new)
	     )
	 )
       )
     )
    
    ;; x values are going to run from min x1 to max x2: we establish a pair of
    ;; vectors which are going to represent horizons, top and bottom.
    (if r22
	(if (<= (min x1) (min x2))
	    (setq base-line-x x1 base-line-y y1)
	  (setq base-line-x x2 base-line-y y2)
	  )
      (setq base-line-x (combine x1 (rest x2))
	    base-line-y (combine y1 (rest y2)))
      )
    (setq top-line-x base-line-x top-line-y base-line-y)
    
    ;; Now we want to rove over the rows and columns in the appropriate order:
    (setq collist (if (< 1 closest-y) (iseq (1+ closest-y) 1) (iseq 0 n2)))
    
    (dotimes
     (k (- (min (list n1 n2)) 0))
     (setq collist (rest collist))
     (let ((j (first collist)))
       (setq x3 (column j p) y3 (column j q))
       (if (> (first x3) (first (last x3)))
	   (setq x3 (reverse x3) y3 (reverse y3))
	 )
       (if (> fineness *spin-points*)
	   (setq
	    tmp (lin-spline x3 y3 (min mf (* fineness *spin-points*)))
	    x3 (first tmp)
	    y3 (second tmp)
	    )
	 )
       (setq
	newy (second (lin-spline top-line-x top-line-y x3 stol 0))
	top-line-new (pmax newy y3))
       (if (/= 0 (norm (heaviside (- top-line-new newy))))
	   (let()
	     (setq
	      top-line-new (pmax-interp x3 newy y3 ptol)
	      newx (first top-line-new)
	      top-line-new (second top-line-new)
	      tmp (insert-y-in-x top-line-x newx top-line-y top-line-new)
	      top-line-x
	      (remove-close-xvals (first tmp) (second tmp) ctol #'max)
	      top-line-y (second top-line-x)
	      top-line-x (first top-line-x))
	     (send myplot :add-lines newx top-line-new)
	     )
	 )
       
       (setq newy (second (lin-spline base-line-x base-line-y x3 stol 1))
	     base-line-new (pmin newy y3))
       
       (if (/= 0 (norm (heaviside (- newy base-line-new)))) ;; note switch
	   (let()
	     (setq
	      tmp (pmin-interp x3 newy y3 ptol)
	      newx (first tmp)
	      base-line-new (second tmp)
	      tmp
	      (insert-y-in-x base-line-x newx base-line-y base-line-new)
	      base-line-x
	      (remove-close-xvals (first tmp) (second tmp) ctol #'min)
	      base-line-y (second base-line-x)
	      base-line-x (first base-line-x))
	     (send myplot :add-lines newx base-line-new)
	     )
	 )
       )
     )
    (send myplot :add-lines base-line-x base-line-y :color 'blue)
    (send myplot :add-lines top-line-x top-line-y :color 'yellow)
    (send myplot :adjust-to-data)
    myplot
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                hidsurf-movie
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun hidsurf-movie (f rad z)
  "
;; Examples:
 (func f(x y) x*y)
 (hidsurf-movie #'f 10 10)
 (func f(x y) x*x)
 (hidsurf-movie #'f 10 10)
"
  (let (
	(myplot (hidsurf f (list rad 0 z)))
	)
    (dotimes (i 36)
	     (send myplot :start-buffering)
	     (send myplot :clear-lines :draw nil)
	     (hidsurf
	      f 
	      (list
	       (* rad (cos (/ (* i 2pi) 36)))
	       (* rad (sin (/ (* i 2pi) 36)))
	       z
	       )
	      :plot myplot
	      )
	     (send myplot :buffer-to-screen)
	     )
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                hid
;;
#|
	This version of hid uses triangles. If you would prefer quads in a
certain situation, then
(alias hid hid-quad)
would do it.
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun hid (&key
	       plot
	       patch
	       (up-color 'blue)
	       (down-color 'red)
	       (line-color 'black)
	       )
  "
;; Description: function hid
 
      Patches plot (i.e. hides hidden lines) of a function. This can be called
from a 2-d plot, with a simple 'b' typed in the window.

;; Arguments:
	(&key
	       plot
	       patch
	       (up-color 'blue)
	       (down-color 'red)
	       (line-color 'black)

;; See also: :hidden method for andyspin-proto.

;; Examples:
 (hid :plot (plot (lambda(x y) (* x y))))
 (hid :plot (plot (lambda(x y) (* x y))) :patch t)
"
  (andy-print "in hid")
  (let* (
	 (plot (if plot plot (first (agw))))
	 (lines 
	  (if *real-axes* (rest (send plot :getlines)) (send plot :getlines)))
	 (m (if (matrixp (first (send plot :slot-value 'plot-item)))
		(second (size (first (send plot :slot-value 'plot-item))))
	      nil
	      )
	    )
	 (lines (if m (select lines (iseq m)) lines))
	 (trans (send plot :transformation))
	 (mat (matprod trans
		       (diagonal-to-matrix (/ (send plot :scale (iseq 3))))))
	 (a (remove-duplicates (combine (mapcar (lambda(x) (first x)) lines))))
	 (b (remove-duplicates (combine (mapcar (lambda(x) (second x)) lines))))
	 (grid (apply #'bind-rows (mapcar (lambda(x) (third x)) lines)))

	 (grid (if m (transpose grid) grid))
	 (m (1- (length a)))
	 (n (1- (length b)))
	 (eye (solve mat '(0 0 1.7))) ;; got the 1.7 value out of calcsphere...
	 (eyex (first eye))
	 (eyey (second eye))
	 (eyez (third eye))
	 (myplot (send andyplot-proto :new 2 :show nil))
	 (xmin 1e31) (ymin 1e31)
	 (xmax -1e31) (ymax -1e31)
	 (xfactor .1)
	 (yfactor .1)
	 proj d xshift yshift
	 )
    ;; get distances to vertices and directions (up or down). Directions will
    ;; be used to decide the color of a patch.
    (andy-print "get dist")
    (dotimes
     (i m)
     (let* (
	    (ii (1+ i))
	    (x1 (elt a i))
	    (x2 (elt a ii))
	    (xmean (mean (list x1 x2)))
	    )
       (dotimes
	(j n)
	(let* (
	       (jj (1+ j))
	       (y1 (elt b j))
	       (y2 (elt b jj))
	       ;; four adjacent points from the grid...
	       (z11 (aref grid i j))
	       (z12 (aref grid i jj))
	       (z21 (aref grid ii j))
	       (z22 (aref grid ii jj))
	       (ymean (mean (list y1 y2)))
	       (zmean (mean (list z11 z12 z21 z22)))
	       )
	  (setq d
		(append d
			(list
			 (distance eye (list
					(mean (list x1 x1 xmean))
					(mean (list y1 y2 ymean))
					(mean (list z11 z12 zmean))
					)
				   )
			 (distance eye (list
					(mean (list x1 x2 xmean))
					(mean (list y1 y1 ymean))
					(mean (list z11 z21 zmean))
					)
				   )
			 (distance eye (list
					(mean (list x1 x2 xmean))
					(mean (list y2 y2 ymean))
					(mean (list z12 z22 zmean))
					)
				   )
			 (distance eye (list
					(mean (list x2 x2 xmean))
					(mean (list y1 y2 ymean))
					(mean (list z21 z22 zmean))
					)
				   )
			 )))
	  (setf
	   proj
	   (append
	    proj
	    (list
	     (list 
	      (signum (dot eye
			   (cross
			    (- (list xmean ymean zmean) (list x1 y1 z11))
			    (- (list x1 y2 z12) (list x1 y1 z11))
			    )))
	      (list x1 y1 z11)
	      (list x1 y2 z12)
	      (list xmean ymean zmean)
	      )
	     (list
	      (signum (dot eye
			   (cross
			    (- (list x2 y1 z21) (list x1 y1 z11))
			    (- (list xmean ymean zmean) (list x1 y1 z11))
			    )))
	      (list x1 y1 z11)
	      (list x2 y1 z21)
	      (list xmean ymean zmean)
	      )
	     (list
	      (signum (dot eye
			   (cross
			    (- (list xmean ymean zmean) (list x1 y2 z12))
			    (- (list x2 y2 z22) (list x1 y2 z12))
			    )))
	      (list x1 y2 z12)
	      (list x2 y2 z22)
	      (list xmean ymean zmean)
	      )
	     (list
	      (signum (dot eye
			   (cross
			    (- (list x2 y2 z22) (list x2 y1 z21))
			    (- (list xmean ymean zmean) (list x2 y1 z21))
			    )))
	      (list x2 y1 z21)
	      (list x2 y2 z22)
	      (list xmean ymean zmean)
	      )
	     )
	    )
	   )
	  )
	)
       )
     )
    
    (andy-print "sort vert")
    ;; vertices are now sorted by distance, and the projections, and positions
    ;; are also sorted (note that they are reversed: farthest first!).
    (setq d (simul-sort d proj))
    (setq proj (reverse (second d)))
    (setq d (reverse (first d)))
    
    ;; find the projections of the coordinates onto 2-d:
    (let ((i -1))
      (dolist
       (p proj)
       (let ((val (list (first p))))
	 (dolist
	  (q (rest p))
	  (let* (
		 (x (first q))
		 (y (second q))
		 (z (third q))
		 (tt (matmult mat q))
		 )
	    (if (> tt 0)
		(let (
		      (xval (first tt))
		      (yval (second tt))
		      )
		  (setf val (append val (list (list xval yval))))
		  (if (> xval xmax) (setq xmax xval))
		  (if (< xval xmin) (setq xmin xval))
		  (if (> yval ymax) (setq ymax yval))
		  (if (< yval ymin) (setq ymin yval))
		  )
	      )
	    )
	  )
	 (setq i (1+ i))
	 (setf (elt proj i) val)
	 )
       )
      )

    (andy-print "limits:" xmin xmax ymin ymax)
    (setq xshift (* xfactor (- xmax xmin)))
    (setq yshift (* yfactor (- ymax ymin)))
    (send myplot :range 0 (- xmin xshift) (+ xmax xshift))
    (send myplot :range 1 (- ymin yshift) (+ ymax yshift))

    (mapcar
     (lambda(points)
       (let* (
	      (ud (first points))
	      (x1 (first (second points)))
	      (y1 (second (second points)))
	      (x2 (first (third points)))
	      (y2 (second (third points)))
	      (x3 (first (fourth points)))
	      (y3 (second (fourth points)))
	      (tmp (list
		    (send myplot :scaled-to-canvas x1 y1)
		    (send myplot :scaled-to-canvas x2 y2)
		    (send myplot :scaled-to-canvas x3 y3)
		    (send myplot :scaled-to-canvas x1 y1)
		    )
		   )
	      (color (if (= ud 1) up-color down-color))
	      )
	 (send myplot :add-plot-item
	       (if patch
		   (list 'path tmp color line-color)
		 (list 'path tmp 'white color)
		 )
	       :draw nil
	       )
	 )
       )
     proj
     )
    (send myplot :andy-options (list :axes nil :box nil))

    (defmeth myplot :do-click (x y m1 m1) nil)
    (defmeth myplot :redraw () (send self :plot-item) nil)

    (andy-print "leaving hid")
    (send myplot :show-window)
    myplot
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                hid-quad
;;
#|
	The problem with hid-quad as it currently stands is that I'm using
quads, rather than triangles. I need to rewrite this, breaking quads into
triangles. (That's function hid.)
|#
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun hid-quad (&key
	       plot
	       patch
	       (up-color 'blue)
	       (down-color 'red)
	       (line-color 'black)
	       )
  "
;; Description: function hid-quad
 
      Patches plot (i.e. hides hidden lines) of a function

;; Arguments:
	(&key
	       plot
	       patch
	       (up-color 'blue)
	       (down-color 'red)
	       (line-color 'black)

;; Examples:
 (hid-quad :plot (plot (lambda(x y) (* x y))))
"
  (andy-print "in hid-quad")
  (let* (
	 (plot (if plot plot (first (agw))))
	 (lines 
	  (if *real-axes* (rest (send plot :getlines)) (send plot :getlines)))
	 (m (if (matrixp (first (send plot :slot-value 'plot-item)))
		(second (size (first (send plot :slot-value 'plot-item))))
	      nil
	      )
	    )
	 (lines (if m (select lines (iseq m)) lines))
	 (trans (send plot :transformation))
	 (mat (matprod trans
		       (diagonal-to-matrix (/ (send plot :scale (iseq 3))))))
	 (a (remove-duplicates (combine (mapcar (lambda(x) (first x)) lines))))
	 (b (remove-duplicates (combine (mapcar (lambda(x) (second x)) lines))))
	 (grid (apply #'bind-rows (mapcar (lambda(x) (third x)) lines)))
	 (grid (if m (transpose grid) grid))
	 (grid (flesh (select-rows grid
				   (iseq (floor (half (first (size grid))))))
		      :type 'linear))
	 (a (rseq (min a) (max a) (first (size grid))))
	 (b (rseq (min b) (max b) (second (size grid))))

	 (m (1- (length a)))
	 (n (1- (length b)))
	 (eye (solve mat '(0 0 1.7))) ;; got the 1.7 value out of calcsphere...
	 (eyex (first eye))
	 (eyey (second eye))
	 (eyez (third eye))
	 (myplot (send andyplot-proto :new 2 :show nil))
	 (xmin 1e31) (ymin 1e31)
	 (xmax -1e31) (ymax -1e31)
	 (xfactor .1)
	 (yfactor .1)
	 proj d xshift yshift
	 )
    ;; get distances to vertices and directions (up or down). Directions will
    ;; be used to decide the color of a patch.
    (andy-print "get dist")
    (dotimes
     (i m)
     (let* (
	    (ii (1+ i))
	    (x1 (elt a i))
	    (x2 (elt a ii))
	    (xmean (mean (list x1 x2)))
	    )
       (dotimes
	(j n)
	(let* (
	       (jj (1+ j))
	       (y1 (elt b j))
	       (y2 (elt b jj))
	       ;; four adjacent points from the grid...
	       (z11 (aref grid i j))
	       (z12 (aref grid i jj))
	       (z21 (aref grid ii j))
	       (z22 (aref grid ii jj))
	       (ymean (mean (list y1 y2)))
	       (zmean (mean (list z11 z12 z21 z22)))
	       )
	  (setq d
		(append d
			(list
			 (distance eye (list
					xmean
					ymean
					zmean
					)
				   )
			 )))
	  (setf
	   proj
	   (append
	    proj
	    (list
	     (list 
	      (signum (dot eye
			   (cross
			    (- (list x2 y1 z21) (list x1 y1 z11))
			    (- (list x1 y2 z12) (list x1 y1 z11))
			    )))
	      (list x1 y1 z11)
	      (list x1 y2 z12)
	      (list x2 y2 z22)
	      (list x2 y1 z21)
	      )
	     )
	    )
	   )
	  )
	)
       )
     )
    
    (andy-print "sort vert")
    ;; vertices are now sorted by distance, and the projections, and positions
    ;; are also sorted (note that they are reversed: farthest first!).
    (setq d (simul-sort d proj))
    (setq proj (reverse (second d)))
    (setq d (reverse (first d)))
    
    ;; find the projections of the coordinates onto 2-d:
    (let ((i -1))
      (dolist
       (p proj)
       (let ((val (list (first p))))
	 (dolist
	  (q (rest p))
	  (let* (
		 (x (first q))
		 (y (second q))
		 (z (third q))
		 (tt (matmult mat q))
		 )
	    (if (> tt 0)
		(let (
		      (xval (first tt))
		      (yval (second tt))
		      )
		  (setf val (append val (list (list xval yval))))
		  (if (> xval xmax) (setq xmax xval))
		  (if (< xval xmin) (setq xmin xval))
		  (if (> yval ymax) (setq ymax yval))
		  (if (< yval ymin) (setq ymin yval))
		  )
	      )
	    )
	  )
	 (setq i (1+ i))
	 (setf (elt proj i) val)
	 )
       )
      )

    (andy-print "limits:" xmin xmax ymin ymax)
    (setq xshift (* xfactor (- xmax xmin)))
    (setq yshift (* yfactor (- ymax ymin)))
    (send myplot :range 0 (- xmin xshift) (+ xmax xshift))
    (send myplot :range 1 (- ymin yshift) (+ ymax yshift))

    (mapcar
     (lambda(points)
       (let* (
	      (ud (first points))
	      (x1 (first (second points)))
	      (y1 (second (second points)))
	      (x2 (first (third points)))
	      (y2 (second (third points)))
	      (x3 (first (fourth points)))
	      (y3 (second (fourth points)))
	      (x4 (first (fifth points)))
	      (y4 (second (fifth points)))
	      (tmp (list
		    (send myplot :scaled-to-canvas x1 y1)
		    (send myplot :scaled-to-canvas x2 y2)
		    (send myplot :scaled-to-canvas x3 y3)
		    (send myplot :scaled-to-canvas x4 y4)
		    (send myplot :scaled-to-canvas x1 y1)
		    )
		   )
	      (color (if (= ud 1) up-color down-color))
	      )
	 (send myplot :add-plot-item
	       (if patch
		   (list 'path tmp color line-color)
		 (list 'path tmp 'white color)
		 )
	       :draw nil
	       )
	 )
       )
     proj
     )
    (send myplot :andy-options (list :axes nil :box nil))
    
    (defmeth myplot :do-click (x y m1 m1) nil)
    (defmeth myplot :redraw () (send self :plot-item) nil)

    (andy-print "leaving hid-quad")
    (send myplot :show-window)
    myplot
    )
  )
