;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; simpson Doc:    Creation Date:12/18/95        Author: Andy Long
;;
(defmacro simpson (&rest e)
  "
;; Description:

	(simpson f x0 xm &optional (eps 1e-6))
  
;; Arguments

;; Examples:

 (func f(x) sin x)
 (simpson f 0 pi) ;; true value is 2
 (simpson f 0 pi 1e-12) ;; let's decrease the tolerance eps
 (simpson sinc 0 pi 1e-6)

;; See Also: integrate-simpson-recursive
"
  (let* (
	 (f (first e))
	 (f (eval `(turn-it-into-a-function ,f)))
	 (x0 (second e))
	 (xm (third e))
	 (eps (if (>= (length e) 4) (fourth e) 1e-6))
	 newval
	 j
	 end-values
	 )
    (if (not (eval `(listp ,xm)))
	`(integrate-simpson-recursive ,f ,x0 ,xm ,eps)
      (let* ((int-values (make-array (eval `(length ,xm)))))
	(setq end-values (eval `,xm) j 0)
	(dolist (i end-values)
		(setq newval
		      (eval `(integrate-simpson-recursive ,f ,x0 ,i ,eps)))
		(setf (aref int-values j) newval)
		(setq j (1+ j))
		)
	`(coerce ,int-values 'list)
	)
      )
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun integrate-simpson-recursive (f x0 xm eps &optional int-last)
  "
;; Examples:
 (integrate-simpson-recursive #'sinc 0 pi .0049)
 (integrate-simpson-recursive #'sinc 0 pi .0001)

 ;; Really screws up this one: should get 1!
 (integrate-simpson-recursive (lambda(z) (let ((val (* z z)))
			 (cond
			  ((> val 1) 0)
			  ((> val .25) (* (/ 8 3) (^ (- 1 (abs z)) 3)))
			  (t (- (/ 4 3) (* 8 val (- 1 (abs z)))))
			  ))) -1.99 2 1e-8)
 (integrate-simpson-recursive (lambda(z) (let ((val (* z z)))
			 (cond
			  ((> val 1) 0)
			  ((> val .25) (* (/ 8 3) (^ (- 1 (abs z)) 3)))
			  (t (- (/ 4 3) (* 8 val (- 1 (abs z)))))
			  ))) -1 1 1e-8)
"
  (let* (
	 (ff (eval `(turn-it-into-a-function ,f)))
	 (xmean (mean (list x0 xm)))
	 (int-l (simple-simpson ff x0 xmean))
	 (int-r (simple-simpson ff xmean xm))
	 (int (+ int-l int-r))
	 )
    (if (not int-last)
	(setq int-last (simple-simpson ff x0 xm))
      )
    (if (< (abs (- int int-last)) (* 15 eps)) int
      (+ (integrate-simpson-recursive ff x0 xmean (/ eps 2) int-l)
	 (integrate-simpson-recursive ff xmean xm (/ eps 2) int-r))
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun simple-simpson (f x0 xm)
  "
;; Description: (simple-simpson f x0 xm)

	One step in the simpson process (one panel).
"
  (let* (
	 (f0 (funcall f x0))
	 (f1 (funcall f (* 0.5 (+ x0 xm))))
	 (fm (funcall f xm))
	 (h  (* 0.5 (- xm x0)))
	 )
    (* (/ h 3.0) (+ f0 (* 4.0 f1) fm))
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; integral Doc:    Creation Date://        Author: Andy Long
;;
;;
(defmacro integral (fi f x0 var &optional (tol 1e-6))
  "
;; Description:

	(integral fi f x0 var &optional (tol 1e-6))

;; Arguments

;; See Also: simpson, integrate-simpson-recursive

;; Notes: You may get a stack overflow error, like this:
			error: system stack overflow
if your tolerance is too small (you'll force too many recursions) or if
you get outside of the domain of the function (as you can easily do with the ln
function below).

;; Examples:

 (integral sinci sinc 0 x .001)
 (sinci pi)
 (plot sinci)

 (integral ln.int ln 1 x .01)
 (plot ln.int .1 2)
"
  (let (
	(bing
	 (eval
	  `(defun ,fi (,var)
	     (eval (append (list 'simpson ',f ',x0 ,var ',tol))))))
	)
    `(function ,bing)
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Attempting to get a list of elements which 
(defun isr (f x0 xm eps &optional int-last)
  "
 (plot (isr #'sinc 0 1 .0001))
 (setq sincint (isr #'sinc 0 1 1e-5))
 (setq sincint (isr #'sinc 0 1 1e-6))
 (setq sincint (isr #'sinc 0 1 1e-7))
 (setq sincint (isr #'sinc 0 1 1e-7))
 (plot (firsts sincint) (seconds sincint))
 (setq sincint (isr #'ln 1 (exp 1) 1e-7))
 (plot (firsts sincint) (seconds sincint))
"
  (let* (
	 (xmean (mean (list x0 xm)))
	 (int-l (simple-simpson f x0 xmean))
	 (int-r (simple-simpson f xmean xm))
	 (int (+ int-l int-r))
	 )
    (if (not int-last) (setq int-last (simple-simpson f x0 xm)))

    (if (< (abs (- int int-last)) (* 15 eps)) (list xm int)
      (append (isr f x0 xmean (/ eps 2) int-l)
	      (isr f xmean xm (/ eps 2) int-r))
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; integral2 Doc:    Creation Date://        Author: Andy Long
;;
(defmacro integral2 (f x0 xm &optional (eps 1e-6))
;;
  "
;; Description: macro

	(integral2 f x0 xm &optional (eps 1e-6))
  
Produces a list of the integral values obtained using recursive simpsons for
the interval from x0 to xm at the nodes chosen (which won't be equally spaced,
in general).

;; See Also: integral

;; Examples:

 (integral2 sinc 0 pi 1e-6)
 (plot (integral2 sinc 0 4pi 1e-6))
"
  (let* (
	 (bing (eval `(turn-it-into-a-function ,f)))
	 (int-values (append (list `,x0 0) (eval `(isr ,bing ,x0 ,xm ,eps))))
	 (end-values (seconds int-values))
	 )
    (dotimes (i (1- (length end-values)))
	     (setq ii (1+ i))
	     (setl end-values ii (+ (nth ii end-values) (nth i end-values)))
	     )
    (setq results (list (firsts int-values) end-values))
    `(echo results)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;	Rectangle Rules: left, right, midpoint, trapezoidal, simpson
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Left RR, rrr:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun left-rectangle-rule (f a b n)
  (let* (
	 (interval (- b a))
	 (h (/ interval n))
	 (x_i (grid a (- b h) (max (1- n) 1)))
	 (f_i (mapcar `,f x_i))
	 )
    (if (= 1 n) (* h (first f_i)) (* h (sum f_i)))
    )
  )

(defmacro lrr (f &rest e)
  "
;; Description/Arguments:

        (lrr f a b n)
  
Integrates f on [a,b], using n steps.

;; See Also: rrr, simpson

;; Examples:

 (func f(x) 2 * x)
 (lrr f 0 1 100) ; should get something very close to 1!
"
  `(left-rectangle-rule #',f ,@e)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Right RR, rrr:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun right-rectangle-rule (f a b n)
  (let* (
	 (interval (- b a))
	 (h (/ interval n))
	 (x_i (grid (+ a h) b (max (1- n) 1)))
	 (f_i (mapcar `,f x_i))
	 )
    (if (= 1 n) (* h (first f_i)) (* h (sum f_i)))
    )
  )
(defmacro rrr (f &rest e)
  "
See lrr.
"
  `(right-rectangle-rule #',f ,@e)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Midpoint RR, mid:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun midpoint-rectangle-rule (f a b n)
  (let* (
	 (interval (- b a))
	 (h (/ interval n))
	 (x_i (+ (half h) (grid a (- b h) (max (1- n) 1))))
	 (f_i (mapcar `,f x_i))
	 )
    (if (= 1 n) (* h (first f_i)) (* h (sum  f_i)))
    )
  )
(defmacro mid (f &rest e)
  "
;; Description/Arguments:

        (mid f a b n)
  
Integrates f on [a,b], using n steps.

;; Notes:

;; See Also: lrr, rrr, trap, simpson

;; Examples:

 (func f(x) 2 * x)
 (mid f 0 1 100) ; should get something very close to 1!

"
  `(midpoint-rectangle-rule #',f ,@e)
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Trapezoidal RR, trap:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro trap (f &rest e)
  "
Description: trap
	Trapezoidal rule for the estimation of an integral. Works out to the
average of the left and right rectangle rules.

Notes:
	See lrr, rrr, trap-data

This is not a \"workhorse\" routine: it's not defined efficiently, but serves
for purposes of demonstration.

See Also: simpson for serious integration.

;; Examples:
 (func f (x) x ^ 2)
 (trap f 0 1 10)
 (mid f 0 1 10)
 (lrr f 0 1 10)
 (rrr f 0 1 10)

"
  `(half (+ (left-rectangle-rule #',f ,@e) (right-rectangle-rule #',f ,@e)))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Simpson's RR, simp:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro simp (f &rest e)
  "
Description: simp
        Simpson's rule for the estimation of an integral. Works out to a
weighted average of the midpoint and trapezoidal rectangle rules:

                  simp(n) = (2*mid(n) + trap(n))/3

Notes:
	See lrr, rrr, mid, trap

This is not a \"workhorse\" routine: it's not defined efficiently, but serves
purposes of demonstration. See simpson for serious integration.

;; Examples:
 (func f (x) x ^ 2)
 (simp f 0 1 10)
 (trap f 0 1 10)
 (mid f 0 1 10)
 (lrr f 0 1 10)
 (rrr f 0 1 10)
"
  (let ((bing (eval `(turn-it-into-a-function ,f))))
    `(/ (+ 
	 (* 2 (midpoint-rectangle-rule ,bing ,@e)) ;; twice midpoint
	 (half (+ (left-rectangle-rule ,bing ,@e)  ;; trapezoidal rule
		  (right-rectangle-rule ,bing ,@e)))
	 )
	3)
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun qgaus (f a b)
  "
Description: gaussian integration is good for improper integrals, since it
	doesn't use the endpoints.

;; Examples:
 (func f(x) 1/sqrt(16-x^2))
 (qgaus #'f 0 4)
"
  (let
      (
       (x '(0.14887434 0.4333954 0.67940956 0.86506337 0.9739065))
       (w '(0.29552424 0.26926672 0.21908636 0.14945135 0.06667134))
       (dx 0.0)
       (xr (* 0.5 (+ b (- a))))
       (xm (* 0.5 (+ b a)))
       (ss 0)
       )
    (dotimes (j 5)
	     (setq dx (* xr (nth j x)))
	     (setq ss
		   (+ ss
		      (* (nth j w)
			 (+
			  (eval `(,f (+ ,xm ,dx)))
			  (eval `(,f (+ ,xm (- ,dx))))
			  )
			 )
		      )
		   )
	     )
    (setq ss (* xr ss))
    )
  )

(defun qgauss (f a b n)
  (let* ((step (/ (- b a) n))(temp-a a)(temp-b (+ a step))(ss 0))
    (dotimes (i n)
	     (setq ss (+ ss (eval `(qgaus ,f ,temp-a ,temp-b))))
	     (setq temp-a temp-b)
	     (setq temp-b (+ temp-b step))
	     )
    ss
    )
  )
	   
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			gaussian integration, gauss:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro gauss (f &rest e)
  "
Description: (gauss f a b n)

	Gaussian rule for the estimation of an integral. This is especially
useful for the calculation of improper integrals.

Notes: uses a five-point scheme.

;; Examples:

 (func f(x) 1 / sqrt (16 - x ^ 2))
 (gauss f 0 4 10)

 (func f(x) 1 / (1 + x ^ 2))
 (gauss f 0 400 10)
 (gauss (func f(x) 1 / (1 + x ^ 2)) 0 400 10)
"
  (let ((bing (eval `(turn-it-into-a-function ,f))))
    `(qgauss ,bing ,@e)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; trap-data Doc:    Creation Date://        Author: Andy Long
;;
;;
(defun trap-data (x &optional (step 1))
  "
;; Description/Arguments: computes the trapezoidal rule on a bunch of data
;; given as a list.

;; Notes: makes use of car, cdr, and reverse, on a list.

;; See Also: trap, cdr, car, reverse

;; Examples:
 (data x 26 22 19 14 6 3 1) ;; tree-trunk circumferences
 (setq x (calc (x / (2 * pi)) ^ 2 * pi)) ;; converted to areas of cross-sections
 (trap-data x 20) ;; estimate for volume of tree
"
  (/ (* (sum (* 2 (cdr (reverse (cdr x)))) (car x) (car (reverse x))) step) 2)
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; integrate Doc:    Creation Date:4/1/96        Author: Andy Long
;;
;;
(defmacro integrate (f &rest e)
  "
;; Description:

	(integrate f xmin xmax &optional tolerance) ;; univariate
or
	(integrate f xmin xmax ymin ymax &optional steps tolerance) ;; multi

	Should give a pretty good numerical approximation to a function over a
rectangular interval, either univariate or bi-variate.

;; See Also: simp2, integrate2

;; Notes: Integrate uses simpson's rule for univariate functions, and a
roughly simpson's rule for the bivariate functions.

;; Examples:
;; univariate:
 (integrate sinc 0 10 .001)
 (integrate (func f(x) x) 0 1)
 (integrate ln 1 2 .01)

 ;; Should get 1:
 (integrate (lambda(z) (let ((val (* z z)))
			 (cond
			  ((> val 1) 0)
			  ((> val .25) (* (/ 8 3) (^ (- 1 (abs z)) 3)))
			  (t (- (/ 4 3) (* 8 val (- 1 (abs z)))))
			  ))) -2 2 1e-8)

;; multivariate:
 (integrate (func f(x y) x * y) 0 1 0 1)
 (integrate (func f(x y) x * x) 0 1 0 1 100)
 (integrate (func f(x y) pi) 0 1 0 1)
"
  (let* (
	 (bing (eval `(turn-it-into-a-function ,f)))
	 (nvars (length (second (eval `(get-lambda-expression ,bing)))))
	 (lene (length `,e))
	 )
    (if (= 1 nvars) ;; univariate case
	(if (= lene 2) `(integrate-simpson-recursive ,bing ,@e 1e-6)
	  `(integrate-simpson-recursive ,bing ,@e)
	  )
;;      `(integrate2 ,bing ,@e)
      `(simp2 ,bing ,@e) ;; multivariate case
      )
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; integrate2 Doc:    Creation Date:4/1/96        Author: Andy Long
;;
;;
(defun integrate2(f xmin xmax ymin ymax step-size &key xstep ystep)
  "
;; Description/Arguments:

       (integrate2)
  
;; Notes:

;; See Also:

;; Examples:
 (integrate2 (func f(x y) x * y) 0 1 0 1 .1)
 (integrate2 (func f(x y) pi) 0 1 0 1 .1)
"
  (let* (
	 (xstep (if (not xstep) step-size))
	 (ystep (if (not ystep) step-size))
	 (msteps (ceiling (/ (- xmax xmin) xstep)))
	 (nsteps (ceiling (/ (- ymax ymin) ystep)))
	 (sum3 0)
	 (xval1 xmin)
	 (xval2 (+ xval1 xstep))
	 )
    (dotimes
     (i msteps)
     (let* (
	    (yval1 ymin)
	    (yval2 (+ yval1 ystep))
	    )
       (dotimes
	(j nsteps)
	(setq
	 sum3 (+ sum3
		 (eval `(,f (mean (list ,xval1 ,xval2))
			    (mean (list ,yval1 ,yval2)))))
	 yval1 yval2
	 yval2 (+ yval1 ystep)
	 )
	)
       )
     (setq xval1 xval2 xval2 (+ xval1 xstep))
     )
    (* sum3 xstep ystep)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
(defun integrate-matrix (matrix &optional (xstep 1) (ystep 1))
  "
;; Description: (integrate-matrix matrix &optional (xstep 1) (ystep 1))

	'Integrates' a matrix by adding up the components and scaling by xstep
and ystep (if they are provided).

;; Examples:
 (table beef 5 4 
       2.65 2.59 2.51 2.43
       4.14 4.05 3.94 3.88
       5.11 5.00 4.97 4.84
       5.35 5.29 5.19 5.07
       5.79 5.77 5.60 5.53
       )
 (plot beef)
 (integrate-matrix beef .2 .2)
"
  (* xstep ystep (sum matrix))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; integrate-xy Doc:    Creation Date:4/9/96        Author: Andy Long
;;
;;
(defmacro integrate-xy (f xmin xmax f1 f2 &rest e)
  "
;; Description:

	(integrate-xy f xmin xmax f1 f2 (tol 1e-6))

	Should give a pretty good numerical approximation to a function over an
region bounded by two curves, y=f1(x) and y=f2(x). Thus the user must input
three functions.

;; Notes: this currently uses a weird mix of midpoint and trapezoidal: it
;; should be rethought!

;; See Also: integrate-xy2

;; Examples:
 (func f(x y) x * y)
 (func f1 (x) 0)
 (func f2 (x) 1)
 (integrate-xy f 0 1 f1 f2 .1)

 (integrate-xy (func f(x y) 1) 0 1 (func f1 (x) - x) (func f2 (x) x) .1)
 (integrate-xy (func f(x y) x * y) 0 1 (func f1 (x) - x) (func f2 (x) x) .1)
"
  (let* (
	 (bing (eval `(turn-it-into-a-function ,f)))
	 (bang (eval `(turn-it-into-a-function ,f1)))
	 (bong (eval `(turn-it-into-a-function ,f2)))
	 )
    `(integrate-xy2 ,bing ,xmin ,xmax ,bang ,bong ,@e)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; integrate-xy2 Doc:    Creation Date:4/9/96        Author: Andy Long
;;
(defun integrate-xy2(f xmin xmax f1 f2 step-size &key xstep ystep)
  "
;; Description/Arguments:

       (integrate-xy2)
  
;; See Also: integrate-xy

;; Examples:
 (integrate-xy2 (func f(x y) x*y) 0 1 (func f1(x) 0) (func f2 (x) 1) .1)
 (integrate-xy2 (func f(x y) x*y) 0 1 (func f1(x) -x) (func f2 (x) x) .1)
"
  (let* (
	 (xstep (if (not xstep) step-size))
	 (msteps (ceiling (/ (- xmax xmin) xstep)))
	 (xval1 xmin)
	 (xval2 (+ xval1 xstep))
;;	 (sum2 0)
	 (sum3 0)
	 )
    (dotimes
     (i msteps)
     (let* (
	    (ystep (if (not ystep) step-size))
	    (ymin (eval `(,f1 ,xval1)))
	    (ymax (eval `(,f2 ,xval1)))
	    (nsteps (max (list 0 (ceiling (/ (- ymax ymin) ystep)))))
	    (ystep (if (= 0 nsteps) 1 (/ (- ymax ymin) nsteps)))
	    (yval1 ymin)
	    (yval2 (+ yval1 ystep))
	    (sum1 0)
	    )
       (dotimes
	(j nsteps)
	(setq
	 sum1 (+ sum1
		 ;; This is a midpoint calculation:
		 (eval `(,f (mean (list ,xval1 ,xval2))
			    (mean (list ,yval1 ,yval2)))))
	 yval1 yval2
	 yval2 (+ yval1 ystep)
	 )
	)
       ;; this is a sort of trapezoidal method:
       ;;       (if (> i 0) (setq sum3 (+ sum3 (* sum1 ystep))))
       ;;       (if (< i (1- msteps)) (setq sum2 (+ sum2 (* sum1 ystep))))
       (setq sum3 (+ sum3 (* sum1 ystep)))
       )
     (setq xval1 xval2 xval2 (+ xval1 xstep))
     )
    ;; this is that trapezoidal stuff:
    ;;    (* (/ (+ sum2 sum3) 2) xstep)
    (* sum3 xstep)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; simp2 Doc:    Creation Date:4/22/96        Author: Andy Long
;;
;;
(defun simp2(f xmin xmax ymin ymax &optional (steps 20) (tol 1e-6))
  "
;; Description:

        (simp2 f xmin xmax ymin ymax &optional (steps 20) (tol 1e-6))

Uses simpson in one direction, and a hermite method on the results of those
simpson's calculations (or the same order).

;; Examples:
 (simp2 (func f(x y) x * y) 0 1 0 1)

;; Note: Using the hermite integration scheme on the bunch:
;  H = (h/24)(9(y1+yn+1) + 28(y2+yn) + 23(y3+yn-1) +24(y4 + y5 + ...+ yn-2)
;    = (sum yi) + (- 15*(y1+yn+1) + 4*(y2+yn) - (y3+yn-1))/24
"
  (let* (
	 (row-vals '())
	 (ff (eval `(turn-it-into-a-function ,f)))
	 (xstep (/ (- xmax xmin) (1- steps)))
	 )
    (dolist
     (x (rseq xmin xmax steps))
     (setq
      row-vals
      (append
       row-vals
       (list (eval `(simpson (lambda (y) (,ff ,x y)) ,ymin ,ymax ,tol)))))
     )
    (if (> (length row-vals) 2)
	(let (
	      (sum-em-up (sum row-vals))
	      (backwards (reverse row-vals))
	      )
	  (* xstep (+
		    sum-em-up
		    (/ (+
			(* -15 (+  (first row-vals)  (first backwards)))
			(*   4 (+ (second row-vals) (second backwards)))
			(-     (+  (third row-vals)  (third backwards)))
			)
		       24
		       ))))
      (* (/ (+ (sum (rest row-vals)) (sum (butlast row-vals))) 2) xstep))
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SIMPSONS-MATRIX Doc:    Creation Date:4/28/96        Author: Andy Long
;;
;;
(defun SIMPSONS-MATRIX(n)
  "
;; Description/Arguments:

       (SIMPSONS-MATRIX n)

creates an n by n matrix which can be used much in the same way that Simpson's
rule is used in one dimension. In fact this two dimensional generalization of
Simpson's rule was suggested in the Harvard projects multivariate calculus
text, section 15.4.
  
;; Examples:
 (setq simpsons (simpsons-matrix 9))
 (setq funcvals (outer-product (rseq 1 5 9) (rseq 2 6 9)
			      (func f(x y) sqrt (x ^ 2 + y ^ 2))))
 (* 16 (/ (sum (* simpsons funcvals)) (sum simpsons)))
;; for comparison purposes, here's a more adaptive version of Simpson's rule:
 (integrate f 1 5 2 6)
"
  (let ((nlen (/ (- n 3) 2)))
    (matrix (list n n)
	    (combine
	     (combine
	      (list 1 4)
	      (repeat '(2 4) nlen)
	      '(1)
	      )
	     (repeat 
	      (combine
	       (combine
		(list 4 16)
		(repeat '(8 16) nlen)
		'(4)
		)
	       (combine
		(list 2 8)
		(repeat '(4 8) nlen)
		'(2)
		)
	       )
	      nlen
	      )
	     (combine
	      (list 4 16)
	      (repeat '(8 16) nlen)
	      '(4)
	      )
	     (combine
	      (list 1 4)
	      (repeat '(2 4) nlen)
	      '(1)
	      )
	     )
	    )
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; hermite Doc:    Creation Date:6/21/96	        Author: Andy Long
;;
;;
(defun hermite(&rest e)
  "
;; Description:

        (hermite f xmin xmax &key (steps 20) derivative type)
or
        (hermite xvals fvals)

        Performs Hermite integration of either a set of equally spaced points,
or a function on a given interval, with a given step-size.

;; Note: The default hermite integration scheme (h3) on the data is

   H3 =(h/24)(9(y_1+y_{n+1})+28(y_2+y_n)+23(y3+y_{n-1})+24(y_4+y_5+...+y_{n-2})
      = h[(sum y_i)+(-15*(y_1+y_{n+1})+4*(y_2+y_n)-(y_3+y_{n-1}))/24]
      = trapezoidal - h[3*(y_1+y_{n+1})-4*(y_2+y_n)+(y_3+y_{n-1})]/24

   H5 = trapezoidal - h[25*(y_1+y_{n+1})-48*(y_2+y_n)+36(y_3+y_{n-1})
			-16*(y_4+y_{n-2})+3(y_5+y_{n-3})]/24

while the true Hermite's Rule requires the derivative, and is given by

   H  = trapezoidal - h^2[y'_n - y'_1]/12

;; Examples:
 (hermite (func f(x) 2*x) 0 1)
 (hermite (func f(x) 2*x) 0 1 :steps 100)
 (hermite (grid 0 1 100) (calc 2 * (grid 0 1 100)))
 (hermite (grid 0 1 100) (calc 2 * (grid 0 1 100)) :type 'h5)

 ;; Hermite gets any polynomial exactly right, as the error is proportional to
 ;; the fourth derivative (notice that we're only using 2 steps!):
 (hermite (func f(x) 4*x^3) 0 1 :steps 2)
 (hermite (func f(x) 3*x^2) 0 1 :steps 2)
 (hermite (func f(x) 2*x) 0 1 :steps 2)

 ;; but it can't get a quartic right as easily:
 (hermite (func f(x) 5*x^4) 0 1 :steps 2) ;; should be 1
 (hermite (func f(x) 5*x^4) 0 1 :steps 4 :type 'h5)
 ;; it helps if we can give the derivative function:
 (hermite (func f(x) 5*x^4) 0 1 :steps 4 :deriv (func g(x) 20*x^3))
 ;; with a few more steps we get darned close (default: 20)!
 (hermite (func f(x) 5*x^4) 0 1 :deriv (func g(x) 20*x^3))

 ;; Hermite seems to have a hard time with this function:
 (hermite (func f(x) (abs (x-1))) 0 1 :steps 2)
 (hermite (func f(x) (abs (x-1))) 0 1 :steps 2
	  :deriv (defun g(x) (if (= x 1) 0 (if (< x 1) -1 1))))

 (func f(x) (abs (x-pi/4)))
 (func g(x) (signum (x-pi/4)))
 (simp f 0 2 40)
 (hermite #'f 0 2 :steps 80)
 (hermite #'f 0 2 :steps 80 :deriv #'g)

 (hermite (func f(x) (signum (x-pi/4))*(square (x-pi/4)))
	 0 2 :steps 80 :deriv (func g(x) (abs (x-pi/4))))
 (hermite #'f 0 2 :steps 80)
 (hermite #'f 0 2 :steps 80 :type 'h5)
 (simp f 0 2 40)	 
 (/ (- (cube (- 2 pi/4)) (cube pi/4)) 3)
"
  (let* (
	 (f (first e))
	 (xmin (second e))
	 (keys (catch-keys '(:steps :deriv :type) e))
	 (steps (first keys))
	 (steps (if steps steps 20))
	 (deriv (second keys))
	 (type (third keys))
	 (type (if type type (if deriv 'h 'h3)))
	 xmax xstep xvals fvals
	 )
    (if (listp f)
	(setq xvals f
	      fvals xmin
	      xstep (- (second xvals) (first xvals))
	      )
      (setq ff (eval `(turn-it-into-a-function ,f))
	    xmax (third e)
	    xstep (/ (- xmax xmin) steps)
	    xvals (grid xmin xmax steps)
	    fvals (mapcar ff xvals)
	    )
      )
    (if (> (length fvals) 2)
	(let (
	      (trap (* xstep
		       (- (sum fvals) (half (+ (first fvals) (final fvals))))))
	      (r (reverse fvals))
	      )
	  (+ trap
	     (* xstep
		(cond
		 ((eq type 'h3)
		  (/ (+
		      (* -3 (+  (first fvals)  (first r)))
		      (*  4 (+ (second fvals) (second r)))
		      (-    (+  (third fvals)  (third r)))
		      )
		     24
		     ))
		 ((eq type 'h5)
		  (/ (+
		      (* -25 (+ (elt fvals 0) (elt r 0)))
		      (*  48 (+ (elt fvals 1) (elt r 1)))
		      (* -36 (+ (elt fvals 2) (elt r 2)))
		      (*  16  (+ (elt fvals 3) (elt r 3)))
		      (*  -3 (+ (elt fvals 4) (elt r 4)))
		      )
		     144))
		 (deriv
		  (* xstep
		     (/ (- (funcall deriv (first xvals))
			   (funcall deriv (final xvals))
			   )
			12
			)))
		 (t 
		  (error "Unknown hermite type in function hermite: ~s" type)
		  )))))
      (* (/ (+ (sum (rest fvals)) (sum (butlast fvals))) 2) xstep))
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 integration
;;
(defmacro integration(&rest that)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          integration Doc:
;;
;; Author: Andy Long
;; Created: Sat Nov 8 17:27:34 1997
;;
;; Description/Arguments:

       (integration expression)
  
	Where expression is a given infix expression. This macro attempts to
perform integration on an expression given as a function of x. It currently
handles only functions of one variable.

;; Notes: I created this in my excitement after finding Peter Norvig's
;; simplifier routine. I knew that I could improve on that routine a little by
;; using the functions I'd created for infix to prefix stuff. Now his and mine
;; are both loaded, so you may be interested to see his (e.g. infix->prefix,
;; prefix->infix, etc.).

;; See Also: simplifier.

;; Examples:
 (integration (3*x+2)^2)
 (simplifier '(int (3 * x + 2) ^ 2 d x))

 (integration (x*sin(x^2)))
 (simplifier '(int x * sin(x ^ 2) d x))

 (integration ((3*x^3)-1/(3*x^3)))
 (simplifier '(int ((3 * x ^ 3) - 1 / (3 * x ^ 3)) d x))

 (integration (sin(x)^2*cos(x)))
 (simplifier '(int sin(x) ^ 2 * cos(x) d x))

 (integration (sin(x)/(1+cos(x))))
 (simplifier '(int sin(x) / (1 + cos(x)) d x))

 (integration ((2*x+1)/(x^2+x-1)))
 (simplifier '(int (2 * x + 1) / (x ^ 2 + x - 1) d x))

 (integration (8*x^2/(x^3+2)^3))
 (simplifier '(int 8 * x ^ 2 / (x ^ 3 + 2) ^ 3 d x))

 ;; ael: fixed a problem so that these two are now 'effectively' equivalent...
 (integration (3*x+2)^(-2/3))
 (simplifier '(int (3 * X + 2) ^ (- 2 / 3) d x))
 ;; Simplifier is a little better if you use the actual rational number -2/3:
 (simplifier '(int (3 * x + 2) ^ -2/3 d x))

 ;; You can use other variables of integration (even t!). Just be sure to use
 ;; the :var key to indicate the variable of integration:
 (integration t^2 :var t)
 (integration 2+alpha^2 :var alpha)
"
  (let* (
	 (dash (first (catch-keys '(:dash) that)))
	 (that (remove-keys '(:dash) that))
	 (var (first (catch-keys '(:var) that)))
	 (that (remove-keys '(:var) that))
	 (var (if var var 'x))
	 (a (eval
	     `(no-space 
	       ,(eval
		 `(list-to-string ',that)
		 )
	       :dash ,dash
	       )
	     )
	    )
	 (a (recursive-grep-replace '(+ -) '(-) a))
	 (a (recursive-grep-replace '(/ -) '(* -1 /) a))
	 (a (recursive-grep-replace '(* -) '(* -1 *) a))
	 (a (recursive-grep-replace '(- -) '(+) a))
	 )
    `(simplifier '(int ,@a d ,var))
    )
  )
