;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;
#|
	This is my modification of Jason Bond's label.lsp. I "personalized"
the method, to alleviate the cross-plot referencing that he was doing: I don't
see why we have to keep track of all the plots that have labels, etc.: each
plot will take care of itself.

	I'd really like to change reference from absolute coordinates to
relative coordinates (so that labels stay in position when the screen is
resized), but I don't have the courage or need yet!

							Andy Long
							6/10/97
|#
;;;;  Program to display labels on plots inheriting from the graph-proto
;;;;  prototype.  Labels can be inserted, removed, aligned, and dragged
;;;;  by clicking in the desired plots.  Calls to the function should be 
;;;;  of the form:
;;;;  (make-labels (list plot1 plot2 ... plotn)).  Plots supported are
;;;;  all scatterplots, histograms, and spin-plots.  These labels will
;;;;  be saved and printed along with the plot image.
;;;;
;;;;      Jason Bond (jbond@laplace.stat.ucla.edu)  3/30/95
;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                jason-add-labels
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth graph-proto :jason-add-labels (labels coords)
  (send self :jason-label-coordinates 
	(append (send self :jason-label-coordinates) (list coords)))
  (send self :jason-plot-labels
	(append (send self :jason-plot-labels) (list labels)))
  (send self :jason-horiz-vert
	(append
	 (send self :jason-horiz-vert)
	 (list (send self :jason-c-horiz-vert))))
  (send self :jason-horiz
	(append
	 (send self :jason-horiz)
	 (list (send self :jason-c-horiz))))
  (send self :jason-vert
	(append
	 (send self :jason-vert)
	 (list (send self :jason-c-vert)))))

  #| - ael - Sun May 24 08:55:30 1998
;; moved redraws to a clear-content method in andyplot.lsp:
 - lea - |#

(defmeth graph-proto :jason-remove-label (plot x y)
  (let* (
	 (removeplot '(0))
	 (coords (send self :jason-label-coordinates))
	 (labels (send self :jason-plot-labels))
	 (h-v (send self :jason-horiz-vert))
	 (h (send self :jason-horiz))
	 (v (send self :jason-vert))
         )
    (when
     removeplot
     (let* (
	    (xremove (which (mapcar #'(lambda (i) (< (min x) i (max x)))
				    (mapcar #'first coords))))
	    (yremove (which (mapcar #'(lambda (i) (< (min y) i (max y)))
				    (mapcar #'second coords))))
	    (to-remove (intersection xremove yremove))
	    )
       (when to-remove
	     (let (
		   (seq (set-difference
			 (iseq (length labels))
			 (if (listp to-remove) to-remove (list to-remove))))
		   )
	       (send self :jason-horiz-vert (select h-v seq))
	       (send self :jason-horiz (select h seq))
	       (send self :jason-vert (select v seq))
	       (send self :jason-plots-with-labels (list self))
	       (send self :jason-label-coordinates (select coords seq))
	       (send self :jason-plot-labels (select labels seq))))))))

(defmeth graph-proto :jason-align (plot x y)
  (let* (
	 (coords (send self :jason-label-coordinates))
	 (horiz-align (send self :jason-horiz-align))
         )
    (let* (
	   (xalign (which (mapcar #'(lambda (i) (< (min x) i (max x)))
				  (mapcar #'first coords))))
	   (yalign (which (mapcar #'(lambda (i) (< (min y) i (max y)))
				  (mapcar #'second coords))))
	   (to-align (intersection xalign yalign))
	   )
      (when
       to-align
       (cond (horiz-align
	      (let* (
		     (newcoords (select coords to-align))
		     (y-val (floor (mean (mapcar #'second newcoords))))
		     )
		(mapcar #'(lambda (y) (setf (second y) y-val)) newcoords)
		(setf (select coords to-align) newcoords)
		(send self :jason-label-coordinates coords)))
	     (t 
	      (let* (
		     (newcoords (select coords to-align))
		     (x-val (floor (mean (mapcar #'first newcoords))))
		     )
		(mapcar #'(lambda (x) (setf (first x) x-val)) newcoords)
		(setf (select coords to-align) newcoords)
		(send self :jason-label-coordinates coords))))))))

(defmeth graph-proto :jason-drag-labels (x y m1 m2)
  (let* (
	 (coords (send self :jason-label-coordinates))
	 (b-c (send self :jason-box-coords))
	 (good-click (if (and (< (min (first b-c)) x 
				 (max (first b-c)))
			      (< (min (second b-c)) y
				 (max (second b-c)))) t nil))
	 )
    (when
     good-click
     (let* (
	    (xdrag
	     (which
	      (mapcar #'(lambda (i) 
			  (< (min (first b-c)) i (max (first b-c))))
		      (mapcar #'first coords))))
	    (ydrag
	     (which
	      (mapcar #'(lambda (i) 
			  (< (min (second b-c)) i (max (second b-c))))
		      (mapcar #'second coords))))
	    (to-drag (intersection xdrag ydrag))
	    )
       (when
	(not to-drag)
	(send self :redraw)
	(send self :mouse-mode 'jason-get-drag-labels)
	)
       (when
	to-drag
	(let* ( 
	       (v-h (select (send self :jason-horiz-vert) to-drag))
	       (horiz (select (send self :jason-horiz) to-drag))
	       (vert (select (send self :jason-vert) to-drag))
	       (method
		(mapcar
		 #'(lambda (x) (if (= x 0) :draw-text :draw-text-up))
		 v-h))
	       (crds (select coords to-drag))
	       (labels (select (send self :jason-plot-labels) to-drag))
	       (xdiff (apply #'- (first b-c)))
	       (ydiff (apply #'- (second b-c)))
	       (lx (first b-c))
	       (ly (second b-c))
	       (ox x)
	       (oy y)
	       (n-c nil)
	       )
	  (send
	   self
	   :while-button-down
	   #'(lambda (x1 y1)
	       (send self :draw-mode 'xor)
	       (let (
		     (xmove (- x1 ox))
		     (ymove (- y1 oy))
		     )
		 (mapcar #'(lambda (x y z h v) 
			     (send self z x (first y) (second y) h v))
			 labels crds method horiz vert)
		 (setf crds
		       (mapcar #'(lambda (x) 
				   (setf x (+ x (list xmove ymove)))) crds))
		 (send self :draw-line (first lx) (first ly) 
		       (first lx) (second ly))
		 (send self :draw-line (first lx) (second ly)
		       (second lx) (second ly))
		 (send self :draw-line (second lx) (second ly)
		       (second lx) (first ly))
		 (send self :draw-line (second lx) (first ly)
		       (first lx) (first ly))
		 
		 (send self :draw-line (+ (first lx) xmove)
		       (+ (first ly) ymove)
		       (+ (first lx) xmove)
		       (+ (second ly) ymove))
		 (send self :draw-line (+ (first lx) xmove)
		       (+ (second ly) ymove)
		       (+ (second lx) xmove)
		       (+ (second ly) ymove))
		 (send self :draw-line (+ (second lx) xmove)
		       (+ (second ly) ymove)
		       (+ (second lx) xmove)
		       (+ (first ly) ymove))
		 (send self :draw-line (+ (second lx) xmove)
		       (+ (first ly) ymove)
		       (+ (first lx) xmove)
		       (+ (first ly) ymove))
		 (mapcar #'(lambda (x y z h v) 
			     (send self z x (first y) (second y) h v))
			 labels crds 
			 method horiz vert)
		 (setf ox x1)
		 (setf oy y1)
		 (setf lx (+ lx xmove))
		 (setf ly (+ ly ymove))
		 (setf n-c (list (min lx) (min ly))))))
	  (unless n-c (setf n-c (list (min lx) (min ly))))
	  (send self :draw-mode 'normal)
	  (let (         
		(c-d
		 (cond ((and (< xdiff 0) (< ydiff 0))
			(- (select n-c (list 0 1)) (mapcar #'first b-c)))
		       ((and (> xdiff 0) (< ydiff 0))
			(- (select n-c (list 0 1)) 
			   (list (second (first b-c)) (first (second b-c)))))
		       ((and (> xdiff 0) (> ydiff 0))
			(- (select n-c (list 0 1)) (mapcar #'second b-c)))
		       (t (- (select n-c (list 0 1)) 
			     (list (first (first b-c)) 
				   (second (second b-c)))))))
		(newcoords (select coords to-drag))
		)
	    (setf newcoords (mapcar #'(lambda (y) (setf y (+ y c-d))) 
				    newcoords))
	    (setf (select coords to-drag) newcoords)
	    (send self :jason-label-coordinates coords))
	  )
	(send self :mouse-mode 'jason-get-drag-labels)
	)
       )
     (send self :redraw)
     )
    )
  )

       
(defmeth graph-proto :jason-draw-label (x y m1 m2)
  (let* ((loc (send self :location))
	 (newx (+ x (first loc) (- 80)))
	 (newy (+ y (second loc) (- 80)))
	 (put-string (get-string-dialog "Enter a Label:" 
					:location (list newx newy))))
    (if (and put-string (> (length put-string) 0))
	(send self :jason-add-labels put-string (list x y))))
  (send self :redraw))

(defmeth graph-proto :jason-erase-label (x y m1 m2)
  (let ((coords (send self :jason-draw-box x y)))
    (send self :jason-remove-label self 
	  (list x (first coords)) (list y (second coords)))
    (send self :redraw)))

(defmeth graph-proto :jason-align-labels (x y m1 m2)
  (let ((coords (send self :jason-draw-box x y))
	(horiz (choose-item-dialog "Align:" 
				   (list "Horizontally" "Vertically"))))
    (when horiz (send self :jason-horiz-align (if (= horiz 0) t nil))
	  (send self :jason-align self
		(list x (first coords)) (list y (second coords))))
    (send self :redraw)))

(defmeth graph-proto :jason-get-drag-labels (x y m1 m2)
  (let ((coords (send self :jason-draw-box x y))
	(plots (list self))
	)
    (send self :jason-box-coords (list (list x (first coords))
					      (list y (second coords))))
    (mapcar #'(lambda (x) (send x :mouse-mode 'jason-drag-labels)) plots)
    )
  )

(defmeth graph-proto :jason-draw-box (x y)
  (let ((newx x)
	(newy y))
    (send self :draw-mode 'xor)
    (send self
	  :while-button-down
	  #'(lambda (x1 y1) 
	      (send self :draw-line x y x y1)
	      (send self :draw-line x y x1 y)
	      (send self :draw-line x y1 x1 y1)
	      (send self :draw-line x1 y1 x1 y)
	      (send self :draw-line x y x newy)
	      (send self :draw-line x y newx y) 
	      (send self :draw-line x newy newx newy)
	      (send self :draw-line newx newy newx y)
	      (setf newx x1)
	      (setf newy y1)))
    (send self :draw-mode 'normal)
    (list newx newy)))

(defmeth graph-proto :make-labels (&optional (launch t))
  "
;; Description: defmeth graph-proto :make-labels ()

	This is a method for adding/editting labels in a plot. It seems to work
okay. Have a try!

;; Examples:
  (plot sin)
  (send my-plot :add (list 'points (rand.u 10) (rand.u 10)))
  (send my-plot :make-labels)
  (send my-plot :jason-add-labels \"HI\" '(200 50))
  (send my-plot :jason-draw-label 100 100 100 100)

  (plot (lambda(x y) (* x y)))
  (send my-plot :make-labels)

  (def my-plot (histogram (rand.u 100)))
  (send my-plot :make-labels)
"
  (if (send self :has-slot 'jason-box-coords) nil
    (let ()
      (dolist
       (i '(jason-plots jason-plots-with-labels jason-label-coordinates
			jason-plot-labels jason-horiz-vert jason-vert jason-horiz
			(jason-c-horiz-vert 0) (jason-c-vert 0) (jason-c-horiz 0)
			(jason-horiz-align t) jason-box-coords
			))
       (if (listp i)
	   (let ((j (second i)) (i (first i)))
	     (eval-string (format nil "(send self :add-slot '~a ~s)" i j))
	     (eval-string
	      (format nil "(defmeth self :~a (&optional (val nil set))
  (if set (setf (slot-value '~a) val) (slot-value '~a)))" i i i))
	     )
	 
	 (let ()
	   (eval-string (format nil "(send self :add-slot '~a)" i))
	   (eval-string
	    (format nil "(defmeth self :~a (&optional (val nil set))
  (if set (setf (slot-value '~a) val) (slot-value '~a)))" i i i)))))
      
      (send self :add-mouse-mode 'jason-get-drag-labels
	    :title "Get Drag Labels"
	    :cursor 'hand
	    :click :jason-get-drag-labels)
      
      (send self :add-mouse-mode 'jason-draw-label
	    :title "Draw Label"
	    :cursor 'arrow
	    :click :jason-draw-label)
      
      (send self :add-mouse-mode 'jason-erase-label
	    :title "Erase Label"
	    :cursor 'brush
	    :click :jason-erase-label)
      
      (send self :add-mouse-mode 'jason-align-labels
	    :title "Align Labels"
	    :cursor 'finger
	    :click :jason-align-labels)
      
      (send self :add-mouse-mode 'jason-drag-labels
	    :title "Drag Labels"
	    :cursor 'arrow
	    :click :jason-drag-labels)
      )
    )
  (if launch (make-label-dialog self))
  )

(defun make-label-dialog (plot)
  (let* (
	 (v-h-ask (send text-item-proto :new "Place Label: "))
	 (v-h-get
	  (send choice-item-proto :new (list "Horizontally" "Vertically")
		:action #'(lambda () (send plot :jason-c-horiz-vert
					   (send self :value)))))
	 (h-just-ask (send text-item-proto :new "Horiz/Vert Justification:"))
	 (h-just-get
	  (send choice-item-proto :new 
		(list "Left" "Centered" "Right")
		:action #'(lambda () (send plot :jason-c-horiz 
					   (send self :value)))))
	 (v-just-ask (send text-item-proto :new "Vert/Horiz Justification"))
	 (v-just-get (send choice-item-proto :new (list "Above" "Below")
			   :action #'(lambda () (send plot :jason-c-vert 
						      (send self :value)))))
	 (p-r-ask (send text-item-proto :new "Action:"))
	 (p-r-get
	  (send
	   choice-item-proto
	   :new 
	   (list "Draw Labels" "Erase Labels" "Align Labels" "Drag Labels")
	   :action
	   #'(lambda ()
	       (let ((p-r (send self :value)))
		 (case
		  p-r
		  (0 (send plot :mouse-mode 'jason-draw-label))
		  (1 (send plot :mouse-mode 'jason-erase-label))
		  (2 (send plot :mouse-mode 'jason-align-labels))
		  (3 (send plot :mouse-mode 'jason-get-drag-labels))
		  )))))
	 (dialog
	  (send dialog-proto :new (list
				   (list v-h-ask v-h-get)
				   (list h-just-ask h-just-get)
				   (list v-just-ask v-just-get)
				   (list p-r-ask p-r-get))))
	 (mouse-mode (send plot :mouse-mode))
	 )
    (send plot :mouse-mode 'jason-draw-label)

    ;; I change the close method to reset the mouse-mode to its previous value.
    (defmeth dialog :close()
      (call-next-method)
      (send plot :mouse-mode mouse-mode)
      )
    ))

