(defun normal-vector (f &rest p)
  "
;; Description: function (normal-vector f &rest p)

	Calculates the normal vector to a function of two variables f at point p:
that is, the cross product of (1,0,grad_x) and (0,1,grad_y).

;; Examples:
 (func f(x y) x*y)
 (normal-vector #'f 0 0)

 ;; You can define a function using variables other than x and y:
 (func f(u v) -u*v)
 (normal-vector #'f 0 0)

 ;; Getting fancy now:
 (plot f)
 (def a (rand.u 10))
 (def b (rand.u 10))
 (mapcar
  (lambda(a b)
    (def normal (coerce (normal-vector #'f a b) 'list))
    (send my-plot :add-plot-item
	  (list 'lines
		(list a (+ a (first normal)))
		(list b (+ b (second normal)))
		(list (f a b) (+ (f a b) (third normal)))
		)
	  )
    )
  a b
  )
 (def stuff (send my-plot :plot-item))
"
  ;; Calculate the gradient at point p:
  (let ((grad (apply (gradient2 f) p)))
    ;; then the cross-product:
    (cross (list 1 0 (first grad)) (list 0 1 (second grad)))
    )
  )

