;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                     *object* method :apropos
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#|
 I have an :apropos method in my
 <http://galen.med.virginia.edu/~sdm7g/XES/Sources/genutils.lsp>
 which makes it easier to find method names:

 Also, look at some other code for examples -- there's a lot of stuff 
 on http://www.stat.ucla.edu, and I have quite a lot of graphics code
 on http://galen.med.virginia.edu/~sdm7g/XES/


 ---|  Steven D. Majewski   (804-982-0831)  <sdm7g@Virginia.EDU>  |---
 ---|  Department of Molecular Physiology and Biological Physics  |---
 ---|  University of Virginia             Health Sciences Center  |---
 ---|  P.O. Box 10011            Charlottesville, VA  22906-0011  |---
 All power corrupts and obsolete power corrupts obsoletely. - Ted Nelson
|#
(defmeth *object* :apropos (str &key help)
  "
Author: Steven Majewski

	:apropos makes it easier to find method names of an object.

;; Examples:
 (send andyplot-proto :apropos \"draw\")
 (send andyplot-proto :apropos \"draw\" :help t)
 (send histogram-proto :apropos \"plot\")
"
  (dolist 
   (name (send self :doc-topics))
   (when (string-search str (symbol-name name)) 
	 (if help 
	     (progn (send self :help name) (terpri))
	   (format t "~s~%" name))))
  (format t "~a : ~a SLOTS:~%" (slot-value 'proto-name) self)
  (dolist 
   (name (send self :slot-names))
   (when (string-search str (symbol-name name))
	 (format t "~s : ~a~%" name (slot-value name)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				:dispose
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth *object* :dispose (&rest e)
  "
 Defined this so that calls to call-next-method have someplace to end
 up. This method doesn't do anything!
"
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				:add
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth *object* :add (&rest e)
  "
 Defined this so that calls to call-next-method have someplace to end
 up. This method doesn't do anything!
"
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				:class-slots
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth *object* :class-slots ()
  "
;; Author: Andy Long
;; Created: April, 1998

	Reports - but does not attempt to change - the number of slots that are
class slots and not instance slots (see (help defproto) for a mini-notion of
the difference). 

	Using method :add-slot adds a slot to the class variables rather than
the instance variables, an important distinction as the instance slots are
inherited by children (and can be modified by them), but the class instances
are inherited but may not be modified by a child.
"
  (set-difference
   (remove-dups (send self :slot-names))
   (send self :slot-value 'instance-slots))
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                      *object* method  :save-self
;;
(defmeth *object* :save-self (&optional name)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          :save-self Doc:
;;
;; Author: Andy Long
;; Created: Sat Nov 1 08:57:02 1997
;;
;; Description/Arguments:

       *object* method :save-self (&optional name)
  
	The name is a symbol of the proto necessary to save the object. This
method asks for the values of :own-slots, and supplies the appropriate save
command to re-create the object.

	I created this method as a helper for the :save method I create in
macro uber-proto. I couldn't think of any slicker way to solve the :save
problem there.

;; See Also: uber-proto

;; Examples:
 
 (uber-proto longs (hio hoi) (thisn))
 (uber-proto andy (hio hoi) (thisn) longs)
 (def andyroo (send andy :new :hio 1 :hoi 2))
 (send andyroo :save-self)
 (send andy :save-self 'longs)
"
  (let* (
	 (slots (send self :own-slots))
	 (vals (mapcar (lambda(x) (slot-value x)) slots))
	 (slots (mapcar #'tack-colon-onto-symbol slots))
	 (vals (smoosh (transpose (list slots vals))))
	 (name (if name name (send self :slot-value 'proto-name)))
	 )
    `(apply #'send ,name :new ',vals)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				:become
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
#| - ael - Sun Apr 26 11:47:23 1998
From: "Steven D. Majewski" <sdm7g@Virginia.EDU>
To: Xlisp List <stat-lisp-news@umnstat.stat.umn.edu>
Subject: Rearranging the hierarchy -- :BECOME (was: dialog focus)

I bought the latest issue of MacTech magazine, just because it
had an article "Rhapsody in Purple" -- about how to "backpatch" 
applications in OpenStep/NextStep. The example they give is 
similar to the sort of problems I've been wrestling with. 
 In their example, they want to replace part of the standard
framework (NSWindow) with a custom variant (PurpleWindow) so 
that descendants of NSWindow inherit from PurpleWindow ( which
inherits from NSWindow). 
 They mention that ObjectiveC has a PoseAsClass method, which can
be used to rearrange the class hieracrhy. The example they
give is going from:


           [ NSWindow ]
            /        \
[PurpleWindow]     [ NSPanel ]


to:

          [ NSWindow ]
               |
          [ PurpleWindow]
                    \
                   [ NSPanel ] 


XlispStat has a similar ability to rearrange the hierarchy with
:REPARENT. There's a slightly higher level variant of :REPARENT
defined in objects.lsp: :RETYPE, which is restricted to only
work on instances.   A prototype is defined as an object with
it's own 'INSTANCE-SLOTS slot, and :RETYPE wants it's SELF to
not be a proto and it's arg to be a proto. ( :RETYPE is however
the method to use for the example in another thread on how to
temporarily change the behaviour of a graph window to be a 
PS-GRAPH-PROTO or LATEX-GRAPH-PROTO so that the effects of a 
:REDRAW can be redirected. ) 
 - lea - |#
(defmeth *object* :become (proto)
  (send self :reparent proto)
  (dolist (s (send proto :slot-value 'instance-slots))
           (send self :add-slot s (send proto :slot-value s)))
  (setf (slot-value 'instance-slots)
         (delete-duplicates 
           (append (send proto :slot-value 'instance-slots)
                    (slot-value 'instance-slots))))
  nil
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				find-descendants-of
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun find-descendants-of (proto)
  "
;; Description: (find-descendants-of proto)

;; Author: Steven D. Majewski

	'The :become method has a problem: the changes in instance-slots for
descendants of the proto which were defined before the redefinition takes
place, don't get those added slots.

	'The only solution I can think of, is to search thru all of the symbols
 to find the descendants and to fix them. This function by itself might be a
 useful utility for exploring the XlispStat object hierarchy....'

;; Key functions: do-all-symbols, symbol-value, push, kind-of-p, boundp

;; Examples:
 (find-descendants-of graph-proto)
"
  ;; remove the proto itself from this list:
  (remove
   proto
   (let (descendants)
     (do-all-symbols
      (s descendants)
      (when (boundp s)
	    (let ((v (symbol-value s)))
	      (when (kind-of-p v proto)
		    (push v descendants))))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				:new-by-vals
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmeth *object* :new-by-vals (&rest args)
  "
;; Description:

	:new-by-vals is a method which allows one to give (all!) the
instance-slot values one after another, without using the 

;; Examples:
 (defproto
   trial-proto 
   '(num-vars lambda domain-test-function derivative anti-derivative analytic)
   )
 ;; the default is that, to create a new example of a prototype, you have to
 ;; set the values of required slots as follows:
 (def
  stuff
  (send 
   trial-proto
   :new
   :num-vars 1
   :lambda (lambda(x) (sin x)) 
   :derivative (lambda(x) (cos x))
   :anti-derivative (lambda(x) (- (cos x)))
   :analytic t
   ))
 (show stuff)

 ;; :new-by-vals lets us simply provide a set of values (respecting the order
 ;; of the instance-slots) to create the new function examples.
 ;; (here's the order to respect - same as in the definition above)
 (send trial-proto :slot-value 'instance-slots)
 (def
  stuff
  (send trial-proto
        :new-by-vals
        1 (lambda(x) (sin x)) nil (lambda(x) (cos x)) (lambda(x) (- (cos x))) t
        ))
 (show stuff)
"
  (let (
	(instance (send self :new))
	(slots (send self :slot-value 'instance-slots))
	)
    (mapcar (lambda(x y) (send instance :slot-value x y)) slots args)
    instance
    )
  )

;; add to manual: add-normal-assessors
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 tack-colon-onto-symbol
;;
(defun tack-colon-onto-symbol(x)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          tack-colon-onto-symbol Doc:
;;
;; Author: Andy Long
;; Created: Thu Oct 30 23:45:55 1997
;;
;; Description/Arguments:

       (tack-colon-onto-symbol x)
  
;; Notes:

;; See Also:

;; Examples:
 (tack-colon-onto-symbol 'hi)
 (tack-colon-onto-symbol '(ho hi))
"
  (if (listp x) (mapcar #'tack-colon-onto-symbol x)
    (read-from-string (strcat ":" (symbol-name x))))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                 new-assessor-method
;;
(defmacro new-assessor-method (proto name)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                          new-assessor-method Doc:
;;
;; Author: Andy Long
;; Created: Thu Nov 6 20:42:49 1997
;;
;; Description/Arguments: macro (proto name)

	Writes the usual assessor method, which either sets a slot by the given
name if given an argument, or returns its value.

;; Notes:

;; See Also: add-normal-assessors

;; Examples:
 (defproto neat '(layers))
 (new-assessor-method neat layers)
 (send neat :layers)
 (send neat :layers 4)
 (send neat :layers)
"
  (let ((name1 (tack-colon-onto-symbol name)))
    (eval `(normal-assessor ,name1 ,name ,proto))))
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				add-normal-assessors
;;
(defun add-normal-assessors(proto &optional slots)
  "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;			add-normal-assessors Doc:
;;
;; Author: Andy Long
;; Created: Mon Apr 27 10:59:47 1998
;;
;; Description/Arguments: add-normal-assessors

	Adds the normal assessors to a proto-type (that is, for each instance
slot a method is created which allows one to easily change the slot value or to
check its value. See the example.

;; Arguments: prototype, usually one freshly created.

;; Returns: nil

;; Notes: Don't know why they used 'assessor': I'd have used accessor. But
;; since I find this function often, I'll leave it as is.

;; See Also: 

;; Key Functions: normal-assessor symbol-name :slot-value

;; Examples:
 ;; create a proto of *object* with two new instance slots
 (defproto stuff '(hi lo))
 ;; (send stuff :hi 3)
 ;; would cause an error: one is obliged to change the slots like this:
 (send stuff :slot-value 'hi 2)
 (send stuff :slot-value 'hi)
 ;; as soon as we add the normal assessors, we can access slots directly:
 (add-normal-assessors stuff)
 (send stuff :hi 3)	;; changes the slot-value from 2 to 3
 (send stuff :hi)	;; returns the slot-value 3
 (send stuff :lo)	;; nil
"
  (mapcar
   (lambda(x)
     (let ((y (tack-colon-onto-symbol x)))
       (eval `(normal-assessor ,y ,x ,proto))))
   (if slots slots (send proto :slot-value 'instance-slots)))
  nil
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				active-windows
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun active-windows ()
  "Args: ()
  Returns list of active windows. For some reason this wasn't exported in the
  standard distribution.

;; Examples:
 (plot sin)
 (hulp)
 (active-windows)
"
  (remove-if-not #'(lambda (x) (kind-of-p x window-proto))
		 (mapcar #'third *hardware-objects*)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				close-all-windows
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun close-all-windows ()
  "A quick and dirty way of closing all window-protos.

;; Examples:
 (close-all-windows)
"
  (map-elements #'send (active-windows) :close))
