#|
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                                            ;;
;;			Parser.lsp: Function Parser                           ;;
;;                                                                            ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	I worked long and hard for this baby! My parser, so that we can use
more natural (infix) notation for function definitions in lisp. 


	The workhorse is parser, although the macro 

				func

	will probably get the use:


	Based loosely on the differentiation stuff contained in deriv.lsp.
Hoping that my success there will rub off!

	The idea is that "it ought to be easy" to use a fortran function
definition to define a function in lisp. The user never need know about reverse
polish notation! I guess that this is an infixer.

--------------------------------------------------------------------------------
		Problem: I want to be giving instructions like these:

(func f (x) x + 3)
(get-lambda-expression #'f)

(func f (x) sin x)
(get-lambda-expression #'f)

(func f (x) x ^ 3 + tan x)
(get-lambda-expression #'f)

(func f (x) x ^ 3 + (tan x) ^ 2)
(get-lambda-expression #'f)

(func f (x y) y ^ 3 + ln x)
(spin-function #'f 1 4 0 1)

--------------------------------------------------------------------------------
				So....
I'll need to pick off the function name,
	f,
of variables
	(x),
Defined by the expression
	x ^ 3 + (tan x) ^ 2
which I will have to turn into the lsp function:

(defun f (x)
  (+ (^ x 3) (^ (tan x) 2))
  )

--------------------------------------------------------------------------------

				Plan:

	My plan, then, is to make a list of the precedence of the operations,
and then go through the expression e in order, arranging things correctly for
the particular case.

	I'll get it working for a few simple cases, and then build on it as I
have for the differentiation rules of deriv.lsp.

--------------------------------------------------------------------------------


						Andy Long, Fall 1995


--------------------------------------------------------------------------------
Notes: 

			Setting an element of a list:

(setq mylist '(a b c d))
(setf (nth 3 mylist) 'here-i-am)
mylist

(setl list index value)

Now defined in matrix.lsp. check.


			precedence list from Bison example:

     exp:      NUM                { $$ = $1;         }
             | exp '+' exp        { $$ = $1 + $3;    }
             | exp '-' exp        { $$ = $1 - $3;    }
             | exp '*' exp        { $$ = $1 * $3;    }
             | exp '/' exp        { $$ = $1 / $3;    }
             | '-' exp  %prec NEG { $$ = -$2;        }
             | exp '^' exp        { $$ = pow ($1, $3); }
             | '(' exp ')'        { $$ = $2;         }

I assume that the unary functions come after those, and we'll need to do
something special to handle the two separate negative "-" cases.

--------------------------------------------------------------------------------
|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				 Special variables:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(def *top-functions* '(^ ** < > = >= <= /=))

;; Note: "- asdf" will be turned into + (- asdf) next, so "-" has next priority
(def *top-op* '(-))
;; This is a preprocessing step, however: we'll change subtraction into
;; addition for the binary operations.
(def *unary-functions* 
     '(sin cos tan cot sqrt log ln exp cosh sinh tanh abs
	   acos asin atan acosh asinh atanh))
(def *binary-functions* '(/ *))
(def *precedence-list* (combine
			*unary-functions*
			*top-functions*
			*binary-functions*
			*top-op*
			'+
			)
     )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				 Parser, the master:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun parser (e)
  "
	Parser is the workhorse of the prefix-to-infix code, although as a
function it isn't as handy from the command line. It's used by calc, and func.

;; Examples:
 (parser '((1 / (x - exp (sqrt 3))) * (tan x) ^ (3 / x)))
 (parser  '(- 3 ** (y - x) / (tan (x) ^ 3)))
 (parser '(if (x < 0) 0 (sin (x + 3))))

 (parser '((list 1 2) * (list 3 4)))
 (parser '((1 2) * (3 4)))
"
  (let (
	(len-e (length e))
	(new-e (copy-list e))
	(e-member nil)
	(j nil)
	)
    (cond
     ;; parser handles "if"s:
     ((eq (first e) 'if)
      (setq new-e (list 'if
			(if (listp (second e)) (parser (second e)) (second e))
			(if (listp (third e)) (parser (third e)) (third e))
			(if (listp (fourth e)) (parser (fourth e)) (fourth e))
			)
	    )
      )
     ;; parser will keep explicit lists as explicit lists:
     ((eq (first e) 'list)
      (setq new-e (list 'list))
      (dolist (j (rest e))
	      (setq new-e (combine new-e (if (listp j) (parser j) j)))
	      )
      )
     ;; parser will shake off quotes:
     ((eq (first e) 'quote)
      (setq new-e (parser (rest e)))
      )
     ;; make parser indifferent to calc:
     ((eq (first e) 'calc)
      (setq new-e e)
      )
     #|
;; this doesn't work, as it defuses expressions like (1 + 3)
     ((numberp (first e))
      (setq new-e (eval `(combine '(list) ,@e)))
      )
|#
     ;; so I'll try the following: make what appear to be lists into explicit
     ;; lists:
     ((and (> len-e 1) (numberp (first e)) (numberp (second e)))
      (setq new-e (eval `(combine '(list) ,@e)))
      )
     (t
      (list
       (dotimes (j len-e)
		(setq e-member (nth j new-e))
		(if (listp e-member) (setl new-e j (parser e-member)))
		)
       ;; Now each list represents a coherent lisp whole.
       (if (= (length new-e) 1) (setq new-e (single-list new-e))
	 (setq new-e (parse-whole new-e)))
       )
      )
     )
    new-e
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; here's a helper:

(defun single-list (x)
  "
;; Examples:

 (single-list '(x))
 (single-list '((x)))
 (single-list '((((x)))))
"
  (if (and (listp x) (= (length x) 1)) (single-list (first x)) x)
  )
#|
;; formerly:
(defun single-list (x)
  (if (and (listp x) (= (length x) 1))
      (if (and (listp (first x)) (= (length (first x))))
	  (single-list (first x))
	(first x)
	)
    x
    )
  )
|#

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Parse-whole, the composer:
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun parse-whole (e)
  "
This reflects my newest precedence:

	unaries first, so that

		x ^ tan x
will be properly processed, then 

	exponentials
	/ *
	-
	+

Notice that parse-binaries gets called twice:
the first time handle (/ *), then a second time
after (-) gets turned into addition, for (+).
"
  (let (
	(new-e e)
	)
    ;;  (if *andy.debug* (print (list "parser" new-e)))
    (setq new-e (parse-unaries new-e))
    ;;  (if *andy.debug* (print (list "parse-unaries" new-e)))
    (setq new-e (parse-top-function new-e))
    ;;  (if *andy.debug* (print (list "parse-top-function" new-e)))
    (def *binary-functions* '(/ *))
    (setq new-e (parse-binaries new-e))
    ;;  (if *andy.debug* (print (list "parse-binaries" new-e)))
    (setq new-e (parse-top-op new-e))
    ;;  (if *andy.debug* (print (list "parse-top-op" new-e)))
    (def *binary-functions* '(+))
    (setq new-e (parse-binaries new-e))
    ;;  (if *andy.debug* (print (list "parse-binaries" new-e)))
    (def *binary-functions* '(/ * +))
    new-e
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				parse-top-function
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun parse-top-function (expression)
  (let (
	(e expression)
	(len-e (length expression))
	(new-e nil)		;; we'll write new-e
	)
    ;;    (where-am-i (list "top-function" e))
    (dotimes
     (j len-e)
     (setq e-member (nth j e))
     (cond
      ;; if nil, collapse:
      ((not e-member) nil)
      ;; if a list or cons, then skip:
      ((consp e-member) (setq new-e (append new-e (list e-member))))
      ((listp e-member) (setq new-e (append new-e (list e-member))))
      ;; if a top-function, then:
      ((member e-member *top-functions*)
       (list
	(setq op (first (member e-member *top-functions*)))
	(if (eq op '**) (setq op '^))
	(list
	 (setq e-base (last new-e))
	 (setq e-power (list (nth (1+ j) e)))
	 (setl new-e (1- (length new-e)) (append (list op) e-base e-power))
	 (setl e (1+ j) nil)
	 )
	)
       )
      (t (setq new-e (append new-e (list e-member))))
      )
     )
    (if (= (length new-e) 1) (first new-e) new-e)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				parse-top-op
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun parse-top-op (expression)
  (let (
	(len-e (length expression))
	(e expression)
	(new-e nil)		;; we'll write new-e
	(e-member nil)
	(j nil)
	)
;    (print (list "top-op" e))
    (dotimes
     (j len-e)
     (setq e-member (nth j e))
     (cond
      ;; if nil, collapse
      ((not e-member) nil)
      ;; if a list or cons, then skip:
      ((consp e-member)
       (setq new-e (append new-e (list e-member))))
      ((listp e-member)
       (setq new-e (append new-e (list e-member))))
      ;; if a top-op, then:
      ((member e-member *top-op*)
       (setq op (first (member e-member *top-op*)))
       (cond
	((eq op '-)
	 (list
	  (cond
	   ((= j 0) (list
		     (setq new-e (list (append '(-) (list (second e)))))
		     (setl e 1 nil)
		     )
	    )
	   (t (list
	       (setq new-e
		     (append new-e '(+)
			     (list (append '(-) (list (nth (1+ j) e)))))
		     )
	       (setl e (1+ j) nil)
	       )
	      )
	   )
	  )
	 )
	)
       )
      (t (setq new-e (append new-e (list e-member))))
      )
     )
    (if (= (length new-e) 1) (first new-e) new-e)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				parse-binaries
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun parse-binaries (expression)
;;  (where-am-i (list "parse-binaries" expression (length expression)))
  (let (
	(e expression)
	(len-e (length expression))
	(new-e nil)		;; we'll write new-e
	(e-member nil)
	)
    (dotimes
     (j len-e)
     (setq e-member (nth j e))
     (cond
      ((= 1 len-e) (setq new-e (list e)))
      ;; if nil, collapse:
      ((not e-member) nil)
      ;; if a list or cons, then skip:
      ((consp e-member) (setq new-e (append new-e (list e-member))))
      ((listp e-member) (setq new-e (append new-e e-member)))
      ;; if a binary-function, then:
      ((member e-member *binary-functions*)
       (list
	(setq op (first (member e-member *binary-functions*)))
	(list
	 (setq e-base (last new-e))
	 (setq e-power (list (nth (1+ j) e)))
	 (setl new-e (1- (length new-e)) (append (list op) e-base e-power))
	 (setl e (1+ j) nil)
	 )
	)
       )
      (t (setq new-e (append new-e (list e-member))))
      )
     )
    (if (= (length new-e) 1) (first new-e) new-e)
    )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				parse-unaries
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun parse-unaries (expression)
  ;;  (where-am-i (list "parse-unaries" expression (length expression)))
  (let (
	(e expression)
	(len-e (length expression))
	(new-e nil)		;; we'll write new-e
	(e-member nil)
	)
    (dotimes
     (j len-e)
     (setq e-member (nth j e))
     (cond
      ;; if a list, then skip:
      ((= 1 len-e) (setq new-e (list e)))
      ((not e-member) nil)
      ((consp e-member)
       (setq new-e (append new-e (list e-member))))
      ((listp e-member)
       (setq new-e (append new-e (list e-member))))
      ;; if a unary-function, then:
      ((member e-member *unary-functions*)
       (list
	(setq op (first (member e-member *unary-functions*)))
	(list
	 (setq e-power (list (nth (1+ j) e)))
	 (if new-e (setq new-e (append new-e (list (append (list op) e-power))))
	   (setq new-e (list (append (list op) e-power))))
	 (setl e (1+ j) nil)
	 )
	)
       )
      (t (setq new-e (append new-e (list e-member))))
      )
     )
    (if (= (length new-e) 1) (first new-e) new-e)
    )
  )

#|
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun test-parser ()
  (print (parser  '(- 3)))
  (print (parser  '(2 - 3)))
  (print (parser  '(- 3 ^ x)))
  (print (parser  '(- 3 ^ (- x))))
  (print (parser  '(- 3 ^ (y - x))))
  (print (parser  '(- 3 ^ (y - x) / 6)))
  (print (parser  '(- 3 ^ (y - x) * (x ^ 3))))
  (print (parser  '(- 3 ^ (y - x) + (x ^ 3))))
  (print (parser  '(- 3 ** (y - x) / (tan x ^ 3))))
  (print (parser  '(- 3 ** (y - x) / (tan (x) ^ 3))))

  (print (parser '(x + 3)))
  (print (parser '(sin x)))
  (print (parser '(x ^ 3 + tan x)))
  (print (parser '(x ^ 3 + (tan x) ^ 2)))
  (print (parser '(y ^ 3 + ln x)))
  (print (parser '(ln (y ^ 3 + ln x))))
  (print (parser '(cosh x - x)))
  (print (parser '((x - 4) / (2 * x + 4) ^ 2)))
  (print (parser '(((((x)))))))
  (print (parser '(x - exp (sqrt 3))))
  (print (parser '(x ^ 3 - 2 * x + 3)))
  (print (parser '(if (x < 0) (0) (sin x / 3))))
  (print (parser '(if (x < 0) 0 (sin x))))
  nil
  )

The most horrible function that I can think of:

(parser '((1 / (x - exp (sqrt 3))) * (tan x) ^ (3 / x)))
(defun horrible (x) (* (/ 1 (+ X (- (EXP (SQRT 3))))) (^ (TAN X) (/ 3 X))) )
(plot-function #'horrible .5 1)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Description for FUN:
;;
(defmacro fun (&rest e)
  "
;; Examples:
 (fun (1 / (x - 3)) * (tan x) ^ (3 / x))
 (fun 3 * cos 2)
 ;; these also work:
 (fun (1 / (x - 3)))
 (fun (3 * cos 2))
 ;; fun doesn't mind spurious parenthesis about!
"
  `(parser ',e)
  )
|#
