;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;				Plotit
;;			the workhorse of plot
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun plotit(x &key special-plot-args)
  "
	Plotit is the workhorse of plot. It takes a single argument (which may
be a list of info), and goes through a check-list to figure out what to do with
the stuff.

See: plot
"
  (let (
	(p.range *range*)
	(p.domain (if *domain* *domain* '(-1 1)))
	(npoints (if *points* *points* 100))
	(tplot (if *lines* 'lines 'points))
	(titem (if *lines* *line-width* *plot-symbol*))
	(mat.type (if *lines* #'matrix-plot #'matrix-point-plot))
	nvars
	dimen
	plotit.x
	plotit.y
	plot-items
	funcs
	)
    
    ;; okay, this is where the various options come into play:
    (cond 
     
     ;; x is a function, or a symbol which has a function definition bound to
     ;; it (i.e. fboundp):
     ((or 
       (functionp x)
       (and (symbolp x) (fboundp x)))
      (if (symbolp x) (setq x (eval `(turn-it-into-a-function ,x))))

      ;; Decide whether it's one- or two-d:
      ;; nvars is the number of independent variables (from the lambda exp)
      (setq nvars (length (second (get-lambda-expression x))))
      (setq dimen (1+ nvars))
      (cond
       ;; added the nvars=0 case for closures, which have 0 args:
       ((or (= nvars 1) (= nvars 0))
	(if (not *domain*) (setq *domain* '(-1 1)))
	(setq plotit.x (apply #'rseq (combine *domain* npoints))
	      plotit.y (mapcar x plotit.x))
	(def my-plot (send andyplot-proto :new 2 :show nil))
	(apply #'send my-plot :range 0 *domain*)
	(setq
	 plot-items (list
		     (list 'function x *draw-color* *lines* *line-width*
			   *line-type*))
	 p.range (remove-if #'null (remove-if #'complexp plotit.y))
	 p.range (if p.range p.range '(-1 1))
	 p.range (get-nice-range (min p.range) (max p.range) *y-tics*)) 
	)
       ((= nvars 2)
	(if (not *spin-domain*) (setq *spin-domain* '(-1 1 -1 1)))
	(if (not *spin-points*) (setq *spin-points* 6))
	(def
	 my-plot
	 (spin-andy
	  (list x)
	  (first *spin-domain*) (second *spin-domain*)
	  (third *spin-domain*) (fourth *spin-domain*)
	  :num-points *spin-points*
	  :spline *spin-spline*
	  )
	 )
	(setq plot-items (list x))
	)))
     
     ;;  x is a list:
     ((listp x) ; x is a list of some sort:
      (let ((x_len (length x)))   
	(setq dimen 2)
	
	;; Given that it's a list, these are the options:
	(cond
	 
	 ;; x is a list containing (at least one, maybe some) functions:
	 ((functionp (first x))
	  
	  ;; Decide whether it's one- or two-d function:
	  ;; nvars is the number of independent variables (from the lambda exp)
	  (setq nvars (length (second (get-lambda-expression (first x)))))

	  #| - Sun Mar 29 12:02:37 1998
I'm going to establish the rule that if it's a function, but doesn't have a
lambda-expression, then it's a subroutine or a closure with a single argument.
|#
	  (if (= nvars 0) (setq nvars 1))
	  ;;ael

	  (setq dimen (1+ nvars))
	  
	  (cond
	   
	   ((functionp (second x)) ;; it's a list of a bunch of functions!
	    
	    (cond
	     ((= nvars 1)
	      ;;  might have some domain info at the tail end:
	      (let (
		    (sl (nth (- x_len 2) x))
		    (final (final x))
		    )
		(if (and (numberp sl) (numberp final))
		    (setq
		     p.domain (list sl final)
		     funcs (butlast (butlast x))
		     )
		  (setq funcs (rest x))
		  ))
	      (setq
	       plotit.x (apply #'rseq (combine p.domain npoints))
	       plotit.y (mapcar (first x) plotit.x))
	      (def my-plot (send andyplot-proto :new 2 :show nil))
	      (apply #'send my-plot :range 0 p.domain)
	      (dolist
	       (f funcs)
	       (setq plotit.y (combine plotit.y (mapcar f plotit.x))))
	      (setq
	       plot-items (append (list (first x)) funcs)
	       p.range (remove-if #'null (remove-if #'complexp plotit.y))
	       p.range (if p.range p.range '(-1 1))
	       p.range (get-nice-range (min p.range) (max p.range) *y-tics*))
	      )
	     ((= nvars 2)
	      (setq funcs (rest x))
	      (if (not *spin-domain*) (setq *spin-domain* '(-1 1 -1 1)))
	      (if (not *spin-points*) (setq *spin-points* 6))
	      (def
	       my-plot
	       (spin-andy
		x 
		(first *spin-domain*) (second *spin-domain*)
		(third *spin-domain*) (fourth *spin-domain*)
		:num-points *spin-points*
		:spline *spin-spline*
		)
	       )
	      (setq plot-items x)
	      )
	     (t (error "Haven't a clue how to plot this: sorry!"))
	     ))
	   
	   ;; x is a list containing some domain information:
	   ((= (length x) 3)
	    (setq p.domain (list (second x) (third x)))
	    (def my-plot (send andyplot-proto :new 2 :show nil))
	    (setq
	     plot-items (list (first x))
	     plotit.x (apply #'rseq (combine p.domain npoints))
	     plotit.y (mapcar (first x) plotit.x)
	     p.range (remove-if #'null (remove-if #'complexp plotit.y))
	     p.range (if p.range p.range '(-1 1))
	     p.range (get-nice-range (min p.range) (max p.range) *y-tics*)) 
	    )
	   
	   ;; not sure what's going on: just plot the first function:
	   (t
	    (format
	     t "~%What's that? I'll cogitate on it: plot the first one...~%")
	    (def my-plot (send andyplot-proto :new 2 :show nil))
	    (setq plot-items (list (first x)))
	    )
	   )
	  )
	 
	 ;; x is really a straight list of numbers:
	 ((and (not (listp (first x)))
	       (or (typep (first x) 'float)
		   (typep (first x) 'complex)
		   (typep (first x) 'fixnum)))
	  (if (typep (first x) 'complex)
	      (let (
		    (reals (realpart x))
		    (imags (imagpart x))
		    )
		(setq dimen 2)
		(def my-plot (send andyplot-proto :new 2 :show nil))
		(send my-plot :add-plot-item
		      (list tplot reals imags *draw-color* titem
			    (cond
			     (*lines* *line-type*)
			     ((if (and *point-labels*
				       (= (length *point-labels*)
					  (length reals)))
				  *point-labels*)))))
		(setq p.domain (get-nice-range (min reals) (max reals) *x-tics*))
		)
	    (let ((seq (iseq 1 x_len)))
	      (setq dimen 1)
	      (def my-plot (send andyplot-proto :new 2 :show nil))
	      (send my-plot :add-plot-item
		    (list tplot seq x *draw-color* titem
			  (cond
			   (*lines* *line-type*)
			   ((if (and *point-labels*
				     (= (length *point-labels*)
					x_len))
				*point-labels*)))))

	      (setq p.domain (get-nice-range 1 x_len *x-tics*))
	      ))
	  )
	 
	 ;; x is a pair of lists:
	 ((= x_len 2 )
	  (setq dimen 2)
	  (let* (
		 (xx (first x))
		 (yy (second x))
		 ;; in this previous incarnation, we removed all nil values of
		 ;; xx and yy; this seemed like a good idea, but what we should
		 ;; really do is split them into separate lists:
		 #|
		 (gooduns (sort-data (intersection (which xx) (which yy))))
		 (xx (select xx gooduns))
		 (yy (select yy gooduns))
|#
		 (where (union (where xx nil #'eq) (where yy nil #'eq)))
		 (split-x (copy-list xx))
		 (split-y (copy-list yy))
		 nils
		 new-expressions
		 )

	    (def my-plot (send andyplot-proto :new 2 :show nil))

	    (if (= (length where) 0)
		(setq new-expressions 
		      (list (list tplot xx yy *draw-color* titem
			    (cond
			     (*lines* *line-type*)
			     ((if (and *point-labels*
				       (= (length *point-labels*)
					  (length xx)))
				  *point-labels*))))))
	      (let (
		    (gooduns (sort-data (intersection (which xx) (which yy))))
		    )
		(setf nils (repeat nil (length where))
		      xx (select xx gooduns)
		      yy (select yy gooduns)
		      (select split-x where) nils
		      (select split-y where) nils
		      )
		(mapcar
		 (lambda(x y)
		   (setq
		    new-expressions 
		    (append
		     new-expressions
		     (list
		      (smoosh
		       (list tplot (list x) (list y) *draw-color* titem))))))
		 (list-split nil split-x)
		 (list-split nil split-y)
		 )
		)
	      )
	    (apply #'send my-plot :add-plot-item new-expressions)
	    
	    (setq
	     p.domain (remove-if #'null (remove-if #'complexp xx))
	     p.domain (if p.domain p.domain '(-1 1))
	     p.domain (get-nice-range (min p.domain) (max p.domain) *x-tics*)
	     )
	    
	    ))
	 
	 ;; three points/lists:
	 ((= x_len 3)
	  (setq dimen 3)
	  (let ((seq (select x (iseq 3))))
	    (def my-plot (spin-plot seq))
	    (if *lines* (send my-plot :add-lines seq))
	    (setq plot-items (list seq))
	    )
	  )
	 
	 ;; ? what's going on here?
	 (t
	  (format t "~%What's that derned thing you passed me? I'll try!~%")
	  (cond
	   ((= (length (first x)) 2)
	    (let (
		  (xx (mapcar #'first x))
		  (yy (mapcar #'second x))
		  )
	      (setq
	       dimen 2
	       p.domain (remove-if #'null (remove-if #'complexp xx))
	       p.domain (if p.domain p.domain '(-1 1))
	       p.domain (get-nice-range (min p.domain) (max p.domain) *x-tics*))
	      (def my-plot (send andyplot-proto :new 2 :show nil))
	      (send my-plot :add-plot-item
		    (list tplot xx yy *draw-color* titem
			  (cond
			   (*lines* *line-type*)
			   ((if (and *point-labels*
				     (= (length *point-labels*)
					(length xx)))
				*point-labels*)))))

	      )
	    )
	   ((= (length (first x)) 3)
	    (format
	     t "~%Let's try a three-d plot, since there are three lists....~%")
	    (setq dimen 3)
	    (def my-plot
		 (spin-plot
		  (list
		   (mapcar #'first x) (mapcar #'second x) (mapcar #'third x)
		   )))
	    #| - ael - Sun Apr 26 09:44:05 1998
	    (send my-plot :add-slot 'plot-item)
	    (send my-plot :add-slot 'lines-to-kill)
	    (send my-plot :add-slot 'andy-options)
 - lea - |#
	    )
	   (t
	    (format
	     t "~%Let me paste the columns together and see what happens....~%")
	    (setq dimen 3)
	    (def my-plot (funcall mat.type (apply #'bind-columns x)))
	    )
	   )
	  )
	 )
	)
      )
     
     ;;  x is a matrix, rather than a list:
     ((matrixp x)
      (setq dimen 3)
      (def my-plot (funcall mat.type x))
      )
     
     ;;  x is a vector of numbers, possibly complex:
     ((and (vectorp x) (not (typep x 'string)))
      (cond
       ((realp (aref x 0))
	(setq
	 dimen 1
	 p.domain (get-nice-range 1 (length x) *x-tics*))
	(def my-plot (send andyplot-proto :new 2 :show nil))
	(send my-plot :add-plot-item
	      (list tplot (iseq 1 (length x)) x *draw-color* titem
		    (cond
		     (*lines* *line-type*)
		     ((if (and *point-labels*
			       (= (length *point-labels*)
				  (length x)))
			  *point-labels*)))))
	
	)
       ((complexp (aref x 0))
	(setq
	 dimen 2
	 p.domain
	 (get-nice-range (min (realpart x)) (max (realpart x)) *x-tics*))
	(def my-plot (send andyplot-proto :new 2 :show nil))
	(send my-plot :add-plot-item
	      (list tplot (realpart x) (imagpart x) *draw-color* titem
		    (cond
		     (*lines* *line-type*)
		     ((if (and *point-labels*
			       (= (length *point-labels*)
				  (length (realpart x))))
			  *point-labels*)))))
	
	)
       ))
     
     ;;  x may as well be a comic book:
     (t
      (error
       "Sorry, I can't figure out how to handle that thing you passed me.
It may as well be a comic book, for all I can tell. Try typing

                          (help plot)

for some examples of the use of the plot function. Or better yet, fix me
so that I can plot this sort of thing!")
      )
     )
     
    ;; Here we do a little last minute fenagling with the plot:
    (if (< dimen 3)
	(let()
	  (send my-plot :adjust-to-data)
	  (if *variable-labels*
	      (let()
		(send my-plot
		      :variable-label '(0 1) (select *variable-labels* '(0 1)))
		(send my-plot :y-axis t t *y-tics*)
		(send my-plot :x-axis t t
		      (if (third p.domain) (third p.domain) *x-tics*))
		))
	  
	  (if p.range (send my-plot :range 1 (first p.range) (second p.range)))
	  (send my-plot :range 0 (first p.domain) (second p.domain))
	  
	  (if *x-axis* (send my-plot :add-lines (send my-plot :range 0) '(0 0)))
	  (if *y-axis* (send my-plot :add-lines '(0 0) (send my-plot :range 1)))
	  
	  ;; This is useful! use those mouse-buttons:
	  (send my-plot :mouse-mode (if *andy-mode* *mouse-mode* 'selecting))
	  
	  (if (/= 0 (send my-plot :num-points))
	      (let()
		(send my-plot :selection
		      (iseq 0 (- (send my-plot :num-points) 1)))
		(send my-plot :point-symbol
		      (send my-plot :selection) *plot-symbol*)
		)
	    )
	  (send my-plot :showing-labels t)
	  )
      
      (if my-plot
	  (let()
	    (send my-plot :mouse-mode 'hand-rotate)
	    (send my-plot :showing-axes (if *spin-axes* t nil))
	    (send my-plot :depth-cuing (if *spin-depth*  t nil))
	    (if (/= 0 (send my-plot :num-points))
		(let()
		  (send my-plot :selection
			(iseq 0 (- (send my-plot :num-points) 1)))
		  (send my-plot :point-symbol
			(send my-plot :selection) *spin-symbol*)
		  )
	      )
	    (if *real-axes* (send my-plot :real-axes))
	    )
	)
      )
    (if *title* (send my-plot :title *title*))
    (send my-plot :andy-options (list ':axes t))
    (mapcar (lambda(arg) (apply #'send my-plot arg)) special-plot-args)
    (send my-plot :plot-item plot-items)
    (if *show-plots* (send my-plot :show-window))
    my-plot
    )
  )

(defun andy-mode (&optional (on/off t))
  (if on/off
      (defmeth andyplot-proto :isnew (&rest args)
	(apply #'call-next-method args)
	(send self :add-overlay (send scatterplot-control-overlay-proto :new))
	(send self :margin 0 0 0 20)
	)
    (let()
     (defmeth andyplot-proto :isnew (&rest args)
       (apply #'call-next-method args)
       (send self :mouse-mode 'selecting)
       )
     (setq *andy-mode* nil)
     )
    )
  )

(defun verbose (&optional (on-off nil on-off-set))
  "Makes xlisp-stat a little more talkative!"
  (if on-off-set (setq *verbose* on-off)
    (setq *verbose* (not *verbose*))
    )
  )
