(defun splus-contour
  (matrix 
   &key
   x
   y
   limits
   (title "splus-contour")
   (xlabel "x")
   (ylabel "y")
   (levels 10)
   add
   hpgl
   (psfile "splus-contour.ps")
   rotate
   )
  "
;; Description: (splus-contour matrix 
   &key
    x
    y
    limits
    (title \"splus-contour\")
    (xlabel \"x\")
    (ylabel \"y\")
    (levels 10)
    add
    hpgl
    (psfile \"splus-contour.ps\")
    rotate
    )

splus-contour is meant to be compatible with contour-matrix. It just gives me a
little prettier plot of the same information. It does include the 'limits'
options, which allows one to set the limits of interest for the plot. It also
has additional features:

	add is for an additional matrix to be added to the same plot: it can be
either a matrix, or a list of xvals, yvals, and a matrix.
	hpgl is a file name, if you would rather have an hpgl file than X11
output.
	psfile is the postscript file name.
	rotate set to t will use landscape mode.

;; Examples:
 (splus-contour
  (matrix (list 33 13) (rand.u (* 33 13)))
  :x (rseq 18 82 33)
  :y (rseq -1.9025190710898217 2.6644401322623494 13)
  )
 (splus-contour
  (matrix (list 33 13) (rand.u (* 33 13)))
  :x (rseq 18 82 33)
  :y (rseq -1.9025190710898217 2.6644401322623494 13)
  :hpgl \"out.hpgl\"
  )
"
  (let* (
         (file (open "splus-contour.mat" :direction :output))
         (splusfile (open "splus-contour.666" :direction :output))
	 (size (size matrix))
	 (nrows (first size))
	 (ncols (second size))
	 (xvals (if x x (iseq nrows)))
	 (yvals (if y y (iseq ncols)))
	 (nlevels (if (listp levels) 'levels 'nlevels))
	 (limitson (if limits 'T 'F))
	 (rotate (if rotate 'T 'F))
	 (levels
	  (if
	      (numberp levels) levels
	    (grep-replace " " "," (format nil "~g" levels))
	    )
	  )
         )
    (dolist
     (i (row-list matrix))
     (mapcar #'(lambda(x) (format file "~g " x)) (coerce i 'list))
     (format file "~%")
     )
    (close file)
    (if add
	(let ()
	  (setq file (open "splus-contour.matadd" :direction :output))
	  (dolist
	   (i (row-list (if (listp add) (third add) add)))
	   (mapcar #'(lambda(x) (format file "~g " x)) (coerce i 'list))
	   (format file "~%")
	   )
	  (close file)
	  ))

    (if hpgl
	(format splusfile "~a~%"
		(strcat
		 "hpgl(width=11,height=8.5,color=1,speed=100,file=\""
		 hpgl
		 "\",hw.control=F)"))
      (format splusfile "X11()~%"))

    (format splusfile "x <- scan(\"splus-contour.mat\")~%")
    (if limits
	(format splusfile
		"
u <- seq(~g,~g,~g)
v <- seq(~g,~g,~g)
z <- 0*t(matrix(x,nrow=~d,ncol=~d))
r.ak <- c(diff(range(u)),diff(range(v)),diff(range(z)))
par(mfrow=c(1,1), oma=c(0,0,5,0))
contour(u, v, z, add=F,nlevels=c(1),labex=.8, triangles=T,lty=1)
"
	(first limits) (second limits) (/ (- (second limits) (first limits)) (1- (length xvals)))
	(third limits) (fourth limits) (/ (- (fourth limits) (third limits)) (1- (length yvals)))
	ncols nrows
	)
      )
    (format splusfile
	    "
u <- seq(~g,~g,~g)
v <- seq(~g,~g,~g)
z <- t(matrix(x,nrow=~d,ncol=~d))
r.ak <- c(diff(range(u)),diff(range(v)),diff(range(z)))
par(mfrow=c(1,1), oma=c(0,0,5,0))
"
	    (min xvals) (max xvals) (/ (- (max xvals) (min xvals)) (1- (length xvals)))
	    (min yvals) (max yvals) (/ (- (max yvals) (min yvals)) (1- (length yvals)))
	    ncols nrows
	    )

    (if (equal nlevels 'nlevels)
	(format
	 splusfile
	 "contour(u, v, z, add=~a,nlevels=~g,labex=.8, triangles=T,lty=1)"
	 limitson levels)
      (format
       splusfile
       "contour(u, v, z, add=~a,levels=c~g,labex=.8, triangles=T,lty=1)"
       limitson levels)
      )

    (if add
	(let ()
	  (if (listp add)
	      (let ()
		(setq xvals (first add))
		(setq yvals (second add))
		(setq add (third add))
		))
	  (format
	   splusfile
	   "
x <- scan(\"splus-contour.matadd\")
u <- seq(~g,~g,~g)
v <- seq(~g,~g,~g)
z <- t(matrix(x,nrow=~d,ncol=~d))
contour(u, v, z, add=T,levels=c~g,labex=.8, triangles=T,lty=2)"
	   (min xvals) (max xvals)
	   (/ (- (max xvals) (min xvals)) (1- (length xvals)))
	   (min yvals) (max yvals)
	   (/ (- (max yvals) (min yvals)) (1- (length yvals)))
	   (second (size add)) (first (size add)) levels))
      )
  
    (format splusfile
	    "
title(main=~s, xlab=~s,ylab=~s)
printgraph(horizontal=~a,print=F,file=\"~a\",maximize=T)
"
title xlabel ylabel rotate psfile
)
    (if (not hpgl)
	(format splusfile "
!xmessage -geometry 450x70+500+0 -message \"Hit redraw; click the quit button to obliterate the plot.\"
"))
    (close splusfile)
    (system "Splus BATCH splus-contour.666 splus-contour.667")
    )
  (values)
  )


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            splus-dat
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun splus-dat (filenames
		  &key
		  x
		  y
		  (title "splus-dat")
		  (xlabel "x")
		  (ylabel "y")
		  (plots '(1 1))
		  (pch '("*" "+" "o" "x" "1" "2" "3" "4" "5"))
		  (lty '("1" "2" "3" "4" "5" "6"))
		  add
		  lines
		  hpgl
		  command
		  (wait t)
		  (psfile "splus-dat.ps")
		  rotate
		  )
  "
;; Documentation: (splus-dat filenames
		  &key
		  x
		  y
		  (title \"splus-dat\")
		  (xlabel \"x\")
		  (ylabel \"y\")
		  (plots '(1 1))
		  (pch '(\"*\" \"+\" \"o\" \"x\" \"1\" \"2\" \"3\" \"4\" \"5\"))
		  (lty '(\"1\" \"2\" \"3\" \"4\" \"5\" \"6\"))
		  add
		  lines
		  hpgl
		  command
		  (wait t)
		  (psfile \"splus-dat.ps\")
		  )

	Plots on different plots, or plots on the same plot (depends on add).
The list of filenames refers to files of two columns of a matrix. I should
really make it nicer, by just adding an option for it to be a list, or list of
lists. Default is points, although if lines is set to true, lines will be
plotted.

	If you want to add an splus command following the plot, give a string
to the command argument.

	The x and y options should be lists of pairs (the minimum and maximum
values that should be shown). Plots is the way that you want them plotted: if
you give a list (list nrow ncol) then Splus will make multiple plots, with nrow
many rows and ncol many columns.

	pch and lty are splus options for point and line types; add will put
two (or more) plots on top of each other; lines makes lines instead of points;
hpgl dictates that an hpgl file should be created; command is an optional splus
command, contained in a string; wait puts up a dialogue (from within XWindows)
to keep the plot active until you decide to let it go (otherwise it will be
eliminated once the hpgl or postscript plots are made).

;; Examples:
 (write-matrix-to-file (bind-columns (rand.u 100) (rand.n 100)) \"ctmp.out1\")
 (write-matrix-to-file (bind-columns (rand.u 100) (rand.u 100)) \"ctmp.out2\")
 (splus-dat (list \"ctmp.out1\" \"ctmp.out2\") :add t)
"
  (let* (
         (splusfile (open "splus-dat.666" :direction :output))
	 file
	 (i (first filenames))
	 (rotate (if rotate 'T 'F))
         )
    (format splusfile "
X11()
par(mfrow=c(~d,~d), oma=c(0,0,5,0))
x<-scan(\"~a\")
generic.mat<-matrix(x,ncol=2,byrow=T)
u<-generic.mat[,1]
v<-generic.mat[,2]
" (first plots) (second plots) i)
    (if (and x (listp x))
	(format splusfile "xval<-c(~g,~g)~%" (first x) (second x))
      (format splusfile "xval<-c(min(u),max(u))~%")
      )
    (if (and y (listp y))
	(format splusfile "yval<-c(~g,~g)~%" (first y) (second y))
      (format splusfile "yval<-c(min(v),max(v))~%")
      )
    (format splusfile "
plot(u,v,xlim=xval,ylim=yval,xlab=\"~a\",ylab=\"~a\",~a=~a)
"
	    xlabel ylabel
	    (if lines "lty" "pch")
	    (if lines (first lty) (strcat "\"" (first pch) "\"" )))
						
    (if lines (format splusfile "lines(u,v)"))
    (if command (format splusfile "~%~a" command))

    (dolist
     (i (rest filenames))
     (format splusfile "
x<-scan(\"~a\")
generic.mat<-matrix(x,ncol=2,byrow=T)
u<-generic.mat[,1]
v<-generic.mat[,2]
" i)
     (setq pch (circulate pch))
     (if add 
	 (format splusfile
		 (if lines
		     "lines(u,v,lty=\"~a\")~%"
		   "points(u,v,pch=\"~a\")~%"
		   )
		 (if lines (first lty) (first pch)))
       (format splusfile
	       "plot(u,v,xlab=\"~a\",ylab=\"~a\",pch=\"~a\")~%"
	       xlabel ylabel (first pch))
       )
     )
    (format splusfile
	    "
par(mfrow=c(1,1), oma=c(0,0,5,0))
title(main=~s)
printgraph(horizontal=~a,print=F,file=\"~a\",maximize=T)
" title rotate psfile)
    (if wait
	(format splusfile "
!xmessage -geometry 450x70+500+0 -message \"Hit redraw; click the quit button to obliterate the plot.\"
")
      )
    (close splusfile)
    (system "Splus BATCH splus-dat.666 splus-dat.667")
    (values)
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                            splus-lists
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun splus-lists (lists
		  &key
		  x
		  y
		  (title "splus-lists")
		  (xlabel "x")
		  (ylabel "y")
		  (plots '(1 1))
		  (pch '("*" "+" "o" "x" "1" "2" "3" "4" "5"))
		  (lty (iseq 1 6))
		  add
		  lines
		  hpgl
		  command
		  (wait t)
		  (psfile "splus-lists.ps")
		  rotate
		  )
  "
;; Documentation: (splus-lists lists
		  &key
		  x
		  y
		  (title \"splus-lists\")
		  (xlabel \"x\")
		  (ylabel \"y\")
		  (plots '(1 1))
		  (pch '(\"*\" \"+\" \"o\" \"x\" \"1\" \"2\" \"3\" \"4\" \"5\"))
		  (lty (iseq 1 6))
		  add
		  lines
		  hpgl
		  command
		  (wait t)
		  (psfile \"splus-lists.ps\")
		  )

	Plots on different plots, or plots on the same plot (depends on add).
The list of lists refers to files of two columns of a matrix. I should
really make it nicer, by just adding an option for it to be a list, or list of
lists. Default is points, although if lines is set to true, lines will be
plotted.

	If you want to add an splus command following the plot, give a string
to the command argument.

	The x and y options should be lists of pairs (the minimum and maximum
values that should be shown). Plots is the way that you want them plotted: if
you give a list (list nrow ncol) then Splus will make multiple plots, with nrow
many rows and ncol many columns.

	pch and lty are splus options for point and line types; add will put
two (or more) plots on top of each other; lines makes lines instead of points;
hpgl dictates that an hpgl file should be created; command is an optional splus
command, contained in a string; wait puts up a dialogue (from within XWindows)
to keep the plot active until you decide to let it go (otherwise it will be
eliminated once the hpgl or postscript plots are made).

;; Examples:
 (splus-lists (list (list (rand.u 100) (rand.u 100))
		    (list (rand.u 100) (rand.n 100)))
   :x '(0 1)
   :y '(-3 3)
   :add t)

 (splus-lists (list (list (rand.u 100) (rand.u 100))
		    (list (rand.u 100) (rand.n 100)))
   :x '(0 1)
   :y '(-3 3)
   :plots '(1 2)
   )

 (splus-lists (list (list (rand.u 100) (rand.u 100))
		    (list (rand.u 100) (rand.n 100)))
   :x '(0 1)
   :y '(-3 3)
   :lines t
   :add t)
"
  (let* (
         (splusfile (open "splus-lists.666" :direction :output))
	 file
	 (i (first lists))
	 (xx (first i))
	 (yy (second i))
	 (rotate (if rotate 'T 'F))
         )
    (format splusfile "
X11()
par(mfrow=c(~d,~d), oma=c(0,0,5,0))
u<-~a
v<-~a
" (first plots) (second plots) (splus-vector xx) (splus-vector yy))
    (if (and x (listp x))
	(format splusfile "xval<-c(~g,~g)~%" (first x) (second x))
      (format splusfile "xval<-c(min(u),max(u))~%")
      )
    (if (and y (listp y))
	(format splusfile "yval<-c(~g,~g)~%" (first y) (second y))
      (format splusfile "yval<-c(min(v),max(v))~%")
      )
    (format splusfile "
plot(u,v,xlim=xval,ylim=yval,xlab=\"~a\",ylab=\"~a\",lty=~d,pch=\"~a\")~%"
	    xlabel ylabel (first lty) (first pch))
    (if lines (format splusfile "lines(u,v,lty=~d)" (first lty)))
    (if command (format splusfile "~%~a~%" command))

    (dolist
     (i (rest lists))
     (format splusfile "~%u<-~a~%v<-~a~%"
	      (splus-vector (first i)) (splus-vector (second i)))
     (setq pch (circulate pch))
     (setq lty (circulate lty))
     (if add 
	 (format splusfile
		 (if lines (format nil "lines(u,v,lty=~d)" (first lty))
		   (strcat "points(u,v,pch=\"" (first pch) "\")")))
       (format splusfile "
plot(u,v,xlim=xval,ylim=yval,xlab=\"~a\",ylab=\"~a\",lty=~d,pch=\"~a\")~%"
	       xlabel ylabel (first lty) (first pch))
       )
     )
    (format splusfile
	    "
par(mfrow=c(1,1), oma=c(0,0,5,0))
title(main=~s)
printgraph(horizontal=~a,print=F,file=\"~a\",maximize=T)
" title rotate psfile)
    (if wait
	(format splusfile "
!xmessage -geometry 450x70+500+0 -message \"Hit redraw; click the quit button to obliterate the plot.\"
")
      )
    (close splusfile)
    (system "Splus BATCH splus-lists.666 splus-lists.667")
    (values)
    )
  )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun splus-lines(x y &optional (lty 1))
  "
	(splus-lines x y &optional (lty 1))

	Generates an splus line command from a pair of lists, as a string.
;; Examples:
 (splus-lines (rand.u 10) (rand.u 10))
"
  (strcat
   "lines(c("
   (apply #'strcat
          (mapcar
           #'(lambda(x) (format nil "~g," x))
           (butlast x)))
   (format nil "~g),c(" (final x))
   (apply #'strcat
          (mapcar
           #'(lambda(x) (format nil "~g," x))
           (butlast y)))
   (format nil "~g),lty=~d)~%" (final y) lty)
   )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun splus-points(x y &optional (pch "*"))
  "
	(splus-points x y &optional (pch 1))

	Generates an splus line command from a pair of lists, as a string.
;; Examples:
 (splus-points (rand.u 10) (rand.u 10))
"
  (strcat
   "points(c("
   (apply #'strcat
          (mapcar
           #'(lambda(x) (format nil "~g," x))
           (butlast x)))
   (format nil "~g),c(" (final x))
   (apply #'strcat
          (mapcar
           #'(lambda(x) (format nil "~g," x))
           (butlast y)))
   (format nil "~g),pch=\"~a\")~%" (final y) pch)
   )
  )

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun splus-vector(x)
  "
	(splus-vector x)

	Generates an splus-vector from a list, as a string.

;; Examples:
 (splus-vector (rand.u 10))
"
  (strcat
   "c("
   (apply #'strcat
          (mapcar
           #'(lambda(x) (format nil "~g," x))
           (butlast x)))
   (format nil "~g)" (final x))
   )
  )
