;; Just checking...

;; Stole the next two routines from horner.lisp, in contrib. Here are the
;; details:
;------------------------------------------------------------------------------;
;;; -*- Mode: LISP; Syntax: Common-lisp; Package: USER -*-

;;;; Horner's Rule

;;;  (c) Copyright Gerald Roylance 1982, 1984, 1987
;;;      All Rights Reserved.
;;;  This file may be distributed noncommercially provided
;;;  that this notice is not removed.

;;; Bugs and Fixes
;;;   Horner should use terms and termlists
;;;   poly and poly-expand should be flushed

(defun poly-expand (var coefs)
  (cond
   ((null coefs)       0.0)
   ((null (cdr coefs)) (car coefs))
   (t
    `(+ ,(car coefs) (* ,var ,(poly-expand var (cdr coefs))))))
  )

#|
(defmacro poly (var . coefs)
  (poly-expand var coefs)
  )
|#
(defmacro poly (var &rest coefs)
  "
;; Examples:
 (poly 1 1 2) ;; represents (+ 1 (* 2 1))
 (poly 1 1 2 3) ;; represents (+ 1 (* 2 1) (* 3 1))
"
  (eval `(poly-expand ,var ',coefs))
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                                taylor-coef
;;
;; Modified: 2/2/98 - ael
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun taylor-coef (f n a &optional (coefs nil))
  "
;; Description:
	Taylor-coef provides the Taylor series coefficients for the Taylor
series expansion of a univariate function.

;; Examples:
 (func f(x) x ^ 4)
 (taylor-coef #'f 3 1)

 (func g(x) sin x)
 (taylor-coef #'g 4 pi)
"
  (let* (
	 (vars (second (eval `(get-lambda-expression ,f))))
	 (xvar (first vars))
	 (fpx (lambda-deriv f xvar))
	 )
    (if (not coefs) (setq coefs (funcall f a)))
    (setq coefs (combine (funcall fpx a) coefs))
    (if (= n 1)
	(/ (reverse coefs) (mapcar #'factorial (iseq (length coefs))))
      (taylor-coef fpx (1- n) a coefs)
      )
    )	 
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; taylor2 Doc:    Creation Date:3/5/96        Author: Andy Long
;;
;; Modified: 2/2/98 - ael
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defun taylor2 (f n a &optional taylorfunc)
  "
;; Description/Arguments: function

	 (taylor2 f n a &optional taylorfunc)
  
	Creates the taylor series expansion to a function f about a, optionally
creating a function with value this expansion.

;; See Also: taylor-coef, taylor

;; Examples:
 (func f(x) x^4)
 (taylor2 #'f 3 1 'ft)

 (new-plot)
 (plot ft f 0 2)
 (plot (taylor2 #'f 3 1) f 0 2)
"
  (let* (
	 (taylorcoefs (taylor-coef f n a))
	 (vars (second (get-lambda-expression f)))
	 (xvar (first vars))
	 (bing (poly-expand (list '- xvar a) taylorcoefs))
	 )
    (if taylorfunc 
	(eval `(defun ,taylorfunc ,vars ,bing))
      (eval `(lambda ,vars ,bing))
      )
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; taylor Doc:    Creation Date:3/5/96        Author: Andy Long
;;
;; Modified: 2/2/98 - ael
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defmacro taylor (f ft &rest e)
  "
;; Description/Arguments:

	macro (taylor f tp n a)
  
	Creates the taylor polynomial tp of degree n to f at a. tp can then
be plotted, etc.: treated like any other function.
 
;; Notes: basically calls function taylor2.

;; See Also: taylor-coef, taylor2

;; Examples:
 (title \"Cubic fit to x^4 at 1\")
 (func f(x) x^4)
 (taylor f ft 3 1)
 (plot f ft .5 1.5)

 (title \"Sixth degree fit to sin(x) at pi/2\")
 (func g(x) sin x)
 (taylor g gt 6 (/ pi 2))
 (range -2 1)
 (plot g gt -pi/2 pi)

;; Tangent line has n of 1:
 (title \"Tangent line approximation at 2\")
 (taylor g gt 1 2)
 (plot g gt 1 3)
 (what gt)
"
  (let ((ff (eval `(turn-it-into-a-function ,f))))
    `(taylor2 ,ff ,@e ',ft)
    )
  )
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
