(message-execute "
;; =================
                                ;; Section  5.6.1
;; =================

;; ael substeps to make-projection 
;; (if (matrixp cols) cols (apply #'bind-columns cols))

;; (select (column-list u) (which (> s-vals 0.0)))

;; (matmult u1 (matmult y u1))
")
(message-execute "
(defun make-projection (cols)
  (let* ((x (if (matrixp cols)
                cols
              (apply #'bind-columns cols)))
         (svd (sv-decomp x))
         (u (first svd))
         (s-vals (second svd))
         (basis (select (column-list u)
                        (which (> s-vals 0.0))))
         (u1 (apply #'bind-columns basis)))
    #'(lambda (y) (matmult u1 (matmult y u1)))))

")
(message-execute "
;; =================
                                ;; Section  5.6.2
;; =================
;; ael substeps to robust-coefs
;; (+ (first beta) (matmult x (rest beta)))

;; (do ((last nil beta)
;;      (count 0 (+ count 1))
;;      (beta (reg-coefs weights) (improve-guess beta)))
;;     ((good-enough-p last beta count) beta))

;; (let* ((resids (- y (fitvals beta)))
;;        (scale (/ (median (abs resids)) .6745))
;;        (wts (funcall wf (/ resids scale))))
;;   (reg-coefs wts))

;; (flet ((rel-err (x y) (mean (/ (abs (- x y)) (+ 1 (abs x))))))
;;   (or (> count count-limit)
;;       (and last (< (rel-err beta last) tol))))
")

(message-execute "(defun robust-coefs (x y wf &key (weights (repeat 1 (length y)))
                       (tol .0001) (count-limit 20))
  (let ((x (if (matrixp x) x (apply #'bind-columns x))))
    (labels ((as-list (x) (coerce (compound-data-seq x) 'list))
             (rel-err (x y) (mean (/ (abs (- x y)) (+ 1 (abs x)))))
             (reg-coefs (weights)
                        (let* ((m (make-sweep-matrix x y weights))
                               (p (array-dimension x 1)))
                          (as-list
                           (select (first (sweep-operator m (iseq 1 p)))
                                   (1+ p) (iseq 0 p)))))
             (fitvals (beta) (+ (first beta) (matmult x (rest beta))))
             (improve-guess (beta)
                            (let* ((resids (- y (fitvals beta)))
                                   (scale (/ (median (abs resids)) .6745))
                                   (wts (funcall wf (/ resids scale))))
                              (reg-coefs wts)))
             (good-enough-p (last beta count)
                            (if (> count count-limit)
                                (format t \"Iteration limit exceeded~%\"))
                            (or (> count count-limit)
                                (and last (< (rel-err beta last) tol)))))
            (do ((last nil beta)
                 (count 0 (+ count 1))
                 (beta (reg-coefs weights) (improve-guess beta)))
                ((good-enough-p last beta count) beta)))))")
(message-execute "
(defun make-wf (name &optional
                     (k (case name
                              (biweight 4.685)
                              (cauchy 2.385)
                              (huber 1.345))))
  #'(lambda (r)
      (let ((u (abs (/ r k))))
        (case name
              (biweight (^ (- 1 (^ (pmin u 1) 2)) 2))
              (cauchy (/ 1 (+ 1 (^ u 2))))
              (huber (/ 1 (pmax u 1)))))))
")
(message-execute "
(load-data \"stackloss\")

(regression-model (list air temp conc) loss)

(robust-coefs (list air temp conc) loss (make-wf 'huber))
")
