(tutorial-dialog (combine 
(list "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                      Differential Equations (simple stuff!)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;      If you've got a pesky differential equations to solve, then you might
;; consider the rather primitive diffeq solver \"euler\":
;;
;;      Euler computes the euler approximate solution for a given function
;; g(x,y), representing the right hand side of a first-order differential
;; equation.
;;                            dy/dx=g(x y)

;;      Note that the function g should include both variables (dependent y and
;; independent x) in its definition.
 
 (title \"dy/dx=y(1-y)\")
 (func g(x y) y * (1 - y)) ;; for the logistic diffeq  dx/dt=x(1-x)
 (euler g .5 0 5 .1) ;; computes values of the solution from 0 to 5, 5 steps
")
(list "
;;      Another variant of that is the function 2euler, which uses a VERY crude
;; discrete approximation for second order equations of the form

;;                            d^2y/dx^2=g(x y)

;;     2euler computes the 2euler approximate solution for a given function g
;; representing the right hand side of a second-order differential
;; equation. E.g.,

 (title \"d^2y/dx^2=-32\")
 (func g(x y) -32) ;; for the trajectory of chalk
 (2euler g 6 11.76 0 2 .2 (func f(x) -16 * x ^ 2 + 32 * x + 6))
")
(list "
;;      Another useful routine is \"slope-field\", which produces a slope-field
;; (oddly enough) for a given first-order differential equation. For example,
;; the slope-field corresponding to a differential equation of the form

;;                            dy/dx = f(x y)

 (title \"Slope field of -x/y\")
 (func f (x y) (-  x / y))
 (slope-field f -2 2 -2 2)

;;      If you click inside the plot you'll set an initial value and euler will
;; compute and plot the corresponding trajectory. Why are some of them so weird
;; (and wrong!)?
")
(list "
;;      For more sophisticated problems, you may want to consider the functions
;; euler-nd, rk-4, etc. For more information on them, try them!

       (try-out rk-4)
")
(list "
       (try-out euler-nd)
")
(list "
       (try-out phase-plane)
")

) :title "diffeqs")
