(tutorial-dialog (combine 
(list "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             Fourier Series
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;         Fourier series provide a means for fitting functions globally: that
;; means that we seek an approximation that is pretty good over an interval (which
;; we will take to be [-pi,pi]), rather than at a single point (that's the Taylor
;; series's job).

;;         The main routine for us will be \"fourier\", which computes the Fourier
;; series coefficients by direct numerical integration. It returns the
;; coefficients, as well as the approximating fourier series of a particular
;; order n.

        (new-plot)
        (title \"A saw tooth function\")
        (func f(x) x * (bump x (- pi) pi))
        (fourier f 7)
        (plot f fourier-approx -pi pi)
")
(list "
;;        Now look at several approximating fourier series for a \"bump\"
;; function: we'll look at the n=3, n=7, and n=15 approximations.

        (new-plot)
        (func f(x) (bump x 0 pi))
        (title \"f is just a bump on a log\")
        (fourier f 3 hippo)
        (plot f hippo -pi pi)
        (fourier f 7 hippo)
        (plot f hippo -pi pi)
        (fourier f 15 hippo)
        (plot f hippo -pi pi)
")
(list "
;;        If you have a function which is not on the interval [-pi,pi] to begin
;; with, then you will need to scale the domain: for example, suppose that you
;; want to approximate the function x^2-x on the interval [-1,3]. We could replace
;; the independent variable x by ((4 * (x + pi) / 2pi) - 1) and we would have it 
;; on the interval [-pi,pi]. (Check that this is a linear transformation that
;; takes the interval [-pi,pi] and shifts it over to [-1,3]. Can you see how I
;; came up with it?)

        (func ff(x) x^2-x)
        (func h(x) (ff (4*(x+pi)/2pi-1)))
        (fourier h 5 hippo)
        (title \"Function shifted [-pi,pi]\")
        (plot h hippo -pi pi)
")
(list "
;;        One important application of Fourier series is in attempting to
;; identify trends in data. If you have some regularly-spaced data (for example,
;; measurements of rainfall on a daily basis) which seems periodic, then it may
;; pay to \"fourier analyze\" it - that is, to try to find out what frequencies
;; make it up.

;;        If prediction is a goal, then it would be useful to have a function
;; which interpolates the data. So let's find a fourier approximation to the
;; process that is producing the data. As an example, we'll look at some
;; precipitation levels for the month of March in the Minneapolis-St. Paul area
;; over a thirty-year period:

        (lines off)
        (load-data \"tutorial\")
        (title \"precipitation data\")
        (variable-labels \"year\" \"inches of water\")
        (plot (iseq 1 (length precipitation)) precipitation) 
")
(list "
;;         Now that the data is loaded, we'd like to carry out a 'discrete'
;; fourier transform (dft) of the data, to find the frequencies and to visualize
;; the approximation to the process:

        (dft precipitation) ;; precipitation.
        (title \"Fourier approximation to data\")
        (lines on)
        (plot fft-approx 0 30)
        (variable-labels \"year from start of study\" \"inches of water\")
        (send my-plot :add-points (iseq (length precipitation)) precipitation)
")

) :title "Fourier Analysis")
