(tutorial-dialog (combine 
(list "
        One important application of Fourier series is in attempting to
identify trends in data. If you have some data which seems periodic, then it
may pay to \"fourier analyze\" it - that is, to try to find out what
frequencies make it up. So we'll attempt to find a fourier approximation to the
process that is producing the data.

")
(load-data "tutorial")
(new-plot)
(title "precipitation data")
(plot (iseq (length precipitation)) precipitation) 

(defun dft(x)
  "
	The discrete fourier transform takes a vector (or list) x, and
expresses it in a basis of trigonometric functions. If the vector is 

"
  (let* ((lenx (length x))(1/lenx (/ 1 lenx))(nfreq (floor (/ lenx 2)))
	 fvals xvals)
    (dotimes
     (k (+ 1 nfreq)) ;; from 0 to [N/2]
     (setq xvals (exp (* (iseq 0 (1- lenx)) 2pi #c(0 1) k 1/lenx)))
     (setq fvals (combine fvals (sum (* x xvals 2 1/lenx)))))
    (setq fvals (rest fvals))
    (setf (select fvals 0)(/ (select fvals 0) 2))
    (if (evenp lenx)
	(setf (select fvals (1- (length fvals)))
	      (/ (select fvals (1- (length fvals))) 2))
      )
    fvals
    )
  )

(defun dft(x)
  "
	The discrete fourier transform takes a vector (or list) x, and
expresses it in a basis of trigonometric functions. Function dft returns the
function which approximates the given data, and optionally prints the
coefficients of the discrete approximation.

;; Examples:
 (load-data \"tutorial\")
 (dft precipitation)
 (plot fft-approximation 0 30)
 (send my-plot :add-points (iseq (length precipitation)) precipitation)
 (title "precipitation data")
 (plot (iseq (length precipitation)) precipitation) 
 precipitation
 (mapcar #'fft-approximation (rseq 0 29 30))
"
 (let* (
	(lenx (length x))
	(1/lenx (/ 1 lenx))
	(fft-x (fft x))
	(ao (* 1/lenx (realpart (first fft-x))))
	)
   (if (evenp lenx)
       (list
	(setq nfreq (/ lenx 2))
	(setq two-parts (split-list fft-x nfreq))
	(setq one-part (* 1/lenx 
			  (combine (rest (first two-parts))
				   (/ (first (second two-parts)) 2))))
	)
     (list
      (setq nfreq (/ (1- lenx) 2))
      (setq two-parts (split-list (rest fft-x) nfreq))
      (setq one-part (* 1/lenx (first two-parts)))
      )
     )
   (defun fft-approximation (x)
     (+
      ao
      (* 2
	 (sum (realpart
	       (* one-part (exp (* #c(0 1) x 2pi (iseq 1 nfreq) 1/lenx))))))
      )
     )
   )
 )
(dft precipitation)
(plot fft-approximation 0 30)
(send my-plot :add-points (iseq (length precipitation)) precipitation)
(title "precipitation data")
(plot (iseq (length precipitation)) precipitation) 
precipitation
(mapcar #'fft-approximation (rseq 0 29 30))

(dft (rest precipitation))
(plot fft-approximation 0 30)
(title "(rest precipitation) data")
(plot (iseq (length (rest precipitation))) (rest precipitation))
(send my-plot
      :add-points
      (iseq (length (rest precipitation))) (rest precipitation)
      )
precipitation
(mapcar #'fft-approximation (rseq 0 29 30))
) :title "dft and fft")
