(tutorial-dialog (combine 
(list "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                       Programming in xlisp-stat
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

	This is a collection of hints, mostly for myself, so that I can try to
avoid reinventing the wheel everytime!

	In general I've found programming in lisp maddening: I can't seem to
get ahold of everything. Success seems elusive! After a year with any other
language, I'd feel comfortable; with xlisp-stat, I seem to bump into lots of
invisible windows (ouch!). But it is enjoyable, in a masochistic kind of way.

                                                       Andy Long
                                                       Spring, 1996
")
(list "
        The first thing that I want to say is that, if you really want to code
in xlisp-stat, then you should look at codes. There's no better way to learn
coding than to steal other people's ideas. Please steal mine! Furthermore, if
you've come at xlisp-stat from the calculus standpoint, you're going to need to
get familiar with the usual lisp problems: my codes can only be used to some
limited extent. If you look at the source for the calculus stuff, you'll see
that I didn't write it using all the 'conveniences' that I provided for the
calc student, but rather in the reverse-polish-notation style of ordinary LISP.
So: you are fore-warned (fore-armed?)!
")

(list "
                         Macros versus Functions

        You can define either in xlisp-stat. We define functions all the time
in mathematics, but you may find yourself defining other stranger functions as
you program in xlisp-stat. For example, you could define a function f-func
that combines two lists:

(defun f-func(x y) (combine x y))
(f-func '(2 3) '(1 2))
(f-func (list 2 3) (list 1 2))

but you'd have to type those annoying quotes all the time, or write lists using
the 'list' command. Alternatively, you could define a macro f-mac:

(defmacro f-mac(x y) `(combine ',x ',y))
(f-mac (2 3) (1 2))
")
(defun f-func(x y) (combine x y))
(f-func '(2 3) '(1 2))
(f-func (list 2 3) (list 1 2))
(defmacro f-mac(x y) `(combine ',x ',y))
(f-mac (2 3) (1 2))
(list "")
(list "
        One problem, however, is that the macro may not expand the elements of
the list as you might like: for example,

(setq a '(2 3))
(setq b '(1 2))
(f-func a b)
(f-mac a b)

f-func does what is intended, but f-mac creates a list which is the combination
of symbols a and b
")
(setq a '(2 3))
(setq b '(1 2))
(f-func a b)
(f-mac a b)
(list "")

(list "
        We can make the definition of the macro much more complicated, and then
we should be okay:
(defmacro f-mac(x y)
  (if (consp x)
      (if (consp (first x))
          `(combine ',x ',y)
        `(combine (list ,@x) (list ,@y))
        )
    (let (
          (xx (eval `,x))
          (yy (eval `,y))
          )
      `(combine (list ,@xx) (list ,@yy))
      )      
    )      
  )
(f-mac (2 3) (1 2))
(f-mac a b)
")
(defmacro f-mac(x y)
  (if (consp x)
      (if (consp (first x))
          `(combine ',x ',y)
        `(combine (list ,@x) (list ,@y))
        )
    (let (
          (xx (eval `,x))
          (yy (eval `,y))
          )
      `(combine (list ,@xx) (list ,@yy))
      )      
    )      
  )
(f-mac (2 3) (1 2))
(f-mac a b)
(list "")

(list "
        Obviously, the trade-off is between programming complexity and ease of
use! Since I'm aiming to help students use this stuff, I have to be ready to
jump through a few hoops. You may not be, and would just prefer throwing around
a couple of extra ' symbols now and then!
        The same is true for more complicated objects, such as functions. If
'f' is the name of a function (or macro), then you can not pass that to another
function: you have to pass #'f, or (function f), or some similarly clunky
thing. I've gone in the opposite direction (that is, of macros).
")
(list "
        I've taken to 'pre-treating' functions passed to both functions and
macros with the 'turn-it-into-a-function' macro.

        It appears that if I use that in macros, I have to use an eval to make
it happen, whereas I may get along without it in functions. This is another
difference between macros and functions (defmacro versus defun): in the next
few examples I want to look at other differences between these.

        Defun is definitely the most straightforward from the programming
standpoint, so I've taken to first defining a function (calling it the same
thing as the macro, with maybe a '2' on the end), and then using the macro to
send info to the function. This is a fairly straightforward and easy technique.
")

(list "
        To continue with our previous example, I could have defined the macro
in terms of f-func as follows:

(defmacro f-mac(x y)
  (if (consp x) ;; if it's a list (cons)
      (if (consp (first x)) ;; if the first thing in it's a list, then
          `(f-func ',x ',y) ;; otherwise, pass the contents of the list:
        `(f-func '(,@x) '(,@y))
        ) ;; on the other hand, if x isn't a list, then
    (let (
          (xx (eval `,x)) ;; we'll ask what's in these symbols
          (yy (eval `,y)) ;; we'll ask what's in these symbols
          )
      `(f-func '(,@xx) '(,@yy))
      )      
    )      
  )
(f-mac (2 3) (1 2))
(f-mac a b)
")
(defmacro f-mac(x y)
  (if (consp x)
      (if (consp (first x))
          `(f-func ',x ',y)
        `(f-func '(,@x) '(,@y))
        )
    (let (
          (xx (eval `,x))
          (yy (eval `,y))
          )
      `(f-func '(,@xx) '(,@yy))
      )      
    )      
  )
(f-mac (2 3) (1 2))
(f-mac a b)
(list "")
(list "
        This would be especially attractive if the function f-func were doing
something terribly complicated (rather than just combining two lists, as it is
in this example).
        In the code above you see that we needed to consider a variety of
cases, and those could well be a bit mysterious and strange.
")
(list "
        Here's a more useful example: I'll create a macro to call the matrix
function native to xlisp-stat (which is clunky like f-func above):

(defmacro matcro(x y)
  (if (consp x) ;; if it's a list (cons)
      (if (consp (first x)) ;; if the first thing in it's a list, then
          `(matrix ',x ',y) ;; otherwise, pass the contents of the list:
        `(matrix '(,@x) '(,@y))
        ) ;; on the other hand, if x isn't a list, then
    (let (
          (xx (eval `,x)) ;; we'll ask what's in these symbols
          (yy (eval `,y)) ;; we'll ask what's in these symbols
          )
      `(matrix '(,@xx) '(,@yy))
      )      
    )      
  )
(matcro (2 2) (3 2 1 2))
(setq a '(2 2))
(setq b '(3 2 1 2))
(matcro a b)
")
(defmacro matcro(x y)
  (if (consp x) ;; if it's a list (cons)
      (if (consp (first x)) ;; if the first thing in it's a list, then
          `(matrix ',x ',y) ;; otherwise, pass the contents of the list:
        `(matrix '(,@x) '(,@y))
        ) ;; on the other hand, if x isn't a list, then
    (let (
          (xx (eval `,x)) ;; we'll ask what's in these symbols
          (yy (eval `,y)) ;; we'll ask what's in these symbols
          )
      `(matrix '(,@xx) '(,@yy))
      )      
    )      
  )
(matcro (2 2) (3 2 1 2))
(setq a '(2 2))
(setq b '(3 2 1 2))
(matcro a b)
(list "")
(list "
        One more time, replacing matcro by a routine which will handle
everything that matrix will (so that I can replace matrix!):

(defmacro mat(x y)
  (cond ;; cond is used to go through a list of alternatives in order:
   ((consp x) ;; if it's a list (cons) then
    (cond
     ((consp (first x))  ;; if the first thing in it's a list, then
      `(matrix ',x ',y))
     (t ;; otherwise,
      (cond
       ((eq (first x) 'quote) ;; If we've got a quote, then old-style...
        (let ((xx (second x))
              (yy (second y)))
          `(matrix '(,@xx) '(,@yy))
          ))
       (t ;; otherwise, pass the contents of the list:
        `(matrix '(,@x) '(,@y))
        )))))
   (t ;; on the other hand, if x isn't a list, then
    (let ((xx (eval `,x)) ;; we'll ask what's in these symbols,
          (yy (eval `,y))) ;; and pass that:
      `(matrix '(,@xx) '(,@yy))
      )))
  )
  (mat '(2 2) '(3 2 1 2))
  (mat (2 2) (3 2 1 2))
  (mat a b)
")
(defmacro mat(x y)
  (cond ;; cond is used to go through a list of alternatives in order:
   ((consp x) ;; if it's a list (cons) then
    (cond
     ((consp (first x))  ;; if the first thing in it's a list, then
      `(matrix ',x ',y))
     (t ;; otherwise,
      (cond
       ((eq (first x) 'quote) ;; If we've got a quote, then old-style...
        (let ((xx (second x))
              (yy (second y)))
          `(matrix '(,@xx) '(,@yy))
          ))
       (t ;; otherwise, pass the contents of the list:
        `(matrix '(,@x) '(,@y))
        )))))
   (t ;; on the other hand, if x isn't a list, then
    (let ((xx (eval `,x)) ;; we'll ask what's in these symbols,
          (yy (eval `,y))) ;; and pass that:
      `(matrix '(,@xx) '(,@yy))
      )))
  )
  (mat '(2 2) '(3 2 1 2))
  (mat (2 2) (3 2 1 2))
  (mat a b)
(list "")
(list "
        Now it's only sort of true that I could replace matrix: because there
are lots of routines out there right now that would break if I replaced the
function matrix by the macro mat: as I mentioned above, if a macro gets
called from a macro, then it needs to be evaluated. Here's an example:

(defun trial1 (x y)
  (matrix x y))
(defun trial2 (x y)
  (mat x y))
(trial1 '(2 2) '(3 2 1 2))
;; #2A((3 2) (1 2))
(trial2 '(2 2) '(3 2 1 2))
;; Error: The variable X is unbound.
;; Happened in: #<FSubr-LET: #4c5c80>
")
(defun trial1 (x y)
  (matrix x y))
(defun trial2 (x y)
  (mat x y))
(trial1 '(2 2) '(3 2 1 2))
"(trial2 '(2 2) '(3 2 1 2))"
";; Error: The variable X is unbound."
";; Happened in: #<FSubr-LET: #4c5c80>"
(list "")
(list "
        In order to fix that, I'll evaluate the macro: 

(defun trial2 (x y)
  (eval `(mat ,x ,y)))
(trial2 '(2 2) '(3 2 1 2))
;; (trial2 (2 2) (3 2 1 2)) ;; function can't handle this!
(trial2 a b)

(defmacro trial3 (x y)
  (eval `(mat ,x ,y)))
(trial3 '(2 2) '(3 2 1 2))
(trial3 (2 2) (3 2 1 2))
(trial3 a b)

")
(defun trial2 (x y)
  (eval `(mat ,x ,y)))
(trial2 '(2 2) '(3 2 1 2))
;; (trial2 (2 2) (3 2 1 2)) ;; function can't handle this!
(trial2 a b)

(defmacro trial3 (x y)
  (eval `(mat ,x ,y)))
(trial3 '(2 2) '(3 2 1 2))
(trial3 (2 2) (3 2 1 2))
(trial3 a b)
(list "")

(list "
                                     looping

	On to looping. If I need to fake the fortran loop, it can be done using
the tagbody, and a set of tags (and go statements), as demonstrated in the
following piece of code (this is a loop to print some square roots, from 1 to
m, for every m up to a given n):
")
(list "
(defun print-some-sqrts (n)
  (let* ((i 1)(temp n))
    (tagbody

     loop
     (if (> i temp)(go end))
     (print (sqrt i))
     (setf i (+ i 1))
     (go loop)

     l5545
     (print-some-sqrts (1- temp))
     (go done)

     end
     (if (> temp 0) (go l5545))
     (print \"hi all!\")

     done
     (print 'done)
     )
    )
  \"finished!\"
  )
(print-some-sqrts 5)
")
(defun print-some-sqrts (n)
  (let* ((i 1)(temp n))
    (tagbody

     loop
     (if (> i temp)(go end))
     (print (sqrt i))
     (setf i (+ i 1))
     (go loop)

     l5545
     (print-some-sqrts (1- temp))
     (go done)

     end
     (if (> temp 0) (go l5545))
     (print "hi all!")

     done
     (print 'done)
     )
    )
  "finished!"
  )
(print-some-sqrts 5)
(list "")
(list "
        That's kind of a meaningless example, but it should serve as a decent
template. I got it from the loop.lsp file, in the loop subdirectory.

	The trouble with tagbodies is that they don't compile right, so if
you're going to want to streamline things by compiling, don't use them!
")

(list "
	There are other looping constructs, including the loop, the do loop and
the while loop. Here's an example of the loop:

 (setq i 0)
 (loop 
  (setq i (1+ i))
  (if (> i 10) (return \"done\") (print \"not yet!\"))
  )
")
(setq i 0)
(loop 
 (setq i (1+ i))
 (if (> i 10) (return "done") (print "not yet!"))
 )

(list "
	Here's an example of the while loop:

(def i 0)
(while (< i 10) (print i) (setq i (1+ i)))
")
(def i 0)
(while (< i 10) (print i) (setq i (1+ i)))

(list "
	The do loop is a little more complicated, but uses local variables for
looping indices (certainly more convenient):

(do
 ((i 0 (1+ i)))
 ((>= i 10))
 (print i))

Notice that it *stops* when i is >= 10 (it's using an 'until').
")
(do
 ((i 0 (1+ i)))
 ((>= i 10))
 (print i))

(list "
	One last hint: don't use lists when you can use lets: e.g.,

    (if (< a1 b1)
	(setq vals (list a1))
      (list
	(setq vals (list b1))
	(setq switch 't)
	)
      )

should really be

    (if (< a1 b1)
	(setq vals (list a1))
      (let()
	(setq vals (list b1))
	(setq switch 't)
	)
      )

These won't compile correctly either (if you're into compiling!).
")


(list "
	Looking at what I can do with the &rest and &key arguments. Turns out
that I can use both, if I'm willing to use the &rest with an argument, e.g.
:rest! Isn't that curious. It might prove useful if I want to send things off
to myself, otherwise - not really.

(defun crumcake
  (x &rest e &key hi)
  (print (list x e hi))
  )
(crumcake 1)
(crumcake 1 :hi 7)
(crumcake 1 :rest 2)
(crumcake 1 :e 2 3 2)
(crumcake 1 :e 2 3 2 :hi 7)
(crumcake 1 :e '(2 3 2 :hi 7))
")
(defun crumcake
  (x &rest e &key hi)
  (print (list x e hi))
  )
(crumcake 1)
(crumcake 1 :hi 7)
(crumcake 1 :rest 2)
(crumcake 1 :e 2 3 2)
(crumcake 1 :e 2 3 2 :hi 7)
(crumcake 1 :e '(2 3 2 :hi 7))
(list "")

(list "
(defun crumcake
  (x &rest e &key (hi nil) ho)
  (print (list x e hi ho))
  )
(crumcake 1)
(crumcake 1 :hi 7)
(crumcake 1 :rest 2)
(crumcake 1 :e 2 3 2)
(crumcake 1 :e 2 3 2 :hi 7)
(crumcake 1 :e '(2 3 2 :hi 7))
(crumcake 1 :e 2 3 2 :hi 7 :ho 8)
")
(crumcake 1)
(crumcake 1 :hi 7)
(crumcake 1 :rest 2)
(crumcake 1 :e 2 3 2)
(crumcake 1 :e 2 3 2 :hi 7)
(crumcake 1 :e '(2 3 2 :hi 7))
(crumcake 1 :e 2 3 2 :hi 7 :ho 8)

(list "")

(list "
(defun crumcake
  (x &rest e)
  (print (list x e))
  )
(crumcake 1 2)
(crumcake 1 2 :hi 2)
")
(defun crumcake
  (x &rest e)
  (print (list x e))
  )
(crumcake 1 2)
(crumcake 1 2 :hi 2)


(list "
        That's all folks. As I gather more hints, I'll put them together,
probably dividing this file up so that you'll have more options (so that you
won't have to see everything if you only want to see some particular coding
secrets).
")
) :title "Programming")
