(tutorial-dialog (combine 
(list "
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;                             Taylor Series
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;         The Taylor polynomial approximations to a function are useful *local*
;; approximations. They work very hard to get the function right at a point, x=a,
;; and if we're lucky that approximation will still be good in a neighborhood
;; around x=a.

;;         Have a look at the following second degree taylor polynomial
;; approximation to the cubic function f: the following commands will
;; (a) define f;
;; (b) find ft, the quadratic (degree 2) Taylor approximation at -.5; and
;; (c) plot both together from -2 to 2.

        (title \"Taylor series\")
        (func f(x) x ^ 3 - 2 * x + 3) ;; f(x)=x^3-2*x+3
        (taylor f ft 2 -.5) ;; Find taylor poly ft to f of degree 2 at -.5
        (plot f ft -2 2)
")
(list "
;;      (It's hard to be a quadratic wanting to be a cubic!)
")
(list "
        (title \"Cubic fit to x^4 at 1\")
        (func f(x) x ^ 4)
        (taylor f ft 3 1)
        (plot f ft .5 1.5)
")
(list "
        (title \"Sixth degree fit to sin(x) at pi/2\")
        (func g(x) sin x)
        (taylor g gt 6 (/ pi 2))
        (range -2 1)
        (plot g gt -pi/2 pi)
        (new-plot)
")
(list "
        ;; Tangent line has n of 1:
        (title \"Tangent line approximation at 2\")
        (taylor g gt 1 2)
        (plot g gt 1 3)
        (what gt) ;; (what tells you how a function is defined)
")
(list "

;;        This next example illustrates the concept of 'radius of convergence'.
;; There are basically two kinds of functions: those whose Taylor series just get
;; better as the degree of the polynomial goes to infinity, and those that have
;; some 'problem' that keeps the polys from getting close for the whole domain of
;; the function.

;;         Cosine is 'nice': but f(x) = 1/(1+x) isn't! Watch as the Taylor
;; polynomials try to get this function right at x=-1.

        (title \"Second degree fit to 1/(1+x) at 0\")
        (func g(x) 1 / (1 + x))
        (taylor g gt 2 0)
        (range 0 3)
        (plot g gt -.9 2)
")
(list "
        (title \"Second and Fourth fit to 1/(1+x) at 0\")
        (taylor g g4 4 0)
        (plot g gt g4 -.9 2)
")
(list "
        (title \"Second, Fourth, and Sixth fit\")
        (taylor g g6 6 0)
        (plot g gt g4 g6 -.9 2)

;; Notice that the sixth degree Taylor polynomial is doing worse than the second
;; and fourth as x approaches 2. As we increase the degree, the polys will do
;; better up to 1, but worse from there on. This function has radius of
;; convergence 1 about zero.
")
)
:title "Taylor Series")
