;;########################################################################
;; dashobj1.lsp
;; Contains datasheet menu functions, datasheet constructor function,
;; code to define the datasheet prototype and its slot accessor methods,
;; and a few additional methods.
;; Copyright (c) 1994-5 by Forrest W. Young
;;########################################################################

(defun new-data ()
  (let* ((txt1 (send text-item-proto :new "Create New Data Named:"))
         (name (send edit-text-item-proto :new "NewData" :text-length 14))
         (txt2 (send text-item-proto :new "The New Data Will Be:"))
         (type (send choice-item-proto :new
                     '("Multivariate Data" "Matrix Data" "Classification (Table) Data")))
         (OK   (send modal-button-proto :new "OK" :action #'(lambda () 
                    (list (send name :text) (send type :value)))))
         (cancel (send modal-button-proto :new "Cancel"))
         (dialog (send modal-dialog-proto :new
                       (list txt1 name txt2 type (list OK cancel)) 
                       :default-button OK))
         (return (send dialog :modal-dialog))
         (choice nil) )
    (when return
          (setf choice (second return))
          (case choice 
            (0 (show-datasheet (data (first return)
                :variables '("Var0") :labels '("Obs0") :data '(nil))
                               :new-data t))
            (1 (show-datasheet (data (first return)
                :variables '("Var0") :labels '("Var0") :data '(nil)
                :shapes '("Symmetric") :matrices '("Matrix 0"))
                               :new-data t))
            (t (show-datasheet (data (first return)
                :variables '("Response" "Way1") 
                :types '("Numeric" "Category")
                :labels '("Obs0") :data '(nil nil)) ))))))

(defun show-datasheet (&optional data-object &key new-data)
  (when (not data-object) (setf data-object current-data)) 
  (when (send data-object :ways) 
        (setf data-object 
              (create-data (strcat "MV-"(send data-object :name)))))
  (let ((dsob (send data-object :datasheet-object))) 
    (cond 
      ((not dsob)
           (setf dsob (datasheet data-object :editable t)))
      (t 
       (send dsob :editable t)
       (send dsob :enable-menu-items t)
       (send dsob :hot-cell nil)
       (send dsob :hot-cell-ready nil)
       (send dsob :hot-cell-string nil)
       (send dsob :scroll 0 0)
       (send dsob :set-window-size 
             (send dsob :field-width)
             (send dsob :field-height)
             (send dsob :label-width)
             (send dsob :nvar)
             (send dsob :nobs))
       (send dsob :redraw)
       (send dsob :show-window)))
    (when dsob
          (send dsob :enable-vista-menus&tools nil)
          (send dsob :new-data new-data)
          (send dsob :edited nil))
    t))

(defun browse-data (&optional data-object)
  (when (not data-object) (setf data-object current-data)) 
  (cond 
    ((not (send data-object :datasheet-object))
     (datasheet data-object :editable nil)
     )
    (t
     (send (send data-object :datasheet-object) :editable nil)
     (send (send data-object :datasheet-object) :show-window)))
  (send (send data-object :datasheet-object) :enable-menu-items nil)
  t)

(defun datasheet (data-object &key editable)
  (let* ((object (send datasheet-proto :new 2
                       :title (send data-object :name)
                       :size full-graph-size ;'(590 380)
                       :location location11 ;'(22 22)
                       :show nil)) 
         (nvar   (send data-object :nvar))
         (nobs   (send data-object :nobs))
         (dsmenu (send menu-proto :new "DataSheet"))
         (setfw  (send menu-item-proto :new "Width of Columns"
                       :action #'(lambda ()
                                   (send object :set-fw-dialog))))
         (setdec (send menu-item-proto :new "Number of Decimals"
                       :action #'(lambda ()
                                   (send object :set-dec-dialog))))
         (new1st (cond 
                   ((send data-object :matrices)
                    (send menu-item-proto :new "New Row and Column"
                          :action 'new-row-and-column))
                   (t
                    (send menu-item-proto :new "New Observation"
                          :action 'new-observation))))
         (new2nd (cond
                   ((send data-object :matrices)
                    (send menu-item-proto :new "New Matrix"
                          :action 'new-matrix))
                   (t
                    (send menu-item-proto :new "New Variable"
                       :action 'new-variable))))
         (newlab (cond
                   ((send data-object :matrices)
                    (send menu-item-proto :new "Change Matrix Names"
                          :action 'change-matrix-names))
                   (t
                    (send menu-item-proto :new "Switch Label Variable"
                          :action 'switch-label-variable))))
         (tw     (send object :text-width "9"))
         (data   (cond
                   ((send data-object :matrices)
                    (send data-object :data))
                   ((send data-object :ways)
                    (combine (send data-object :data)))
                   (t
                    (send data-object :active-data '(numeric ordinal)))))
         (ndigit (if (select data 0) 
                     (ceiling (log (max (abs (remove 'nil data))) 10)) 6))
         (ndciml 2)
         (ncols  (max 6 (+ ndigit ndciml)))
         (dcimlw (send object :text-width "."))
         (signw  (send object :text-width "-"))
         (fw     (+ (* tw ncols) dcimlw signw 6))
         (fh     (+ (send object :text-ascent) 
                    (send object :text-descent) 3))
         (lw     nil))
    (send object :variable-strings 
          (copy-list (send data-object :variables)))
    (send object :matrix-strings 
          (copy-list (send data-object :matrices)))
    (send object :type-strings (copy-list (send data-object :types))) 
    (send object :create-label-strings data-object)
    (setf lw (max (* 10 tw) fw (+ 6 (max 
             (map-elements #'send object :text-width 
                           (send object :label-strings))))))
    (send dsmenu :append-items setfw setdec (send dash-item-proto :new)
          new1st new2nd (send dash-item-proto :new) newlab)
    (send object :data-object data-object)
    (send object :nvar nvar)
    (send object :nobs nobs) 
    (send object :newvar 0)
    (send object :newobs 0)
    (send object :newmat 0)
    (when (send data-object :matrices)
          (send object :nmat (send data-object :nmat)))
    (send object :field-width fw) 
    (send object :field-height fh)
    (send object :label-width  lw)
    (send object :number-of-decimals ndciml)
    (send object :number-of-columns ncols)
    (send object :create-data-matrix-strings)
    (send object :editable editable)
    (send object :menu dsmenu)
    (send object :show-window)
    (send object :set-window-size fw fh lw nvar nobs)
    (send object :h-scroll-incs fw (* 4 fw))
    (send object :v-scroll-incs fh (* 10 fh))
    (setf *datasheet* object)
    (send data-object :datasheet-object object)
    (send data-object :datasheet-open t)
    object))
    
(defproto datasheet-proto '(data-object 
   data-matrix-strings variable-strings label-strings type-strings 
   editable edited nvar nobs nmat field-width field-height label-width 
   hot-cell hot-cell-ready hot-cell-string menu-states new-data
   number-of-columns number-of-decimals newvar newobs newmat
   matrix-strings) nil graph-proto)

(defmeth datasheet-proto :isnew (&rest args) 
  (apply #'call-next-method args))

(defmeth datasheet-proto :data-object (&optional (obj-id nil set))
"Message args: (&optional obj-id)
 Sets or retrieves the object-id of the data object for this datasheet."
  (if set (setf (slot-value 'data-object) obj-id))
  (slot-value 'data-object))

(defmeth datasheet-proto :data-matrix-strings (&optional (matrix nil set))
"Message args: (&optional matrix)
 Sets or retrieves the data matrix with data as strings."
  (if set (setf (slot-value 'data-matrix-strings) matrix))
  (slot-value 'data-matrix-strings))

(defmeth datasheet-proto :variable-strings (&optional (list nil set))
"Message args: (&optional list)
 Sets or retrieves the list of variable strings."
  (if set (setf (slot-value 'variable-strings) list))
  (slot-value 'variable-strings))

(defmeth datasheet-proto :label-strings (&optional (list nil set))
"Message args: (&optional list)
 Sets or retrieves the list of label strings."
  (if set (setf (slot-value 'label-strings) list))
  (slot-value 'label-strings))

(defmeth datasheet-proto :type-strings (&optional (list nil set))
"Message args: (&optional list)
 Sets or retrieves the list of type strings."
  (if set (setf (slot-value 'type-strings) list))
  (slot-value 'type-strings))

(defmeth datasheet-proto :matrix-strings (&optional (list nil set))
"Message args: (&optional list)
 Sets or retrieves the list of matrix name strings."
  (if set (setf (slot-value 'matrix-strings) list))
  (slot-value 'matrix-strings))

(defmeth datasheet-proto :editable (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether the datasheet is editable."
  (if set (setf (slot-value 'editable) logical))
  (slot-value 'editable))

(defmeth datasheet-proto :edited (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether the datasheet is editable."
  (if set (setf (slot-value 'edited) logical))
  (slot-value 'edited))

(defmeth datasheet-proto :nobs (&optional (number nil set))
"Message args: (&optional number)
 Sets or retrieves the number of observations in the datasheet."
  (if set (setf (slot-value 'nobs) number))
  (slot-value 'nobs))

(defmeth datasheet-proto :nvar (&optional (number nil set))
"Message args: (&optional number)
 Sets or retrieves the number of variables in the datasheet."
  (if set (setf (slot-value 'nvar) number))
  (slot-value 'nvar))

(defmeth datasheet-proto :newobs (&optional (number nil set))
"Message args: (&optional number)
 Sets or retrieves the number of observations in the datasheet."
  (if set (setf (slot-value 'newobs) number))
  (slot-value 'newobs))

(defmeth datasheet-proto :newvar (&optional (number nil set))
"Message args: (&optional number)
 Sets or retrieves the number of variables in the datasheet."
  (if set (setf (slot-value 'newvar) number))
  (slot-value 'newvar))

(defmeth datasheet-proto :newmat (&optional (number nil set))
"Message args: (&optional number)
 Sets or retrieves the number of variables in the datasheet."
  (if set (setf (slot-value 'newmat) number))
  (slot-value 'newmat))

(defmeth datasheet-proto :nmat (&optional (number nil set))
"Message args: (&optional number)
 Sets or retrieves the number of variables in the datasheet."
  (if set (setf (slot-value 'nmat) number))
  (slot-value 'nmat))

(defmeth datasheet-proto :field-width (&optional (number nil set))
"Message args: (&optional number)
 Sets or retrieves the field width of the datasheet."
  (if set (setf (slot-value 'field-width) number))
  (slot-value 'field-width))

(defmeth datasheet-proto :field-height (&optional (number nil set))
"Message args: (&optional number)
 Sets or retrieves the field height of the datasheet."
  (if set (setf (slot-value 'field-height) number))
  (slot-value 'field-height))

(defmeth datasheet-proto :label-width (&optional (number nil set))
"Message args: (&optional number)
 Sets or retrieves the label width of the datasheet."
  (if set (setf (slot-value 'label-width) number))
  (slot-value 'label-width))

(defmeth datasheet-proto :hot-cell (&optional (number-list nil set))
"Message args: (&optional number-list)
 Sets or retrieves a list of the row and column of the highlighted cell."
  (if set (setf (slot-value 'hot-cell) number-list))
  (slot-value 'hot-cell))

(defmeth datasheet-proto :hot-cell-ready (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether the hot cell is ready for typing."
  (if set (setf (slot-value 'hot-cell-ready) logical))
  (slot-value 'hot-cell-ready))

(defmeth datasheet-proto :hot-cell-string (&optional (string nil set))
"Message args: (&optional string)
 Sets or retrieves the hot cell string."
  (if set (setf (slot-value 'hot-cell-string) string))
  (slot-value 'hot-cell-string))

(defmeth datasheet-proto :menu-states (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether the hot cell is ready for typing."
  (if set (setf (slot-value 'menu-states) logical))
  (slot-value 'menu-states))

(defmeth datasheet-proto :new-data (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether the hot cell is ready for typing."
  (if set (setf (slot-value 'new-data) logical))
  (slot-value 'new-data))

(defmeth datasheet-proto :number-of-decimals (&optional (number nil set))
"Message args: (&optional number)
 Sets or retrieves the number of decimals displayed in the datasheet."
  (if set (setf (slot-value 'number-of-decimals) number))
  (slot-value 'number-of-decimals))

(defmeth datasheet-proto :number-of-columns (&optional (number nil set))
"Message args: (&optional number)
 Sets or retrieves the number of columns displayed in the datasheet."
  (if set (setf (slot-value 'number-of-columns) number))
  (slot-value 'number-of-columns))

(defmeth datasheet-proto :set-window-size (fw fh lw nv no)
  (send self :has-h-scroll 
        (max (select (screen-size) 0) (+ 1 lw (* fw (+ 1 nv)))))
  (send self :has-v-scroll 
        (max (select (screen-size) 1) (+ 1 (* fh (+ 3 no))))))

(defmeth datasheet-proto :enable-vista-menus&tools 
  (&optional (logical nil set)) 
  (when set 
        (cond 
          (logical ;enable appropriate menus and tools
             (cond
               ((send current-data :matrices)
                (send current-data :set-menu&tool-states "Matrix"))
               ((send current-data :ways)
                (send current-data :set-menu&tool-states "Table"))
               (t (send current-data :set-menu&tool-states "MV"))))
          (t       ;disable all menus and tools
             (send current-data :set-menu&tool-states "Disabled")
             (send self :menu-states 
                   (list (send *data-menu* :enabled) 
                         (send *trans-menu* :enabled)
                         (send *tools-menu* :enabled) 
                         (send *model-menu* :enabled))))))
  (send self :menu-states))


(defmeth datasheet-proto :create-data-matrix-strings ()
"Message args: nil
 Creates and stores the string version of the data matrix." 
  (let* ((mat (send (send self :data-object) :data-matrix))
         (numobs (send self :nobs))
         (numvar (send self :nvar))
         (nummat (send self :nmat))
         (ndec (send self :number-of-decimals))
         (ncol (send self :number-of-columns))
         (matst (make-array (list numobs numvar)))
         (k 0)) 
    (cond 
      ((= 0 (length (send current-data :matrices)))
       (dotimes (i numobs)
          (dotimes (j numvar)
             (setf (aref matst i j) (string-trim " " ;was string-right-trim
                   (format nil "~v,vf" ncol ndec (aref mat i j)))))))
      (t
       (send self :nobs (* numvar nummat))
       (dotimes (L nummat)
          (setf k 0)
          (dotimes (i numvar)
             (dotimes (j numvar)
                (setf (aref matst (+ i (* numvar L)) j) (string-trim " "
                      (format nil "~v,vf" ncol ndec (aref mat k L))))
                (setf k (+ k 1))))))) 
    (send self :data-matrix-strings matst))) 

(defmeth datasheet-proto :create-label-strings (data-object)
  (cond 
    ((send data-object :ways)
     (send self :label-strings (repeat (send data-object :labels) 
                                       (send data-object :cellfreqs))))
    ((send data-object :matrices)
     (send self :label-strings (repeat (send data-object :variables)
                                       (send data-object :nmat))))
    (t
     (send self :label-strings (copy-list (send data-object :labels))))))