;;########################################################################
;; dashobj3.lsp
;; Contains code to manipulate and close datasheet.
;; Copyright (c) 1994-5 by Forrest W. Young
;;########################################################################


(defun change-matrix-names () (send *datasheet* :change-matrix-names))

(defmeth datasheet-proto :change-matrix-names () 
  (let* ((result (send (send self :matrix-names-dialog-box) :modal-dialog))
         (names (copy-list (send self :matrix-strings))))
    (when result
          (cond ((first result) 
                 (setf (select names (first result)) (second result))
                 (send self :edited t)
                 (send self :matrix-strings names))
            (t (error-message "You must select a name to change."))))))

(defmeth datasheet-proto :matrix-names-dialog-box ()
"Returns list of matrix names, or nil if canceled."
  (let* ((heading (send text-item-proto :new "Change Matrix Names:"))
         (step1   (send text-item-proto :new "1) Select Name to Change:"))
         (matlist (send list-item-proto :new (send self :matrix-strings)))
         (step2   (send text-item-proto :new "2) Type New Name:"))
         (newname (send edit-text-item-proto :new "" :text-length 24))
         (cancel  (send modal-button-proto :new "Cancel"))
         (ok      (send modal-button-proto :new "OK" :action #'(lambda () 
                    (list (send matlist :selection) 
                          (send newname :text))))))
    (send modal-dialog-proto :new
               (list heading step1 matlist step2 newname (list ok cancel))
          :default-button ok)))

(defun switch-label-variable () (send *datasheet* :switch-label-variable))

(defmeth datasheet-proto :switch-label-variable ()
  (let* ((varnum (send (send self :label-dialog-box) :modal-dialog))
         (data (send self :data-matrix-strings))
         (labels (send self :label-strings))
         (names (send *datasheet* :variable-strings))
         (types (send self :type-strings))
         (var nil)
         (nobs nil)
         )
    (when varnum
          (setf varnum (first varnum))
          (cond 
            ((not varnum) (error-message "You must select a variable."))
            (t
             (setf var (col data varnum))
             (setf nobs (length var))
             (send self :label-strings var) 
             (setf (select data (iseq nobs) varnum) 
                   (matrix (list nobs 1) labels))
             (send self :data-matrix-strings data)
             (setf (select types varnum) "Category")
             (setf (select names varnum) "Label")
             (send self :edited t)
             (send self :redraw))))))
          
(defmeth datasheet-proto :label-dialog-box ()
"Returns list of selected variables, or (nil) if none selected, or nil if canceled."
  (let* ((heading (send text-item-proto :new 
           (format nil "Switch the Label Variable~%with the Data Variable~%you select below:")))
         (varlist (send list-item-proto :new (send self :variable-strings)))
         (cancel    (send modal-button-proto :new "Cancel"))
         (ok        (send modal-button-proto :new "OK" :action #'(lambda () 
                    (list (send varlist :selection))))))
    (send modal-dialog-proto :new
               (list heading varlist (list ok cancel))
          :default-button ok)
        ))

(defmeth datasheet-proto :enable-menu-items (nilt)
  (let ((menu (send self :menu)))
    (send (select (send menu :items) 3) :enabled nilt)
    (send (select (send menu :items) 4) :enabled nilt)
    (send (select (send menu :items) 6) :enabled nilt)))
    
(defmeth datasheet-proto :close ()
  (let* ((choice nil)
         (result nil)
         (new-name nil)
         (dob (send self :data-object))
         (parents (send dob :dob-parents))
         (children (send dob :dob-children))
         (old-name (send dob :name))
         (dms (send self :data-matrix-strings))
         (num-choices 3)
         (table nil)
         (new-table nil)
         (types (mapcar #'string-capitalize (send self :type-strings)))
         (tok (list "Category" "Ordinal" "Numeric"))
         )
    (send self :type-strings types)
    (dotimes (i (send self :nvar))
             (when (not (find (select types i) tok :test 'equal))
                   (message-dialog 
                    "Variable types must be Category, Ordinal or Numeric")
                   (error "Bad Variable Type"))
             (when (not (equal (select types i) "Category"))
                   (map-elements #'convert-number-from-string (col dms i)))
             )
    (when (or parents children) (setf num-choices 2))
    (when
     (and
      (equal (first   types) "Numeric")
      (=     (length (remove-duplicates (rest types) :test 'equal)) 1)
      (equal (first  (remove-duplicates (rest types) :test 'equal)) 
             "Category"))
     (setf table t)
     (setf num-choices 2)) 
    (cond
      ((and (send self :edited) (send self :editable) 
            (not (send self :new-data)))
       (setf result (send self :close-dialog old-name num-choices table))
       (setf choice (first result))
       (setf new-name (second result))
       (setf new-table (third result)))
      ((send self :new-data)     ;new data treated as update current dob
       (setf choice 2)
       (setf new-name old-name))
      ((or (not (send self :edited)) ;unedited or uneditable data discarded  
           (not (send self :editable)))
       (setf choice 1))) 
    (when (not new-table) (setf new-table 1))
    (when choice ;0=create new; 1=discard; 2=update current
          (send dob :datasheet-open nil)
          (send self :hide-window)
  ;       (when (send *workmap* :gui) (send *workmap* :show-window))
          (send self :enable-vista-menus&tools t)
          (when (/= 1 choice) ;use changes to create new or update current
                (send self :update-data-object choice new-name new-table)
                (cond
;current-data changed from dob in next two statements 12/17/95 fwy
                  ((send current-data :matrices) (send *vista* :show-mats))
                  ((send current-data :ways) (send *vista* :show-cells))
                  (t (send *vista* :show-obs)))
                (send *vista* :show-labels))
          (when (< choice 2) ;leave old unchanged - discard or new
                (send self :nvar (- (send self :nvar) (send self :newvar)))
                (send self :nobs (- (send self :nobs) (send self :newobs)))
                (when (send self :nmat)
                      (send self :matrix-strings (send dob :matrices))
                      (send self :nmat (- (send self :nmat) 
                                          (send self :newmat))))
                (send self :variable-strings 
                                          (copy-list (send dob :variables)))
                (send self :type-strings  (copy-list (send dob :types)))
                (send self :create-label-strings dob)
                (send self :create-data-matrix-strings))
          (send self :newvar 0)
          (send self :newobs 0)
          (send self :newmat 0))))

(defmeth datasheet-proto :update-data-object (choice title table)
  (let* ((dob (send self :data-object))
         (matdata nil)
         (new-mat-names nil)
         (noldmat nil)
         (nnewmat nil)
         (dob-nvar (send dob :nvar))
         (dob-nobs (send dob :nobs))
         (dsob-nvar (send self :nvar))
         (dsob-nobs (send self :nobs)))
    (when (= choice 0) ;create new dob
          (cond
            ((send dob :matrices) 
             (data title
                   :created (send *desktop* :selected-icon)
                   :data (combine (send self :matrix-from-strings-matrix))
                   :variables (send self :variable-strings)
                   :matrices (send self :matrix-strings)
                   :labels (send self :label-strings)
                   :types (send self :type-strings)))
;if we could make a table dob here we wouldn't have to make both mv and tab
            (t
             (data title
                   :created (send *desktop* :selected-icon)
                   :data (combine (send self :matrix-from-strings-matrix))
                   :variables (send self :variable-strings)
                   :labels (send self :label-strings)
                   :types (send self :type-strings))))
          (when (= 0 table)
                (send current-data :make-table-data 
                      (first (send current-data :variables)))))
    (when (= choice 2) ;update old dob
          (cond 
            ((send dob :matrices) 
             (setf matdata (combine (send self :matrix-from-strings-matrix)))
             (send dob :matrices (send self :matrix-strings))
             (send dob :nmat (send self :nmat))
             (send dob :nele (/ (length matdata) (send dob :nmat)))
             (send dob :mat-states (repeat 'normal (send dob :nmat)))
             (send dob :shapes (repeat (first (send dob :shapes)) 
                                       (send dob :nmat)))
             (send dob :data (combine (transpose 
                (matrix (list (send dob :nmat) (send dob :nele)) matdata)))))
            (t
             (send dob :data (combine 
                              (send self :matrix-from-strings-matrix)))
             ))
          (send dob :labels (send self :label-strings))
          (send dob :variables (send self :variable-strings))
          (send dob :types (send self :type-strings))
          (when (or (/= dob-nvar dsob-nvar) (/= dob-nobs dsob-nobs))
                (send dob :nvar dsob-nvar)
                (send dob :nobs dsob-nobs)
                (send dob :obs-states (repeat 'normal dsob-nobs))
                (send dob :var-states (repeat 'normal dsob-nvar))))
    ))

(defmeth datasheet-proto :close-dialog (old-name num-choices table)
"Args: old-name num-choice table
Dialog Box for Closing Datasheets. Arguments are name of old data object, number of choice for saving changes, and t if datasheet represents a table. Returns nil if cancelled, or a list of three items. First is 0 for create; 1 for discard; 2 for update. Second is name of new data object.  Third is Nil when table is nil, if table is t then 0 of create table, 1 for create multivariate."
  (let* ((title (send text-item-proto :new 
                      "Options for Saving DataSheet:"))
         (choice (if (= num-choices 2)
                      (send choice-item-proto :new (list
                            "Discard All DataSheet Changes"
                            "Create New Data Object named:")
                            :value 1)
                      (send choice-item-proto :new (list
                            "Update Current Data Object"
                            "Discard All DataSheet Changes"
                            "Create New Data Object named:")
                            :value 2)))
         (y-loc (if (= num-choices 2) 82 102))
#+msdos  (y-loc (+ y-loc 10))
         (name (send edit-text-item-proto :new (strcat "Edit-of-" old-name)
                     :location (list 27 y-loc)))
 ;                   :size (list 200 16)))
#-msdos  (y-loc2 24)
#+msdos  (y-loc2 34)
         (type-title (send text-item-proto :new
                      "Create:"
                     :location (list 12 (+ y-loc y-loc2))))
         (type (send choice-item-proto :new (list
                     "Table and Multivariate Data"
                     "Multivariate Data Only")
                     :value 0
                     :location (list 78 (+ y-loc (- y-loc2 2)))))
         (ok (send modal-button-proto :new "OK"
                   :action #'(lambda ()
                               (list (- num-choices 1 (send choice :value)) 
                                     (send name :text)
                                     (if table (send type :value))))))
         (cancel (send modal-button-proto :new "Cancel"))
         (dialog (send modal-dialog-proto :new
                       (cond
                         (table (list title choice name 
                                      type-title type
                                      (list ok cancel)))
                         (t (list title choice name (list ok cancel))))
                       :default-button ok))
         )
    (when table (setf y-loc (+ y-loc 60)))
    (send dialog :modal-dialog)))

(defmeth datasheet-proto :make-new-labels-and-cells ()
"Args: none
Used to write out edited table data.  Sorts table cell labels into order and makes a new table data cell list based on the sorted cell labels. Returns the sorted labels and the data cell list."
  (let* ((label-strings (send self :label-strings))
         (sorted-table (sort-and-permute-dob 
                        (send self :data-matrix-strings) 
                        label-strings label-strings nil))
         (sorted-data   (combine (first sorted-table)))
         (sorted-labels (second sorted-table))
         (nobs (length sorted-labels))
         (data-cell-list nil)
         (start 0)
         (finish nil)) 
    (dotimes (i (1- nobs))
       (when (not (eq (select sorted-labels i) (select sorted-labels (1+ i))))
             (setf finish i)
             (setf data-cell-list (add-element-to-list 
                                   data-cell-list 
                                  (select sorted-data (iseq start finish))))
             (setf start (1+ i))))
    (setf data-cell-list (add-element-to-list 
                          data-cell-list 
                         (select sorted-data (iseq start (1- nobs)))))
    (list sorted-labels data-cell-list)))
             
(defmeth datasheet-proto :matrix-from-strings-matrix () 
  (let* ((dob  current-data)
         (dms  (send self :data-matrix-strings))
         (type (send self :type-strings))
         (nvar (send self :nvar))
         (nobs (send self :nobs))
         (dm   nil))
     (dotimes (i nvar)
              (cond 
                ((equal (select type i) "Category")
                 (setf dm (combine dm (col dms i))))
                (t
                 (setf dm (combine dm
                     (map-elements #'number-from-string (col dms i)))))))
    (transpose (matrix (list nvar nobs) (rest dm)))))

(defmeth datasheet-proto :show-window () 
  (send (send self :data-object) :datasheet-open t)
  (call-next-method))

(defmeth datasheet-proto :set-fw-dialog () 
  (let* ((min-ncols 1)
         (tw        (send self :text-width "9"))
         (signw     (send self :text-width "-"))
         (dcimlw    (send self :text-width "."))
         (ndecimals (send self :number-of-decimals))
         (old-ncols (send self :number-of-columns))
         (new-ncols (get-value-dialog "Width of Columns:" 
                                      :initial old-ncols))
         )
    (when new-ncols 
          (when (not new-ncols) (setf new-ncols (list min-ncols)))
          (when (not (first new-ncols)) (setf new-ncols (list min-ncols)))
          (when (< (first new-ncols) min-ncols) 
                (setf new-ncols (list min-ncols)))
          (send self :field-width (+ (* tw (first new-ncols)) dcimlw signw 6))
          (send self :number-of-columns (first new-ncols))
          (send self :scroll 0 0)
          (send self :has-h-scroll 
                (max (select (screen-size) 0)
                     (+ 1 (send self :label-width) (* (send self :field-width) 
                                           (+ 1 (send self :nvar))))))
          (send self :redraw))))

(defmeth datasheet-proto :set-dec-dialog ()
  (let* ((odec (send self :number-of-decimals))
         (ndec (get-value-dialog "Number of Decimals:" 
                                 :initial odec))) 
    (when ndec
          (setf ndec (first ndec))
          (when (not ndec) (setf ndec odec))
          (when (< ndec 0) (setf ndec odec))
          (send self :number-of-decimals ndec)
          (send self :create-data-matrix-strings)
          (send self :redraw))))

(defun convert-number-from-string (str)
    (let ((result (ignore-errors (number-from-string str))))
      (when (not result) 
            (when (not (equal "nil" (string-downcase str)))
                  (message-dialog 
          "A Numeric or Ordinal variable has non-numeric data.")
                  (error "Bad Numeric or Ordinal Data")))
      result))

(setf *datasheet* nil)