;;#########################################################################
;; dataobj2.lsp
;; Copyright (c) 1991-95 by Forrest W. Young
;; This file contains the data object's menu methods and conversion methods. 
;; Together, dataobj1.lsp, dataobj2.lsp, and dataobj3.lsp contain
;; all the code to implement multivariate data objects.
;;#########################################################################

;;-------------------------------------------------------------------------
;; slot accessor methods (continued)
;;-------------------------------------------------------------------------

(defmeth mv-data-object-proto :dob-parents 
                                    (&optional (object-id-list nil set))
"Message args: (&optional object-id-list)
 Sets or retrieves the list of parent data and model objects." 
  (if set (setf (slot-value 'dob-parents) object-id-list))
  (slot-value 'dob-parents))

(defmeth mv-data-object-proto :dob-children 
                                  (&optional (object-id-list nil set))
"Message args: (&optional object-id-list)
 Sets or retrieves the list of children data and model objects." 
  (if set (setf (slot-value 'dob-children) object-id-list))
  (slot-value 'dob-children))

(defmeth mv-data-object-proto :guidemap-number (&optional (number nil set))
"Message args: (&optional number)
 Sets or retrieves the guidemap number (just for identification)." 
  (if set (setf (slot-value 'guidemap-number) number))
  (slot-value 'guidemap-number))

(defmeth mv-data-object-proto :guidemap-ancestors 
  (&optional (number-list nil set))
"Message args: (&optional number-list)
Sets or retrieves the guidemap ancestors list. This is a list of guidemap numbers created by link and used by return to traverse the guidemap hypertext." 
  (if set (setf (slot-value 'guidemap-ancestors) number-list))
  (slot-value 'guidemap-ancestors))

(defmeth mv-data-object-proto :add-parent (parent-object)
  (send self :dob-parents 
      (add-element-to-list (send self :dob-parents) parent-object)))

(defmeth mv-data-object-proto :add-child (child-object)
  (send self :dob-children 
        (add-element-to-list (send self :dob-children) child-object)))

(defmeth mv-data-object-proto :needs-computing (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether the object needs computing (t) or not (nil)." 
  (if set (setf (slot-value 'needs-computing) logical))
  (slot-value 'needs-computing))

(defmeth mv-data-object-proto :auto-compute (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether the object automatically re-computes (t) or not (nil) when needs-computing is set to t." 
  (if set (setf (slot-value 'auto-compute) logical))
  (slot-value 'auto-compute))

(defmeth mv-data-object-proto :datasheet-object 
  (&optional (object-id nil set))
"Message args: (&optional object-id)
 Sets or retrieves the object id of the data objects datasheet." 
  (if set (setf (slot-value 'datasheet-object) object-id))
  (slot-value 'datasheet-object))

(defmeth mv-data-object-proto :datasheet-open 
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether the datasheet is open." 
  (if set (setf (slot-value 'datasheet-open) logical))
  (slot-value 'datasheet-open))

;;-------------------------------------------------------------------------
;; methods to convert mv to table data
;;-------------------------------------------------------------------------

(defmeth mv-data-object-proto :make-table-data 
                         (response-variable-name &optional menu-name)
;1 make new labels from category variables
  (let* ((cat-matrix (send self :active-data-matrix '(category)))
         (cat-variables  (send self :active-variables '(category)))
         (cell-labels (make-cell-labels cat-matrix cat-variables))
         (ungrouped-resp-var (send self :variable response-variable-name))
         (nobs (length ungrouped-resp-var))
;2 sort new labels into order and create data cells
         (urv-mat (send self :active-data-matrix '(all)))
         (sorted-labels-and-grouped-data 
          (sort-labels-and-group-data cell-labels urv-mat))
;3 create new table data object 
         (table nil))
    (when (not menu-name) 
          (setf menu-name (strcat "Table-" (send self :title))))
    (setf table (data menu-name
                      :created (send *desktop* :selected-icon)
                      :data (second sorted-labels-and-grouped-data)
                      :variables (list response-variable-name)
                      :ways cat-variables
                      :classes (third sorted-labels-and-grouped-data)))
#| I don't think the following is needed, nor do I think :set-labels is used anywhere else. FWY 4/11/95
    (send table :set-labels 
          (remove-duplicates (first sorted-labels-and-grouped-data)
                             :test 'equal)) |#
    table))

(defun make-cell-labels (cat-matrix cat-variables)
  (let ((row nil)
        (labels nil)
        (string "")
        (nobs (first (size cat-matrix)))
        (nvar (second (size cat-matrix)))
        )
    (dotimes 
     (i nobs)
     (setf row (row (send current-data :active-data-matrix '(category)) i))
     (setf string "")
     (dotimes 
      (j nvar)
      (setf string (strcat string 
           (select cat-variables j) "[" (select cat-matrix i j) "] ")))
     (setf labels (add-element-to-list labels string)))
    labels))  

#|I don't think this function is used. FWY 4/11/95
(defun make-classes ()
  (let* ((cat-vars (column-list 
                    (send current-data :active-data-matrix '(category))))
         (ncatvars (length cat-vars))
         (result nil))
    (dotimes (i ncatvars)
             (setf result (add-element-to-list result
                (coerce (remove-duplicates (select cat-vars i) :test 'equal) 
                        'list))))
    result))|#

(defun sort-labels-and-group-data (cell-labels ungrouped-data)
"Args: cell-labels ungrouped-data
Cells-labels is a list of cell-labels and ungrouped-data is a data-matrix. Sorts table cell labels into order and uses the sorted cell labels to make a new table data cell list and classes list . Returns the sorted labels, the data cell list, and the sorted classes list."
  (let* ((sorted-table (sort-and-permute-dob 
                        ungrouped-data cell-labels cell-labels nil))
         (sorted-data (first sorted-table))
         (sorted-resp-var (combine (col sorted-data 0)))
         (sorted-labels (second sorted-table))
         (nobs (length sorted-labels))
         (nways (1- (second (size ungrouped-data)))) 
         (data-cell-list nil)
         (classes-list nil)
         (start 0)
         (finish nil)) 
    (dotimes (i (1- nobs))
       (when (not (equal (select sorted-labels i) 
                         (select sorted-labels (1+ i))))
             (setf finish i)
             (setf data-cell-list (add-element-to-list data-cell-list 
                          (select sorted-resp-var (iseq start finish))))
             (setf start (1+ i))))
    (setf data-cell-list (add-element-to-list data-cell-list 
                         (select sorted-resp-var (iseq start (1- nobs)))))
    (dotimes (i nways)
             (setf classes-list (add-element-to-list classes-list
                   (remove-duplicates (combine (col sorted-data (1+ i)))
                                  :test 'equal))))
    (list sorted-labels data-cell-list classes-list)))

#|This was in the above method but was taken out when blanks removed from data
(dotimes (i nways)
             (setf classes-list (add-element-to-list classes-list
                   (remove-duplicates 
                    (map-elements #'string-right-trim " "
                                  (combine (col sorted-data (1+ i))))
                                  :test 'equal))))|#

;;---------------------------------------------------------------------------
;;data menu methods 
;;---------------------------------------------------------------------------
     
(defmeth mv-data-object-proto :set-menu&tool-states (data-mode)
"Method Args: data-mode
Sets states of tools to gray or normal depending on data-mode. Mode may be a string which is Table, Matrix, MV, or Disabled." 
  (let ((tools (send *toolbox* :icon-list)))
    (cond 
      ((equal data-mode "MV")
       (send *data-menu*  :enabled t) 
       (send *trans-menu* :enabled t)
       (send *tools-menu* :enabled t)
       (send (select tools 0) :icon-state "normal")  ;help
       (send (select tools 1) :icon-state "gray")    ;anova  
       (send (select tools 2) :icon-state "normal")  ;coresp
       (send (select tools 3) :icon-state "gray")    ;factor
       (send (select tools 4) :icon-state "gray")    ;mdscal
       (send (select tools 5) :icon-state "normal")  ;mulreg
       (send (select tools 6) :icon-state "normal")  ;prncmp
       (send (select tools 7) :icon-state "normal")  ;regres
       (send (select tools 8) :icon-state "normal")  ;univar
       (send (select tools 9) :icon-state "normal")  ;nonpar  TAC
       )
      ((equal data-mode "Table")
       (send *data-menu*  :enabled t) 
       (send *trans-menu* :enabled nil)
       (send *tools-menu* :enabled t)
       (send (select tools 0) :icon-state "normal")
       (send (select tools 1) :icon-state "normal")
       (send (select tools 2) :icon-state "gray")
       (send (select tools 3) :icon-state "gray")
       (send (select tools 4) :icon-state "gray")
       (send (select tools 5) :icon-state "gray")
       (send (select tools 6) :icon-state "gray")
       (send (select tools 7) :icon-state "gray")
       (send (select tools 8) :icon-state "normal")
       (send (select tools 9) :icon-state "normal") ; TAC
       )
      ((equal data-mode "Matrix")
       (send *data-menu*  :enabled t) 
       (send *trans-menu* :enabled nil)
       (send *tools-menu* :enabled t)
       (send (select tools 0) :icon-state "normal")
       (send (select tools 1) :icon-state "gray")
       (send (select tools 2) :icon-state "gray")
       (send (select tools 3) :icon-state "gray")
       (send (select tools 4) :icon-state "normal")
       (send (select tools 5) :icon-state "gray")
       (send (select tools 6) :icon-state "gray")
       (send (select tools 7) :icon-state "gray")
       (send (select tools 8) :icon-state "gray")
       (send (select tools 9) :icon-state "gray") ; TAC
       )
      ((equal data-mode "Disabled")
       (send *data-menu*  :enabled nil) 
       (send *trans-menu* :enabled nil)
       (send *tools-menu* :enabled nil)
       (send (select tools 0) :icon-state "normal")
       (send (select tools 1) :icon-state "gray")
       (send (select tools 2) :icon-state "gray")
       (send (select tools 3) :icon-state "gray")
       (send (select tools 4) :icon-state "gray")
       (send (select tools 5) :icon-state "gray")
       (send (select tools 6) :icon-state "gray")
       (send (select tools 7) :icon-state "gray")
       (send (select tools 8) :icon-state "gray")
       (send (select tools 9) :icon-state "gray") ; TAC
       ))
    t))
        
(defmeth mv-data-object-proto :save-data (&optional file)
"Args: (&optional file)
FILE is a string. The data-object is written to the file FILE.lsp in a form suitable for use with the load-data command."  
 (when (not file) (setf file
#+macintosh(set-file-dialog "Save Data in File:" "" t)
#+msdos    (set-file-dialog "Save Data in File:")
#+X11      (get-string-dialog "Save Data in File:"  )
                        ))
  (when file
        (setf file (string-downcase file))
        (when (and (> (length file) 3)
              (string= ".lsp" file
                       :start2 (- (length file) 4)))
              (setf file (string-right-trim "lsp" file))
              (setf file (string-right-trim "." file)))
        (format t "; saving ~s~%" file)
        (let ((f (open (strcat (string file) ".lsp") :direction :output))
              (oldbreak *breakenable*))
          (setq *breakenable* nil)
          (send self :save-data-template f)
          (setq *breakenable* oldbreak)
          (close f)
          (format t "; finished saving ~s~%" file)
          f)))

(defmeth mv-data-object-proto :save-data-template (f)
  (unwind-protect
   (print 
    `(data       ,(send self :name)
                 :title      ,(send self :title)
                 :variables ',(send self :active-variables '(all))
                 :types     ',(send self :active-types '(all))
                 :labels    ',(send self :active-labels)
                 :data      ',(send self :active-data '(all)))
    f)))

(defmeth mv-data-object-proto :delete-data ()
"Message args: none
Deletes a data object and all its child objects."
  (error-message "Not yet implemented."))

(defmeth mv-data-object-proto :create-data (&optional name &key (table nil)) 
"Message args: (&optional name &key (table nil))
Creates a new data object from the current active data.  Can optionally convert mv-table data to table data. If NAME is specified, the data object is named NAME (a string) and, if table is t, is converted to be a table object. If NAME not specified a dialog is presented for name and table. Returns object identification of the new data object."
  (if (not (eq current-object self)) (setcd self))
  (setf merge-dob nil)
  (let* ((result nil)
         (menu-name name)
         (types (send self :active-types '(all)))
         (rest-types (remove-duplicates (rest types) :test 'equal))
         (numeric-var (first (send current-data :active-variables '(all))))
         )
    (when table
          (if (not (send self :can-make-table))
            (error "Cannot convert these multivariate data to table data.")))
    (when (not menu-name)
          (setf table (send self :can-make-table))
          (setf result (send self :close-dialog table))
          (setf menu-name (first result))
          (setf table (second result)))
    (when (and menu-name (= (length menu-name) 0)) 
          (error-message "You must specify a name."))
    (when (and menu-name (> (length menu-name) 0))
          (if table
              (send self :make-table-data numeric-var menu-name)
              (data menu-name
                    :created (send *desktop* :selected-icon)
                    :title (strcat "Created from " (send self :title))
                    :data (send self :active-data '(all))
                    :variables (send self :active-variables '(all))
                    :labels (send self :active-labels)
                    :types (send self :active-types '(all))
                    )))))

(defmeth mv-data-object-proto :can-make-table ()
"Message args: none
Returns t if data are convertable to table data, nil otherwise"
  (let* ((types (send self :active-types '(all)))
         (rest-types (remove-duplicates (rest types) :test 'equal))
         (numeric-var (first (send current-data :active-variables '(all)))))
    (and (equal (first   types) "Numeric")
         (= (length rest-types) 1)
         (equal (first rest-types) "Category"))))

(defmeth mv-data-object-proto :close-dialog (table)
"Gets name of new data object and whether it is to be a table or not."
  (let* ((title (send text-item-proto :new "Name of the New Data Object:"))
         (name (send edit-text-item-proto :new 
                     (strcat "Created-From-" (send self :name))))
         (type-title (send text-item-proto :new "Make It:"))
         (type (send choice-item-proto :new (list
                     "Table Data"
                     "Multivariate Data")
                     :location (list 84 72)
                     :value 0))
         (ok (send modal-button-proto :new "OK"
                   :action #'(lambda ()
                               (cond 
                                 (table (list (send name :text) 
                                              (= 0 (send type :value))))
                                 (t (list (send name :text) nil))))))
         (cancel (send modal-button-proto :new "Cancel"))
         (dialog (send modal-dialog-proto :new
                       (cond 
                         (table
                          (list title name type-title type (list ok cancel)))
                         (t (list title name (list ok cancel))))
                        :default-button ok))
         )
    (send dialog :modal-dialog)
    ))

(defun visualize-data (&key dialog)
"Function Args: dialog
Load-on-demand function for data visualization."
;real function is in graphics file
  (load (strcat *vista-dir-name* "graphics"))
  (load (strcat *vista-dir-name* "graphobj"))
  (send current-data :visualize :dialog dialog)
  t)

(defmeth mv-data-object-proto :visualize (&key dialog) 
"Message args: dialog
Presents a visualization of the active portion of the data. When dialog is t and there are more than 5 variables, a dialog is presented to provide choice of spreadplot or guided-tour.  Otherwise a spreadplot is presented."
  (if (not (eq current-object self)) (setcd self))
  (let ((choice nil))
    (if (and dialog (> (send self :active-nvar '(all)) 5))
        (setf choice (choose-item-dialog "Choose Visualization Consisting of:"
                      '("Scatterplot, Spinning Plot, Splom & Histogram"
                        "Guided Tour Plot")))
        (setf choice 0))
    (when choice
          (cond
            ((= choice 0) (send self :visualize-continuous-data))
            ((= choice 1) (send self :visualize-tour-plot))))))

(defmeth mv-data-object-proto :visualize-continuous-data ()
  (if (<= (send self :active-nvar '(numeric ordinal)) 1)
      (error-message "Visualization not possible with less than two numeric or ordinal variables.")
      (send self :visualize-data 
            (send self 
                  :active-data-matrix '(numeric ordinal))
            (send self :active-labels)
            (send self :active-variables '(ordinal numeric))
            v-options)))

(defmeth mv-data-object-proto :visualize-tour-plot ()
  (load (strcat *vista-dir-name* "visuals"))
  (send current-data :visualize-tour-plot))

(defmeth mv-data-object-proto :visualize-data 
  (data-matrix point-labels axis-labels graphs)
"Message args: (data-matrix point-labels axis-labels graphs)
Temporary method for visualization that presents scat-mat, scat-plot,
histogram and spin-plot."
  (let* ((variables (column-list data-matrix))
         (model-p nil)
         (test (repeat 0 6)))
    (setf graphs (select graphs 0))
    (dotimes (i (length graphs)) (setf (select test (select graphs i)) 1))
    (when (= 1 (select test 0)) 
          (spreadplot variables point-labels axis-labels 
                      :scale-type 'variable 
                      :data-object t))
    (when (= 1 (select test 1))
          (visuals variables point-labels axis-labels model-p
                   :scale-type 'variable)
          (send tour-plot :show-window))
    (when (= 1 (select test 2))
          (setf  boxplot (boxplot variables :show nil)))
    (when (= 1 (select test 3))
          (dotimes (i (length variables))
                   (quantile-plot (select variables i)))
          (spreadplot (mapcar #'sort-data variables) point-labels axis-labels
                      :scale-type 'variable))
    (when (= 1 (select test 4))
;;in here goes code for probability plots
          )
    (when (= 1 (select test 5))
          (send current-data :visualize-time-series))))

(defun report-data ()
  (send current-data :report)
  t)

(defmeth mv-data-object-proto :report (&optional ok-types)
"Method Args:  (&optional ok-types)
Presents a listing of the data that includes the data object name, 
variable names and types and observation labels.  Reports only the active
ok-types data when the optional argument is used. Ok-types must be one of 
the following strings: all, numeric, ordinal, category, label. 
On Macintoshes the listing is presented in a separate window whose object 
identification is returned.  On other machines the listing is in the listener
window, T is returned."
  (if (not (eq current-object self)) (setcd self))
  (if (not ok-types) (setf ok-types '(all)))
  (let* ((data (send self :active-data ok-types))
         (n (length (send self :active-variables ok-types)))
         (m (/ (length data) n))
         (w nil)
         (dat-mat (matrix (list m n) data))
         (dat-lab (send self :active-labels))
         (both (bind-columns dat-lab dat-mat)))
    (setf w (report-header (strcat (send self :name) " Data Listing")))
    (display-string (format nil "Title: ~a" (send self :title)) w)
    (display-string
        (format nil "~%Data:  ~a~2%" (send self :name)) w)
    (display-string
        (format nil "Variable Names: ~a~%" 
                (send self :active-variables ok-types)) w)
    (display-string
        (format nil "Variable Types: ~a~2%" 
                (send self :active-types ok-types)) w)
    (display-string
          (format nil "Labeled Data Matrix: ~%") w)
    (print-matrix-to-window dat-mat w :labels dat-lab)
    w
    ))

(defun report ()
"Args: none
Generic function to ask an object for a report of itself"
  (send current-object :report))

(defmeth mv-data-object-proto :select-variables (var-name-list)
"Args: VAR-NAME-LIST
Selects the variables in VAR-NAME-LIST from the list of variables.  Displays
the selection in the variable window when it is open."
  (let* ((w (send *vista* :var-window-object))
         (var-num-list ($position var-name-list (send self :variables)))
         (states (repeat 'NORMAL (send self :nvar)))
         )
    (cond 
      (w (send w :selection var-num-list)
         (send self :var-states 
               (send w :point-state (iseq (send w :num-points)))))
      (t (setf (select states var-num-list)
                     (repeat 'SELECTED (length var-num-list)))
         (send self :var-states states)))))

(defmeth mv-data-object-proto :select-observations (obs-label-list)
"Args: OBS-LABEL-LIST
Selects the observations in OBS-LABEL-LIST from the list of observation labels.  Displays the selection in the observation window when it is open."
  (let* ((w (send *vista* :obs-window-object))
         (obs-num-list ($position obs-label-list (send self :labels)))
         (states (repeat 'NORMAL (send self :nobs)))
         )
    (cond 
      (w (send w :selection obs-num-list)
         (send self :obs-states 
               (send w :point-state (iseq (send w :num-points)))))
      (t (setf (select states obs-num-list)
                     (repeat 'SELECTED (length obs-num-list)))
         (send self :obs-states states)))))
 
(provide "dataobj2")
