;;##########################################################################
;; systmobj.lsp
;; Copyright (c) 1991-95 by Forrest W. Young
;; code for ViSta system object
;;##########################################################################

(defproto vista-system-object-proto 
  '(var-window-object obs-window-object mat-window-object 
                      cells-window-object help-window-object expert
                      guidemap guidemap-name guidemap-number
                      need-new-guidemap guidemap-slots long-menus
                      button-down delay-update delay-return instant-return
                      vis-window-status help-window-status
                      guide-window-location guide-window-size show-load-vista
                      show-guidemap hide-workmap show-toolbar show-varobs
                      show-listener show-open-data show-long-menus
                      report-window-id-list))

(defmeth vista-system-object-proto :isnew ()
  (let ((var-window (name-list '("dummy")
                               :show nil
                               :size var-obs-window-size
                               :location var-window-location 
                               :title "Vars"))
        (cancel-var-selection
              (send menu-item-proto :new "Cancel Selection" :enabled nil
                    :action #'(lambda ()
                                (send *var-window* :unselect-all-points)
                                (send (fourth (send (send *var-window* :menu) 
                                                    :items)) :enabled nil))))
        (obs-window (name-list '("dummy")
                               :show nil
                               :size var-obs-window-size
                               :location obs-window-location 
                               :title "Obs"))
        (cancel-obs-selection
              (send menu-item-proto :new "Cancel Selection" :enabled nil
                    :action #'(lambda ()
                                (send *obs-window* :unselect-all-points)
                                (send (fourth (send (send *obs-window* :menu) 
                                                    :items)) :enabled nil))))
        )
    (send self :expert nil)
    (send self :guidemap nil)
    (send self :guidemap-name "load-dat")
    (send self :vis-window-status nil)
    (send self :help-window-object nil)
    (send self :help-window-status nil)
    (send self :need-new-guidemap t)
;;var-window
    (send var-window :clear-points)
    (send var-window :new-menu "Var"
          :items '(ERASE-SELECTION FOCUS-ON-SELECTION SHOW-ALL))
    (send (send var-window :menu) :append-items cancel-var-selection)
    (defmeth var-window :close () (send self :hide-window))
    (send self :var-window-object var-window)
    (defmeth var-window :unselect-all-points ()
      (call-next-method)
      (send current-data :var-states
            (send self :point-state (iseq (send self :num-points)))))
    (defmeth var-window :do-select-click (x y m1 m2)
      (when (> (send self :num-points) 0)
            (call-next-method x y m1 m2)
            (send (fourth (send (send self :menu) :items)) :enabled t)
            (send current-data :var-states
                  (send self :point-state (iseq (send self :num-points))))))
    (defmeth var-window :erase-selection ()
      (call-next-method)
      (send (fourth (send (send self :menu) :items)) :enabled nil)
      (send current-data :var-states
            (send self :point-state (iseq (send self :num-points)))))
    (defmeth var-window :focus-on-selection ()
      (call-next-method)
      (send current-data :var-states
            (send self :point-state (iseq (send self :num-points)))))
    (defmeth var-window :show-all-points ()
      (call-next-method)
      (send current-data :var-states
            (send self :point-state (iseq (send self :num-points)))))
;;obs-window, cells-window & mat-window
    (send obs-window :clear-points)
    (send obs-window :new-menu "Obs"
          :items '(ERASE-SELECTION FOCUS-ON-SELECTION SHOW-ALL))
    (send (send obs-window :menu) :append-items cancel-obs-selection)
    (defmeth obs-window :close () (send self :hide-window))
    (send self :obs-window-object obs-window)
    (send self :mat-window-object obs-window)
    (send self :cells-window-object obs-window)
    (defmeth obs-window :unselect-all-points ()
      (call-next-method)
      (send *vista* :set-obs-mats-states obs-window))
    (defmeth obs-window :do-select-click (x y m1 m2)
      (when (> (send self :num-points) 0)
            (call-next-method x y m1 m2)
            (send (fourth (send (send self :menu) :items)) :enabled t)
            (send *vista* :set-obs-mats-states obs-window)))
    (defmeth obs-window :erase-selection ()
      (call-next-method)
      (send (fourth (send (send self :menu) :items)) :enabled nil)
      (send *vista* :set-obs-mats-states obs-window))
    (defmeth obs-window :focus-on-selection ()
      (call-next-method)
      (send *vista* :set-obs-mats-states obs-window))
    (defmeth obs-window :show-all-points ()
      (call-next-method)
      (send *vista* :set-obs-mats-states obs-window))
    ))

(defmeth vista-system-object-proto :set-obs-mats-states (w)
  (if (send current-data :matrices)
          (send current-data :mat-states 
                (send w :point-state (iseq (send w :num-points))))
          (send current-data :obs-states
                (send w :point-state (iseq (send w :num-points))))))

(defmeth vista-system-object-proto :show-labels ()
  (let ((n (send current-data :nvar))
        (w (send self :var-window-object))
        )
#+msdos(send w :clear)
#-msdos(send w :clear :draw nil)
    (send w :add-points n :draw nil :point-labels
          (mapcar #'concatenate (repeat 'string n) 
                  (send current-data :variables) 
                  (repeat " (" n) (send current-data :types) 
                  (repeat ")" n)))
    (send w :point-state (iseq n) (send current-data :var-states))
    (send w :redraw)
    ))

(defmeth vista-system-object-proto :show-obs ()
  (let* ((n (send current-data :nobs))
         (w (send self :obs-window-object))
         (m (send w :menu)))
#+msdos(send w :clear)
#-msdos(send w :clear :draw nil)
    (send w :title "Obs")
    (send w :add-points n :draw nil :point-labels (send current-data :labels))
    (send w :point-state (iseq n) (send current-data :obs-states))
    (send w :redraw)
    (send show-obs-menu-item :title "List Observations")
    (send show-obs-popup-menu-item :title "List Observations")
    (send m :remove)
    (send m :title "Obs")))         

(defmeth vista-system-object-proto :show-mats ()
  (let* ((n (send current-data :nmat))
         (w (send self :mat-window-object))
         (m (send w :menu)))
#+msdos(send w :clear)
#-msdos(send w :clear :draw nil)
    (send w :title "Mats")
    (send w :add-points n :draw nil 
            :point-labels (send current-data :matrices))
    (send w :point-state (iseq n) (send current-data :mat-states))
    (send w :redraw)
    (send show-obs-menu-item :title "List Matrices")
    (send show-obs-popup-menu-item :title "List Matrices")
    (send m :remove)
    (send m :title "Mats")))

(defmeth vista-system-object-proto :show-cells ()
  (let* ((n (send current-data :ncells))
         (w (send self :cells-window-object))
         (m (send w :menu)))
#+msdos(send w :clear)
#-msdos(send w :clear :draw nil)
    (send w :title "Cells")
    (send w :add-points n :draw nil 
            :point-labels (send current-data :labels))
    (send w :redraw)
    (send show-obs-menu-item :title "List Cells")
    (send show-obs-popup-menu-item :title "List Cells")
    (send m :remove)
    (send m :title "Cells")))

(defmeth vista-system-object-proto :store-slot-info (slot-list)
  (send self :guidemap-slots 
        (append (send self :guidemap-slots) slot-list)))

(defmeth vista-system-object-proto :store-return-info 
  (instant-return guidemap-number)
  (if (not guidemap-number)
      (send self :instant-return
            (append (send self :instant-return) (list instant-return)))
      (setf (select (send *vista* :instant-return) guidemap-number) 
             instant-return)))

(defun vista-system ()
  (send vista-system-object-proto :new))

(defmeth vista-system-object-proto :expert 
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves the expert mode of the system."
  (if set (setf (slot-value 'expert) logical))
  (slot-value 'expert))

(defmeth vista-system-object-proto :guidemap 
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves the guidemap mode of the system."
  (if set (setf (slot-value 'guidemap) logical))
  (slot-value 'guidemap))

(defmeth vista-system-object-proto :button-down 
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether a guidemap button is down."
  (if set (setf (slot-value 'button-down) logical))
  (slot-value 'button-down))

(defmeth vista-system-object-proto :delay-update 
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether the updating of the guidemap window has been delayed."
  (if set (setf (slot-value 'delay-update) logical))
  (slot-value 'delay-update))

(defmeth vista-system-object-proto :long-menus
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether the data and model menus show object names."
  (if set (setf (slot-value 'long-menus) logical))
  (slot-value 'long-menus))

(defmeth vista-system-object-proto :delay-return 
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether returning to the parent guidemap window has been delayed."
  (if set (setf (slot-value 'delay-return) logical))
  (slot-value 'delay-return))

(defmeth vista-system-object-proto :instant-return 
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether returning to the parent guidemap window is to instantly occur before map is displayed."
  (if set (setf (slot-value 'instant-return) logical))
  (slot-value 'instant-return))

(defmeth vista-system-object-proto :guidemap-name 
  (&optional (string nil set))
"Message args: (&optional string)
 Sets or retrieves the current guidemap name."
  (if set (setf (slot-value 'guidemap-name) string))
  (slot-value 'guidemap-name))

(defmeth vista-system-object-proto :guidemap-number 
  (&optional (number nil set))
"Message args: (&optional number)
 Sets or retrieves the current guidemap load sequence number."
  (if set (setf (slot-value 'guidemap-number) number))
  (slot-value 'guidemap-number))

(defmeth vista-system-object-proto :need-new-guidemap 
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether a new guidemap needs to be loaded."
  (if set (setf (slot-value 'need-new-guidemap) logical))
  (slot-value 'need-new-guidemap))

(defmeth vista-system-object-proto :guidemap-slots 
  (&optional (list nil set))
"Message args: (&optional list)
 Sets or retrieves list of guidemap slot values."
  (if set (setf (slot-value 'guidemap-slots) list))
  (slot-value 'guidemap-slots))

(defmeth vista-system-object-proto :show-guidemap
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether to show the guidemap at startup."
  (if set (setf (slot-value 'show-guidemap) logical))
  (slot-value 'show-guidemap))

(defmeth vista-system-object-proto :hide-workmap
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether to hide the workmap at startup."
  (if set (setf (slot-value 'hide-workmap) logical))
  (slot-value 'hide-workmap))

(defmeth vista-system-object-proto :show-toolbar
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether to show the toolbar at startup."
  (if set (setf (slot-value 'show-toolbar) logical))
  (slot-value 'show-toolbar))

(defmeth vista-system-object-proto :show-varobs
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether to show the variable and observation windows at startup."
  (if set (setf (slot-value 'show-varobs) logical))
  (slot-value 'show-varobs))

(defmeth vista-system-object-proto :show-long-menus
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether to show long-menus at startup."
  (if set (setf (slot-value 'show-long-menus) logical))
  (slot-value 'show-long-menus))

(defmeth vista-system-object-proto :show-listener
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether to show the listener at startup."
  (if set (setf (slot-value 'show-listener) logical))
  (slot-value 'show-listener))

(defmeth vista-system-object-proto :show-load-vista
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether to show the load vista dialog at startup."
  (if set (setf (slot-value 'show-load-vista) logical))
  (slot-value 'show-load-vista))

(defmeth vista-system-object-proto :show-open-data
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves whether to show the open-data-dialog at startup."
  (if set (setf (slot-value 'show-open-data) logical))
  (slot-value 'show-open-data))

(defmeth vista-system-object-proto :report-window-id-list
  (&optional (list nil set))
"Message args: (&optional list)
 Sets or retrieves the list of report-window object ids."
  (if set (setf (slot-value 'report-window-id-list) list))
  (slot-value 'report-window-id-list))

(defmeth vista-system-object-proto :show-bug-list ()
  (let ((help-file-name 
#+macintosh(strcat *vista-dir-name* "help:"  "bug-list.hlp")
#+msdos    (strcat *vista-dir-name* "help\\" "bug-list.hlp")
#+X11      (strcat *vista-dir-name* "help/"  "bug-list.hlp")
         ))
    #+macintosh (file-to-window help-file-name "Bug List" 
                                (send self :help-window-object))
    #-macintosh (file-to-stream help-file-name "Bug List")))

(defmeth vista-system-object-proto :preferences ()
  (let* ((choices nil)
         (prefs nil)
         (file nil)
         (oldbreak *breakenable*))
    (when (send self :hide-workmap)    (setf choices (append choices '(0))))
    (when (send self :show-guidemap)   (setf choices (append choices '(1))))
    (when (send self :show-toolbar)    (setf choices (append choices '(2))))
    (when (send self :show-open-data)  (setf choices (append choices '(3))))
    (when (send self :show-long-menus) (setf choices (append choices '(4))))
    (when (send self :show-varobs)     (setf choices (append choices '(5))))
    (when (send self :show-load-vista) (setf choices (append choices '(6))))
    (when (send self :show-listener)   (setf choices (append choices '(7))))
    (setf prefs (choose-subset-dialog "Select Startup Preferences:"
                   '("Hide WorkMap" "Show GuideMap" "Show Toolbar" 
                     "Show Open Data Dialog" "Show Long Menus"
                     "Show Var and Obs Lists" "Show Load Vista? Dialog"
#+macintosh          "Show XLISP-STAT"
                           ) :initial choices))
    (when 
     (> (length prefs) 0)
     (setf file 
           (open (strcat *ViSta-dir-name* "prefs.lsp") :direction :output))
     (setf file2
           (open (strcat *ViSta-dir-name* "prefload.lsp") :direction :output))
     (setq *breakenable* nil)
     (unwind-protect
      (cond ((member 0 (first prefs)) 
             (princ "(hide-workmap)"    file)
             (princ "(send *vista* :hide-workmap t)" file)
             (send self :hide-workmap t))
        (t (send self :hide-workmap nil)))
      (cond ((member 1 (first prefs)) 
             (princ "(show-guidemap)"   file)
             (princ "(send *vista* :show-guidemap t)" file)
             (send self :show-guidemap t))
        (t (send self :show-guidemap nil)))
      (cond ((member 2 (first prefs)) 
             (princ "(show-toolbar)"    file)
             (princ "(send *vista* :show-toolbar t)" file)
             (send self :show-toolbar t))
        (t (send self :show-toolbar nil)))
      (cond ((member 4 (first prefs)) 
             (princ "(make-long-menus)" file)
             (princ "(send *vista* :show-long-menus t)" file)
             (send self :show-long-menus t))
        (t (send self :show-long-menus nil)))
      (cond ((member 5 (first prefs)) 
             (princ "(show-varobs)" file)
             (princ "(send *vista* :show-varobs t)" file)
             (send self :show-varobs t))
        (t (send self :show-varobs nil)))
      (cond ((member 6 (first prefs)) 
             (princ "(setf ask-load-vista t)" file2)
             (princ "(setf memory-message t)" file2)
             (setf ask-load-vista t)
             (send self :show-load-vista t))
        (t 
         (princ "(setf ask-load-vista nil)" file2)
         (princ "(setf memory-message t)" file2)
         (setf ask-load-vista nil)
         (send self :show-load-vista nil)))
      (cond ((member 7 (first prefs)) 
             (princ "(show-xlisp-stat)" file)
             (princ "(send *vista* :show-listener t)" file)
             (send self :show-listener t))
        (t (send self :show-listener nil)))
      (cond ((member 3 (first prefs)) 
             (princ "(open-data)"       file)
             (princ "(send *vista* :show-open-data t)" file)
             (send self :show-open-data t))
        (t (send self :show-open-data nil)))
      )
     (setq *breakenable* oldbreak)
     (close file)
     (close file2)
     )
    ))

(defmeth vista-system-object-proto :vis-window-status
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves the visualization windows status: t=open, nil=closed." 
  (if set (setf (slot-value 'vis-window-status ) logical))
  (slot-value 'vis-window-status ))

(defmeth vista-system-object-proto :help-window-status
  (&optional (logical nil set))
"Message args: (&optional logical)
 Sets or retrieves the help window status: t=open, nil=closed." 
  (if set (setf (slot-value 'help-window-status ) logical))
  (slot-value 'help-window-status ))

(defmeth vista-system-object-proto :var-window-object 
  (&optional (objid nil set))
"Message args: (&optional objid)
 Sets or retrieves the object identification of the variable window."
  (if set (setf (slot-value 'var-window-object) objid))
  (slot-value 'var-window-object))

(defmeth vista-system-object-proto :obs-window-object 
  (&optional (objid nil set))
"Message args: (&optional objid)
 Sets or retrieves the object identification of the observation window."
  (if set (setf (slot-value 'obs-window-object) objid))
  (slot-value 'obs-window-object))

(defmeth vista-system-object-proto :mat-window-object 
  (&optional (objid nil set))
"Message args: (&optional objid)
 Sets or retrieves the object identification of the matrix window."
  (if set (setf (slot-value 'mat-window-object) objid))
  (slot-value 'mat-window-object))

(defmeth vista-system-object-proto :cells-window-object 
  (&optional (objid nil set))
"Message args: (&optional objid)
 Sets or retrieves the object identification of the table cells window."
  (if set (setf (slot-value 'cells-window-object) objid))
  (slot-value 'cells-window-object))

(defmeth vista-system-object-proto :help-window-object 
  (&optional (objid nil set))
"Message args: (&optional objid)
 Sets or retrieves the object identification of the help window."
  (if set (setf (slot-value 'help-window-object) objid))
  (slot-value 'help-window-object))

(defmeth vista-system-object-proto :guide-window-size 
  (&optional (size-list nil set))
"Message args: (&optional size-list)
 Sets or retrieves the size of the guidemap window."
  (if set (setf (slot-value 'guide-window-size) size-list))
  (slot-value 'guide-window-size))

(defmeth vista-system-object-proto :guide-window-location 
  (&optional (location-list nil set))
"Message args: (&optional location-list)
 Sets or retrieves the locationof the guidemap window."
  (if set (setf (slot-value 'guide-window-location) location-list))
  (slot-value 'guide-window-location))

(defmeth vista-system-object-proto :list-vars ()
"(alias for list-variables)"
  (send (send self :var-window-object) :show-window))

(defmeth vista-system-object-proto :list-var ()
"(alias for list-variables)"
  (send (send self :var-window-object) :show-window))

(defmeth vista-system-object-proto :list-variables ()
  (send (send self :var-window-object) :show-window))

(defmeth vista-system-object-proto :list-obs ()
"(alias for list-observations)" 
  (send (send self :obs-window-object) :show-window))

(defmeth vista-system-object-proto :list-observations () 
  (send (send self :obs-window-object) :show-window))

(defmeth vista-system-object-proto :list-mat ()
"(alias for list-matrices)" 
  (send (send self :mat-window-object) :show-window))

(defmeth vista-system-object-proto :list-mats ()
"(alias for list-matrices)" 
  (send (send self :mat-window-object) :show-window))

(defmeth vista-system-object-proto :list-matrices () 
  (send (send self :mat-window-object) :show-window))

(defmeth vista-system-object-proto :list-cells () 
  (send (send self :cells-window-object) :show-window))

(defmeth vista-system-object-proto :save-preferences ()
  (let ((f (open (strcat *vista-dir-name* "prefs2.lsp") :direction :output))
        (oldbreak *breakenable*))
    (break)
    (setq *breakenable* nil)
    (unwind-protect
     (if (send *workmap* :toolbar)
         (print `(send *workmap* :show-toolbar) f)
         (print `(send *workmap* :hide-toolbar) f)))
    (setq *breakenable* oldbreak)
    (close f)
    f))

(provide "systmobj")
