# $Id: textundo.tcl,v 1.1 1998/06/18 08:13:29 ael Exp ael $

# ael:  I found this textundo stuff on the net, and think it's fine. \
	I want to point out the strategy as I see it: it's simply a means of taking care of insertions and deletions (see checkpoint): nothing else figures in the history. \
	For the moment I'll hold off, but I'd ultimately like to make this into an infinite undo, a la emacs. \
	To do that, I'd need to change the structure: for example, you do a few insertions. After these insertions, you decide that you want to undo a few of them. You could either do what textundo currently does, which is "back out" actions from the stack, or you can add to the stack the actions that undo the previous actions. I'd like to do the latter. \
	Then when you get to the "end" of the (i.e. you've undone every action) you report that fact. If the user wishes to continue undoing however, the sequence of actions repeats from the beginning. \
	June 24, 1998.

proc textUndoer__textUndoer {id widget {depth 2147483647}} {
    global textUndoer

    if {[string compare [winfo class $widget] Text]!=0} {
        error "textUndoer error: widget $widget is not a text widget"
    }
    set textUndoer($id,widget) $widget
    set textUndoer($id,originalBindingTags) [bindtags $widget]
    bindtags $widget \
	    [concat $textUndoer($id,originalBindingTags) UndoBindings($id)]

    bind UndoBindings($id) <Control-u> "textUndoer__undo $id"

    # self destruct automatically when text widget is gone
    bind UndoBindings($id) <Destroy> "delete textUndoer $id"

    # ael:  if I may interrupt here: this is really cool! We're renaming the \
	    text widget to something else, appropriating all the commands \
	    destined for that widget and then having our way with them. This \
	    reminds me of a virus!
    # lea

    # rename widget command
    rename $widget \
	    [set textUndoer($id,originalCommand) textUndoer__original$widget]

    # and intercept modifying instructions before calling original command
    proc $widget {args} \
	    "textUndoer__checkpoint $id \$args; \
	    eval $textUndoer($id,originalCommand) \$args"

    set textUndoer($id,commandStack) [new lifo $depth]
    set textUndoer($id,cursorStack) [new lifo $depth]
}

proc textUndoer__~textUndoer {id} {
    global textUndoer

    bindtags $textUndoer($id,widget) $textUndoer($id,originalBindingTags)
    rename $textUndoer($id,widget) ""
    rename $textUndoer($id,originalCommand) $textUndoer($id,widget)
    delete lifo $textUndoer($id,commandStack)
    delete lifo $textUndoer($id,cursorStack)
}

proc textUndoer__checkpoint {id arguments} {
    global textUndoer

    # do nothing if non modifying command
    if {[string compare [lindex $arguments 0] insert]==0} {
        textUndoer__processInsertion $id [lrange $arguments 1 end]
    }
    if {[string compare [lindex $arguments 0] delete]==0} {
        textUndoer__processDeletion $id [lrange $arguments 1 end]
    }
}

proc textUndoer__processInsertion {id arguments} {
    global textUndoer

    set number [llength $arguments]
    set length 0
    # calculate total insertion length while skipping tags in arguments
    for {set index 1} {$index<$number} {incr index 2} {
        incr length [string length [lindex $arguments $index]]
    }
    if {$length>0} {
        set index [$textUndoer($id,originalCommand) index [lindex $arguments 0]]
        lifo__push $textUndoer($id,commandStack) \
		"delete $index $index+${length}c"
        lifo__push $textUndoer($id,cursorStack) \
		[$textUndoer($id,originalCommand) index insert]
    }
}

proc textUndoer__processDeletion {id arguments} {
    global textUndoer

    set command $textUndoer($id,originalCommand)
    lifo__push $textUndoer($id,cursorStack) [$command index insert]
    set start [$command index [lindex $arguments 0]]
    if {[llength $arguments]>1} {
        lifo__push $textUndoer($id,commandStack) \
		"insert $start {[$command get $start [lindex $arguments 1]]}"
    } else {
        lifo__push $textUndoer($id,commandStack) \
		"insert $start {[$command get $start]}"
    }
}

proc textUndoer__undo {id} {
    global textUndoer

    if {[catch {set cursor [lifo__pop $textUndoer($id,cursorStack)]}]} {
        return
    }
    eval $textUndoer($id,originalCommand) \
	    [lifo__pop $textUndoer($id,commandStack)]
    # now restore cursor position
    $textUndoer($id,originalCommand) mark set insert $cursor
    # make sure insertion point can be seen
    $textUndoer($id,originalCommand) see insert
}

proc textUndoer__reset {id} {
    global textUndoer

    lifo__empty $textUndoer($id,commandStack)
    lifo__empty $textUndoer($id,cursorStack)
}
