proc tk_Ask_ text {
   global tk_Ready
   catch {destroy .tk_askwin}
   toplevel .tk_askwin -class Dialog
   wm title .tk_askwin $text
   wm iconname .tk_askwin $text

   frame .tk_askwin.top -relief raised -bd 1
   pack .tk_askwin.top -side top -expand 1 -fill both
   frame .tk_askwin.bot -relief raised -bd 1
   pack .tk_askwin.bot -side bottom
   entry .tk_askwin.answer -width 20 -textvariable tk_Answer
   set tk_Answer ""
   bind .tk_askwin.answer <Return> {
      destroy .tk_askwin
      set tk_Ready 1
   }
   pack .tk_askwin.answer -in .tk_askwin.top -side top -expand 1 -fill both
   button .tk_askwin.cancel -text "Cancel" -command {
      destroy .tk_askwin
      set tk_Answer ""
      set tk_Ready 1
   }
   button .tk_askwin.ok -text "OK" -command {
      destroy .tk_askwin
      set tk_Ready 1
   }
   pack .tk_askwin.cancel .tk_askwin.ok -in .tk_askwin.bot -side left
   focus .tk_askwin.answer
}

proc tk_Ask text {
   global tk_Ready tk_Answer
   set tk_Ready 0
   tk_Ask_ $text
   while {!$tk_Ready} {
      after 50
      update
   }
   return $tk_Answer
}

bind Text <BackSpace> {
    if {[%W tag nextrange sel 1.0 end] != ""} {
	%W delete sel.first sel.last
    } elseif [%W compare insert != 1.0] {
	%W delete insert-1c
	%W see insert
    }
}

set control_k_spot ""
bind Text <Control-k> {
    # I define the global variable control_k_spot to keep track of a \
	    succession of control-ks. If they occur together I should \
	    append the yankbuffer to the existing yankbuffer, rather \
	    than replace it.
    global control_k_spot tk_Emacs_markSet
    set tk_Emacs_markSet 1
    set tmp [%W index insert]
    set ind [%W index "insert lineend"]
    if {$tmp == $ind} {
	# then we're at the end of a line, so I'll swollow up the \n:
	set ind [%W index "insert lineend + 1c"]
    }
    # here's where we either append or replace, according to control_k_spot:
    if {$tmp == $control_k_spot} {
	set tk_YankBuffer \
		[join \
		[list $tk_YankBuffer [%W get insert $ind]] \
		""]
    } else {
	set tk_YankBuffer [%W get insert $ind]
    }
    %W delete insert $ind
    set control_k_spot $tmp
    %W mark set emacs_mark insert
    %W see insert
}
# These control-l features don't do what I really want, which is to center the \
	window at the line containing the insert
bind Text <Control-l> {
    %W yview insert
    set tmp [%W yview]
    set diff [expr [expr [lindex $tmp 1] - [lindex $tmp 0]] / 2.0]
    %W yview moveto [expr [lindex $tmp 0] - $diff]
    %W see insert
}
bind Text <Control-L> {
    %W yview scroll -5 units
}
bind Text <Control-w> {
    global tk_Emacs_markSet
    # if there is no mark set, return:
    if {$tk_Emacs_markSet == 1} {
	if {[%W get insert emacs_mark] == ""} {
	    set tk_YankBuffer [%W get emacs_mark insert]
	    %W delete emacs_mark insert
	} else {
	    set tk_YankBuffer [%W get insert emacs_mark]
	    %W delete insert emacs_mark
	}
	%W mark set emacs_mark insert
    }
}
bind Text <Control-y> {
    global tk_Emacs_markSet
    set tk_Emacs_markSet 1
    set tmp [%W index insert]
    if {[info exists tk_YankBuffer]} {
	%W insert insert $tk_YankBuffer
    }
    %W see insert 
    %W mark set emacs_mark $tmp
    set tmp [%W index insert]
}

set tk_Emacs_markSet 0
bind Text <Control-space> {
    global tk_Emacs_markSet
    set tk_Emacs_markSet 1
    %W mark set emacs_mark insert
    %W mark gravity emacs_mark left
}

bind Text <Control-n> {
    # modify the default Control-n so that, if we're at the end of a file, \
	    we insert a fresh line.
    if !$tk_strictMotif {
	if [%W compare {insert lineend} == {end-1c}] {
	    %W insert {insert lineend} \n
	}
	tkTextSetCursor %W [tkTextUpDownLine %W 1]
    }
}
bind Text <Control-v> {
    # modified the control-v binding to actually set the insert at the top of \
	    the next page.
    if !$tk_strictMotif {
	%W mark set insert [tkTextScrollPages %W 1]
    }
}


# explicitly make sure that these two are modifiers in text windows:

bind Text <Control-x> {#nothin}
bind Text <Control-c> {#nothin}

# standard emacs search controls:
bind Text <Control-s> {
    PerformSearch %W 1 $searchString
}
bind Text <Control-r> {
    PerformSearch %W 0 $searchString
}
bind Text <Control-x><Control-s> {
    if {$saveFile == ""} {
	saveFile
    } else {
	set f [open $saveFile w]
	puts $f [%W get 1.0 end]
	close $f
    }
}
bind Text <Control-x><Control-x> {
    set tmp [%W index emacs_mark]
    %W mark set emacs_mark insert
    %W mark set insert $tmp
    %W see insert
}
bind Text <Control-x><Control-c> {
    saveFile
    destroy .
}
# can get the names of other weird keys from the %K in a bind to Any-Key
bind Text <Control-question> {
    puts "control-question is not yet bound."
}
bind Text <Control-underscore> {
    puts "control-_ is not yet bound."
}
bind Text <Tab> {+
    return 0
}
bind Text <Control-m> {
    global inputHeight outputHeight
    incr inputHeight
    if {$inputHeight > 31} {set inputHeight 31}
    set outputHeight [expr 34-$inputHeight]
    .input configure -height $inputHeight
    .output configure -height $outputHeight
}
bind Text <Control-M> {
    global inputHeight outputHeight
    incr inputHeight -1
    if {$inputHeight < 3} {set inputHeight 3}
    set outputHeight [expr 34-$inputHeight]
    .input configure -height $inputHeight
    .output configure -height $outputHeight
}
# take care of tabbing:
foreach key {<Tab> <Control-i>} {
    bind Text $key {
	# tab now does nothing, unless it's at the beginning of a line.
	%W mark set insert "insert linestart"
	parenthesisIndent [howFarInToIndent]
	focus %W
	break
    }
}
foreach key {<Control-q><Tab> <Control-q><Control-i>} {
    bind Text $key {
	tkTextInsert %W \t
	focus %W
	break
    }
}

# allow easy access to the beginning and end of input workspace:
bind Text <Alt-greater> {
    if !$tk_strictMotif {
	tkTextSetCursor %W end-1c
    }
}
bind Text <Alt-less> {
    if !$tk_strictMotif {
	tkTextSetCursor %W 1.0
    }
}

# if return is active, we want to add the following command:
bind Text <Return> {
    if $returnActive {
	removePromptAndExecute \
		[%W get "insert linestart" "insert lineend"]
    }
    tkTextInsert %W \n
    if {$matchParens} {parenthesisIndent [howFarInToIndent]}
}
#
# Make control-Return active in any case:
#
bind Text <Control-Return> {
    set linestart [%W index "insert linestart"]
    set lineend [%W index "insert lineend"]
    removePromptAndExecute [%W get $linestart $lineend]
}
bind Text <Double-3> {
    tkTextButton1 %W %x %y
    set val [%W index @%x,%y]
    %W tag remove sel 0.0 end
    set linestart [%W index "$val linestart"]
    set lineend [%W index "$val lineend"]
    removePromptAndExecute [%W get $linestart $lineend]
}

bind Text <Control-x><Control-r> {
    global tk_Emacs_markSet
    # if there is no mark set, return:
    if {$tk_Emacs_markSet == 1} {
	if {[%W get insert emacs_mark] == ""} {
	    set tmp [%W get emacs_mark insert]
	} else {
	    set tmp [%W get insert emacs_mark]
	}
	saveStringtoFile $tmp .asdf overwrite
	executeCommandsInFile .asdf
    }
}
bind Text <Control-g> {
    break
}
bind Text <Control-x><Control-g> {
    set l [tk_Ask "LineNumber ?"]
    if {$l != ""} {%W mark set insert $l.0}
    focus %W
    %W see insert
}

bind Text <Control-c><Control-e> {
    set tmp [CatchWholeContainingCommand]
    saveStringtoFile $tmp .asdf overwrite
    executeCommandsInFile .asdf
}
bind Text <Control-c><Control-r> {
    # insert a comment:
    %W insert insert "#|
    
|#
    "
    tkTextSetCursor %W [tkTextUpDownLine %W -2]
}
