#!/bin/sh
#
# the next line restarts using wish \
exec wish "$0" "$@"
#
##############################################################################
#
# ael: xls --  a GUI to xlispstat/RGM (Ripon Good Math) under UNIX \
	that won't make students wanna womit. 

# Written while on a Fulbright at the \
	Institute de Mathematiques et de Sciences Physiques, 1998.

# This script has been tested under \
	Tcl/Tk/wish 8.0, and \
	Tcl 7.6 and Tk/wish 4.2.
#
##############################################################################
#
#				initializations
#
# Note that XLSPATH has to be set in the environment: all other necessary \
	filenames are derived on its definition. It is the name of the \
	directory in which the file xls.tcl sits, and is usually set in \
	another file (e.g. xls) which then executes this one.
# By default, the Ripon Good Math (RGM) directory is the parent of the xlsdir \
	directory, but doesn't have to be. If it is empty, certain parameters \
	will change, to provide an interface to the vanilla xlispstat:
set xlsdir $env(XLSPATH)
set RGM $env(RGM)
set XLS_FILENAME $env(XLS_FILENAME)

# if RGM is not set, then we revert to the old fashioned xlispstat mode:
if {$RGM==""} {
    wm title . "Xlispstat meets Tcl/Tk"
    set disableState disabled
    set searchString "plot-points"
    set helpCommand "plot-points"
    set interactiveCommand "(plot-function #'cos -6.28 6.28)"
    set firstLine "(plot-points (normal-rand 10) (normal-rand 10))"
    set secondLine "(plot-function #'tan -1 1)"
    set hotfile $xlsdir/xls.hot
} else {
    wm title . "Xlispstat and Ripon Good Math (RGM) meet Tcl/Tk"
    set disableState normal
    set searchString "plot"
    set helpCommand "plot"
    set interactiveCommand "(plot cos -pi pi)"
    set firstLine "(plot sin -pi pi)"
    set secondLine "(plot tan)"
    set hotfile $xlsdir/rgm.hot
}
#
#	Some window geometry, and size of the text windows:
#
wm geometry . +100+0
set inputHeight 17
set outputHeight 17
#
#	Set the font to something fixed and "soft":
#
# set font *-Courier-Bold-R-Normal-*-120-*
set font *-Courier-Bold-R-Normal-*-20*
#
# open the xlispstat channel:
#
set xls [open "| xlispstat" r+]
#
# turn blocking off (0): this is necessary to use gets and a command pipe, \
	evidently: help me, someone!
#
fconfigure $xls -blocking 0
#
#	Set the hotlist to user default commands in file $hotfile
#
set HLinitialized ""
if [file readable $hotfile] {
    set f [open $hotfile]
    while {[gets $f line] >= 0} {
	set HLinitialized [concat $HLinitialized [format "%s" $line]]
    }
    close $f
}
#
# Set the other global variables that figure in the intial state of the \
	various entry boxes/labels.
#
set saveFile ""
set messageAlert ""
# patience is for the parenthesis matching routine. The problem is that this \
	can be very time consuming if it goes too deep (like if, at the end \
	of a long file you type an unmatched ")"). I'll default it to 50:
set patienceLevel 50
# return is active at startup if 1, inactive if 0:
set returnActive 0
# matching parentheses can also be turned off.
set matchParens 1
#
#######################################################################
#
#	Files sourced for the textundo feature of the input window:
#
########
#
# ael: from textundo-1.0
#
source $xlsdir/new.tcl
source $xlsdir/lifo.tcl
source $xlsdir/textundo.tcl
#
# Note: initialization of variable id is necessary later on in this file, \
	once the input window .input is defined.
#
########
#
# ael - my own utilities to source, stolen/butchered from others of course:
#
source $xlsdir/xlsutils.tcl
#
##############################################################################
#
#				Window Frames:
#
##############################################################################
#
#				Menu frame:
#
frame .menu -relief raised -borderwidth 2
pack .menu -side top -fill x

#######################################################################
#
#  I'm going to add some buttons and entry to the menu bar: why not?
#
button .menu.searchbutton -text "Search" \
	-command "SearchPanel \$searchString .input"
button .menu.abort -text "Abort" -command {exec kill -2 [pid $xls]}
button .menu.hotlist -text "Hot List" -command "hotList"
entry .menu.searchentry -font $font -width 20 -textvariable searchString
bind .menu.searchentry <Return> {.menu.searchbutton invoke}
#
###############
#
menubutton .menu.file -text File -menu .menu.file.m -underline 0
menu .menu.file.m
.menu.file.m add command -label "Save Input to File" -underline 0 \
	-command {saveFile}
.menu.file.m add separator
.menu.file.m add command -label "Import File to Input Workspace" -underline 0 \
	-command {importFile .input}
.menu.file.m add command -label "Append File to Input Workspace" -underline 0 \
	-command {importFile .input append}
.menu.file.m add separator
.menu.file.m add command -label "Load File into xlispstat" -underline 0 \
	-command {loadFile "."}
.menu.file.m add command -label "Load Example into xlispstat" -underline 5 \
	-state $disableState -command {loadRGM "examples"}
.menu.file.m add command -label "Load Module into xlispstat" -underline 5 \
	-state $disableState -command {loadModules}
.menu.file.m add command -label "Change font size" -underline 7 \
	-state $disableState -command {loadFonts}
.menu.file.m add command -label "Change window size" -underline 7 \
	-state $disableState -command {changeSize}
.menu.file.m add separator
.menu.file.m add command -label "Quit" -underline 0 -command "destroy ."

menubutton .menu.tutorials -text Tutorials -underline 0 \
	-state $disableState -menu .menu.tutorials.m 
menu .menu.tutorials.m
.menu.tutorials.m add command -label "Tutorial Directory" \
	-underline 0 -command {loadRGM "tutorial"}
.menu.tutorials.m add command -label "Choose Tutorial" \
	-underline 0 -command {my-exec "(tutorials)"}
.menu.tutorials.m add separator
.menu.tutorials.m add command -label "Luke's Book" \
	-underline 0 -command {my-exec "(book)"}

menubutton .menu.help -text Help -menu .menu.help.m -underline 0
menu .menu.help.m
.menu.help.m add command -label "Graphical Help" -underline 0 \
	-state $disableState -command {my-exec "(hulp)"}
.menu.help.m add command -label "Tcl/Tk Help" -underline 0 \
	-command {exec tclhelp &}

menubutton .menu.edit -text Edit -menu .menu.edit.m -underline 0
menu .menu.edit.m
.menu.edit.m add separator
.menu.edit.m add command -label "Copy" -accelerator "Alt+c" \
	-underline 0 -command {tk_textCopy .input}
.menu.edit.m add command -label "Paste" -accelerator "Alt+v" \
	-underline 0 -command {tk_textPaste .input}
.menu.edit.m add command -label "Cut" -accelerator "Alt+x" \
	-underline 2 -command {tk_textCut .input}
.menu.edit.m add separator
.menu.edit.m add command -label "Undo" -accelerator "Ctrl+u" \
	-underline 0 -command {global id; textUndoer__undo $id}
.menu.edit.m add command -label "Forward Search" -accelerator "Ctrl+s" \
	-underline 0 \
	-command {eval [bind all <Control-s>]}
.menu.edit.m add command -label "Backward Search" -accelerator "Ctrl+r" \
	-underline 0 \
	-command {eval [bind all <Control-r>]}
.menu.edit.m add command -label "Neon Search:" -underline 0 \
	-command {genericHighlightSearch $searchString {.input .output}}
.menu.edit.m add command -label "Go to line number..." -accelerator "Ctrl+g" \
	-underline 0 \
	-command {
    set l [tk_Ask "LineNumber ?"]
    if {$l != ""} {.input mark set insert $l.0}
    focus .input
    .input see insert
}
.menu.edit.m add separator
.menu.edit.m add command -label "Hotlist" -underline 0 \
	-command {hotList}
.menu.edit.m add command -label "Add selected text to hotlist" -underline 0 \
	-command {
    if {[selection own] != ""} {
	global HLinitialized
	set HLinitialized [linsert $HLinitialized 0 \
		[stripStringtoCommand [selection get]]]
	hotList
    }
}
.menu.edit.m add separator
.menu.edit.m add command -label "Clear Workspace" -underline 6 \
	-command {.input delete 1.0 end; focus .input}
.menu.edit.m add command -label "Abort Operation" -underline 6 \
	-command {.menu.abort invoke}
.menu.edit.m add checkbutton -label "Return active" -variable returnActive
.menu.edit.m add checkbutton -label "Parenthesis Matching" -variable matchParens

menubutton .menu.about -text About -menu .menu.about.m -underline 0
menu .menu.about.m
.menu.about.m add command -label "Active Return" -underline 0 \
	-command {aboutMessage "Active Return"}
.menu.about.m add command -label "Bugs" -underline 0 \
	-command "aboutMessage bugs"
.menu.about.m add command -label "Hot Command" -underline 4 \
	-command {aboutMessage "Hot Command"}
.menu.about.m add command -label "Emacs" -underline 0 \
	-command "aboutMessage emacs"
.menu.about.m add command -label "GUI" -underline 0 \
	-command "aboutMessage GUI"
.menu.about.m add command -label "Hot List" -underline 0 \
	-command "aboutMessage Hotlist"
.menu.about.m add command -label "Linked Commands" -underline 0 \
	-command "aboutMessage links"
.menu.about.m add command -label "Misc." -underline 0 \
	-command "aboutMessage misc"
.menu.about.m add command -label "RGM (Ripon Good Math)" -underline 0 \
	-command {aboutMessage "Ripon Good Math"}
.menu.about.m add command -label "reScaling windows" -underline 2\
	-command "aboutMessage rescaling"
.menu.about.m add command -label "To do" -underline 0 \
	-command "aboutMessage todo"

pack .menu.file .menu.edit .menu.about .menu.searchbutton -side left
pack .menu.help .menu.tutorials .menu.abort .menu.hotlist -side right
pack .menu.searchentry -expand yes -fill x
#
#############################################################################
#
#
#########			Text input window
#
frame .inp
pack .inp -side top -fill x

text .input -relief sunken -bd 2 -yscrollcommand ".scrod set" -setgrid 1 \
	-width 82 -height $inputHeight -font $font
scrollbar .scrod -command ".input yview"

pack .scrod -in .inp -side right -fill y
pack .input -in .inp -expand yes -fill both
#
#	Fill up the input space with some initial text and a few tags:
#
if {$XLS_FILENAME == ""} {
    .input insert 0.0 {
;;				}
.input insert end {Input Workspace} wsp
.input insert end "

;; 	This is the input workspace. You can type any xlispstat commands/code
;; here. There are at least three different ways to execute a command:
;;	1. hitting Control and Return simultaneously on a line executes that line;
;;	2. double clicking the right mouse button on a line does the same; and
;;	3. selecting the text and punching the \"Execute Selected\" button.
;; Try any of those methods with the following line:
$firstLine
#|
Alternatively, clicking on \"Execute Workspace\" will plot the sin function as
well as the tangent function (below); the rest of the text in this input 
workspace is commentary.
|#
$secondLine"

    if {$RGM != ""} {
	.input insert end "
#|
             Some Demonstration links (left mouse button to activate):\n\n"
    #
    #	hypertext facility provides easy access to some "fun" demos:
    #
    .input insert end "    1. A density demonstration: illustrates various density and cumulative plots for the standard distributions of statistics, based on some work of Jan de Leeuw." d1
    .input insert end "\n\n"
    .input insert end "    2. A pretty butterfly plot (or is it a baboon behind a butterfly?), provided by Kjetil Halvorsen." d2
    .input insert end "\n\n"
    .input insert end "    3. A TI-30 calculator, by ?" d3
    .input insert end "\n|#\n"
    }
} else {
    global saveFile
    set saveFile $XLS_FILENAME
    if [file exists $XLS_FILENAME] {
	set f [open $XLS_FILENAME]
	while {![eof $f]} {.input insert end [read $f 1000]}
	close $f
	.input mark set insert end
	.input see insert
    } else {
	exec touch $XLS_FILENAME
    }
}
#
# configure the appearance of the tagged hypertext:
#
if {[winfo depth .] > 1} {
    set bold "-background #43ce80 -relief raised -borderwidth 1"
    set normal "-background {} -relief flat"
} else {
    set bold "-foreground white -background black"
    set normal "-foreground {} -background {}"
}
#
#		Create bindings for the tags:
#
foreach tag {d1 d2 d3 wsp} {
    .input tag bind $tag <Any-Enter> ".input tag configure $tag $bold"
    .input tag bind $tag <Any-Leave> ".input tag configure $tag $normal"
}
.input tag bind d1 <1> {xlsFunc "load-calc" \"densdemo\"}
.input tag bind d2 <1> {xlsFunc "load-calc" \"buttafly\"}
.input tag bind d3 <1> {xlsFunc "load-calc" \"calculat\"}
# the last clears the input workspace:
.input tag bind wsp <1> {
    %W delete 1.0 end
    focus .input
}
.input mark set insert 0.0
#
#		Create bindings for the input window input events:
#
bind .input ) {
    ParenthesisCheck
    # now shift over the requisite number of indentations \
	    if at the start of a line:
    if {$matchParens && [%W compare insert == {insert linestart}]} {
	parenthesisIndent [howFarInToIndent]
    }
}
#
# This is kind of fun: I can turn lisp expressions between parentheses into \
	links, which can invoke the surrounding command. \
	The counter hyperCommands is used to make them unique.
# I indicate such a command by binding to a left paren followed by any other key:
#
bind .input (<Any-Key> "updateTag"
#
# Initialize the undoer id, and create an undoIt proc:
#
set id [new textUndoer .input]
#
#############################################################################
#
#
#########			Text output window
#
frame .outp
pack .outp -side top -fill x
text .output -relief sunken -bd 2 -yscrollcommand ".scrodout set" -setgrid 1 \
	-height $outputHeight -font $font
scrollbar .scrodout -command ".output yview"
pack .scrodout -in .outp -side right -fill y
pack .output -in .outp -expand yes -fill both
.output insert 0.0 {
				Output Workspace

}
#
#########			Help frame:
#
# Create the Help entry window at the bottom of the window, and button:
#
frame .bottom -relief raised -borderwidth 2
pack .bottom -side bottom -fill x

if {$RGM == ""} {
    button .bottom.help -text Help: \
	    -command {my-exec [xlsFuncQuoted "help" $helpCommand]}
    button .bottom.apropos -text Apropos \
	    -command {my-exec [xlsFuncQuoted "apropos" $helpCommand]}
} else {
    button .bottom.help -text Help: \
	    -command {my-exec [format "?%s" $helpCommand]}
    button .bottom.apropos -text Apropos -command {xlsFunc "about" $helpCommand}
}
entry .bottom.helpCommand -font $font -relief sunken -borderwidth 2 \
	-textvariable helpCommand
button .bottom.examples -text "Examples" \
	-state $disableState -command {xlsFunc "try-out" $helpCommand}
label .bottom.message -textvariable messageAlert \
	-width 30 -background white -anchor w

pack .bottom.help -in .bottom -side left
pack .bottom.message .bottom.apropos .bottom.examples -in .bottom -side right 
pack .bottom.helpCommand -in .bottom -expand yes -fill x
bind .bottom.helpCommand <Return> {.bottom.help invoke}
#
#########			Command frame:
#
# Create the Command entry window at the bottom of the window, and button:
#
frame .buttons -relief raised -borderwidth 2
pack .buttons -side bottom -fill x

entry .buttons.interact -font $font -relief sunken -borderwidth 2 \
	-textvariable interactiveCommand
button .buttons.dismiss -text "Dismiss" -command "destroy ."
button .buttons.selected -text "Execute Selected" -command "doSelected"
button .buttons.workspace -text "Execute Workspace" \
	-command {
    saveTexttoFile .input .asdf overwrite
    executeCommandsInFile .asdf
    file delete .asdf
}
button .buttons.commanddo -text "Hot Command:" \
	-command {my-exec $interactiveCommand}

pack .buttons.workspace .buttons.selected .buttons.commanddo \
	-in .buttons -side left
pack .buttons.interact -in .buttons -expand yes -fill x
bind .buttons.interact <Return> {
    my-exec $interactiveCommand; set interactiveCommand ""
}
#
#############################################################################
#
#		Now for the procs that commmunicates with xlispstat:
#
#############################################################################
#
#		my-exec
#
proc my-exec {command {newline ""}} {
    # The General Execution procedure
    global xls
    .output insert end [join [list $command "\n"] ""]
    if {$newline == ""} {
	puts $xls $command
    } else {
	puts -nonewline $xls $command
    }
    # waitingforGodot is collecting the output...
}
#
#		waitingforGodot - not sure how to spell Godot, but heh....
#
proc waitingforGodot {} {
    # this is a silly procedure that I've been forced to create: it keeps \
	    checking to see if any output becomes available (you'd think \
	    that it would become available at the time the command is \
	    issued, but no!). Seems like a waste, but I can't seem to get \
	    at it any other way....
    global xls
    flush $xls
    set tmp [read $xls]
    while {$tmp != ""} {
	.output insert end $tmp
	.output see end
	set tmp [read $xls]
    }
    # leaves instructions to fire itself back up in a bit \
	    (each tenth of a second)
    after 100 {waitingforGodot}
}
# Note: we now fire up the procedure:
waitingforGodot
#
#		xlsFunc
#
proc xlsFunc {function word} {
    # executes an xlispstat function and its argument:
    my-exec [format "(%s %s)" $function $word]
}
proc xlsFuncQuoted {function word} {
    # the quoted version of xlsFunc
    my-exec [format "(%s '%s)" $function $word]
}
#
#		parenthesisIndent
#
proc parenthesisIndent {parenthesisCount {win .input}} {
    # indents to the proper level, determined by the value of parenthesisCount:
    set i 0
    while {$i < $parenthesisCount} {
	if ![stringEqual [$win get insert "insert+1c"] " "] {
	    $win insert insert " "
	} else {
	    $win mark set insert "insert+1c"
	}
	incr i
    }
    # eat the excess:
    while {[stringEqual [$win get insert "insert+1c"] " "]} {
	$win delete insert
    }
}
#
#		executeCommandsInFile
#
proc executeCommandsInFile {file} {
    # this procedure reads the commands in a temporary file, then \
	    sends them to the lisp process.
    global xls
    set f [open $file]
    set i 0
    while {[gets $f line] >= 0} {
	set tmp [stripStringtoCommand $line]
	if {$tmp != ""} {
	    # according to my limited understanding of things, using the \
		    -nonewline option means that these commands are actually \
		    considered to be on the same line in the lisp process.
	    # in other words, I'm passing many lines, but they're all getting \
		    globbed together. This is why I have to continue stripping \
		    out the semicolon comment (;;) lines.
	    my-exec $tmp -nonewline
	    flush $xls
	    set tmp [read $xls]
	    while {$tmp != ""} {
		.output insert end $tmp
		.output see end
		set tmp [read $xls]
	    }
	}
    }
    my-exec ""
    close $f
}
#
#		removePromptAndExecute
#
proc removePromptAndExecute {command} {
    # checks a line to see if it starts by "> ", in which case that part \
	    gets stripped off.
    if [stringEqual [string range $command 0 1] "> "] {
	set start 2
    } else {
	set start 0
    }
    my-exec [string range $command $start end]
}
#
#		CatchWholeContainingCommand
#
proc CatchWholeContainingCommand {} {
    global messageAlert
    set messageAlert ""

    # Catch the whole surrounding parenthetical environment:
    set w .input

    # iback is the index of the initiating point:
    set iback [$w index insert]

    # now search for the ending parenthesis. Subtract one for each ")",\
	    and add one for each "(", until we reach 0:
    set sum 1
    set index "$iback"
    set i 0
    while {$sum != 0} {
	incr i
	if {$i >= 50} {
	    set messageAlert "Foreward Capture Failure"
	    after 1000 "set messageAlert \"Use Execute Selected\""
	    return
	}
	set i1 [$w search -forwards "(" $index end]
	set i2 [$w search -forwards ")" $index end]
	if {$i1==""} {
	    set i1 -1.0
	    set li1 -1
	} else {
	    set li1 [string length [$w get $index $i1]]
	}
	if {$i2==""} {
	    set i2 -1.0
	    set li2 -1
	} else {
	    set li2 [string length [$w get $index $i2]]
	}
	if [expr [expr $li1 == $li2] || [expr $i2 < 0]] {
	    return
	} elseif [expr [expr $li1 <= $li2] && [expr $li1 >= 0]] {
	    set sum [incr sum]
	    set index "$i1+1char"
	} else {
	    set sum [incr sum -1]
	    set index "$i2+1char"
	}
    }
    set rightIndex $index

    # now search for the beginning parenthesis. Subtract one for each "(",\
	    and add one for each ")", until we reach 0:
    set sum 1
    set index "$iback"
    set i 0
    while {$sum != 0} {
	incr i
	if {$i >= 50} {
	    set messageAlert "Backward Capture Failure"
	    after 1000 "set messageAlert \"Use Execute Selected\""
	    return
	}
	set i1 [$w search -backwards ")" $index 1.0]
	set i2 [$w search -backwards "(" $index 1.0]
	if {$i1==""} {set i1 -1.0}
	if {$i2==""} {set i2 -1.0}
	set li1 [string length [$w get $i1 $index]]
	set li2 [string length [$w get $i2 $index]]
	# puts "$i1 $i2 $li1 $li2 $sum"
	if [expr [expr $i1==-1.0] && [expr $i2==1.0]] {
	    if {$sum == 1} {
		set sum 0
		set index $i1
	    } else {
		return
	    }
	} elseif [expr [expr $li1 == $li2] || [expr $i2 < 0]] {
	    return
	} elseif [expr [expr $li1 <= $li2] && [expr $li1 >= 0]] {
	    set sum [incr sum]
	    set index "$i1-1char"
	} else {
	    set sum [incr sum -1]
	    if {$sum == 0} {
		set index "$i2"
	    } else {
		set index "$i2-1char"
	    }
	}
    }
    # index is now the index of the left paren:
    $w get $index $rightIndex
}
#
#		ParenthesisCheck
#
proc ParenthesisCheck {} {
    # Parenthesis matching procedure:
    global messageAlert patienceLevel
    set w .input
    set index insert
    set sum 1
    set i 0
    while {$sum != 0 && $i <= $patienceLevel} {
	incr i
	set i1 [$w search -backwards "(" $index 1.0]
	set i2 [$w search -backwards ")" $index 1.0]
	if {$i1==""} {set i1 -1.0}
	if {$i2==""} {set i2 -1.0}
	set li1 [string length [$w get $i1 $index]]
	set li2 [string length [$w get $i2 $index]]
	if {$i == $patienceLevel} {
	    set messageAlert "Undetermined match..."
	    return
	} elseif [expr [expr $i1==1.0] && [expr $i2==-1.0]] {
	    if {$sum == 1} {
		set sum 0
		set index $i1
	    } else {
		set messageAlert "Mismatched parentheses"
		return
	    }
	} elseif [expr [expr $li1 == $li2] || [expr $i1 < 0]] {
	    set messageAlert "Mismatched parentheses"
	    return
	} elseif [expr $li1 <= $li2] {
	    set sum [incr sum -1]
	    set index $i1
	} else {
	    set sum [incr sum]
	    set index $i2
	}
    }
    set messageAlert [join [list \
	    [string range [$w get  $index "$index lineend"] 0 25] "...)"]]
    $w see $index
}
#
#		howFarInToIndent
#
proc howFarInToIndent {} {
    global messageAlert patienceLevel
    set w .input
    set index insert
    set sum 1
    set i 0
    while {$sum != 0 && $i <= $patienceLevel} {
	incr i
	set i1 [$w search -backwards "(" $index 1.0]
	set i2 [$w search -backwards ")" $index 1.0]
	if {$i1==""} {set i1 -1.0}
	if {$i2==""} {set i2 -1.0}
	set li1 [string length [$w get $i1 $index]]
	set li2 [string length [$w get $i2 $index]]
	# puts "$sum $index $i1 $i2 $li1 $li2"
 	if {$i == $patienceLevel} {
	    return
	} elseif [expr [expr $i1==1.0] && [expr $i2==-1.0]] {
	    if {$sum == 1} {
		set sum [incr sum -1]
		set index 1.0
	    } else {
		return 0
	    }
	} elseif [expr [expr $li1 == $li2] || [expr $i1 < 0]] {
	    return
	} elseif [expr $li1 <= $li2] {
	    set sum [incr sum -1]
	    set index $i1
	} else {
	    set sum [incr sum]
	    set index $i2
	}
    }
    set messageAlert [join [list \
	    [string range [$w get  $index "$index lineend"] 0 25] "...)"]]
    return [expr [string length [$w get "$index linestart" $index]]+1]
}
#
#		trimComment
#
proc trimComment {value} {
    # trimComment procedure: \
	    This works in the simple obvious cases, but may break if I have a \
	    semicolon in a string extending over several lines (for example). \
	    Another problem would occur if someone followed a comment \
	    line by a #| or |# - it would be removed and not found in the next \
	    procedure (removeComment).
    set comment [split $value ";"]
    if {[evenp [repCount "\"" [lfirst $comment]]]} {
	return [lfirst $comment]
    } else {
	set new [lfirst $comment]
	set comment [lrest $comment]
	while {[evenp [repCount "\"" [lfirst $comment]]] \
		&& [expr [llength $comment] >= 2]} {
	    set new [join [list $new [lfirst $comment]] ";"]
	    set comment [lrest $comment]
	}
	if {[oddp [repCount "\"" [lfirst $comment]]] \
		&& [expr [llength $comment] == 1]} {
	    set new [join [list $new [lfirst $comment]] ";"]
	}
	return $new
    }
}
#
#		doSelected
#
proc doSelected {} {# to paste commands into xlispstat:
    # first see if the selection belongs to our application:
    if {[selection own] != ""} {
	# selections's ours: but is there really any selection? \
		There is if we succeed in setting the variable tmp:
	catch [set tmp [selection get]]
	if {[info exists tmp]} {
	    # we've succeeded, so execute the selected text:
	    saveStringtoFile $tmp .asdf overwrite
	    executeCommandsInFile .asdf
	    file delete .asdf
	}
    }
}
#
#		stripStringtoCommand
#
proc stripStringtoCommand {tmp} {
    #   Proc to remove comments from a string which is actually any number of \
	    lines of text, each line ending in \n. It returns the a single \
	    line of text, containing all the xlispstat commands joined together.
    set new ""
    foreach i [split $tmp \n] {
	if {$i != ""} {
	    set i [trimComment $i]
	    set new $new$i
	}
    }
    # set new [stringSnipOutMatchedPairs "#|" "|#" $new]
    return [stripEnds $new]
}
#
#		hyperCommands
#
set hyperCommands 0
proc updateTag {} {
    # For tagging left parentheses for hypercommands:
    global bold normal hyperCommands

    # increment the tag number, so that each is unique:
    incr hyperCommands
    set tag [join [list "" $hyperCommands ""] "_"]

    # create the tag:
    .input tag add $tag "insert-1chars" insert

    # now bind the usual stuff to it:
    .input tag bind $tag <Any-Enter> ".input tag configure $tag $bold"
    .input tag bind $tag <Any-Leave> ".input tag configure $tag $normal"
    .input tag bind $tag <1> {
	%W mark set insert "@%x,%y + 1c"
	my-exec [stripStringtoCommand [CatchWholeContainingCommand]]
    }
}
#
#		loadFile
#
proc loadFile {dir} {
    # To load a file into xlispstat
    set file [chooseFile $dir]
    if {$file != ""} {
	my-exec [format "(load %s :print t)" \"$file\"]
    }
}
#
#		saveFile
#
proc saveFile {} {
    global saveFile
    set saveFile [genericEntry "" "Filename in which to save text:"]
    saveTexttoFile .input $saveFile
}
#
#		loadRGM
#
proc loadRGM {dir} {
    # To load an RGM file into xlispstat
    global RGM
    loadFile [join [list $RGM $dir] "/"]
}
#
#		loadModules
#
proc loadModules {} {
    # To load a module into xlispstat:

    # create a generic box listing the modules available:
    set listBox [genericListBox {"RGM" "ViSta"} .gnames "Choose Module:"]

    # then bind a double click to load the module:
    bind $listBox <Double-1> {
	set tmp [join [selection get] ""]
	set tmp2 [join [list "\"/calculus/" $tmp "load\"" ] ""]
	set tmp3 \
		[join \
		[list "(load (concatenate 'string *default-path* " $tmp2 "))"] \
		""]
	my-exec $tmp3
	destroy .gnames
    }
}
#
#		Fonts
#
proc loadFonts {} {
    global font
    # To change font size

    # create a generic box listing the modules available:
    set listBox [genericListBox {"10" "12" "14" "17" "20"} .gfonts "Choose Module:"]

    # then bind a double click to load the module:
    bind $listBox <Double-1> {
	set font [join [selection get] ""]
	set font [join [list "*-Courier-Bold-R-Normal-*-" $font "*" ] ""]
	destroy .gfonts
	.input configure -font $font
	.output configure -font $font
    }
}
#
#		changeSize
#
proc changeSize {} {
    global inputHeight outputHeight
    # To change window size

    # create a generic box listing the modules available:
    set inputHeight  [genericEntry "$inputHeight" "Lines in input window:"]
    set outputHeight [genericEntry "$outputHeight" "Lines in output window:"]
    if {$inputHeight == ""} {
	set inputHeight 17
    }
    if {$outputHeight == ""} {
	set outputHeight 17
    }
    .input configure -height $inputHeight
    .output configure -height $outputHeight
}
#
#		hotList
#
proc hotList {} {
    # The hotlist window procedure:
    global HLinitialized listBox

    # we now create a generic box with the contents of :
    set listBox [genericListBox $HLinitialized .gnomes "Hot List:"]

    # then bind a double click to execute the selected line:
    bind $listBox <Double-1> {
	my-exec [join [selection get] " "]
    }
    # default binding of third double deletes entry
    bind $listBox <Double-3> {
	global listBox
	$listBox delete @%x,%y
	set HLinitialized [$listBox get 0 end]
    }
}
#
#		aboutMessage
#
proc aboutMessage {name} {
    # "About" Messages
    switch $name {
	bugs {
	    set message "
    - Don't nest comments of the form
	#| -- comment -- |#
I.e., don't do this:
	#| #| hi |# |#
(unless you like error messages).

    - Undo doesn't put tags back into place (tagged text loses its tags); it's also not an \"infinite\" undo: you undo something, it's undone! Gone. Parti.

    - semicolons in strings may be treated as indicating comments in certain cases, which could lead to them (and following characters) being removed by accident (OOPS!). This is a BAD THING, which should be fixed someday. For example if you have a string which covers several lines, e.g.
\"
;
\"
You'll be in trouble! Since text is treated line by line, there's nothing to suggest that the semicolon isn't a comment, to be removed...

    - you might notice that the output window isn't quite like the usual output window (xterm): in particular, the \">\" doesn't appear until the output comes out. This seems to be a \"feature\" of the various flavors of the read function.
"}
	emacs {
	    set message "
	Most of the normal emacs bindings are in place. Notes:

    - C-u	undo, rather than repeat command. But it's not an infinite undo: if you undo something you wanted, you can't get it back! Bummer...
    - C-xC-g 	go to line number (supplied by you!)
    - C-xC-x	go to mark - additional C-x flits between the two points..
    - C-xC-r	execute region
    - C-cC-e	execute command containing cursor
"
	} GUI {
	    set message "
    This GUI interface to xlispstat and Ripon Good Math (RGM) was created to provide students an interface under UNIX more similar to the Windows and Mac interfaces.

    It is written in Tcl/Tk by Andy Long in 1998, while a Fulbright scholar at the Institute de Mathematiques et Sciences Physiques in Benin, W. Africa. It has been tested using versions tcl 7.6 and tk/wish 4.2, as well as version 8.0 of both. The GUI and RGM are in the public domain; xlispstat is freely available. Be Free!

    You are encouraged to improve this GUI, but please let the author know what you've done so that he can improve too! He can be contacted at:
	ael@maya.medctr.luc.edu
or see his web page:
	http://maya.medctr.luc.edu/~ael 
"
	} Hotlist {
	    set message "
    The hot list is provided for quick access to commands that you find yourself using frequently. It is initialized from a file called either xls.hot or rgm.hot (by default), depending on whether RGM is defined or not. This file is found in the same directory as the xls file.

    Initial commands in the hot file should be written within double quotes, e.g. \"(defun f(x) (sin x))\".

    Double clicking the first mouse button on an entry executes it; double clicking the third mouse button on an entry deletes it from the list.
"
	} links {
	    set message "
    You might notice that the left parentheses (\"(\") you type are highlighted as the mouse passes over them; if you now click on them with the left (first) mouse button, the next complete lisp expression starting from that parenthesis will be sent to the xlispstat process to evaluate.

    The words \"Input Workspace\" also serve as a link: clicking on it will clear the input workspace.

    If RGM is installed, then there should be several other linked lines in the initial input workspace, linking to some interesting demo files.
"
	} misc {
	    set message "
    - Double clicking the third mouse button on a line executes that line.

    - Typing Control-m (that is, the control and m keys simultaneously) and Control-M determine the ratio of input to output window. Try them!

    - The input workspace can be cleared by clicking the first mouse button on the words \"Input Workspace\"

    - Notice that, as you type left parentheses, they become links: clicking on a highlighted left parenthesis will execute the command which ends with the right parenthesis which closes it.
"
	} "Hot Command" {
	    set message "
The hot command button exists for two reasons:

    - It allows one to interact quickly with the xlispstat session (for example, sometimes it is necessary to type a response to an xlispstat function) without worrying about whether return is active or not.

    - If there is a command that you're typing over and over, you can put it into this entry box and simply punch the button to execute it.
"
	} "Active Return" {
	    set message "
    Return is either active or not: that is, either hitting carriage return in the input window sends the command on the given line directly to xlispstat or it doesn't.

    By default return is inactive, so that as you are simply editting your own text which can be sent to the lisp process later. You can use selections to send your commands to the xlispstat process; type control-return to send a line; control-c followed by control-e executes the command enclosing the insertion cursor. You can also send individual lines using the third mouse button: double-clicking on a line sends that line to the process. Try it!

    Control-Return will always result in active return, regardless of the state of active return. In addition, no carriage return will appear in the input workspace.
"
	} "reScaling windows" {
	    set message "	Use control-m and control-M to adjust the relative proportions assigned to the input and output windows.

	If you want to have more lines (you have a large monitor, for example) you need to change the values of inputHeight and outputHeight (they are initially set at 17). You can do this from the File menu.
"
	} "Ripon Good Math" {
	    set message "
	\"Ripon Good Math\" (RGM) began as a calculus lab to aid students at Ripon College, Ripon, WI, USA. It has developed beyond that, however, and now provides many important advantages to anyone using xlispstat, including

   * more natural (infix) function definition;
   * easy plotting of functions, matrices, vectors, and lists;
   * expression simplification;
   * plots save as postscript files;
   * hidden-surface generation;
   * fortran-to-lisp code translation, so that a wide body of code can be easily ported to xlispstat; and more.

	A tutorial is included, accessed by typing the command (tutorial). Also the Tierney book code has been re-written to be run from the command line using the command (book). There is also a graphical help menu, (hulp).

	RGM is available from Andy Long (ael@maya.medctr.luc.edu), accessible from his home page (http://maya.medctr.luc.edu/~ael). With RGM installed, there should be no disabled menus/buttons; otherwise, certain will, by necessity, be disabled."
	} todo {
	    set message "
Still to be carried out:

    *) get additional emacs bindings working (I've got most of them to my satisfaction, although some of them have changed)

    *) Improve undo, turning it into an infinite undo.
"
	}
   }
   set name [join [list "About " $name] ""]
   genericMessageBox $message $name
}
#
###############################################################################
#
# Set up bindings for everyone:
#
#
bind all <Alt-c> {
    tk_textCopy %W
}
bind all <Alt-v> {
    set tmp [.input tag ranges sel]
    if {$tmp != ""} {
	.input delete [lindex $tmp 0] [lindex $tmp 1]
    }
    tk_textPaste %W
}
bind all <Alt-x> {
    tk_textCut %W
}
# my top secret escape mechanism:
bind all <Escape><Escape> {.buttons.dismiss invoke}
#
#######################################################################
#
######		Initialize the neon search option for the two windows:
#
setupNeon {.input .output}
#
######		Special Key Bindings (lots of emacs-type stuff):
#
source $xlsdir/tkedit.tcl
#
######		Start with the focus on the input window:
#
focus .input
#
######				The end
